<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * The RSDirectory controller.
 */
class RSDirectoryController extends JControllerLegacy
{
    /**
     * The class constructor.
     *
     * @access public
     */
    public function __construct($config = array())
    {
		parent::__construct($config);
			
		JToolBarHelper::title('RSDirectory!', 'rsdirectory');
			
		if ( !RSDirectoryConfig::getInstance()->get('credit_cost') && RSDirectoryCredits::requiresCredits() )
		{
			JFactory::getApplication()->enqueueMessage( JText::_('COM_RSDIRECTORY_EMPTY_CREDIT_COST_VALUE_WARNING'), 'warning' );	
		}
    }
	
	public function installsampledata() {
		$db		= JFactory::getDbo();
		$user 	= JFactory::getUser(); // get the logged user data
		$sample = JPATH_COMPONENT.'/assets/sample/sample.sql'; //  the src where the sample data sql file is located
		
		$ids = array(); //  store the id's that we need for parsing
		
		if ($buffer = file_get_contents($sample)) {
			$parts = explode('--#', $buffer); // split the data so that we can manage it more easy
			$dataQ = $this->setQueries($parts);
			
			// the tables needed to store the id's
			$saveIds = array('entries', 'fields', 'uploaded_files', 'forms');
			// the names of the three pictures of the entries in the specific order
			$uploaded_files = array('the-audi-r8-v10-explored.jpg', 'aventador-matte-black.jpg', 'bugatti-veyron-grand-sport.jpg'); 
			
			require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
			
			foreach($dataQ as $type => $queriesBuffer) {
				if ($queries = $db->splitSql($queriesBuffer)) {
					$z = 0;
					foreach ($queries as $i => $query) {
						$query = trim($query);
						if ($query != '') {
							if ($type == 'forms' && RSDirectoryHelper::isJ4()) {
								// replace the rows and cols
								$query = str_replace(array("row-fluid", "span6", "span12"), array('row', 'col-md-6', 'col-md-12'), $query);
							}
							else if ($type == 'entries') {
								// replace the shortcode for the user id
								$query = str_replace("'{rsdir-user_id}'", $user->id, $query);

								if (RSDirectoryHelper::isJ4()) {
									// replace the rows and cols
									$query = str_replace(array("row-fluid", "span6", "span12"), array('row', 'col-md-6', 'col-md-12'), $query);
								}
							}
							else if ($type == 'entries_custom') {
								// replace the shortcode for the user id
								$query = str_replace("'{rsdir-user_id}'", $user->id, $query);
								
								// replace the shortcode for the field_id
								foreach ($ids['fields'] as $key => $id) {
									$query = str_replace("{rsdir-sample-field_id-".$key."}", $id, $query);
								}
								
								// replace the shortcode for the entry_id
								foreach ($ids['entries'] as $key => $id) {
									$query = str_replace("'{rsdir-sample-entry_id-".$key."}'", $id, $query);
								}
								
							} 
							else if ($type == 'aleter_entries_custom') {
								// replace the shortcode for the field_id
								foreach ($ids['fields'] as $key => $id) {
									$query = str_replace("{rsdir-sample-field_id-".$key."}", $id, $query);
								}
							}
							else if ($type == 'fields_properties') {
								// replace the shortcode for the field_id
								foreach ($ids['fields'] as $key => $id) {
									$query = str_replace("'{rsdir-sample-field_id-".$key."}'", $id, $query);
								}
							} 
							else if ($type == 'fields_dependencies') {
								// replace the shortcode for the field_id (only the 22 and 23 key of the array that stores the fields id's)
								$query = str_replace("'{rsdir-sample-field_id-22}'", $ids['fields'][22], $query);
								$query = str_replace("'{rsdir-sample-field_id-23}'", $ids['fields'][23], $query);
							} 
							else if ($type == 'uploaded_files') {
								// create file hash
								$fileHash = RSDirectoryHelper::getHash();
								$src_dir = JPATH_ADMINISTRATOR . '/components/com_rsdirectory/assets/sample/images';
								$dst_dir = RSDirectoryHelper::createEntryDir($ids['entries'][$i]);
								// check if the directory is created
								if ($dst_dir) {
									// copy the file and replace the specific shorcodes
									if (JFile::copy("$src_dir/$uploaded_files[$i]", "$dst_dir/$fileHash.jpg")) {
										$query = str_replace("'{rsdir-user_id}'", $user->id, $query);
										$query = str_replace("{rsdir-picture-hash}", $fileHash, $query);
									} else {
										continue;
									}
								} else {
									continue;
								}
							} 
							else if ($type == 'uploaded_files_fields_relations') {
								// replace the shortcode for the file_id
								foreach ($ids['uploaded_files'] as $key => $id) {
									$query = str_replace("'{rsdir-sample-file_id-".$key."}'", $id, $query);
								}
								// replace the shortcode for the entry_id
								foreach ($ids['entries'] as $key => $id) {
									$query = str_replace("'{rsdir-sample-entry_id-".$key."}'", $id, $query);
								}
								
							} 
							else if ($type == 'forms_custom_fields' || $type == 'forms_fields') {
								// replace the shortcode for the eform_id
								$query = str_replace("'{rsdir-sample-form_id-0}'", $ids['forms'][0], $query);
								
								// replace the shortcode for the field_id
								foreach ($ids['fields'] as $key => $id) {
									$query = str_replace("'{rsdir-sample-field_id-".$key."}'", $id, $query);
								}
							}
							else if ($type == 'fields') {
								// check if we need to update the new stored field
								$replace_id = stripos($query, 'f_{rsdir-this-id}');
							}

							if (RSDirectoryHelper::isJ4() && $type == 'entries_custom') {

								$entries_custom_columns = $db->getTableColumns('#__rsdirectory_entries_custom');
								$old_columns = array();
								foreach($entries_custom_columns as $ec_column => $c_type)
								{
									if (strpos($query, '`'.$ec_column.'`') === false)
									{
										// add the default value
										$proper_value = "''";
										if (stripos($c_type, 'int') !== false || stripos($c_type, 'float') !== false)
										{
											$proper_value  = 0;
										}
										else if (stripos($c_type, 'datetime') !== false)
										{
											$proper_value  = $db->getNullDate();
										}

										$old_columns['`'.$ec_column.'`'] = $proper_value;

									}
								}

								if (!empty($old_columns))
								{
									$old_columns_names = array_keys($old_columns);
									$old_columns_names = implode(', ', $old_columns_names);
									$old_columns_values = implode(",", $old_columns);
									$query = str_replace(') VALUES', ', '.$old_columns_names.') VALUES', $query);
									$query = str_replace('),', ', '.$old_columns_values.'),', $query);
									$query = str_replace(');', ', '.$old_columns_values.');', $query);
								}
							}

							$db->setQuery($query);
							$db->execute();
							if (in_array($type, $saveIds)) {
								$insertid = $db->insertid();
								if ($type == 'fields') {
									$ids[$type][($z + 8)] = $insertid;
									$z++;
								} else {
									$ids[$type][] = $insertid;
								}
								
								// update the stored field if needed
								if ($type == 'fields') {
									if (is_numeric($replace_id)) {
										$query = $db->getQuery(true)
											   ->update( $db->qn('#__rsdirectory_fields') )
											   ->set( $db->qn('column_name') . ' = ' . $db->q('f_'.$insertid) )
											   ->where( $db->qn('id') . ' = '. $db->q( $insertid ));
											   
										$db->setQuery($query);
										$db->execute();
									}
								}
							}
						}
					}
				}
			}
		}
		// set the category and update the entries category_id
		$this->setCategories($ids); 
		$this->setRedirect('index.php?option=com_rsdirectory&view=entries',JText::_('COM_RSDIRECTORY_SAMPLE_DATA_INSTALLED'));
	}
	
	protected function setQueries($parts) {
		$data = array();
		foreach ($parts as $i => $type) {
			$next = $i + 1;
			if ($i%2 == 0 && isset($parts[$next])) {
				$data[$type] = $parts[$next];
			}
		}
		
		return $data;
	}
	
	protected function setCategories($ids) {
		$user = JFactory::getUser();
        $db = JFactory::getDbo();
            
        // Create the sample category.
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_categories/tables');
		$categoryPrefix = RSDirectoryHelper::isJ4() ? 'JTable' : 'CategoriesTable';
		$category = JTable::getInstance('Category', $categoryPrefix);
            
        $category->id = null;
        $category->title = 'Vehicle';
        $category->alias = 'vehicle';
        $category->extension = 'com_rsdirectory';
        $category->setLocation(1, 'last-child');
        $category->created_user_id = $user->id;
        $category->language = '*';
        $category->published = 1;
            
        $params = new JRegistry( array('form_id' => $ids['forms'][0]) );
            
        $category->params = $params->toString();
            
        $category->store();
        $category->rebuildPath($category->id);
        $category->rebuild($category->id, $category->lft, $category->level, $category->path);
            
            
        // Update entries.
		foreach ($ids['entries'] as $id) {
			$query = $db->getQuery(true)
				   ->update( $db->qn('#__rsdirectory_entries') )
				   ->set( $db->qn('category_id') . ' = ' . $db->q($category->id) )
				   ->set( $db->qn('form_id') . ' = ' . $db->q($ids['forms'][0]) )
				   ->set( $db->qn('created_time') . ' = ' . $db->q( JFactory::getDate()->toSql() ) )
				   ->set( $db->qn('published_time') . ' = ' . $db->q( JFactory::getDate()->toSql() ) )
				   ->where( $db->qn('id') . ' = '. $db->q( $id ));
				   
			$db->setQuery($query);
			$db->execute();
		}
	}
}