<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access'); 

/**
 * Email sender class.
 */
class RSDirectoryEmail
{
    /**
     * The email type.
     *
     * @var string
     * 
     * @access protected
     */
    protected $email_type;
        
    /**
     * Form fields.
     *
     * @var array
     * 
     * @access protected
     */
    protected $form_fields;
        
    /**
     * Entry data.
     *
     * @var object
     * 
     * @access protected
     */
    protected $entry;
        
    /**
     * Parent form.
     *
     * @var object
     *
     * @access protected
     */
    protected $form;
	
	// array that holds the placeholders as keys and the values of those placeholders as the array values
	protected $custom_fields;
        
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param string $email_type
     * @param array $form_fields
     * @param object $entry
     * @param object $form
     */
	
    public function __construct($email_type, $form_fields, $entry, $form, $custom_fields = null)
    {
        $this->email_type = $email_type;
        $this->form_fields = $form_fields;
        $this->entry = $entry;
        $this->form = $form;
		$this->custom_fields = $custom_fields;
    }
        
    /**
     * Get RSDirectoryEmail instance.
     *
     * @access public
     * 
     * @static
     * 
     * @param string $email_type
     * @param array $form_fields
     * @param object $entry
     * @param object $form
     * 
     * @return RSDirectoryEmail
     */
    public static function getInstance($email_type, $form_fields, $entry, $form, $custom_fields = null)
    {
        $rsdirectoryemail = new RSDirectoryEmail($email_type, $form_fields, $entry, $form, $custom_fields);
            
        return $rsdirectoryemail;
    }
        
    /**
     * Get the email based on the entry's parent category and email type.
     *
     * @access private
     *
     * @return mixed
     */
    private function getEmail()
    {
        if (!isset($this->entry->skip_entry)) {
            // Initialize the category id.
            $category_id = $this->entry->category_id;

            if (!$category_id)
                return false;


             static $categories;
        }
        static $emails;
            
        $email_type = $this->email_type;
            
        if ( empty($categories) && !isset($this->entry->skip_entry))
        {
            $categories = RSDirectoryHelper::getCategories();
        }
            
        if ( empty($emails) )
        {
            // Get DBO.
            $db = JFactory::getDbo();
                
            $query = $db->getQuery(true)
                   ->select('*')
                   ->from( $db->qn('#__rsdirectory_email_messages') )
                   ->where( $db->qn('published') . ' = ' . $db->q(1) );
                   
            $db->setQuery($query);
                
            $emails = $db->loadObjectList();
        }

        $emails_found = array();

        if (isset($this->entry->skip_entry)) {
            foreach ($emails as $email)
            {
                if ($email_type == $email->type)
                    $emails_found[] = $email;
            }

            return empty($emails_found) ? false : $emails_found;
        }
            
        // Get the category hierarchy.
        $hierarchy = array_reverse( RSDirectoryHelper::getCategoryHierarchy($category_id, $categories) );


        while (true)
        {
            foreach ($emails as $email)
            {
                if ($email_type == $email->type && $email->category_id == $category_id)
                   $emails_found[] = $email;
            }

            if (!empty($emails_found)) return $emails_found;

            $category = next($hierarchy);
                
            if ($category)
            {
                $category_id = $category->id;
            }
            else
            {
                if (!$category_id)
                    return false;
                    
                $category_id = 0;
            }
        }
    }
        
    /**
     * Process and send email.
     *
     * @access public
     *
     * @return bool
     */
    public function send()
    {
        // Get the email.
        $emails = $this->getEmail();
	
        if (!$emails || empty($emails))
            return false;
            
        require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/placeholders.php';

        $sent_emails = array();

        try {
            foreach ($emails as $email) {
                // Process placeholders.
                $to_name = $this->processPlaceholders($email->to_name);
                $to_email = $this->processPlaceholders($email->to_email);
                $cc = $this->processPlaceholders($email->cc);
                $bcc = $this->processPlaceholders($email->bcc);
                $subject = $this->processPlaceholders($email->subject);
                $message = $this->processPlaceholders($email->text);

                // Get mailer.
                $mailer = JFactory::getMailer();

                // Get the RSDirectory! config object.
                $config = RSDirectoryConfig::getInstance();

                // Set sender.
                $mailer->setSender(array($config->get('from_email'), $config->get('from_name')));

                $to_name = explode(',', $to_name);
                $to_email = explode(',', $to_email);


                // check emails
                foreach ($to_email as $key => $email_check) {
                    if (!JMailHelper::isEmailAddress($email_check)) {
                        unset($to_email[$key]);
                    }
                }
                if (count($to_email) == 0) {
                    continue;
                }

                $count_names = count($to_name);
                $count_emails = count($to_email);

                if ($count_emails != $count_names) {
                    // We'll just assume that "to_email" is the correct field.
                    if ($count_emails > $count_names) {
                        for ($i = $count_emails - $count_names; $i < $count_emails; $i++) {
                            $to_name[$i] = $to_email[$i];
                        }
                    } else {
                        // Can't figure out the correct emails - truncate names instead.
                        $to_name = array_slice($to_name, 0, $count_emails);
                    }
                }

                // Add recipient(s).
                $mailer->addRecipient(
                    $to_email,
                    $to_name
                );

                // Set CC.
                if ($cc) {
                    $mailer->addCC(explode(',', $cc));
                }

                // Set BCC.
                if ($bcc) {
                    $mailer->addBCC(explode(',', $bcc));
                }

                // Set subject.
                $mailer->setSubject($subject);

                // Set body.
                $mailer->setBody($message);

                // Send as HTML.
                $mailer->isHtml($email->send_html);

                $send_status = $mailer->Send();
                $sent_emails[] = (is_bool($send_status) && $send_status) ? 1 : 0;
            }
        }
        catch (Exception $e)
        {
            JFactory::getApplication()->enqueueMessage($e->getMessage(), 'warning');
        }

        return in_array(1, $sent_emails);

    }
        
    /**
	 * Process placeholders.
	 *
	 * @access protected
	 *
	 * @param string $text
	 * 
	 * @return string
	 */
	protected function processPlaceholders($text)
	{
		// Process the regular placeholders.
		$text = RSDirectoryPlaceholders::getInstance($text, $this->form_fields, $this->entry, $this->form, $this->custom_fields)
              ->setParams('email')
              ->process();
         
		return $text;
	}
}