<?php
/**
 * @package    RSDirectorys!
 *
 * @copyright  (c) 2010 - 2021 RSJoomla!
 * @link       https://www.rsjoomla.com
 * @license    GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */
defined('_JEXEC') or die('Restricted access');

abstract class RsdirectoryExport
{
    public static function buildCSV($form_id, $data, $fileHash = '')
    {
        // accepted headers and keys
        $accepted_headers = array(
            'published',
            'title',
            'category_id',
            'category',
            'form',
            'form_id',
            'author_id',
            'author',
            'created_time',
            'published_time',
            'modified_time',
            'expiry_time',
            'promoted',
            'renew',
            'paid',
            'id',
            'availability',
        );

        $form_fields = RSDirectoryHelper::getFormFields($form_id);

        // ignore all above and section break
        $ignored_types = $accepted_headers;
        $ignored_types[] = 'section_break';

        $rows = '';
        if ($fileHash == '')
        {
            $headers = array();
            foreach ($accepted_headers as $header)
            {
                $headers[] =   JText::_('COM_RSDIRECTORY_'.strtoupper($header));
            }

            foreach ($form_fields as $field) {
                if (!$field->create_column) {
                    continue;
                }

                if (in_array($field->field_type, $ignored_types)) {
                    continue;
                }

                if ($field->field_type == 'map') {
                    $headers[] = JText::sprintf('COM_RSDIRECTORY_MAP_FIELD_EXPORT_COLUMN_ADDRESS', $field->form_field_name) ;
                    $headers[] = JText::sprintf('COM_RSDIRECTORY_MAP_FIELD_EXPORT_COLUMN_LAT', $field->form_field_name);
                    $headers[] = JText::sprintf('COM_RSDIRECTORY_MAP_FIELD_EXPORT_COLUMN_LNG', $field->form_field_name);
                    continue;
                }

                $headers[] = $field->form_field_name;

            }

            // Add header to rows
            $rows .= '"'.implode('","', $headers).'"'."\n";
        }

        // load the dbo object
        $db  = JFactory::getDbo();

        $use_date_time_display = RSDirectoryConfig::getInstance()->get('export_use_date_time_display', 0);

        // Add the data to rows
        foreach ($data as $i => $entry)
        {
            $row = array_flip($accepted_headers);

            $query   = $db->getQuery(true)
                ->select('*')
                ->from($db->qn('#__rsdirectory_entries_custom'))
                ->where($db->qn('entry_id') . '=' . $db->q($entry->id));

            $custom_values = $db->setQuery($query, 0, 1)->loadObject();

            // remake the array with the fields that we need to output and remodel for a human readable format
            foreach ((array) $entry as $key => $value)
            {
                //skip if is not supposed to be added
                if (!in_array($key,  $accepted_headers))
                {
                    continue;
                }

                switch ($key)
                {
                    case 'created_time':
                    case 'published_time':
                    case 'modified_time':
                    case 'expiry_time':
                        if (!is_null($value) && $value != $db->getNullDate())
                        {
                            $value = RSDirectoryHelper::formatDate($value, ($use_date_time_display ? null : 'Y-m-d H:i:s'));
                        }
                        else
                        {
                            $value = '-';
                        }

                        $row[$key] = $value;
                    break;

                    case 'category_id':
                        $row[$key] = $value;
                        $row['category'] = self::getCategory($value);
                    break;

                    case 'promoted':
                    case 'published':
                    case 'renew':
                    case 'paid':
                        $row[$key] = $value ? JText::_('JYES') : JText::_('JNO');
                    break;

                    default:
                        $row[$key] = $value;
                    break;
                }
            }

            foreach ($form_fields as $field) {
                if (!$field->create_column) {
                    continue;
                }

                if (in_array($field->field_type, $ignored_types)) {
                    continue;
                }

                if ($field->field_type == 'map') {
                    $row[$field->form_field_name.'_address'] = $custom_values->{$field->column_name.'_address'};
                    $row[$field->form_field_name.'_lat'] = $custom_values->{$field->column_name.'_lat'};
                    $row[$field->form_field_name.'_lng'] = $custom_values->{$field->column_name.'_lng'};
                    continue;
                }

                // handle date fields
                if ($field->field_type == 'dropdown_date_picker' || $field->field_type == 'calendar') {
                    $custom_values->{$field->column_name} = RSDirectoryHelper::formatDate($custom_values->{$field->column_name}, ($use_date_time_display ? null : 'Y-m-d H:i:s'));
                }

                $row[$field->form_field_name] = $custom_values->{$field->column_name};

            }

            array_walk($row, array('RsdirectoryExport', 'fixValue'));

            $rows .= '"'.implode('","',$row).'"';
            $rows .="\n";
        }

        return $rows;
    }

    protected static function getCategory($cat_id)
    {
        static $titles = array();

        if (!isset($titles[$cat_id]))
        {
            $category        = RSDirectoryHelper::getCategory($cat_id);
            $titles[$cat_id] =  $category->title;
        }

        return $titles[$cat_id];
    }

    public static function fixValue(&$string, $key)
    {
        if (strlen($string) && in_array(substr($string, 0, 1), array('=', '+', '-', '@')))
        {
            $string = ' ' . $string;
        }

        $string = str_replace(array('\\r', '\\n', '\\t', '"'), array("\015","\012","\011", "'"), $string);
    }

    public static function writeCSV($form_id, $query, $totalItems, $from, $fileHash, $filename)
    {
        if (!is_writable(JFactory::getConfig()->get('tmp_path'))) {
            throw new Exception(JText::sprintf('COM_RSDIRECTORY_TMP_PATH_NOT_WRITABLE', JFactory::getConfig()->get('tmp_path')));
        }

        $export_limit = RSDirectoryConfig::getInstance()->get('export_limit', 100);

        $db	= JFactory::getDbo();
        $db->setQuery($query, $from, $export_limit);
        $data = $db->loadObjectList();

        $fileContent = RsdirectoryExport::buildCSV($form_id, $data, $fileHash);

        // build the file hash if not already created
        if (!$fileHash) {
            $now 		= JHtml::date('now','D, d M Y H:i:s');
            $date 		= JHtml::date('now','Y-m-d_H-i-s');
            $filename 	= $filename.'-'.$date.'.csv';
            $fileHash 	= md5($filename.$now);
        }

        // create or append the hashed file and put content
        if ($fileContent) {
            if (!file_put_contents(self::getTmpPath($fileHash), $fileContent, FILE_APPEND)) {
                throw new Exception(JText::sprintf('COM_RSDIRECTORY_COULD_NOT_EXPORT_CSV_PATH', self::getTmpPath($fileHash)));
            }
        } else {
            throw new Exception(JText::_('COM_RSDIRECTORY_EXPORT_NO_DATA'));
        }

        $newFrom 		= ($from + $export_limit);
        $checkRemaining = $totalItems - $newFrom;

        return (object) array(
            'newFrom' 	=> ($checkRemaining > 0 ? $newFrom : $totalItems),
            'fileHash'	=> $fileHash
        );
    }

    public static function getCSV($fileHash)
    {
        $file 		= self::getTmpPath($fileHash);
        $content 	= is_file($file) ? file_get_contents($file) : '';
        return $content;
    }

    protected static function getTmpPath($fileHash)
    {
        return JFactory::getConfig()->get('tmp_path').'/'.$fileHash;
    }

    public static function buildCSVHeaders($filename)
    {
        // disable caching
        $now = JHtml::date('now','D, d M Y H:i:s');
        $date = JHtml::date('now','Y-m-d_H-i-s');
        $filename = $filename.'-'.$date.'.csv';

        header("Expires: Tue, 03 Jul 2001 06:00:00 GMT");
        header("Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate");
        header("Last-Modified: ".$now." GMT");

        // force download
        header("Content-Type: application/force-download");
        header("Content-Type: application/octet-stream");
        header("Content-Type: application/download");

        // disposition / encoding on response body
        header("Content-Disposition: attachment;filename=".$filename);
        header("Content-Transfer-Encoding: binary");
    }

}