<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

class RSDirectoryFieldHelper
{
    public static function showCustomField($field, $selected = array(), $group ='', $return_field = false)
    {
        if (empty($field) || empty($field->type))
        {
            return false;
        }

        // clean the group
        if (strlen($group))
        {
            $group = RSDirectoryHelper::escapeHTML($group);
        }

        $jform = isset($field->jform) && strlen($field->jform) ? RSDirectoryHelper::escapeHTML($field->jform) : 'jform';

        // remove the jform, because we don't need it
        if (isset($field->jform))
        {
            unset($field->jform);
        }

        // set the name attribute of the field
        $name = strlen($group) ? $jform.'['.$group.']['.$field->name.']' : $jform.'['.$field->name.']';

        // the multiple attribute must be set
        $field->multiple = !empty($field->multiple) ? true : false;

        // set the default value
        $field->default = !isset($field->default) ? '' : $field->default;

        // Create the XML
        $xml = new SimpleXMLElement('<field></field>');

        // set the value
        if (isset($field->value))
        {
            if ($field->multiple)
            {
                $selected[$field->name] = self::explode($field->value);
            }
            else
            {
                $selected[$field->name] = $field->value;
            }
        }

        // if the field has options we need to build them
        if (isset($field->has_options) && $field->has_options)
        {
            self::getOptions($field, $xml, $selected, $values);

            if (!empty($values))
            {
                $value = $field->multiple ? $values : implode('',$values);
            }
            else
            {
                $value = '';
            }

            $value = isset($selected[$field->name]) ? $selected[$field->name] : $value;
        }
        else
        {
            $value = isset($selected[$field->name]) ? $selected[$field->name] : '';
        }

        // if the field is multiple add to the name the square brackets
        if (isset($field->multiple) && $field->multiple)
        {
            $name.= '[]';
        }

        // overwrite the name if provided
        if (isset($field->forcename) && strlen($field->forcename))
        {
            $name = $field->forcename;
        }

        // add to the XML the other attributes
        self::setXmlAttributes($field, $xml);

        // build the Field Options array
        $options = array();
        if (isset($field->controlgroupclass) && $field->controlgroupclass)
        {
            $options['class'] = $field->controlgroupclass;
        }

        // render the field
        if ($jfield = JFormHelper::loadFieldType($field->type))
        {
            // add the name
            $xml->addAttribute('name', $name);

            $jfield->setup($xml, $value);

            if ($return_field)
            {
                return $jfield;
            }

            $html = $jfield->renderField($options);

            // add the additional if is set
            if (isset($field->additional) && strlen($field->additional))
            {
                if (strpos($html, '<input'))
                {
                    $html = str_replace('<input', '<input ' . $field->additional, $html);
                }
                else if (strpos($html, '<textarea'))
                {
                    $html = str_replace('<textarea', '<textarea ' . $field->additional, $html);
                }
                else if(strpos($html, '<select'))
                {
                    $html = str_replace('<select', '<select ' . $field->additional, $html);
                }
            }

            if (isset($field->group_additional) && strlen($field->group_additional))
            {

                $pos = strrpos($html, '</div>');

                if($pos !== false)
                {
                    $html = substr_replace($html, $field->group_additional.'</div>', $pos, strlen($html));
                }

            }

            return $html;
        }

        return '';
    }

    /**
     * Helper Function to set the XML attributes, used for better parsing
     *
     * @param $field - the field that is parsed
     * @param $xml - the XML that the attributes are set via reference
     */
    protected static function setXmlAttributes($field, &$xml)
    {
        $attributes_to_set = array(
            'required'      => true,
            'readonly'      => true,
            'disabled'      => true,
            'multiple'      => 'multiple',
            'showon'        => 'field_value',
            'class'         => 'field_value',
            'rows'          => 'field_value',
            'label'         => 'field_value',
            'description'   => 'field_value',
            'id'            => 'field_value',
            'default'       => 'field_value',
			'hint'          => 'field_value'
        );

        $length_exceptions = array('label');

        // if the field is multiple the default is not needed because it has options that are selected
        if ($field->multiple)
        {
            unset($attributes_to_set['default']);
        }

        foreach ($attributes_to_set as $attr => $type)
        {
            if (isset($field->{$attr}))
            {
                // handle the boolean types
                if (is_bool($type) && $field->{$attr} === $type)
                {
                    $xml->addAttribute($attr, 'true');
                }
                // handle the field properties values
                else if ($type == 'field_value' && (in_array($attr, $length_exceptions) || strlen($field->{$attr})))
                {
                    $xml->addAttribute($attr, $field->{$attr});
                }
                // handle the predefined values (triggered only by the field boolean "true" property)
                else if ($field->{$attr})
                {
                    $xml->addAttribute($attr, $type);
                }
            }
        }
    }

    protected static function getOptions($field, $xml, $selected, &$values)
    {
        $values = array();
        $field->values = self::explode($field->values);
        if ($field->values)
        {
            foreach ($field->values as $string)
            {
                $disabled = false;
                if (strpos($string, '[d]') !== false)
                {
                    $string = str_replace('[d]', '', $string);
                    $disabled = true;
                }

                // check for classes
                $class = '';
                $pattern = '/\[cl="(.+)"]/';

                preg_match($pattern, $string, $matches);
                if ($matches) {
                    $class = $matches[1];
                    $string = str_replace($matches[0], '', $string);
                }

                // <optgroup>
                if (strpos($string, '[g]') !== false)
                {
                    $string = str_replace('[g]', '', $string);
                    $option = $xml->addChild('option', $string);
                    $option->addAttribute('value', '<OPTGROUP>');
                    continue;
                }
                if (strpos($string, '[/g]') !== false)
                {
                    $string = str_replace('[/g]', '', $string);
                    $option = $xml->addChild('option', $string);
                    $option->addAttribute('value', '</OPTGROUP>');
                    continue;
                }

                $checked = false;
                if (strpos($string, '[c]') !== false)
                {
                    $string = str_replace('[c]', '', $string);

                    if (!isset($selected[$field->name]))
                    {
                        $checked = true;
                    }
                }

                if (strpos($string, '|') !== false)
                {
                    list($value, $text) = explode('|', $string, 2);
                }
                else
                {
                    $value = $text = $string;
                }

                if (isset($selected[$field->name]) && in_array($value, (array) $selected[$field->name]))
                {
                    $checked = true;
                }

                $option = $xml->addChild('option', $text);
                $option->addAttribute('value', $value);
                if ($disabled)
                {
                    $option->addAttribute('disabled', 'disabled');
                }
                if ($checked)
                {
                    $values[] = $value;
                }

                if (strlen($class))
                {
                    $option->addAttribute('class', $class);
                }
            }
        }
    }

    protected static function explode($data)
    {
        // if is already array return it this way
        if (is_array($data))
        {
           return $data;
        }
        else
        {
            $data = str_replace(array("\r\n", "\r"), "\n", $data);
            return explode("\n", $data);
        }
    }
}