<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! jQuery Calendar class.
 */
class RSDirectoryJqueryCalendar
{
    protected $params = array();

    public function __construct($params = array()) {
        // Check if they are parameters defined
        if (!is_array($params) || empty($params)) {
            throw new Exception(JText::_('COM_RSDIRECTORY_JQUERY_CALENDAR_PARAMS_ERROR'));
        }

        // the value must always be set
        if (!isset($params['value'])) {
            $params['value'] = '';
        }

        $this->_check_required($params);

        // The default layout should be Popup
        if (!isset($params['layout'])) {
            $params['layout'] = 'popup';
        }

        // The default fake_attributes should bas defined below if not present
        if (!isset($params['fake_attributes'])) {
            $params['fake_attributes'] = 'class="'.(RSDirectoryHelper::isJ4() ? 'form-control' : 'input-small').' rsdir-calendar-input"';
        }

        $this->params = $params;
    }

    protected function _check_required($params) {
        $required_params = array('name', 'fake_id', 'id');

        $errors = array();
        foreach ($required_params as $r_param) {
            if (!isset($params[$r_param])) {
                $errors[] = JText::sprintf('COM_RSDIRECTORY_JQUERY_CALENDAR_PARAM_ERROR', $r_param);
            }
        }

        if (!empty($errors)) {
           $errors = implode('<br/>', $errors);
           throw new Exception($errors);
        }
    }

    public function render() {
        $calendar_layout = $this->params['layout'];

        $datepicker_attrs = array();
        if ($calendar_layout == 'popup') {
            $datepicker_attrs['showOn'] = "'both'";
            $datepicker_attrs['zIndex'] = "999";
            $datepicker_attrs['dateFormat'] = "'" . $this->params['date_mask'] . "'";
            $datepicker_attrs['altField'] = "'#" . $this->params['id'] . "_hidden'";
            $datepicker_attrs['altFormat'] = "'" . $this->params['format_date'] . "'";
        } else {
            $datepicker_attrs['dateFormat'] = "'mm/dd/yy'";
            $datepicker_attrs['altField'] = "'#" . $this->params['id'] . " .rsdir-calendar-input'";
            $datepicker_attrs['altFormat'] = "'" . $this->params['date_mask'] . "'";
        }

        if (isset($this->params['changeMonth']) && $this->params['changeMonth']) {
            $datepicker_attrs['changeMonth'] = "true";
        }
        if (isset($this->params['changeYear']) && $this->params['changeYear']) {
            $datepicker_attrs['changeYear'] = "true";
        }
		
		 if (isset($this->params['min_date']) && $this->params['min_date']) {
            $this->params['min_date'] = RSDirectoryHelper::escapeHTML($this->params['min_date']);

            $min_date = explode( '/', $this->params['min_date'] );
            $datepicker_attrs['minDate'] = 'new Date(' . $min_date[2] . ', ' . ($min_date[0] - 1) . ', ' . $min_date[1] . ')';
        }

        if (isset($this->params['max_date']) && $this->params['max_date']) {
            $this->params['max_date'] = RSDirectoryHelper::escapeHTML($this->params['max_date']);
            $max_date = explode( '/', $this->params['max_date'] );
            $datepicker_attrs['maxDate'] = 'new Date(' . $max_date[2] . ', ' . ($max_date[0] - 1) . ', ' . $max_date[1] . ')';
        }


        $datepicker_attrs['firstDay'] = JFactory::getLanguage()->getFirstDay();

        $unique_id = uniqid();
        $unique_id = md5($this->params['id'].$unique_id);
        $script = 'var rsdir_dp_options_'.$unique_id.' = {'."\n";
        foreach ($datepicker_attrs as $key => $value) {
            $script .= "$key: $value,\n";
        }

        $script .= 'onSelect: calendarSelect'."\n";
        $script .= "}\n";
        $script .= "jQuery.extend(rsdir_dp_options_".$unique_id.", rsdir_datepicker_regional);\n\n";

        // Open the document ready function.
        $script .= "jQuery(function(){\n";
        if ($calendar_layout == 'popup') {
            $script .= "jQuery('#".$this->params['id']."').find('.rsdir-calendar-input').datepicker(rsdir_dp_options_".$unique_id.")";
        } else  {
            $script .= "jQuery('#".$this->params['id']."').find('.rsdir-calendar').datepicker(rsdir_dp_options_".$unique_id.")\n";
        }

        if (isset($this->params['change_id']) && is_array($this->params['change_id'])) {
            $script .= '.on( "change", function() {
                          var ref_date = jQuery("#'.$this->params['id'] .'_hidden").val();

                          // get the parsed date
                          var new_date = jQuery.datepicker.parseDate( "'.$this->params['format_date'].'",  ref_date, {
                                  dayNamesShort: rsdir_datepicker_regional.dayNamesShort,
                                  dayNames: rsdir_datepicker_regional.dayNames,
                                  monthNamesShort: rsdir_datepicker_regional.monthNamesShort,
                                  monthNames: rsdir_datepicker_regional.monthNames
                          });

                          jQuery(\'#'.$this->params['change_id'][0].' .rsdir-calendar-input\').datepicker( "option", "'.$this->params['change_id'][1].'", new_date );

                          // because the button changes need to style it again
                          jQuery(\'#'.$this->params['change_id'][0].'\').find(\'.ui-datepicker-trigger\').addClass(\'btn btn-secondary\').html(\'<span class=\"icon-calendar\" aria-hidden=\"true\"></span>\');
                        })';
        }
        $script .= ";\n";


        $default = empty($this->params['value']) ? '' : $this->params['value'];

        if ($default && $default != '00/00/0000')
        {
            if ($calendar_layout == 'popup') {
                $script .= "jQuery('#" . $this->params['id'] . "').find('.rsdir-calendar-input').val(jQuery.datepicker.formatDate('" . $this->params['date_mask'] . "', new Date('" . $default . "'), {
                                  dayNamesShort: rsdir_datepicker_regional.dayNamesShort,
                                  dayNames: rsdir_datepicker_regional.dayNames,
                                  monthNamesShort: rsdir_datepicker_regional.monthNamesShort,
                                  monthNames: rsdir_datepicker_regional.monthNames
                          }));\n";
            } else {
                $script .= "jQuery('#" . $this->params['id'] . "').find('.rsdir-calendar').datepicker('setDate', new Date('$default'));\n";
            }
        }

        if ($calendar_layout == 'popup') {
            $script .= "jQuery('#" . $this->params['id'] . "').find('.ui-datepicker-trigger').addClass('btn btn-secondary').html('<span class=\"icon-calendar\" aria-hidden=\"true\"></span>');\n";
        }

        if ((!$default || $default == '00/00/0000') && $calendar_layout == 'flat') {
            $script .= "jQuery('#" . $this->params['id'] . "').find('.rsdir-calendar').datepicker('setDate', null);\n";
        }

        // Close the document ready function.
        $script .= "});\n";

        // Build the html
        $html =  '<div id="'.$this->params['id'].'"'.($calendar_layout == 'popup' ? ' class="'.(RSDirectoryHelper::isJ4() ? 'input-group' : 'input-append').'"' : '').'>'."\n";
        $html .= '<input type="text" '.$this->params['fake_attributes'].' placeholder="'.$this->params['date_mask'].'" value="" id="'.$this->params['fake_id'].'"/>'."\n";
        $html .= '<input type="hidden" id="' . $this->params['id'] . '_hidden" name="' .$this->params['name']. '"' . ( !empty($this->params['value']) ? ' value="' . RSDirectoryHelper::escapeHTML($this->params['value']) . '"' : '' ) . " />\n";

        if ($calendar_layout != 'popup') {
            $html .= "<div class=\"rsdir-calendar\">&nbsp;</div>\n";
        }

        $html .= "</div>\n";

        // add the javascript to the DOM
        JFactory::getDocument()->addScriptDeclaration($script);

        return $html;
    }
}