<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/formfield.php';
require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/field.php';
require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/scripting.php';

/**
 * Placeholders processing class.
 */
class RSDirectoryPlaceholders
{
    /**
     * The string to be processed.
     *
     * @var string
     * 
     * @access protected
     */
    protected $str;
        
    /**
     * Form fields.
     *
     * @var array
     * 
     * @access protected
     */
    protected $form_fields;
        
    /**
     * Entry data.
     *
     * @var object
     * 
     * @access protected
     */
    protected $entry;
        
    /**
     * Parent form.
     *
     * @var object
     *
     * @access protected
     */
    protected $form;
        
    /**
     * A list of all default placeholders.
     *
     * @var array
     * 
     * @access protected
     *
     * @static
     */
    protected static $placeholders = array(
		'site.name',
		'site.url',
        'username',
        'name',
        'email',
        'userid',
        'credits-remaining',
        'credits-spent',
        'id',
        'url',
        'category',
        'category-path',
        'images',
        'small-thumb',
        'big-thumb',
        'normal-thumb',
        'publishing-date',
        'publishing-period',
        'expiry-date',
        'title',
        'big-subtitle',
        'small-subtitle',
        'price',
        'price-value',
        'description',
        'activation-url',
    );
        
    /**
     * Excluded field types.
     *
     * @var array
     * 
     * @access protected
     */
    protected $excluded_field_types = array();
        
    /**
     * Excluded placeholders.
     *
     * @var array
     * 
     * @access protected
     */
    protected $excluded_placeholders = array();
        
    /**
     * Process placeholders (e.g.: add excluded field types, add excluded placeholders) based on these params.
     *
     * @var array
     *
     * @access protected
     */
    protected $params = array();
		
	/**
	 * Placeholders => values map.
	 *
	 * @var array
	 *
	 * @access protected
	 */
	protected $map = array();
	
	// array that holds the placeholders as keys and the values of those placeholders as the array values
	protected $custom_fields;
        
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param string $str
     * @param array $form_fields
     * @param object $entry
     * @param mixed $form
     */
	 
	
    public function __construct($str, $form_fields, $entry, $form = null, $custom_fields = null)
    {
        $this->str = $str;
        $this->form_fields = $form_fields;
        $this->entry = $entry;
        $this->form = $form;
		$this->custom_fields = $custom_fields;
    }
        
    /**
     * Get RSDirectoryPlaceholders instance.
     *
     * @access public
     * 
     * @static
     * 
     * @param string $str
     * @param array $form_fields
     * @param object $entry
     * @param mixed $form
     * 
     * @return RSDirectoryPlaceholders
     */
    public static function getInstance($str, $form_fields, $entry, $form = null, $custom_fields = null)
    {
        $rsdirectoryplaceholders = new RSDirectoryPlaceholders($str, $form_fields, $entry, $form, $custom_fields);
            
        return $rsdirectoryplaceholders;
    }
        
    /**
     * Set excluded field types.
     *
     * @access public
     * 
     * @param mixed $excluded_field_types
     * 
     * @return RSDirectoryPlaceholders
     */
    public function addExcludedFieldTypes($excluded_field_types)
    {
        if ( is_array($excluded_field_types) )
        {
            $this->excluded_field_types = array_merge($this->excluded_field_types, $excluded_field_types);
        }
        else
        {
            $this->excluded_field_types[] = $excluded_field_types;
        }
            
        return $this;
    }
        
    /**
     * Remove field types from the excluded field types array.
     *
     * @access public
     * 
     * @param mixed $field_types
     * 
     * @return RSDirectoryPlaceholders
     */
    public function removeExcludedFieldTypes($field_types)
    {
        if ( is_array($field_types) )
        {
            $this->excluded_field_types = array_diff($this->excluded_field_types, $field_types);
        }
        else
        {
            $this->excluded_field_types = array_diff( $this->excluded_field_types, array($field_types) );
        }
            
        return $this;
    }
        
    /**
     * Set excluded placeholders.
     *
     * @access public
     * 
     * @param mixed $excluded_placeholders
     * 
     * @return RSDirectoryPlaceholders
     */
    public function addExcludedPlaceholders($excluded_placeholders)
    {
        if ( is_array($excluded_placeholders) )
        {
            $this->excluded_placeholders = array_merge($this->excluded_placeholders, $excluded_placeholders);
        }
        else
        {
            $this->excluded_placeholders[] = $excluded_placeholders;
        }
            
        return $this;
    }
        
    /**
     * Remove placeholders from the excluded placeholders array.
     *
     * @access public
     * 
     * @param mixed $placeholders
     * 
     * @return RSDirectoryPlaceholders
     */
    public function removeExcludedPlaceholders($placeholders)
    {
        if ( is_array($placeholders) )
        {
            $this->excluded_placeholders = array_diff($this->excluded_placeholders, $placeholders);
        }
        else
        {
            $this->excluded_placeholders = array_diff( $this->excluded_placeholders, array($placeholders) );
        }
            
        return $this;
    }
        
    /**
     * Set the string.
     *
     * @access public
     * 
     * @param string $str
     * 
     * @return RSDirectoryPlaceholders
     */
    public function setStr($str)
    {
        $this->str = $str;
            
        return $this;
    }
        
    /**
     * Set params.
     *
     * @access public
     *
     * @param mixed $params
     *
     * @return RSDirectoryPlaceholders
     */
    public function setParams($params)
    {
        if ( !is_array($params) )
        {
            $params = (array)$params;
        }
            
        foreach ($params as $param)
        {
            if ( !in_array($param, $this->params) )
            {
                if ($param == 'title')
                {
                    $this->addExcludedFieldTypes(array('fileupload', 'image_upload'));
                    $this->addExcludedPlaceholders( array('credits-remaining', 'credits-spent', 'images', 'small-thumb', 'big-thumb', 'normal-thumb', 'title') );
                }
                    
                $this->params[] = $param;
            }
        }
            
        return $this;
    }
		
    /**
     * Remove params.
     *
     * @access public
     *
     * @return RSDirectoryPlaceholders
     */
    public function removeParams($params)
    {
        if ( !is_array($params) )
        {
            $params = (array)$params;
        }
            
        foreach ($params as $param)
        {
            $index = array_search($param, $this->params);
            
            if ($index !== false)
            {
                if ($param == 'title')
                {
                    $this->removeExcludedFieldTypes('fileupload', 'image_upload');
                    $this->removeExcludedPlaceholders( array('credits-remaining', 'credits-spent', 'images', 'small-thumb', 'big-thumb', 'normal-thumb', 'title') );
                }
                    
                unset($this->params[$index]);
            }
        }
            
        return $this;
    }
		
	/**
	 * Method to initialize the required resources.
	 *
	 * @access protected
	 */	
	protected function init()
	{
		$str = $this->str;
        $form_fields = $this->form_fields;
        $entry = $this->entry;
        $form = $this->form;
        $excluded_field_types = $this->excluded_field_types;
        $excluded_placeholders = $this->excluded_placeholders;
        $params = $this->params;
			
		// Get an instance of the RSDirectoryConfig object.
        $config = RSDirectoryConfig::getInstance();
			
		// Get the Joomla! config object.
		$jconfig = JFactory::getConfig();
            
        // Get the user object.
        $user = JFactory::getUser($entry->user_id);
			
		// Initialize the placeholders => values array.
		$map = array();

        // determine if is not related to an entry
        $exclude_entry_related_placeholders = array();
        if (isset($entry->skip_entry)) {
            $exclude_entry_related_placeholders =  array('images', 'small-thumb', 'big-thumb', 'normal-thumb', 'id', 'url',  'category', 'category-path', 'publishing-date', 'publishing-period', 'expiry-date', 'title', 'big-subtitle', 'small-subtitle', 'price', 'description');
        }

        // Hold only image related placeholders
        $image_related = array('images', 'small-thumb', 'big-thumb', 'normal-thumb');

        $max_images_nr = 1;
        // Get the images field.
        $images = RSDirectoryHelper::findFormField('images', $form_fields);
        if ($images)
        {
            $max_images_nr = (int) $images->properties->get('max_files_number');
        }

        for($j = 0; $j < $max_images_nr; $j++)
        {
            self::$placeholders[] = 'small-thumb-src-'.$j;
            self::$placeholders[] = 'big-thumb-src-'.$j;
            self::$placeholders[] = 'normal-thumb-src-'.$j;

            $image_related[] = 'small-thumb-src-'.$j;
            $image_related[] = 'big-thumb-src-'.$j;
            $image_related[] = 'normal-thumb-src-'.$j;

            if (isset($entry->skip_entry)) {
                $exclude_entry_related_placeholders[] = 'small-thumb-src-'.$j;
                $exclude_entry_related_placeholders[] = 'big-thumb-src-'.$j;
                $exclude_entry_related_placeholders[] = 'normal-thumb-src-'.$j;
            }
        }
			
		foreach (self::$placeholders as $placeholder)
        {
            // Skip this placeholder because we have determined that we need to skip the entry related placeholders.
            if ( in_array($placeholder, $exclude_entry_related_placeholders) )
                continue;
            // Skip this placeholder because it is excluded.
            if ( in_array($placeholder, $excluded_placeholders) )
                continue;
                
            // Skip this placeholder because it was not found in the string.
            if ( strpos($str, '{' . $placeholder . '}') === false && strpos($str, '{if ' . $placeholder . '}') === false )
                continue;
                
            // Initialize the placeholder value.
            $placeholder_value = '';

			if (in_array( $placeholder,  $image_related ) )
			{
				// Get the images list.
				if ($images)
				{
					$images_list = RSDirectoryHelper::getFilesObjectList($images->id, $entry->id);
				}
			}
			
			// Skip some placeholders due to improper settings
            $skip_placeholder = false;
				
            switch ($placeholder)
            {
				case 'site.name':
						
					$placeholder_value = $jconfig->get('sitename');
					break;
						
				case 'site.url':
						
					$placeholder_value = JUri::root();
					break;
						
                case 'username':
                        
                    $placeholder_value = $user->username;
                    break;
                        
                case 'name':
                        
                    $placeholder_value = $user->name;
                    break;
                        
                case 'email':
                        
                    $placeholder_value = $user->email;
                    break;
                        
                case 'userid':
                        
                    $placeholder_value = $user->id;
                    break;
                        
                case 'credits-remaining':
                        
                    $credits = RSDirectoryCredits::getUserCredits($user->id);
						
					$placeholder_value = $credits === 'unlimited' ? JText::_('COM_RSDIRECTORY_UNLIMITED') : $credits;
						
                    break;
                        
                case 'credits-spent':
                        
                    $placeholder_value = RSDirectoryCredits::getUserSpentCredits($user->id);
                    break;
                        
                case 'id':
                        
                    $placeholder_value = $entry->id;
                    break;
                        
                case 'url':
                        
                    $placeholder_value = RSDirectoryRoute::getEntryURL($entry->id, $entry->title, '', 0, true);
                    break;
                        
                 case 'category':

                    if ( isset($entry->category_title) )
                    {
                        $placeholder_value = $entry->category_title;
                    }
                    else
                    {
                        $category = RSDirectoryHelper::getCategory($entry->category_id);
                        if (!is_null($category))
                        {
                            $placeholder_value = $category->title;
                        }  else {
                            $skip_placeholder = true;
                        }
                    }

                break;
                        
                case 'category-path':
                    $category = RSDirectoryHelper::getCategory($entry->category_id);
                    if (!is_null($category))
                    {
                        static $categories;

                        if (empty($categories))
                        {
                            $categories = RSDirectoryHelper::getCategories($categories);
                        }

                        $hierarchy = RSDirectoryHelper::getCategoryHierarchy($entry->category_id, $categories);

                        if ($hierarchy)
                        {
                            $path = array();

                            foreach ($hierarchy as $category)
                            {
                                $path[] = $category->title;
                            }

                            if (in_array('database', $params))
                            {
                                $placeholder_value = implode(' » ', $path);
                            }
                            else
                            {
                                $placeholder_value = implode(' &raquo; ', $path);
                            }
                        }
                    } else {
                        $skip_placeholder = true;
                    }
                        
                break;

                        
                case 'images':
                        
                    if ( !empty($images_list) )
                    {
                        if ( in_array('email', $params) )
                        {
                            $placeholder_value = '<table cellpadding="0" cellspacing="5" border="0">';
                            $placeholder_value .= '<tr>';
                                
                            foreach ($images_list as $image)
                            {
                                $placeholder_value .= '<td>';
                                
                                // Set the src.
                                $src = RSDirectoryHelper::getImageURL($image->hash, 'small');
                                    
                                // Set the alt.
                                $alt = JFile::stripExt($image->original_file_name);
                                    
                                $placeholder_value .= '<img src="' . $src . '" alt="' . $alt . '" />';
                                    
                                $placeholder_value .= '</td>';
                            }
                                
                            $placeholder_value .= '</tr>';        
                            $placeholder_value .= '</table>';
                        }
                        else
                        {
                            $placeholder_value = RSDirectoryFormField::getImagesList($images_list, 0, 0);
                        }
                    }
                        
                    break;
                        
                case 'small-thumb':
                        
                    if ( !empty($images_list) )
                    {
                        $src = RSDirectoryHelper::getImageURL($images_list[0]->hash, 'small');
                            
                        // Get the width of the small thumbnail.
                        $width = $config->get('small_thumbnail_width');
                            
                        // Get the height of the small thumbnail.
                        $height = $config->get('small_thumbnail_height');
                            
                        $placeholder_value = '<img src="' . $src . '" alt="" width="' . $width . '" height="' . $height . '" />';
                    }
                        
                    break;
                        
                case 'big-thumb':
                        
                    if ( !empty($images_list) )
                    {
                        $src = RSDirectoryHelper::getImageURL($images_list[0]->hash, 'big');
                            
                        // Get the width of the small thumbnail.
                        $width = $config->get('big_thumbnail_width');
                            
                        // Get the height of the small thumbnail.
                        $height = $config->get('big_thumbnail_height');
                            
                        $placeholder_value = '<img src="' . $src . '" alt="" width="' . $width . '" height="' . $height . '" />';
                    }
                        
                    break;
                        
                case 'normal-thumb':
                        
                    if ( !empty($images_list) )
                    {
                        $src = RSDirectoryHelper::getImageURL($images_list[0]->hash);
                            
                        $placeholder_value = '<img src="' . $src . '" alt="" />';
                    }
                        
                    break;
                        
                case 'publishing-date':
                        
                    if ($entry->published_time == '0000-00-00 00:00:00')
                    {
                        $placeholder_value = JText::_('COM_RSDIRECTORY_ENTRY_NOT_YET_PUBLISHED');
                    }
                    else
                    {
                        $placeholder_value = RSDirectoryHelper::formatDate($entry->published_time);
                            
                        $placeholder_value = $placeholder_value ? $placeholder_value : JText::_('COM_RSDIRECTORY_JUST_PUBLISHED');
                    }
                        
                    break;
					
				case 'publishing-period':
					$placeholder_value = $entry->period ? $entry->period : JText::_('COM_RSDIRECTORY_UNLIMITED');
				break;

                case 'expiry-date':
                        
                    if ($entry->expiry_time == '0000-00-00 00:00:00')
                    {
                        $placeholder_value = JText::_('COM_RSDIRECTORY_NO_EXPIRY');
                    }
                    else
                    {
                        // Entry not yet expired.
                        if ( JFactory::getDate($entry->expiry_time)->toUnix() > JFactory::getDate()->toUnix() )
                        {
                            $placeholder_value = RSDirectoryHelper::formatDate($entry->expiry_time);    
                        }
                        else
                        {
                            $placeholder_value = JText::_('COM_RSDIRECTORY_EXPIRED');
                        }
                    }
                        
                    break;

                case 'title':
                        
                    $placeholder_value = trim($entry->title);
                    break;
                        
                case 'big-subtitle':
                        
                    $placeholder_value = $entry->big_subtitle;
                    break;
                
                case 'small-subtitle':
                        
                    $placeholder_value = $entry->small_subtitle;
                    break;
                        
                case 'price':
                        
                    $placeholder_value = RSDirectoryHelper::formatPrice($entry->price);
                    break;

                case 'price-value':
                    $placeholder_value = $entry->price;
                    break;
                        
                case 'description':
                        
                    $description = RSDirectoryHelper::findFormField('description', $form_fields);
					$placeholder_value = RSDirectoryField::getInstance($description, $entry)->generate();
						
                break;

                case 'activation-url':
                    $placeholder_value = !empty($entry->activation_code) ? JUri::root() . 'index.php?option=com_rsdirectory&task=entry.activate&id=' . $entry->id . '&code=' . $entry->activation_code : '';
                break;

                default:
                    $detect_images_src_placeholders = array('small-thumb-src-', 'big-thumb-src-', 'normal-thumb-src-');

                    foreach ($detect_images_src_placeholders as $src_placeholder)
                    {
                        if (strpos($placeholder, $src_placeholder) !== false)
                        {
                            $image_number = explode('-', $placeholder);
                            $image_number = (int)array_pop($image_number);

                            $src = '';
                            if ( !empty($images_list) && isset($images_list[$image_number]))
                            {
                                if ($src_placeholder == 'small-thumb-src-')
                                {
                                    $src = RSDirectoryHelper::getImageURL($images_list[$image_number]->hash, 'small');
                                }
                                else if ($src_placeholder == 'big-thumb-src-')
                                {
                                    $src = RSDirectoryHelper::getImageURL($images_list[$image_number]->hash, 'big');
                                }
                                else if ($src_placeholder == 'normal-thumb-src-')
                                {
                                    $src = RSDirectoryHelper::getImageURL($images_list[$image_number]->hash);
                                }
                            }

                            $placeholder_value = $src;
                            break 1;
                        }
                    }

                break;
            }
				
			$map[$placeholder] = $placeholder_value;
        }

        if (!is_null($form_fields)) {
            foreach ($form_fields as $form_field) {
                // Skip core placeholders.
                if (in_array($form_field->name, self::$placeholders))
                    continue;

                // Skip the form field if it's in the excluded field types array.
                if (in_array($form_field->field_type, $excluded_field_types))
                    continue;

                // Skip this placeholder because it is excluded.
                if (in_array($form_field->name, $excluded_placeholders))
                    continue;
				
				// handle src image uploads
                if ($form_field->field_type == 'image_upload')
                {
                    $find_in_src = array();

                    if ($max = $form_field->properties->get('max_files_number'))
                    {
                        for ($i = 0; $i < (int)$max; $i++)
                        {
                            $find_in_src[] = '{' . $form_field->name . '-small-thumb-src-'.$i.'}';
                            $find_in_src[] = '{' . $form_field->name . '-big-thumb-src-'.$i.'}';
                            $find_in_src[] = '{' . $form_field->name . '-normal-thumb-src-'.$i.'}';
                            $find_in_src[] = '{if ' . $form_field->name . '-small-thumb-src-'.$i.'}';
                            $find_in_src[] = '{if ' . $form_field->name . '-big-thumb-src-'.$i.'}';
                            $find_in_src[] = '{if ' . $form_field->name . '-normal-thumb-src-'.$i.'}';
                        }
                    }

                    if (!empty($find_in_src)) {
                        foreach ($find_in_src as $find_placeholder)
                        {
                            if (strpos($str, $find_placeholder) !== false)
                            {
                                $map_index = str_replace(array('{if ','{', '}'), '', $find_placeholder);

                                if (isset($map[$map_index]))
                                {
                                    continue 1;
                                }

                                $placeholder_data = str_replace('thumb-src-', '', $map_index);
                                $placeholder_data = explode('-', $placeholder_data);

                                $image_nr = (int) array_pop($placeholder_data);
                                $image_type = array_pop($placeholder_data);

                                $files_list = RSDirectoryHelper::getFilesObjectList($form_field->id, $entry->id);
                                $all_files = count($files_list);
                                $placeholder_value = '';

                                // if the max files allowed is more than the images loaded skip
                                if ($all_files > 0 && $image_nr < $all_files)
                                {
                                    $placeholder_value = RSDirectoryFormField::getImagesList($files_list, 0, 0, array('type'=> $image_type, 'nr'=> $image_nr));
                                }

                                $map[$map_index] = $placeholder_value;
                            }
                        }
                    }
                }

                // Skip this placeholder because it was not found in the string.
                if (strpos($str, '{' . $form_field->name . '}') === false && strpos($str, '{if ' . $form_field->name . '}') === false)
                    continue;

                $properties = isset($form_field->properties) ? $form_field->properties : null;

                // Initialize the placeholder value.
                $placeholder_value = '';

                // Get a new RSDirectoryField instance.
                $rsdirfield = RSDirectoryField::getInstance($form_field, $entry);

                if (in_array($form_field->field_type, array('fileupload', 'image_upload'))) {
                    // Get the files list.
                    $files_list = RSDirectoryHelper::getFilesObjectList($form_field->id, $entry->id);

                    $rsdirfield->setFilesList($files_list);
                }

                if (in_array('description', $params) && $form_field->field_type == 'textarea' && !$form_field->properties->get('allow_html')) {
                    if (isset($entry->{$form_field->column_name})) {
                        $placeholder_value = nl2br($entry->{$form_field->column_name});
                    }
                } else {
                    // Get the field value.
                    $placeholder_value = $rsdirfield->generate();
                }

                $map[$form_field->name] = $placeholder_value;
            }
        }

		if (!is_null($this->custom_fields)) {
		
			foreach($this->custom_fields as $key => $value) {
				 if ( strpos($str, '{' . $key . '}') === false && strpos($str, '{if ' . $key . '}') === false )
                continue;

                // for the invoice table we must allow html
                if ($key == 'invoice-transaction-table') {
                    $placeholder_value = $value;
                } else {
                    $placeholder_value = nl2br(RSDirectoryHelper::escapeHTML($value));
                }
				$map[$key] = $placeholder_value;
			}
			
		}
			
		$this->map = $map;
	}


    /**
     * Process placeholders and return the new string.
     *
     * @access public
     * 
     * @return string
     */
    public function process()
    {
        if (!$this->str || (!$this->form_fields && is_null($this->custom_fields)) || !$this->entry)
            return;

		// Initialize the required resources.
		$this->init();
			
		if ( strpos($this->str, '{/if}') !== false )
		{
            $scripting = RSDirectoryScripting::getInstance(self::$placeholders);
			$this->str = $scripting->processConditional($this->str, $this->map, $this->form_fields, $this->custom_fields);
		}
            
        // Remove the excluded placeholders from the string.
        if ($this->excluded_placeholders)
        {
            foreach ($this->excluded_placeholders as $placeholder)
            {
                $this->str = str_replace('{' . $placeholder . '}', '', $this->str);
            }
        }
			
		// Replace placeholders with their values.
		if ($this->map)
		{
			foreach ($this->map as $placeholder => $value)
			{
				$this->str = str_replace('{' . $placeholder . '}', $value, $this->str);
			}
		}	
        return (string)$this->str;
    }
}