<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access'); 


require_once dirname(__FILE__) . '/version.php';
require_once dirname(__FILE__) . '/config.php';
require_once dirname(__FILE__) . '/credits.php';
require_once dirname(__FILE__) . '/adapter.php';
require_once dirname(__FILE__) . '/model.php';
require_once dirname(__FILE__) . '/route.php';
require_once dirname(__FILE__) . '/formfieldproperties.php';
require_once dirname(__FILE__) . '/fields.php';


/**
 * RSDirectory! Helper class.
 */
abstract class RSDirectoryHelper
{
    /**
     * Check Joomla Version
     *
     * @access public
     *
     * @static
     */
    public static function isJ4() {
        static $version;
        if (is_null($version)) {
            $version = version_compare(JVERSION, '4.0', '>=');
        }

        return $version;
    }

    /**
     * Load the language files.
     *
     * @access public
     *
     * @static
     */
    public static function loadLang()
    {
        $app  = JFactory::getApplication();
        $lang = JFactory::getLanguage();
        $from = $app->isClient('administrator') ? JPATH_ADMINISTRATOR : JPATH_SITE;

        $lang->load('com_rsdirectory', $from, 'en-GB', true);
        $lang->load( 'com_rsdirectory', $from, $lang->getDefault(), true );
        $lang->load('com_rsdirectory', $from, null, true);
    }

    /**
     * Load media.
     *
     * @access public
     *
     * @static
     */
    public static function loadMedia()
    {
        // Load common CSS.
		JHtml::_('stylesheet', 'com_rsdirectory/common.css', array('relative' => true, 'version' => 'auto'));

        // Load the jQuery UI CSS file.
        JHtml::_('stylesheet', 'com_rsdirectory/jquery-ui-1.10.4.custom.min.css', array('relative' => true, 'version' => 'auto'));

		// Load the Joomla! core.js
        JHtml::_('behavior.core');


        if ( JFactory::getApplication()->isClient('administrator') )
        {
            // Load CSS.
            JHtml::_('stylesheet', 'com_rsdirectory/admin/admin-style.css', array('relative' => true, 'version' => 'auto'));

            // Load icons
            JHtml::_('stylesheet', 'com_rsdirectory/icons.css', array('relative' => true, 'version' => 'auto'));

            // Load jQuery.
            JHtml::_('jquery.framework');

            if (!self::isJ4())
            {
                // Load jQuery UI.
                JHtml::_('jquery.ui', array('core', 'sortable'));
                JHtml::_('bootstrap.tooltip');
				
				// Load CSS.
                JHtml::_('stylesheet', 'com_rsdirectory/admin/style30.css', array('relative' => true, 'version' => 'auto'));
            }
            else
            {
                JHtml::_('bootstrap.popover', '.hasPopover', array('trigger' => 'hover focus'));
                // Load CSS.
                JHtml::_('stylesheet', 'com_rsdirectory/admin/style40.css', array('relative' => true, 'version' => 'auto'));
            }

            // Load the common js script.
            JHtml::_('script', 'com_rsdirectory/common.js', array('relative' => true, 'version' => 'auto'));

            // Load the admin script.
            JHtml::_('script', 'com_rsdirectory/admin-script.js', array('relative' => true, 'version' => 'auto'));

            if (self::isJ4())
            {
                $script = 'RSDirectoryAdminConstants.table_error = "table-danger";';
                $script .= 'RSDirectoryAdminConstants.table_info = "table-info";';
                $script .= 'RSDirectoryAdminConstants.table_success = "table-success";';
                $script .= 'RSDirectoryAdminConstants.table_warning = "table-warning";';
                JFactory::getDocument()->addScriptDeclaration($script);
            }
        }
        else
        {
            // Get an instance of the config object.
            $config = RSDirectoryConfig::getInstance();

            // Load style.
            JHtml::_('stylesheet', 'com_rsdirectory/style.css', array('relative' => true, 'version' => 'auto'));

            // Load jQuery.
            if ( $config->get('load_jquery') )
            {
                JHtml::_('jquery.framework');
            }

			// Load FontAwesome.
            if ( $config->get('load_fa') ) {
                JHtml::_('stylesheet', 'com_rsdirectory/font-awesome.min.css', array('relative' => true, 'version' => 'auto'));
			}

            // Load bootstrap.
            if ( $config->get('load_bootstrap') )
            {
                JHtml::_('bootstrap.framework');
                // Load the files too, when the component is not loaded and only RSDirectory modules are, the template css is loaded first and these overwrites the template
                if ( $config->get('load_bootstrap_css', 1) )
                {
                    JHtml::_('bootstrap.loadcss');
                }
            }
            
            
            if (!self::isJ4())
            {
                // load the specific style for J3
                JHtml::_('stylesheet', 'com_rsdirectory/style30.css', array('relative' => true, 'version' => 'auto'));
                
                // Load jQuery UI.
                JHtml::_('jquery.ui', array('core', 'sortable'));
            }
            else
            {
                // load the style for the J4
                JHtml::_('stylesheet', 'com_rsdirectory/style40.css', array('relative' => true, 'version' => 'auto'));
            }
            
            JHtml::_('script', 'com_rsdirectory/jquery.ui.datepicker.min.js', array('relative' => true, 'version' => 'auto'));


            // Load the common js script.
            JHtml::_('script', 'com_rsdirectory/common.js', array('relative' => true, 'version' => 'auto'));

            //Load modal fix
            JHtml::_('script', 'com_rsdirectory/modal-fix.js', array('relative' => true, 'version' => 'auto'));

            // Load script.
            JHtml::_('script', 'com_rsdirectory/script.js', array('relative' => true, 'version' => 'auto'));
        }

        self::loadJSData();
    }

	public static function loadMagnificPopup() {
        JHtml::_('stylesheet', 'com_rsdirectory/magnific-popup.css', array('relative' => true, 'version' => 'auto'));
		JHtml::_('script', 'com_rsdirectory/jquery.magnific-popup.min.js', array('relative' => true, 'version' => 'auto'));

		// Translations
		JText::script('COM_RSDIRECTORY_GAL_CLOSE');
		JText::script('COM_RSDIRECTORY_GAL_LOADING');
		JText::script('COM_RSDIRECTORY_GAL_PREV');
		JText::script('COM_RSDIRECTORY_GAL_NEXT');
		JText::script('COM_RSDIRECTORY_GAL_COUNTER');
		JText::script('COM_RSDIRECTORY_GAL_IMAGE_ERROR');
	}

    /**
     * Load JS data.
     *
     * @access public
     *
     * @static
     */
    public static function loadJSData()
    {
        static $loaded;

        if (!$loaded)
        {
            $loaded = true;

            $data = array(
                'version: "' . RSDirectoryVersion::$version . '"',
                'token: "' . JSession::getFormToken() . '"',
                'root: "' . JUri::root(true) . '/"',
                'base: "' . JUri::base(true) . '/"',
                'isJ4: ' . (RSDirectoryHelper::isJ4() ? "1" : "0") . '',
                'files_limit: {}',
            );

            JFactory::getDocument()->addScriptDeclaration('var rsdir = {' . implode(',', $data) . '};');
        }
    }
  
    /**
     * Initialize the helper.
     *
     * @access public
     *
     * @static
     */
    public static function init()
    {
        // Load the language files.
        self::loadLang();

        // Load media.
        self::loadMedia();
    }

    /**
     * Add the elements of an array into another array starting at the position marked by the $index parameter.
     *
     * @access public
     *
     * @static
     *
     * @param array $toArray
     * @param array $array
     * @param int $index
     */
    public static function addTo(&$toArray, $array, $index)
    {
        $toArray = array_slice($toArray, 0, $index) + $array + array_slice($toArray, $index);
    }

    /**
     * Get an array of options used for populating a dropdown field.
     *
     * @access public
     *
     * @static
     *
     * @param string $values
     *
     * @return array
     */
    public static function getOptions($values)
    {
        $values = explode ( "\n", str_replace( array("\r\n", "\r"), "\n", $values ) );

        $options = array();

        foreach ($values as $value)
        {
            $aux = explode('|', $value);

            if ( !isset($aux[1]) )
            {
                $aux[1] = $aux[0];
            }

            $options[] = JHtml::_('select.option', $aux[0], $aux[1]);
        }

        return $options;
    }

    /**
     * Get an array of grouped options for populating a dropdown field.
     *
     * @access public
     *
     * @static
     *
     * @param array $items
     *
     * @return mixed
     */
    public static function getGroupedListOptions($items)
    {
        if (!$items)
            return;

        // Initialize the options array.
        $options = array();

        // Initiate the credits array
        $credits_all = array();

        foreach ($items as $item)
        {
            if ( stripos($item->value, '[g]') !== false || stripos($item->text, '[g]') !== false )
            {
                $group = array(
                    'text' => str_replace('[g]', '', $item->text),
                    'items' => array(),
                );
            }
            else if ( isset($group) && ( stripos($item->text, '[/g]') !== false || stripos($item->text, '[/g]') !== false || end($items) == $item ) )
            {
                $options[] = $group;
                unset($group);
            }
            else
            {
                if ( stripos($item->value, '[d]') !== false || stripos($item->text, '[d]') !== false )
                {
                    $value = str_replace('[d]', '', $item->value);
                    $text = str_replace('[d]', '', $item->text);

                    $disable = true;
                }
                else
                {
                    $value = $item->value;
                    $text = $item->text;

                    $disable = false;
                }

                // check for credits in the value
                if( preg_match('/\[c-\d+]/', $value, $credits) != false)
                {
                    $value = str_replace($credits[0], '', $value);

                    // get the credits value
                    $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                    $credits_all[$value] = (int)$credit_value;
                }

                // check for credits in the text
                if(preg_match('/\[c-\d+]/', $text, $credits))
                {
                    $text = str_replace($credits[0], '', $text);

                    // get the credits value
                    $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                    // we need to save as reference the actual value
                    if (!isset($credits_all[$value])) {
                        $credits_all[$value] = (int)$credit_value;
                    }
                }

                $option = JHtml::_('select.option', $value, $text, 'value', 'text', $disable);

                if ( isset($group) )
                {
                    $group['items'][] = $option;
                }
                else
                {
                    $options[] = array(
                        'items' => array(
                            $option,
                        ),
                    );
                }
            }
        }

        $grouped = array(
            'items' => $options
        );

        if (!empty($credits_all)) {
            $grouped['credits'] = $credits_all;
        }

        return $grouped;
    }

    /**
     * Validate name.
     *
     * @access public
     *
     * @static
     *
     * @param string $name
     *
     * @return
     */
    public static function validateName($name)
    {
        return preg_match('/^([a-z0-9_])+$/i', $name);
    }

    /**
     * Get $val if $val is numeric or zero if it's not.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $val
     *
     * @return int
     */
    public static function getInt($val)
    {
        return is_numeric($val) ? $val : 0;
    }

    /**
     * Get params.
     *
     * @access public
     *
     * @static
     *
     * @return mixed
     */
    public static function getParams()
    {
        $app = JFactory::getApplication();

        $itemid = $app->input->getInt('Itemid', 0);
        $params = array();

        if ($itemid)
        {
            $menu = $app->getMenu();

            if (!$menu)
                return;

            $active = $menu->getItem($itemid);

            if (!$active)
                return;

            $params = $active->params;
        }

        if ($params)
        {
            $params = $app->getParams();
        }

        return $params;
    }

    /**
     * Get category hierachy.
     *
     * @access public
     *
     * @static
     *
     * @param int $category_id
     * @param array $categories
     *
     * @return mixed
     */
    public static function getCategoryHierarchy($category_id, $categories)
    {
        if (!$category_id || !$categories)
            return;

        // Initialize the hierarchy array.
        $hierarchy = array();

        $i = 0;

        // Get the array count.
        $count = count($categories);

        // Get the hierarchy of the category.
        while ($i < $count)
        {
            if ($categories[$i]->id == $category_id)
            {
                $hierarchy[] = $categories[$i];

                if ($categories[$i]->parent_id < 2)
                    break;

                $category_id = $categories[$i]->parent_id;
                $i = 0;
            }
            else
            {
                $i++;
            }
        }

        return array_reverse($hierarchy);
    }

    /**
     * Check if all characters in a string are alphabetic or belong to the extra values.
     *
     * @access public
     *
     * @static
     *
     * @param string $param
     * @param mixed $extra
     *
     * @return bool
     */
    public static function alpha($param, $extra = null)
    {
        if ( strpos($param, "\n") !== false)
        {
            $param = str_replace( array("\r", "\n"), '', $param);
        }

        $strlen = strlen($param);

        for ($i = 0; $i < $strlen; $i++)
        {
            if ( strpos($extra, $param[$i]) === false && preg_match('#([^a-zA-Z ])#', $param[$i]) )
                return false;
        }

        return true;
    }

    /**
     * Check if all characters in a string are numeric or belong to the extra values.
     *
     * @access public
     *
     * @static
     *
     * @param string $param
     * @param mixed $extra
     *
     * @return bool
     */
    public static function numeric($param, $extra = null)
    {
        if ( strpos($param, "\n") !== false )
        {
            $param = str_replace( array("\r", "\n"), '', $param );
        }

        $strlen = strlen($param);

        for ($i = 0; $i < $strlen; $i++)
        {
            if ( strpos($extra, $param[$i]) === false && !is_numeric($param[$i]) )
                return false;
        }

        return true;
    }

    /**
     * Check if all characters in a string are alphanumeric or belong to the extra values.
     *
     * @access public
     *
     * @static
     *
     * @param string $param
     * @param mixed $extra
     *
     * @return bool
     */
    public static function alphanumeric($param, $extra = null)
    {
        if ( strpos($param, "\n") !== false)
        {
            $param = str_replace( array("\r", "\n"), '', $param);
        }

        $strlen = strlen($param);

        for ($i = 0; $i < $strlen; $i++)
        {
            if ( strpos($extra, $param[$i]) === false && !preg_match('/^[A-Za-z0-9]$/', $param[$i]) )
                return false;
        }

        return true;
    }

    /**
     * Check if the string is a valid email.
     *
     * @access public
     *
     * @static
     *
     * @param string $email
     *
     * @return bool
     */
    public static function email($email)
    {
        return JMailHelper::isEmailAddress( trim($email) );
    }

    /**
     * Check if the string is a valid email.
     *
     * @access public
     *
     * @static
     *
     * @param string $email
     *
     * @return bool
     */
    public static function emaildns($email)
    {
        // Check if it's an email address format.
        if ( !self::email($email) )
            return false;

        $email = trim($email);
        list($user, $domain) = explode('@', $email, 2);

        // checkdnsrr for PHP < 5.3.0
        if ( !function_exists('checkdnsrr') && function_exists('exec') && is_callable('exec') )
        {
            @exec( 'nslookup -type=MX ' . escapeshellcmd($domain), $output );

            foreach ($output as $line)
            {
                if ( preg_match('/^' . preg_quote($domain) . '/', $line) )
                    return true;
            }

            return false;
        }

        // Fallback method.
        if ( !function_exists('checkdnsrr') || !is_callable('checkdnsrr') )
            return true;

        return checkdnsrr($domain, substr(PHP_OS, 0, 3) == 'WIN' ? 'A' : 'MX');
    }

    public static function uniquefield($value, $column, $id = null)
    {
        // Get DBO.
        $db = JFactory::getDBO();

        $query = $db->getQuery(true)
               ->select( $db->qn('id') )
               ->from( $db->qn('#__rsdirectory_entries', 'e') )
               ->rightJoin( $db->qn('#__rsdirectory_entries_custom', 'c') . ' ON ' . $db->qn('e.id') . ' = ' . $db->qn('c.entry_id') )
               ->where( $db->qn($column) . ' = ' . $db->q($value) );

        if ($id) {
            $query->where($db->qn('e.id') . ' != ' . $db->q($id));
        }

        $db->setQuery($query, 0, 1);

        return !$db->loadResult();
    }

    public static function uniquefielduser($value, $column, $id = null)
    {
        // Get DBO.
        $db = JFactory::getDBO();

        // Get the JUser object.
        $user = JFactory::getUser();

        $query = $db->getQuery(true);

        $query->select( $db->qn('id') )
              ->from( $db->qn('#__rsdirectory_entries', 'e') )
              ->rightJoin( $db->qn('#__rsdirectory_entries_custom', 'c') . ' ON ' . $db->qn('e.id') . ' = ' . $db->qn('c.entry_id') )
              ->where( $db->qn($column) . ' = ' . $db->q($value) );

        if ( $user->get('guest') )
        {
            $query->where( $db->qn('e.ip') . ' = ' . $db->q($_SERVER['REMOTE_ADDR']) );
        }
        else
        {
            $query->where( $db->qn('e.user_id') . ' = ' . $db->q( $user->get('id') ) );
        }

        if ($id) {
            $query->where($db->qn('e.id') . ' != ' . $db->q($id));
        }

        $db->setQuery($query, 0, 1);

        return !$db->loadResult();
    }

    public static function uszipcode($value)
    {
        return preg_match("/^([0-9]{5})(-[0-9]{4})?$/i", $value);
    }

    public static function phonenumber($value)
    {
        return preg_match("/\(?\b[0-9]{3}\)?[-. ]?[0-9]{3}[-. ]?[0-9]{4}\b/i", $value);
    }

    public static function creditcard($value)
    {
        $value = preg_replace('/[^0-9]+/', '', $value);

        if (!$value)
            return false;

        if ( preg_match("/^([34|37]{2})([0-9]{13})$/", $value) ) // Amex
            return true;

        if ( preg_match("/^([30|36|38]{2})([0-9]{12})$/", $value) ) // Diners
            return true;

        if ( preg_match("/^([6011]{4})([0-9]{12})$/", $value) ) // Discover
            return true;

        if ( preg_match("/^([51|52|53|54|55]{2})([0-9]{14})$/", $value) ) // MasterCard
            return true;

        if ( preg_match("/^([4]{1})([0-9]{12,15})$/", $value) ) // Visa
            return true;

        return false;
    }

    public static function custom($param, $extra = null)
    {
        if ( strpos($param, "\n") !== false )
        {
            $param = str_replace( array("\r", "\n"), '', $param );
        }

        $strlen = strlen($param);

        for ($i = 0; $i < $strlen; $i++)
        {
            if( strpos($extra, $param[$i]) === false )
                return false;
        }

        return true;
    }

    public static function ipaddress($param)
    {
        return preg_match('#\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b#', $param);
    }

    public static function validurl($param)
    {
        $format = '_^(?:(?:https?|ftp)://)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\x{00a1}-\x{ffff}0-9]+-?)*[a-z\x{00a1}-\x{ffff}0-9]+)(?:\.(?:[a-z\x{00a1}-\x{ffff}0-9]+-?)*[a-z\x{00a1}-\x{ffff}0-9]+)*(?:\.(?:[a-z\x{00a1}-\x{ffff}]{2,})))(?::\d{2,5})?(?:/[^\s]*)?$_iuS';

        return preg_match($format, $param);
    }

    public static function regex($value, $pattern = null)
    {
        return @preg_match($pattern, $value);
    }

    public static function getIp($check_for_proxy = false)
    {
        $config 	= RSDirectoryConfig::getInstance();
        $ip = '0.0.0.0';

        if ($config->get('store_ip', 1)) {
            $ip = $_SERVER['REMOTE_ADDR'];

            if ($check_for_proxy) {
                $headers = array('HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'HTTP_VIA', 'HTTP_X_COMING_FROM', 'HTTP_COMING_FROM');

                foreach ($headers as $header) {
                    if (!empty($_SERVER[$header])) {
                        $ip = $_SERVER[$header];
                    }
                }
            }
        }

        return $ip;
    }

    /**
     * Get unique hash.
     *
     * @access public
     *
     * @static
     *
     * @param string $prefix
     * @param bool $more_entropy
     *
     * @return string
     */
    public static function getHash($prefix = '', $more_entropy = false)
    {
        return self::md5( uniqid($prefix, $more_entropy) );
    }

    /**
     * Get an array of the active supported comment systems.
     *
     * @access public
     *
     * @static
     *
     * @return array
     */
    public static function get_comment_systems()
    {
        $db = JFactory::getDBO();

        $supported = array(
			'com_rscomments',
			'com_komento'
		);

        $systems = array(
            'com_rscomments' 	=> 'RSComments!',
            'facebook'			=> 'Facebook Comments',
            'disqus' 			=> 'Disqus',
			'com_komento' 		=> 'Komento'
        );

        foreach ($supported as $com_name)
        {
            $path = JPATH_SITE . "/administrator/components/$com_name";

            if ( file_exists($path) )
            {
                $query = $db->getQuery(true)
                       ->select( $db->qn('enabled') )
                       ->from( $db->qn('#__extensions') )
                       ->where( $db->qn('type') . ' = ' . $db->q('component') . ' AND ' . $db->qn('element') . ' = ' . $db->q($com_name) );

                $db->setQuery($query, 0, 1);

                if ( !$db->loadResult() )
                {
                    unset($systems[$com_name]);
                }
            }
            else
            {
                unset($systems[$com_name]);
            }
        }

        return $systems;
    }

    /**
     * Check if $haystack starts with $needle.
     *
     * @access public
     *
     * @static
     *
     * @param string $haystack
     * @param string $needle
     *
     * @return bool
     */
    public static function startsWith($haystack, $needle)
    {
        return !strncmp( $haystack, $needle, strlen($needle) );
    }

    /**
     * Check if $haystack ends with $needle.
     *
     * @access public
     *
     * @static
     *
     * @param string $haystack
     * @param string $needle
     *
     * @return bool
     */
    public static function endsWith($haystack, $needle)
    {
        $length = strlen($needle);

        if ($length == 0)
            return true;

        return substr($haystack, -$length) === $needle;
    }

    /**
     * Cut a string to the specified length.
     *
     * @access public
     *
     * @static
     *
     * @param string $str
     * @param int $length
     *
     * @return string
     */
    public static function cut($str, $length)
    {
        return strlen($str) > $length ? substr($str, 0, $length) . '...' : $str;
    }

    /**
     * Get the absolute JRoute.
     *
     * @access public
     *
     * @static
     *
     * @param string $url
     * @param bool $xhtml
     *
     * @return string
     */
    public static function absJRoute($url, $administrator = false, $xhtml = true)
    {
        return JUri::root() . ($administrator ? 'administrator/' : '') . substr( JRoute::_($url, $xhtml), strlen( JUri::base(true) ) + 1 );
    }

    /**
	 * Convert links in a text from relative to absolute.
	 *
	 * @param string $text The text processed
	 *
	 * @return string Text with converted links
	 */
    public static function relToAbs($text)
    {
        $base = JUri::base();
		$text = preg_replace("/(href|src)=\"(?!http|ftp|https|mailto|data)([^\"]*)\"/", "$1=\"$base\$2\"", $text);

        return $text;
    }

    /**
     * Check user permission.
     *
     * @access public
     *
     * @static
     *
     * @param string $permission
     * @param mixed $user Null to automatically load the logged in user, an user id or an user object.
     *
     * @return bool
     */
    public static function checkUserPermission($permission, $user = null)
    {
        static $permissions;

        if ( empty($permissions) )
        {
            // Get DBO.
            $db = JFactory::getDBO();

            // Get the groups.
            $query = $db->getQuery(true)
                   ->select('*')
                   ->from( $db->qn('#__rsdirectory_groups') )
				   ->where( $db->qn('published') . ' = ' . $db->q(1) );

            $db->setQuery($query);

            $groups = $db->loadObjectList();


            // Get the groups relations.
            $query = $db->getQuery(true)
                   ->select('*')
                   ->from( $db->qn('#__rsdirectory_groups_relations') );

            $db->setQuery($query);

            $groups_relations = $db->loadObjectList();

            // Initialize the permissions array.
            $permissions = array();

            foreach ($groups_relations as $group_relation)
            {
                if ( !isset($permissions[$group_relation->jgroup_id]) )
                {
                    foreach ($groups as $group)
                    {
                        if ($group_relation->group_id == $group->id)
                        {
                            $permissions[$group_relation->jgroup_id] = $group;
                        }
                    }
                }
            }
        }

        if (!$permissions)
            return false;

        if ( is_null($user) )
        {
            $user = JFactory::getUser();
        }
        else if ( is_numeric($user) )
        {
            $user = JFactory::getUser($user);
        }

        if ( empty($user) )
            return false;

        $jgroups = $user->groups;

        // Get the user jgroups.
        if ( $user->get('guest') && !$jgroups )
        {
            $jgroups = array(1);
        }

        if (!$jgroups)
            return false;

        $credits_found = array();

        // Check the permissions.
        foreach ($jgroups as $jgroup_id)
        {
            if ( !empty($permissions[$jgroup_id]->$permission) && (bool)$permissions[$jgroup_id]->$permission )
            {
                if ($permission == 'initial_credits')
                {
                    $credits_found[] = (int)$permissions[$jgroup_id]->$permission;
                }
                else
                {
                    return true;
                }
            }
        }

        return $permission == 'initial_credits' ? (count($credits_found) ? max($credits_found) : 0) : false;
    }

    /**
     * Check if an user can contact an entry author.
     *
     * @access public
     *
     * @static
     *
     * @param int $author_id
     *
     * @return bool
     */
    public static function canContactEntryAuthor($author_id)
    {
        if ( empty($author_id) )
            return false;

        $can_contact_entries_authors = self::checkUserPermission('can_contact_entries_authors');

        if (!$can_contact_entries_authors)
            return false;

        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select( $db->qn('enable_contact_form') )
               ->from( $db->qn('#__rsdirectory_users') )
               ->where( $db->qn('user_id') . ' = ' . $db->q($author_id) );

        $db->setQuery($query);
        $result = $db->loadResult();

        return is_null($result) || $result;
    }

    /**
     * Generate a random integer.
     *
     * @access public
     *
     * @static
     *
     * @param int $min
     * @param mixed $max
     *
     * return int
     */
    public static function rand($min = 0, $max = null)
    {
        if ( function_exists('mt_rand') )
        {
            if ( is_numeric($max) )
                return mt_rand($min, $max);
            else
                return mt_rand($min);
        }
        else
        {
            if ( is_numeric($max) )
                return rand($min, $max);
            else
                return rand($min);
        }
    }

    /**
     * Generate a random string.
     *
     * @access public
     *
     * @static
     *
     * @param int $len
     * @param string $chars
     *
     * return string
     */
    public static function randStr($len = 6, $chars = '1234567890abcdefghijklmnopqrstuvwxyz')
    {
        return substr( str_shuffle($chars), 0, $len );
    }

    /**
     * Get payment methods.
     *
     * @access public
     *
     * @static
     *
     * return mixed
     */
    public static function getPaymentMethods()
    {
        // Retrieve the payment options from the payment plugins.
        $payment_methods = JFactory::getApplication()->triggerEvent('onRsdirectoryaddPaymentOptions');

        if ($payment_methods)
        {
            foreach ($payment_methods as $i => $item)
            {
                if (is_array($item))
                {
                    foreach ($item as $j => $pitem)
                    {
                        if (empty($pitem->value))
                        {
                            unset($item[$j]);
                            continue;
                        }

                        $payment_methods[] = $pitem;
                    }

                    // unset the current
                    unset($payment_methods[$i]);
                }
                else
                {
                    if (empty($item->value))
                    {
                        unset($payment_methods[$i]);
                    }
                }
            }
        }

        return $payment_methods;
    }

    /**
     * Method to get a list of credit packages.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $published
     *
     * @return mixed
     */
    public static function getCreditPackages($published = null)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_credit_packages') )
               ->order( $db->qn('ordering') );

        if ( !is_null($published) )
        {
            $query->where( $db->qn('published') . ' = ' . $db->q($published) );
        }

        $db->setQuery($query);

        return $db->loadObjectList();
    }

    /**
     * Method to get a single credit package.
     *
     * @access public
     *
     * @static
     *
     * @param int $id The credit package id.
     *
     * @return mixed
     */
    public static function getCreditPackage($id)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_credit_packages') )
               ->where( $db->qn('id') . ' = ' . $db->q($id) );

        $db->setQuery($query);

        return $db->loadObject();
    }

    /**
     * Method to get a single user transaction.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $id Transaction id or hash.
     *
     * @return mixed
     */
    public static function getUserTransaction($id)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_users_transactions') );

        if ( is_numeric($id) )
        {
            $query->where( $db->qn('id') . ' = ' . $db->q($id) );
        }
        else if ( is_string($id) )
        {
            $query->where( $db->qn('hash') . ' = ' . $db->q($id) );
        }
        else
        {
            return;
        }

        $db->setQuery($query);

        return $db->loadObject();
    }

    /**
     * Save transaction log.
     *
     * @access public
     *
     * @static
     *
     * @param int $transaction_id
     * @param array $log
     *
     * @return mixed
     */
    public static function saveTransactionLog($transaction_id, $log)
    {
        if (!$transaction_id || !$log)
            return;

        $transaction_id = (int)$transaction_id;

        // Get DBO.
        $db = JFactory::getDBO();

        if ( !is_array($log) )
        {
            $log = (array)$log;
        }

        foreach ($log as $i => $item)
        {
            $log[$i] = '<b>' . JFactory::getDate()->format('Y/m/d') . '</b> ' . $item;
        }

        $log = implode('<br />', $log);

        $query = $db->getQuery(true)
               ->update( $db->qn('#__rsdirectory_users_transactions') )
               ->set( $db->qn('gateway_log') . ' = CONCAT(' . $db->qn('gateway_log') . ', ' . $db->q($log) . ')' )
               ->where( $db->qn('id') . ' = ' . $db->q($transaction_id) );

        $db->setQuery($query);

        return $db->execute();
    }

    /**
     * Format price.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $price
     *
     * @return string
     */
    public static function formatPrice($price, $snippets = false)
    {
        $config 				= RSDirectoryConfig::getInstance();
        $currency 				= $config->get('currency');
        $currency_sign 			= $config->get('currency_sign');
        $decimals 				= $config->get('decimals');
        $decimal_point 			= $config->get('decimal_point');
        $thousands_separator 	= $config->get('thousands_separator');
        $payment_mask 			= $config->get('payment_mask');
		$content_price 			= number_format($price, $decimals, $decimal_point, '');
        $price 					= number_format($price, $decimals, $decimal_point, $thousands_separator);

		$replacements = array(
			'{currency}' 		=> $currency,
			'{currency_sign}' 	=> $currency_sign,
			'{price}'			=> $price
		);

		if ($snippets) {
			$payment_mask = self::escapeHTML($payment_mask);

			$payment_mask = str_replace('{price}', '<meta itemprop="priceCurrency" content="'.self::escapeHTML($currency).'" /><span itemprop="price" content="'.$content_price.'">{price}</span>', $payment_mask);
		}

        return str_replace(array_keys($replacements), array_values($replacements), $payment_mask);
    }

	public static function formatExpiryDate($expiry_time) {
		if ($expiry_time == '0000-00-00 00:00:00') {
			return strtolower( JText::_('COM_RSDIRECTORY_NO_EXPIRY') );
		} else if ( JFactory::getDate($expiry_time)->toUnix() < JFactory::getDate()->toUnix() ) {
			return JText::_('COM_RSDIRECTORY_ENTRY_EXPIRED');
		} else {
			return RSDirectoryHelper::formatDate($expiry_time);
		}
	}

    /**
     * Get md5 hash.
     *
     * @access public
     *
     * @static
     *
     * @param string $str
     *
     * @return string
     */
    public static function md5($str)
    {
        return function_exists('hash') ? hash('md5', $str) : md5($str);
    }

    /**
     * Escape HTML.
     *
     * @access public
     *
     * @static
     *
     * @param string $string
     *
     * @return string
     */
    public static function escapeHTML($string)
    {
        return htmlentities($string, ENT_COMPAT, 'UTF-8');
    }

    /**
     * Clean text.
     *
     * @access public
     *
     * @static
     *
     * @param string $text
     * @param bool $js
     * @param bool $a
     *
     * @return string
     */
    public static function cleanText($text, $js = true, $a = true)
    {
        if ($js)
        {
            $text = preg_replace("'<script[^>]*>.*?</script>'si", '', $text);
        }

        if ($a)
        {
            $text = preg_replace('/<a\s+.*?href="([^"]+)"[^>]*>([^<]+)<\/a>/is', '\2 (\1)', $text);
        }

		return $text;
    }

    /**
     * Get the ids of the core field types.
     *
     * @access public
     *
     * @static
     *
     * @return array
     */
    public static function getCoreFieldTypesIds()
    {
        static $ids;

        if (!$ids)
        {
            // Get an instance of the FieldTypes model.
            $fieldtypes_model = RSDirectoryModel::getInstance('FieldTypes');

            // Get the core field types.
            $field_types = $fieldtypes_model->getFieldTypesObjectList(1);

            // Initialize the core field types ids array.
            $ids = array();

            // Populate the core field types ids array.
            if ($field_types)
            {
                foreach ($field_types as $field_type)
                {
                    $ids[] = $field_type->id;
                }
            }
        }

        return $ids;
    }

    /**
     * Get the ids of the field types that are always published.
     *
     * @access public
     *
     * @static
     *
     * @return array
     */
    public static function getAlwaysPublishedFieldTypesIds()
    {
        static $ids;

        if (!$ids)
        {
            // Get an instance of the FieldTypes model.
            $fieldtypes_model = RSDirectoryModel::getInstance('FieldTypes');

            // Get the core field types.
            $field_types = $fieldtypes_model->getFieldTypesObjectList();

            // Initialize the core field types ids array.
            $ids = array();

            // Populate the field types ids array.
            if ($field_types)
            {
                foreach ($field_types as $field_type)
                {
                    if ($field_type->always_published)
                    {
                        $ids[] = $field_type->id;
                    }
                }
            }
        }

        return $ids;
    }

    /**
     * Get an image URL.
     *
     * @access public
     *
     * @static
     *
     * @param string $hash
     * @param string $size
     *
     * @return string
     */
    public static function getImageURL($hash, $size = 'normal')
    {
        static $config, $files;

        // Check the hash.
        if ( !$hash )
            return;

        if ( !is_array($files) )
        {
            $files = array();
        }

        // Get the file.
        if ( empty($files[$hash]) )
        {
            $files[$hash] = RSDirectoryHelper::getFileObject(0, $hash);
        }

        $file = $files[$hash];

        if (!$file)
            return;

        if ( empty($config) )
        {
            // Get an instance of the RSDirectory Config.
            $config = RSDirectoryConfig::getInstance();
        }

        if ($size == 'mapicon')
        {
            $w = $config->get('mapicon_thumbnail_width');
            $h = $config->get('mapicon_thumbnail_height');
        }
        else if ($size == 'small')
        {
            $w = $config->get('small_thumbnail_width');
            $h = $config->get('small_thumbnail_height');
        }
        else if ($size == 'big')
        {
            $w = $config->get('big_thumbnail_width');
            $h = $config->get('big_thumbnail_height');
        }
        else if (preg_match('/^([0-9]+)x([0-9]+)$/', $size, $match))
		{
			$w = $match[1];
			$h = $match[2];
		}
		else
        {
            $w = $config->get('normal_thumbnail_width');
            $h = $config->get('normal_thumbnail_height');
            $far = 0;
        }

        if ( $config->get('watermark_images') && $size == 'normal' )
        {
            $watermark_file = JPATH_BASE . '/' . $config->get('watermark');
            $watermark_position = $config->get('watermark_position');
            $watermark_opacity = $config->get('watermark_opacity');
            $watermark_size = $config->get('watermark_size');

            $ext = JFile::getExt($file->file_name);

            $file_name = hash('md5', $file->file_name . $watermark_file . $watermark_position . $watermark_opacity . $watermark_size) . ($ext ? ".$ext" : '');
        }
        else
        {
            $file_name = $file->file_name;
        }

        // Set the cache dir path. 
        $cache_dir = JPATH_ROOT   . "/components/com_rsdirectory/files/cache/{$w}x{$h}/";

        // Set the cache file path.
        $cache_file = $cache_dir . $file_name;

        if ( file_exists($cache_file) )
        {
            return JUri::root() . "components/com_rsdirectory/files/cache/{$w}x{$h}/$file_name";
        }
        else
        {
            return self::absJRoute("index.php?option=com_rsdirectory&task=image.view&size=$size&hash=$hash");
        }
    }

    /**
     * Get the categories that belong to RSDirectory!
     *
     * @access public
     *
     * @static
     *
     * @return mixed
     */
    public static function getCategories($categories_ids = 0)
    {
		static $cache = array();

        if ( $categories_ids && !is_array($categories_ids) )
        {
            $categories_ids = (array)$categories_ids;
        }

        $categories_ids = self::arrayInt($categories_ids, true, true);
        $categories_ids = array_unique($categories_ids);

		$hash = md5(implode(',', $categories_ids));

		if (!isset($cache[$hash])) {
			// Get DBO.
			$db = JFactory::getDbo();

			$query = $db->getQuery(true)
				   ->select( $db->qn('id') . ', ' . $db->qn('parent_id') . ', ' . $db->qn('title') . ', ' . $db->qn('alias') . ', ' . $db->qn('params') . ', ' . $db->qn('published') )
				   ->from( $db->qn('#__categories') )
				   ->where( '('. $db->qn('extension') . ' = ' . $db->q('com_rsdirectory') . ' OR ' . $db->qn('extension') . ' = ' . $db->q('system') .')');

			if ($categories_ids)
			{
				$query->where($db->qn('id') .'IN (' . implode(',', $categories_ids) . ')');
			}

			$db->setQuery($query);

			$cache[$hash] = $db->loadObjectList();
		}

		return $cache[$hash];
    }

    /**
     * Get a category's status based on the status it's parent categories.
     *
     * @access public
     *
     * @static
     *
     * @param int $category_id
     * @param array $categories
     *
     * @return mixed
     */
    public static function getCategoryStatus($category_id, $categories)
    {
        $hierarchy = self::getCategoryHierarchy($category_id, $categories);

        if (!$hierarchy)
            return false;

        foreach ($hierarchy as $category)
        {
            if (!$category->published)
                return false;
        }

        return true;
    }

    /**
     * Get the inherited form associated to a category.
     *
     * @access public
     * @static
     *
     * @param int $category_id
     * @param mixed $categories
     *
     * @return object
     */
    public static function getCategoryInheritedForm($category_id, $categories = null)
    {
        if ( is_null($categories) )
        {
            // Get the categories.
            $categories = self::getCategories();
        }

        if (!$categories)
            return;

        // Get the form id.
        $form_id = self::getCategoryInheritedFormId($category_id, $categories);

        return self::getForm($form_id);
    }

    /**
     * Get the id of the inherited form associated to a category.
     *
     * @access public
     * @static
     *
     * @param int $category_id
     * @param mixed $categories
     *
     * @return object
     */
    public static function getCategoryInheritedFormId($category_id, $categories = null)
    {
        if ( is_null($categories) )
        {
            // Get the categories.
            $categories = self::getCategories();
        }

        if (!$categories)
            return;

        // Get the category hierarchy.
        $hierarchy = self::getCategoryHierarchy($category_id, $categories);

        if (!$hierarchy)
            return;

        // Get the index of the subject category.
        foreach ($hierarchy as $i => $category)
        {
            if ($category->id == $category_id)
            {
                $index = $i;
            }
        }

        // Exit the function if no category was found with the specified id.
        if ( !isset($index) )
            return;

        // Initialize the form id.
        $form_id = 0;

        for ($i = $index; $i >= 0; $i--)
        {
            // Get the category params.
            $params = new JRegistry($hierarchy[$i]->params);

            // Get the form id.
            $form_id = $params->get('form_id');

            // Exit the loop if we found the inherited form.
            if ($form_id)
                break;
        }

        return $form_id;
    }

    /**
     * Get form.
     *
     * @access public
     *
     * @static
     *
     * @param int $form_id
     *
     * @return object
     */
    public static function getForm($form_id)
    {
        static $forms = array();

        if ( !isset($forms[$form_id]) )
        {
            JTable::addIncludePath(JPATH_ADMINISTRATOR  . '/components/com_rsdirectory/tables');

            // Get the form.
            $form = JTable::getInstance('Form', 'RSDirectoryTable');
            $form->load($form_id);

            if (!$form_id)
            {
                $form->title = JText::_('JNONE');
            }

            $forms[$form_id] = $form;
        }

        return $forms[$form_id];
    }

    /**
     * Get a object list of all RSDirectory! forms.
     *
     * @access public
     *
     * @static
     *
     * @return mixed
     */
    public static function getForms()
    {
        // Get DBO.
        $db = JFactory::getDBO();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_forms') )
               ->order( $db->qn('id') );

        $db->setQuery($query);

        return $db->loadObjectList();
    }

    /**
     * Get the forms count.
     *
     * @access public
     *
     * @static
     *
     * @return int
     */
    public static function getFormsCount()
    {
        // Get DBO.
        $db = JFactory::getDBO();

        $query = $db->getQuery(true)
               ->select('COUNT(*)')
               ->from( $db->qn('#__rsdirectory_forms') );

        $db->setQuery($query);

        return $db->loadResult();
    }

    /**
     * Get category.
     *
     * @access public
     *
     * @static
     *
     * @param int $category_id
     *
     * @return object
     */
    public static function getCategory($category_id)
    {
        // Create a new JCategories object.
        $categories = new JCategories( array('extension' => 'com_rsdirectory', 'table' => '#__rsdirectory_entries', 'field' => 'category_id', 'statefield' => 'published') );

        return $categories->get($category_id);
    }

    /**
     * Get the subcategories of the specified category.
     *
     * @access public
     *
     * @static
     *
     * @param int $category_id
     *
     * @return array
     */
    public static function getSubcategories($category_id)
    {
        // Get the parent category.
        $parent = self::getCategory($category_id);

        if ($parent)
            return $parent->getChildren();
    }

    /**
     * Get categories select.
     *
     * @access public
     *
     * @static
     *
     * @param array $categories
     * @param string $more
     *
     * @return mixed
     */
    public static function getCategoriesSelect($categories, $more = '>')
    {
        // init the output string
        $str = '';

        $options = array(
            JHtml::_( 'select.option', 0, JText::_('JOPTION_SELECT_CATEGORY') ),
        );

        if ($categories)
        {
            foreach ($categories as $category)
            {
                $children = $category->getChildren();

                $text = $category->title;

                if ( $category->getChildren() )
                {
                    $text .= ' ' . $more;
                }

                $attr = array('option.key'=>'value');

                // add the custom attribute to the option if children are present
                if ($children)
                {
                    $attr['attr'] = 'data-children="1"';
                    $attr['option.attr'] = 'custom_attr';
                }

                $options[] =  JHtml::_( 'select.option',$category->id, self::escapeHTML($text), $attr );
            }
        }

        $str .= JHtml::_('select.genericlist', $options, '', array('list.attr'=>'class="rsdir-category-select form-select"', 'option.attr'=>'custom_attr'));
        
        $str .= JHtml::_('image', 'com_rsdirectory/loader.gif', '', 'class="rsdir-loader hide ms-1" width="16" height="11"', true);

        return $str;
    }

    /**
     * Get form fields.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $forms_ids A single integer value or an array of integer values.
     * @param mixed $published
     * @param mixed $expect_value
     * @param mixed $field_type
     *
     * @return mixed
     */
    public static function getFormFields($forms_ids = null, $published = null, $expect_value = null, $field_type = null)
    {
        if ( !is_array($forms_ids) )
        {
            $forms_ids = (array)$forms_ids;
        }

        $forms_ids = self::arrayInt($forms_ids, true, true);

        // Get DBO.
        $db = JFactory::getDbo();

        foreach ($forms_ids as $i => &$form_id)
        {
            $form_id = $db->q($form_id);
        }

        $select = array(
            $db->qn('f') . '.*',
            $db->qn('ft.type', 'field_type'),
            $db->qn('ft.core'),
            $db->qn('ft.create_column'),
            $db->qn('ft.expect_value'),
        );

        $query = $db->getQuery(true)
               ->select($select);

        if ($forms_ids)
        {
            $query->from( $db->qn('#__rsdirectory_forms_fields', 'ff') )
                  ->innerJoin( $db->qn('#__rsdirectory_fields', 'f') . ' ON ' . $db->qn('ff.field_id') . ' = ' . $db->qn('f.id') )
                  ->where( $db->qn('ff.form_id') . ' IN (' . implode(',', $forms_ids) . ')' )
                  ->order( $db->qn('ff.ordering') );
        }
        else
        {
            $query->from( $db->qn('#__rsdirectory_fields', 'f') );
        }

        $query->innerJoin( $db->qn('#__rsdirectory_field_types', 'ft') . ' ON ' . $db->qn('f.field_type_id') . ' = ' . $db->qn('ft.id') );

        if ( !is_null($published) )
        {
            $query->where( $db->qn('f.published') . ' = ' . $db->q($published) );
        }

        if ( !is_null($expect_value) )
        {
            $query->where( $db->qn('ft.expect_value') . ' = ' . $db->q($expect_value) );
        }

        if ( !is_null($field_type) )
        {
            $query->where( $db->qn('ft.type') . ' = ' . $db->q($field_type) );
        }

        $db->setQuery($query);

        // Get the form fields.
        $form_fields = $db->loadObjectList();

        if ($form_fields)
        {
            // Initialize the ids array.
            $ids = array();

            foreach ($form_fields as &$form_field)
            {
                $ids[] = $db->q($form_field->id);

                $form_field->properties = new RSDirectoryFormFieldProperties;
            }

            // Get the fields properties.
            $query = $db->getQuery(true)
                   ->select('*')
                   ->from('#__rsdirectory_fields_properties')
                   ->where( $db->qn('field_id') . ' IN (' . implode(',', $ids) . ')' );

            $db->setQuery($query);

            $properties = $db->loadObjectList();

            // Add the properties to their corresponding fields.
            if ($properties)
            {
                foreach ($properties as $property)
                {
                    foreach ($form_fields as &$form_field)
                    {
                        if ($property->field_id == $form_field->id)
                        {
                            $form_field->properties->set($property->property_name, $property->value);
                            break;
                        }
                    }
                }
            }
        }

        return $form_fields;
    }

    /**
     * Get entry.
     *
     * @access public
     *
     * @static
     *
     * @param int $entry_id
     *
     * @return object
     */
    public static function getEntry($entry_id)
    {
        $items = self::getEntriesObjectListByIds($entry_id);

        return isset($items[0]) ? $items[0] : null;
    }

    /**
     * Get a entries object list, selected by ids.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $ids
     *
     * @return mixed
     */
    public static function getEntriesObjectListByIds($ids)
    {
        if (!$ids)
        {
            return array();
        }

        if ( !is_array($ids) )
        {
            $ids = (array)$ids;
        }

        // Get entry author display info (name, username, email..).
		$author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');

        // Get DBO.
        $db = JFactory::getDbo();

        $select = array(
            $db->qn('e') . '.*',
            $db->qn('ec') . '.*',
            $db->qn('c.title', 'category_title'),
            $db->qn('c.path', 'category_path'),
            $db->qn("u.$author", 'author'),
        );

        // Quote the ids.
        foreach ($ids as &$id)
        {
            $id = $db->q($id);
        }

        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_entries', 'e') )
               ->leftJoin( $db->qn('#__rsdirectory_entries_custom', 'ec') . ' ON ' . $db->qn('e.id') . ' = ' . $db->qn('ec.entry_id') )
               ->leftJoin( $db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id') )
               ->where( $db->qn('e.id') . ' IN (' . implode(',', $ids) . ')' );

        $db->setQuery($query);

        return $db->loadObjectList();
    }

    /**
     * Get a list of entries with forms, form fields and files attached to each entry.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $entry
     *
     * @return array
     */
    public static function getEntryData($entry)
    {
        if (!$entry)
            return;

        if ( !is_array($entry) )
        {
            $entries = array($entry);
        }

        $entries = self::getEntriesData($entries);

        return isset($entries[0]) ? $entries[0] : null;
    }

    /**
     * Get a list of entries with forms, form fields and files attached to each entry.
     *
     * @access public
     *
     * @static
     *
     * @param array $entries
     *
     * @return array
     */
    public static function getEntriesData($entries, $with_custom = false)
    {
        if ( empty($entries) || !is_array($entries) )
            return;

        // Get DBO.
        $db = JFactory::getDbo();

        // Initialize the forms ids array.
        $forms_ids = array();

        // Initialize the entries ids array.
        $entries_ids = array();

        $user_ids = array();

        foreach ($entries as $entry)
        {
            $forms_ids[] = $entry->form_id;
            $entries_ids[] = $entry->id;
            $user_ids[] = $entry->user_id;
        }

        // Remove duplicate forms ids.
        $forms_ids = array_unique($forms_ids);

        // Remove duplicate entries ids.
        $entries_ids = array_unique($entries_ids);

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_forms') )
               ->where( $db->qn('id') . ' IN (' . implode(',', $db->q($forms_ids)) . ')' );

        $db->setQuery($query);

        $forms_list = $db->loadObjectList();

        // Initialize the forms array.
        $forms = array();

        // Place the forms in the array by form id.
        if ($forms_list)
        {
            foreach ($forms_list as $form)
            {
                $form->fields = self::getFormFields($form->id, 1);

                $forms[$form->id] = $form;
            }
        }

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_uploaded_files', 'f') )
               ->innerJoin( $db->qn('#__rsdirectory_uploaded_files_fields_relations', 'fr') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fr.file_id')  )
               ->where( $db->qn('fr.entry_id') . ' IN ' . '(' . implode(',', $db->q($entries_ids)) . ')' )
               ->order( $db->qn('fr.entry_id') . ', ' . $db->qn('fr.ordering') );

        $db->setQuery($query);

        $files_list = $db->loadObjectList();

        // Get favorites
        $user = JFactory::getUser();
        $favorites = array();
        if ($user->id)
        {
            $query = $db->getQuery(true)
                ->select('f.entry_id')
                ->from($db->qn('#__rsdirectory_favorites', 'f'))
                ->where($db->qn('f.entry_id') . ' IN (' . implode(',', $db->q($entries_ids)) . ')')
                ->where($db->qn('f.user_id') . ' = ' . $db->q(JFactory::getUser()->id));

            $favorites = $db->setQuery($query)->loadColumn();
        }

        // Get user data
        $author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');
        $users = array();
        if ($user_ids = array_filter($user_ids))
        {
            $query = $db->getQuery(true)
                ->select($db->qn($author))
                ->select($db->qn('id'))
                ->from($db->qn('#__users'))
                ->where($db->qn('id') . ' IN (' . implode(',', $db->q($user_ids)) . ')')
				->where($db->qn('block') . ' = ' .  $db->q(0));

            $users = $db->setQuery($query)->loadObjectList('id');
        }

        foreach ($entries as $j => $entry)
        {
			if (!isset($users[$entry->user_id])) {
                unset($entries[$j]);
            }
			
			// get the custom values for the entry - used in system plugin
            if ($with_custom)
            {
                $query = $db->getQuery(true)
                    ->select('*')
                    ->from($db->qn('#__rsdirectory_entries_custom'))
                    ->where($db->qn('entry_id') . ' = '. $db->q($entry->id));

                $custom_data = $db->setQuery($query)->loadObject();
                if ($custom_data)
                {
                    foreach($custom_data as $key => $value)
                    {
                        if ($key == 'entry_id')
                        {
                            continue;
                        }
                        $entry->{$key} = $value;
                    }
                }
            }
			
            $entry->faved = in_array($entry->id, $favorites);

            $entry->author = isset($users[$entry->user_id]) ? $users[$entry->user_id]->{$author} : '';

            // Assign the form to the entry.
            $entry->form = empty($forms[$entry->form_id]) ? null : clone $forms[$entry->form_id];

            // Clone and overwrite fields.
            if ( !empty($entry->form->fields) )
            {
                foreach ($entry->form->fields as $i => $field)
                {
                    $entry->form->fields[$i] = clone $field;
                }
            }

            // Add the files to its corresponding field.
            if ($files_list)
            {
                foreach ($files_list as $file)
                {
                    foreach ($entry->form->fields as $field)
                    {
                        if ($entry->id == $file->entry_id && $field->id == $file->field_id)
                        {
                            if ( empty($field->files) )
                            {
                                $field->files = array();
                            }


                            $field->files[] = $file;
                        }
                    }
                }
            }
        }

        return $entries;
    }

    /**
     * Get field.
     *
     * @access public
     *
     * @static
     *
     * @param int $field_id
     *
     * @return mixed
     */
    public static function getField($field_id)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $select = array(
            $db->qn('f') . '.*',
            $db->qn('ft.type', 'field_type'),
            $db->qn('ft.core'),
            $db->qn('ft.all_forms'),
            $db->qn('ft.create_column'),
            $db->qn('ft.expect_value'),
        );

        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_forms_fields', 'ff') )
               ->innerJoin( $db->qn('#__rsdirectory_fields', 'f') . ' ON ' . $db->qn('ff.field_id') . ' = ' . $db->qn('f.id') )
               ->innerJoin( $db->qn('#__rsdirectory_field_types', 'ft') . ' ON ' . $db->qn('f.field_type_id') . ' = ' . $db->qn('ft.id') )
               ->where( $db->qn('f.id') . ' = ' . $db->q($field_id) );

        $db->setQuery($query);

        // Get the field.
        $field = $db->loadObject();

        if (!$field)
            return;

        $field->properties = new RSDirectoryFormFieldProperties;

        // Get the fields properties.
        $query = $db->getQuery(true)
               ->select('*')
               ->from('#__rsdirectory_fields_properties')
               ->where( $db->qn('field_id') . ' = ' . $db->q($field->id) );

        $db->setQuery($query);

        $properties = $db->loadObjectList();

        // Add the properties to their corresponding fields.
        if ($properties)
        {
            foreach ($properties as $property)
            {
                $field->properties->set($property->property_name, $property->value);
            }
        }

        return $field;
    }

    /**
     * Get the fields displayed in the filtering form.
     *
     * @access public
     *
     * @static
     *
     * @param array $categories_ids
     *
     * @return mixed
     */
    public static function getFilterFields($categories_ids)
    {
		if (!$categories_ids)
		{
			$categories_ids = array();
		}

		// Get all the categories.
		$categories = self::getCategories();

		if ( !$categories_ids || in_array(0, $categories_ids) || in_array('', $categories_ids) )
		{
			foreach ($categories as $category)
			{
				if ($category->published)
				{
					$categories_ids[] = $category->id;
				}
			}
		}

		$categories_ids = array_unique($categories_ids);

        // Remove some values.
        $categories_ids = array_diff( $categories_ids, array(0, 1, '') );

		// Initialize the forms ids array.
		$forms_ids = array();

		foreach ($categories_ids as $category_id)
		{
			if ( self::getCategoryStatus($category_id, $categories) )
			{
				$forms_ids[] = self::getCategoryInheritedFormId($category_id, $categories);
			}
		}

		$forms_ids = array_unique($forms_ids);
		$forms_ids = self::arrayInt($forms_ids, true, true);

		if (!$forms_ids)
			return;

		// Initialize the forms fields array.
		$forms_fields = array();

		foreach ($forms_ids as $form_id)
		{
			$forms_fields[] = self::getFormFields($form_id, 1);
		}

		// Initialize holding the common form fields.
		$intersection = array();

		// Get the common form fields.
		foreach ($forms_fields as $form_fields)
		{
			if ($intersection)
			{
                $intersection = self::formFieldsIntersection($intersection, $form_fields);
			}
			else
			{
				$intersection = $form_fields;
			}
		}

		if ($intersection)
		{
			foreach ($intersection as $i => $form_field)
			{
				if ( !$form_field->properties->get('searchable_advanced') )
				{
					unset($intersection[$i]);
				}
			}
		}

		return $intersection;
    }

    /**
     * Compute the intersection between two form fields array.
     *
     * @access public
     *
     * @static
     *
     * @param array $form_fields1
     * @param array $form_fields2
     *
     * @return array
     */
    public static function formFieldsIntersection($form_fields1, $form_fields2)
    {
        $results = array();

        if ( is_array($form_fields1) && is_array($form_fields2) )
        {
           foreach ($form_fields1 as $k1 => $form_field1)
            {
                foreach($form_fields2 as $k2 => $form_field2)
                {
                    if ( isset($form_field1->name, $form_field2->name) && $form_field1->name === $form_field2->name )
                    {
                        $results[] = $form_field1;
                    }
                }
            }
        }

        return $results;
    }

    /**
     * Regenerate the entries titles, big subtitles, small subtitles and descriptions.
     *
     * @access public
     *
     * @static
     *
     * @param array $entries
     * @param array $elements The elements to regenerate: titles, big subtitles, small subtitles and/or descriptions.
     *
     * @return bool
     */
    public static function regenerateEntriesTitles( $entries, $elements = array('title', 'big_subtitle', 'small_subtitle', 'description', 'metadata_title', 'metadata_keywords', 'metadata_description') )
    {
        if ( !$entries || !is_array($entries) )
            return false;

        require_once dirname(__FILE__) . '/placeholders.php';

        // Get DBO.
        $db = JFactory::getDbo();

        // Initialize the forms ids array.
        $forms_ids = array();

        foreach ($entries as $entry)
        {
            $forms_ids[] = $db->q($entry->form_id);
        }

        // Remove duplicate forms ids.
        $forms_ids = array_unique($forms_ids);

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_forms') )
               ->where( $db->qn('id') . ' IN (' . implode(',', $forms_ids) . ')' );

        $db->setQuery($query);

        $forms_list = $db->loadObjectList();

        // Initialize the forms array.
        $forms = array();

        // Place the forms in the array by form id.
        if ($forms_list)
        {
            foreach ($forms_list as $form)
            {
                $forms[$form->id] = $form;
            }
        }

        $form_fields = self::getFormFields();

        foreach ($entries as $entry)
        {
            if ( empty($forms[$entry->form_id]) )
                continue;

            $form = $forms[$entry->form_id];

            $values = (object)array(
                'id' => $entry->id,
            );

            $changed = array();

            if ( in_array('title', $elements) && $form->use_title_template )
            {
                $values->title = RSDirectoryPlaceholders::getInstance($form->title_template, $form_fields, $entry, $form)
                               ->setParams( array('title', 'database') )
                               ->process();
                $changed[] = 'title';
            }

            if ( in_array('big_subtitle', $elements) && $form->use_big_subtitle_template )
            {
                $values->big_subtitle = RSDirectoryPlaceholders::getInstance($form->big_subtitle_template, $form_fields, $entry, $form)
                                      ->setParams( array('big-subtitle', 'database') )
                                      ->process();
                $changed[] = 'big_subtitle';
            }

            if ( in_array('small_subtitle', $elements) && $form->use_small_subtitle_template )
            {
                $values->small_subtitle = RSDirectoryPlaceholders::getInstance($form->small_subtitle_template, $form_fields, $entry, $form)
                                        ->setParams( array('small-subtitle', 'database') )
                                        ->process();
                $changed[] = 'small_subtitle';
            }

            if ( in_array('description', $elements) && $form->use_description_template )
            {
                $values->description = RSDirectoryPlaceholders::getInstance($form->description_template, $form_fields, $entry, $form)
                                     ->setParams( array('description', 'database') )
                                     ->process();
                $changed[] = 'description';
            }


            // handle the metadata
            if ( !empty( array_intersect( $elements, array('metadata_title', 'metadata_keywords', 'metadata_description') ) ) )
            {
                // change the entry values if something above has changed
                if (!empty($changed))
                {
                    foreach ($changed as $col) {
                        $entry->{$col} = $values->{$col};
                    }
                }

                if (in_array('metadata_title', $elements))
                {

                    $values->metadata_title = RSDirectoryPlaceholders::getInstance($form->metadata_title, $form_fields, $entry, $form)
                        ->setParams(array('metadata_title', 'database'))
                        ->process();
                }

                if (in_array('metadata_keywords', $elements))
                {
                    $values->metadata_keywords = RSDirectoryPlaceholders::getInstance($form->metadata_keywords, $form_fields, $entry, $form)
                        ->setParams(array('metadata_keywords', 'database'))
                        ->process();
                }

                if (in_array('metadata_description', $elements))
                {
                    $values->metadata_description = RSDirectoryPlaceholders::getInstance($form->metadata_description, $form_fields, $entry, $form)
                        ->setParams(array('metadata_description', 'database'))
                        ->process();
                }
            }

            $db->updateObject('#__rsdirectory_entries', $values, 'id');
        }

        return true;
    }

    /**
     * Get the files for a certain entry or field.
     *
     * @access public
     *
     * @static
     *
     * @param int $field_id
     * @param int $entry_id
     *
     * @return mixed
     */
    public static function getFilesObjectList($field_id = 0, $entry_id = 0)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_uploaded_files', 'f') )
               ->innerJoin( $db->qn('#__rsdirectory_uploaded_files_fields_relations', 'fr') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fr.file_id')  )
               ->order( $db->qn('fr.ordering') );

        if ( !empty($field_id) )
        {
            $query->where( $db->qn('fr.field_id') . ' = ' . $db->q($field_id) );
        }

        if ( !empty($entry_id) )
        {
            $query->where( $db->qn('fr.entry_id') . ' = ' . $db->q($entry_id) );
        }

        $db->setQuery($query);

        return $db->loadObjectList();
    }

    /**
     * Get a category thumbnail from the database.
     *
     * @access public
     *
     * @static
     *
     * @param int $field_id
     * @param int $category_id
     *
     * @return mixed
     */
    public static function getCategoryThumbObject($file_id = 0, $category_id = 0)
    {
        if (!$file_id && !$category_id)
            return;

        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_uploaded_files', 'f') )
               ->innerJoin( $db->qn('#__rsdirectory_uploaded_files_categories_relations', 'r') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('r.file_id')  );

        if ( !empty($file_id) )
        {
            $query->where( $db->qn('file_id') . ' = ' . $db->q($file_id) );
        }

        if ( !empty($category_id) )
        {
            $query->where( $db->qn('category_id') . ' = ' . $db->q($category_id) );
        }

        $db->setQuery($query, 0, 1);

        return $db->loadObject();
    }
    /**
     * Get a category thumbnail from the database.
     *
     * @access public
     *
     * @static
     *
     * @param int $field_id
     * @param int $category_id
     *
     * @return mixed
     */
    public static function getCategoryMapIconObject($file_id = 0, $category_id = 0)
    {
        if (!$file_id && !$category_id)
            return;

        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_uploaded_files', 'f') )
               ->innerJoin( $db->qn('#__rsdirectory_uploaded_files_cat_mapicons_rel', 'r') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('r.file_id')  );

        if ( !empty($file_id) )
        {
            $query->where( $db->qn('file_id') . ' = ' . $db->q($file_id) );
        }

        if ( !empty($category_id) )
        {
            $query->where( $db->qn('category_id') . ' = ' . $db->q($category_id) );
        }

        $db->setQuery($query, 0, 1);

        return $db->loadObject();
    }

    /**
     * Get file object.
     *
     * @access public
     *
     * @static
     *
     * @param int $file_id
     * @param string $hash
     *
     * @return mixed
     */
    public static function getFileObject($file_id = 0, $hash = 0)
    {
        // Get DBO. 
        $db = JFactory::getDbo();

        $select = array(
            $db->qn('f') . '.*',
            $db->qn('cr.category_id', 'category_id'),
            $db->qn('crm.category_id', 'category_id_map'),
            $db->qn('fr.entry_id'),
            $db->qn('fr.field_id'),
        );

        // Get the file.
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_uploaded_files', 'f') )
               ->leftJoin( $db->qn('#__rsdirectory_uploaded_files_categories_relations', 'cr') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('cr.file_id') )
               ->leftJoin( $db->qn('#__rsdirectory_uploaded_files_cat_mapicons_rel', 'crm') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('crm.file_id') )
               ->leftJoin( $db->qn('#__rsdirectory_uploaded_files_fields_relations', 'fr') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fr.file_id') );


        if ( !empty($file_id) )
        {
            $query->where( $db->qn('f.id') . ' = ' . $db->q($file_id) );
        }

        if ( !empty($hash) )
        {
            $query->where( $db->qn('f.hash') . ' = ' . $db->q($hash) );
        }

        $db->setQuery($query, 0, 1);

        return $db->loadObject();
    }

    /**
     * Delete one or more files from the disk and from the database.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $files_list A single file object or an array of file objects.
     */
    public static function deleteFiles($files_list)
    {
        if ( $files_list && !is_array($files_list) )
        {
            $files_list = array($files_list);
        }

        if (!$files_list)
            return;

        // Get DBO.
        $db = JFactory::getDBO();

        // Initialize the files ids array.
        $files_ids = array();

        foreach ($files_list as $file)
        {
            $path = JPATH_ROOT . "/components/com_rsdirectory/files/entries/$file->entry_id/$file->file_name";

            // Delete the file if it exists.
            if ( file_exists($path) )
            {
                JFile::delete($path);
            }

            $files_ids[] = $db->q($file->id);
        }

        // Get all the cache directories.
        $folders = JFolder::folders(JPATH_ROOT . '/components/com_rsdirectory/files/cache/');

        if ($folders)
        {
            foreach ($folders as $folder)
            {
                foreach ($files_list as $file)
                {
                    $path = JPATH_ROOT . "/components/com_rsdirectory/files/cache/$folder/$file->file_name";

                    // Delete the file if it exists.
                    if ( file_exists($path) )
                    {
                        JFile::delete($path);
                    }
                }
            }
        }

        $files_ids_str = implode(',', $files_ids);

        $query = $db->getQuery(true)
               ->delete( $db->qn('#__rsdirectory_uploaded_files') )
               ->where( $db->qn('id') . ' IN (' . $files_ids_str . ')' );

        $db->setQuery($query);
        $db->execute();

        $query = $db->getQuery(true)
               ->delete( $db->qn('#__rsdirectory_uploaded_files_fields_relations') )
               ->where( $db->qn('file_id') . ' IN (' . $files_ids_str . ')' );

        $db->setQuery($query);
        $db->execute();
    }

    /**
     * Get global placeholders HTML.
     *
     * @access public
     *
     * @static
     *
     * @return string
     */
    public static function getGlobalPlaceholdersHTML()
    {
        return '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_GLOBAL_TITLE') . '</h4>' .
               '<div>{site.name}</div>' .
               '<div>{site.url}</div>';
    }

    /**
     * Get user placeholders HTML.
     *
     * @access public
     *
     * @static
     *
     * @return string
     */
    public static function getUserPlaceholdersHTML()
    {
        return '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_USER_TITLE') . '</h4>'  .
               '<div>{username}</div>' .
               '<div>{name}</div>' .
               '<div>{email}</div>' .
               '<div>{userid}</div>';
    }

	/**
    * Get entry review placeholders HTML.
    *
    * @access public
    *
    * @static
    *
    * @return string
    */
	public static function getReviewPlaceholdersHTML()
	{
		return '<div class="rsdir-placeholders-review">'.
				   '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_REVIEW_ENTRY') . '</h4>'  .
				   '<div>{rev_name}</div>' .
				   '<div>{rev_subject}</div>' .
				   '<div>{rev_email}</div>' .
				   '<div>{rev_score}</div>' .
				   '<div>{rev_review}</div>'.
			   '</div>';
	}

    /**
     * Get credits placeholders HTML.
     *
     * @access public
     *
     * @static
     *
     * @return string
     */
    public static function getCreditsPlaceholdersHTML()
    {
        return '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CREDITS_TITLE') . '</h4>' .
               '<div>{credits-remaining}</div>' .
               '<div>{credits-spent} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CREDITS_SPENT') .  '</div>';
    }

    /**
     * Get credit purchase placeholders HTML.
     *
     * @access public
     *
     * @static
     *
     * @return string
     */
    public static function getEntryCreditsPurchasePlaceholdersHTML()
    {
        return '<div id="rsdir-credits-purchase-placeholders" class="hide">'.
        '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CREDITS_PURCHASE_TITLE') . '</h4>' .
        '<div>{price-credits}</div>' .
        '<div>{tax}</div>' .
        '<div>{tax-type}</div>' .
        '<div>{tax-value}</div>' .
        '<div>{total}</div>' .
        '<div>{credit-title}</div>' .
        '<div>{currency}</div>' .
        '<div>{credits}</div>' .
        '<div>{payment-method}</div>' .
        '<div>{date-purchased}</div>' .
        '</div>';
    }

    /**
     * Get credit purchase placeholders finalized HTML.
     *
     * @access public
     *
     * @static
     *
     * @return string
     */

    public static function getEntryCreditsPurchaseFinalizedPlaceholdersHTML()
    {
        return '<div id="rsdir-credits-purchase-finalized-placeholders" class="hide">'.
        '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CREDITS_PURCHASE_FINALIZED_TITLE') . '</h4>' .
        '<div>{gateway-order-number}</div>' .
        '<div>{gateway-order-type}</div>' .
        '<div>{date-finalized}</div>' .
        '<div>{status}</div>'.
        '</div>';
    }



    /**
     * Get entry general placeholders HTML.
     *
     * @access public
     *
     * @static
     *
     * @return string
     */
    public static function getEntryGeneralPlaceholdersHTML()
    {
        return '<div id="rsdir-general-entry-placeholders">'.
               '<h4 class="mt-3">' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_ENTRY_TITLE') . '</h4>' .
               '<div>{id}</div>' .
               '<div>{url}</div>' .
               '<div>{category} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CATEGORY') . '</div>' .
               '<div>{category-path} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CATEGORY_PATH') . '</div>' .
               '<div>{images} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_IMAGES') . '</div>' .
               '<div>{small-thumb} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_SMALL_THUMB') . '</div>' .
			   '<div>{small-thumb-src-x} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_SMALL_THUMB_SRC_X') . '</div>' .
               '<div>{big-thumb} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_BIG_THUMB') . '</div>' .
			   '<div>{big-thumb-src-x} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_BIG_THUMB_SRC_X') . '</div>' .
               '<div>{normal-thumb} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_NORMAL_THUMB') . '</div>' .
			   '<div>{normal-thumb-src-x} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_NORMAL_THUMB_SRC_X') . '</div>' .
               '<div>{publishing-date} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_PUBLISHING_DATE') . '</div>' .
			   '<div>{publishing-period} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_PUBLISHING_PERIOD') . '</div>' .
               '<div>{expiry-date} - ' . JText::_('COM_RSDIRECTORY_PLACEHOLDERS_EXPIRY_DATE') . '</div>' .
               '<div>{title}</div>' .
               '<div>{big-subtitle}</div>' .
               '<div>{small-subtitle}</div>' .
               '<div>{price}</div>' .
			   '<div>{price-value}</div>' .
               '<div>{description}</div>'.
               '<div id="activation-url">{activation-url}</div>'.
               '</div>';
    }

    /**
     * Get the custom fields placeholders HTML.
     *
     * @access public
     *
     * @static
     *
     * @param int $category_id
     * @param int $form_id
     * @preset mixed $preset
     *
     * @return string
     */
    public static function getCustomFieldsPlaceholdersHTML($category_id, $form_id = 0, $preset = null)
    {
        // Initialize the result string.
        $str = '';

        if ($category_id)
        {
            // Get the inherited form associated to the category.
            $form = self::getCategoryInheritedForm($category_id);

            $form_id = $form->id;
        }

        if (!$form_id)
        {
            $str .= JText::_('COM_RSDIRECTORY_PLACEHOLDERS_CATEGORY_NO_CUSTOM_FIELDS');

            // Close the application.
            return $str;
        }

        // Get the form fields associated to the form.
        $form_fields = self::getFormFields($form_id, null, 1);

        if ($form_fields)
        {
            $found = 0;

            foreach ($form_fields as $i => $form_field)
            {
                if ( $form_field->core || ( $preset == 'title' && in_array( $form_field->field_type, array('fileupload', 'images', 'image_upload') ) ) )
                {
                    unset($form_fields[$i]);
                    continue;
                }

                $found = 1;
            }

            if ($form_fields)
            {
                $table = self::getTableStructure( array_values($form_fields), 4, 'cols' );

                $str .= '<div class="row-fluid">';

                foreach ($table as $cells)
                {
                    $str .= '<div class="span3">';

                    foreach ($cells as $form_field)
                    {
                        if ($form_field)
                        {
                            $complete = array();
                            if ($form_field->field_type == 'image_upload')
                            {
                                $complete[] = '{' . $form_field->name . '-small-thumb-src-x}';
                                $complete[] = '{' . $form_field->name . '-big-thumb-src-x}';
                                $complete[] = '{' . $form_field->name . '-normal-thumb-src-x}';
                                $complete[] = JText::_('COM_RSDIRECTORY_PLACEHOLDERS_IMAGE_UPLOAD_X_NUMBER');
                            }

                            $complete = empty($complete) ? '' : '<br/>'.implode('<br/>', $complete);
                            $str .= '<div>{' . $form_field->name . '}'.$complete.'</div>';
                        }
                    }

                    $str .= '</div>';
                }

                $str .= '</div>';
            }
            else
            {
                $str .= JText::_('COM_RSDIRECTORY_PLACEHOLDERS_NO_CUSTOM_FIELDS');
            }
        }
        else
        {
            $str .= JText::_('COM_RSDIRECTORY_PLACEHOLDERS_NO_CUSTOM_FIELDS');
        }

        return $str;
    }

    /**
     * Format date according to the date settings.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $date A unix timestamp or a date in a valid format (see: http://www.php.net/manual/en/datetime.formats.php).
     * @param mixed $format Valid options: null (to get the global date and time format), "human_readable" or any valid date format.
     *
     * @return string
     */
    public static function formatDate($date, $format = null)
    {
        static $date_and_time_display;
		
		$timeZone = JFactory::getUser()->getParam('timezone', JFactory::getConfig()->get('offset'));

        if ( is_null($format) )
        {
            if ( empty($date_and_time_display) )
            {
                // Get the date and time display method.
                $date_and_time_display = RSDirectoryConfig::getInstance()->get('date_and_time_display');
				$show_time = RSDirectoryConfig::getInstance()->get('show_time');

				if (!$show_time && $date_and_time_display != 'human_readable') {
					list($date_format,$time_format) = explode(' ',$date_and_time_display,2);
					$date_and_time_display = $date_format;
				}
            }

            $format = $date_and_time_display;
        }

		if ($date === '0000-00-00 00:00:00') {
			$time = 0;
		} else {
			$time = is_string($date) ? JFactory::getDate($date)->toUnix() : $date;
		}

        if ($format == 'human_readable')
        {
            $date1 = $time;
            $date2 = JFactory::getDate()->toUnix();

            $diff_secs = abs($date1 - $date2);
            $base_year = min( JFactory::getDate($date1)->format('Y'), JFactory::getDate($date2)->format('Y') );

            $diff = gmmktime(0, 0, $diff_secs, 1, 1, $base_year);

            $data = (object)array(
                'years' => JFactory::getDate($diff)->format('Y') - $base_year,
                'months_total' => ( JFactory::getDate($diff)->format('Y') - $base_year ) * 12 + JFactory::getDate($diff)->format('n') - 1,
                'months' => JFactory::getDate($diff)->format('n') - 1,
                'days_total' => floor( $diff_secs / (3600 * 24) ),
                'days' => JFactory::getDate($diff)->format('j') - 1,
                'hours_total' => floor($diff_secs / 3600),
                'hours' => JFactory::getDate($diff)->format('G'),
                'minutes_total' => floor($diff_secs / 60),
                'minutes' => (int)JFactory::getDate($diff)->format('i'),
                'seconds_total' => $diff_secs,
                'seconds' => (int)JFactory::getDate($diff)->format('s'),
            );

            // Initialize the result array.
            $result = array();

            // Set the precision.
            $precision = 2;

            // Current precision.
            $current_precision = 0;

            $units = array('years', 'months', 'days', 'hours', 'minutes', 'seconds');

            foreach ($units as $unit)
            {
                if ( ($data->$unit || $current_precision) && $current_precision < $precision )
                {
                    if ($data->$unit)
                    {
                        $result[] = JText::plural( 'COM_RSDIRECTORY_NUMBER_OF_' . strtoupper($unit), $data->$unit );
                    }

                    $current_precision++;
                }
            }

            return implode(' ', $result);
        }

        return JFactory::getDate($time)->setTimezone(new DateTimeZone($timeZone))->format($format, true);
    }

    /**
     * Find one or more elements from an array.
     *
     * @access public
     *
     * @static
     *
     * @param array $conditions An array of one or more conditions.
     * @param array $list The array to search in.
     * @param bool $single Return a single element or an array of elements?
     *
     * @return mixed
     */
    public static function findElements($conditions, $list, $single = true)
    {
        if (!$conditions || !$list)
            return;

        // Initialize the restuls array. Will be used only if $single is set to false.
        $results = array();

        foreach ($list as $item)
        {
            foreach ($conditions as $column => $value)
            {
                if ($item->$column != $value)
                    continue 2;
            }

            if ($single)
            {
                return $item;
            }
            else
            {
                $results[] = $item;
            }
        }

        return $results ? $results : null;
    }

    /**
     * Find one or more form fields from an form fields array.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $conditions The search conditions. Can be an array or a string. If it's a string, then the search will be done using the field type.
     * @param array $form_fields The fields array to search in.
     * @param bool $single Return a single element or an array of elements?
     *
     * @return mixed
     */
    public static function findFormField($conditions, $form_fields, $single = true)
    {
        if (!$conditions || !$form_fields)
            return;

        if ( !is_array($conditions) )
        {
            $conditions = array('field_type' => $conditions);
        }

        return self::findElements($conditions, $form_fields, $single);
    }

    /**
     * Get entry meta HTML string.
     *
     * @access public
     *
     * @static
     *
     * @param object $entry
     * @param object $form
     * @param string $section
     *
     * @return string
     */
    public static function getEntryMeta($entry, $form, $section = 'listing_row')
    {
        if ( !$entry || !$form || !in_array( $section, array('listing_row', 'listing_detail') ) )
            return;

        static $date_and_time_display;
        static $view;

        if ( is_null($date_and_time_display) )
        {
            $date_and_time_display = RSDirectoryConfig::getInstance()->get('date_and_time_display');
        }

        if ( is_null($view) )
        {
            $view = JFactory::getApplication()->input->get('view');
        }

        // Initialize the translation string.
        $trans = 'COM_RSDIRECTORY_LISTING_META';

        // Initialize the translation function params array.
        $trans_params = array();

        $show_author = $section . '_show_author';

        if ($form->$show_author && $view != 'myentries')
        {
            $trans .= '_AUTHOR';

            $author_url = RSDirectoryRoute::getUserEntriesURL($entry->user_id, $entry->author);
            $author_title = JText::sprintf( 'COM_RSDIRECTORY_VIEW_ENTRIES_POSTED_BY', self::escapeHTML($entry->author) );

            $trans_params[] = '<i class="fa fa-user '.RSDirectoryHelper::getTooltipClass().'" title="'.JText::_('COM_RSDIRECTORY_ENTRY_TOOLTIP_AUTHOR').'"></i> <a href="' . $author_url . '" title="' . $author_title . '">' . self::escapeHTML($entry->author) . '</a>';
        }

        $show_publishing_time = $section . '_show_publishing_time';

        if ( $form->$show_publishing_time && $entry->published && JFactory::getDate($entry->published_time)->toUnix() <= JFactory::getDate()->toUnix() )
        {
            $trans .= $date_and_time_display == 'human_readable' ? '_TIME_SINCE' : '_TIME';

            $trans_params[] = '<i class="fa fa-calendar '.RSDirectoryHelper::getTooltipClass().'" title="'.JText::_('COM_RSDIRECTORY_ENTRY_TOOLTIP_PUBLISH_DATE').'"></i> ' . self::escapeHTML( RSDirectoryHelper::formatDate($entry->published_time) );
        }

        $show_category = $section . '_show_category';
		$show_category_hierarchy = $section . '_show_category_hierarchy';

        if ($form->$show_category)
        {
            $trans .= '_CATEGORY';
            $category_title = JText::sprintf( 'COM_RSDIRECTORY_VIEW_ENTRIES_POSTED_IN', self::escapeHTML($entry->category_title) );
			$category_route = '<a href="' . RSDirectoryRoute::getCategoryEntriesURL($entry->category_id, $entry->category_title) . '" title="' . $category_title . '">' . self::escapeHTML($entry->category_title) . '</a>';

			if ($form->$show_category_hierarchy) {
				$categories = self::getCategories();
				$hierarchy 	= self::getCategoryHierarchy($entry->category_id, $categories);

				$category_route = array();
				foreach ($hierarchy as $cat) {
					$category_title = JText::sprintf( 'COM_RSDIRECTORY_VIEW_ENTRIES_POSTED_IN', self::escapeHTML($cat->title) );
					$category_route[] = '<a href="' . RSDirectoryRoute::getCategoryEntriesURL($cat->id, $cat->title) . '" title="' . $category_title . '">' . self::escapeHTML($cat->title) . '</a>';
				}
				$category_route = implode(' &rarr; ', $category_route);
			}

            $trans_params[] = '<i class="fa fa-folder-open '.RSDirectoryHelper::getTooltipClass().'" title="'.JText::_('COM_RSDIRECTORY_ENTRY_TOOLTIP_CATEGORY').'"></i> '.$category_route;
        }

        if ($trans_params) {
            return call_user_func_array(array('JText', 'sprintf'), array_merge(array($trans), $trans_params));
        } else {
            return '';
        }
    }

    /**
     * Get the ids of the custom fields for a certain form.
     *
     * @access public
     *
     * @static
     *
     * @param object $entry
     *
     * @return array
     */
    public static function getFormCustomFieldsIds($entry)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select( $db->qn('f.id') )
               ->from( $db->qn('#__rsdirectory_forms_custom_fields', 'fcf') )
               ->innerJoin( $db->qn('#__rsdirectory_fields', 'f') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fcf.field_id') )
			   ->innerJoin( $db->qn('#__rsdirectory_forms_fields', 'ff') . ' ON ' . $db->qn('ff.field_id') . ' = ' . $db->qn('fcf.field_id') . ' AND '.$db->qn('ff.form_id') . ' = ' . $db->q($entry->form_id) )
               ->where( $db->qn('fcf.form_id') . ' = ' . $db->q($entry->form_id) )
               ->where( $db->qn('f.published') . ' = ' . $db->q(1) )
			   ->order($db->qn('ff.ordering'));

        $db->setQuery($query);

        $fields_ids = $db->loadColumn();

        if ($fields_ids)
        {
            $form_fields_aux = self::getFormFields($entry->form_id, true);

            $form_fields = array();

            if ($form_fields_aux)
            {
                foreach ($form_fields_aux as $form_field)
                {
                    $form_fields[$form_field->id] = $form_field;
                }
            }

            // Remove empty fields.
            foreach ($fields_ids as $i => $field_id)
            {
                if ( empty($form_fields[$field_id]) )
                {
                    unset($fields_ids[$i]);
                }
                else
                {
                    if ( isset($entry->{$form_fields[$field_id]->column_name}) )
                    {
                        $value = $entry->{$form_fields[$field_id]->column_name};

                        if ( trim($value) === '' || $value == '0000-00-00 00:00:00' )
                        {
                            unset($fields_ids[$i]);
                        }
                    }
                    else
                    {
                        unset($fields_ids[$i]);
                    }
                }
            }
        }

        return $fields_ids ? array_values($fields_ids) : null;
    }

    /**
     * Get table structure.
     *
     * @access public
     *
     * @static
     *
     * @param array $list
     * @param int $columns
     * @return array $return Organise the result array by cols or by rows.
     *
     */
    public static function getTableStructure($list, $columns = 3, $return = 'rows')
    {
        if (!$list || !$columns)
        {
            return array();
        }

        // Get the list length.
        $list_len = count($list);

        // Calculate the number of rows.
        $rows = ceil($list_len / $columns);

        // Initialize the table array.
        $table = array_fill( 0, $columns, array_fill(0, $rows, 0) );

        $column = 0;
        $row = 0;

        for ($i = 0; $i < $list_len; $i++)
        {
            $table[$column][$row] = 1;

            if ($column < $columns - 1)
            {
                $column++;
            }
            else
            {
                $column = 0;
                $row++;
            }
        }

        $i = 0;

        if ($return == 'cols')
        {
            // Initialize the result array.
            $result = array_fill( 0, $columns, array_fill(0, $rows, null) );

            foreach ($table as $column => $rows)
            {
                foreach ($rows as $row => $value)
                {
                    if (!$value)
                        break;

                    $result[$column][$row] = $list[$i];

                    $i++;
                }
            }
        }
        else
        {
            // Initialize the result array.
            $result = array_fill( 0, $rows, array_fill(0, $columns, null) );

            foreach ($table as $column => $rows)
            {
                foreach ($rows as $row => $value)
                {
                    if (!$value)
                        break;

                    $result[$row][$column] = $list[$i];

                    $i++;
                }
            }
        }

        return $result;
    }

    /**
     * Check if an user posted a review for a certain entry.
     *
     * @access public
     *
     * @static
     *
     * @param int $entry_id
     * @param mixed $user_id
     *
     * @return bool
     */
    public static function hasReview($entry_id, $user_id = null)
    {
        if (!$entry_id)
            return;

        if ( is_null($user_id) )
        {
            $user_id = JFactory::getUser()->id;
        }

        JTable::addIncludePath(JPATH_ADMINISTRATOR  . '/components/com_rsdirectory/tables');

        // Get an instance of the Review table.
        $review = JTable::getInstance('Review', 'RSDirectoryTable');

        $keys = array(
            'entry_id' => $entry_id,
            'user_id' => $user_id,
        );

        if (!$user_id)
        {
            $keys['ip'] = RSDirectoryHelper::getIp(true);
        }

        // Load review.
        $review->load($keys);

        return (bool)$review->id;
    }

    /**
     * Get a review.
     *
     * @access public
     *
     * @static
     *
     * @param int $id
     *
     * @return mixed
     */
    public static function getReview($id)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $select = array(
            $db->qn('r') . '.*',
            $db->qn('u.name', 'author_name'),
            $db->qn('u.username'),
            $db->qn('u.email', 'author_email'),
			$db->qn('e.user_id', 'entry_author_id'),
        );

        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_reviews', 'r') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('r.user_id') . ' = ' . $db->qn('u.id') )
               ->innerJoin( $db->qn('#__rsdirectory_entries', 'e') . ' ON ' . $db->qn('r.entry_id') . ' = ' . $db->qn('e.id') )
               ->where( $db->qn('r.id') . ' = ' . $db->q($id) );

        $db->setQuery($query);

        return $db->loadObject();
    }

    /**
     * Check if an user added an entry to favorites.
     *
     * @access public
     *
     * @static
     *
     * @param int $entry_id
     * @param int $user_id
     *
     * @return bool
     */
    public static function isFavorite($entry_id, $user_id = 0)
    {
        if (!$entry_id)
            return;

        if (!$user_id)
        {
            $user_id = JFactory::getUser()->id;
        }

        if (!$user_id)
            return false;

        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select( $db->qn('entry_id') )
               ->from( $db->qn('#__rsdirectory_favorites') )
               ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) . ' AND ' . $db->qn('user_id') . ' = ' . $db->q($user_id) );

        $db->setQuery($query);

        return (bool)$db->loadResult();
    }

    /**
     * Create a entry folder.
     *
     * @access public
     *
     * @static
     *
     * @param int $entry_id
     *
     * @return mixed The entry dir on succes, or false or failure.
     */
    public static function createEntryDir($entry_id)
    {
        if (!$entry_id)
            return false;

        // Set the target directory.
        $entry_dir = JPATH_ROOT  . "/components/com_rsdirectory/files/entries/$entry_id";

        // Create the target directory if it does not exist.
        if ( !file_exists($entry_dir) && is_writable(JPATH_ROOT . '/components/com_rsdirectory/files/entries') )
        {
            if ( mkdir($entry_dir) )
            {
                // Put an index.html file to prevent snooping around.
                if ( file_put_contents("$entry_dir/index.html", '<html><body bgcolor="#FFFFFF"></body></html>') !== false )
                    return $entry_dir;
            }

            return false;
        }

        return $entry_dir;
    }

    /**
     * Get dependencies.
     *
     * @access public
     *
     * @static
     *
     * @param int $field_id
     * @param int $parent_id
     * @param mixed $value
     *
     * @return mixed
     */
    public static function getDependencies($field_id, $parent_id = 0, $value = null)
    {
        if (!$field_id && !$parent_id)
            return false;

        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_fields_dependencies') );

        if ($field_id)
        {
            $query->where( $db->qn('field_id') . ' = ' . $db->q($field_id) );
        }

        if ($parent_id)
        {
            $query->where( $db->qn('parent_id') . ' = ' . $db->q($parent_id) );
        }

        if ( !is_null($value) )
        {
            $query->where( $db->qn('value') . ' = ' . $db->q($value) );
        }

        $db->setQuery($query);

        return $db->loadObjectList();
    }

	/**
     * Get dependencies items.
     *
     * @access public
     *
     * @static
     *
     * @param obj $dependency
     * @param JRegistry $params
     *
     * @return mixed
     */
	public static function getDependenciesItems($dependency, $params)
	{
		$db = JFactory::getDbo();

		$user = JFactory::getUser();

		$items = array();

		$form_field = RSDirectoryHelper::getField($dependency->field_id);

		$items_array = RSDirectoryHelper::getOptions($dependency->items);

		$num_entries = $params->get('num_entries');

		foreach ($items_array as $item)
		{
			$item_aux = clone $item;
			$item_aux->parent = $dependency->field_id;
			$item_aux->form_field_name = $form_field->form_field_name;

			if ($num_entries)
			{
				$item_aux->entries_count = 0;
			}

			$items[] = $item_aux;
		}

		$column = $db->qn( ($form_field->create_column ? 'ec' : 'e') . ".$form_field->column_name" );

		if ($num_entries)
		{
			$can_view_all_unpublished_entries = RSDirectoryHelper::checkUserPermission('can_view_all_unpublished_entries');
			$can_view_own_unpublished_entries = RSDirectoryHelper::checkUserPermission('can_view_own_unpublished_entries');

			foreach ($items as $item)
			{
				$query = $db->getQuery(true)
					   ->select( 'COUNT(*) AS ' . $db->qn('count') )
					   ->from( $db->qn('#__rsdirectory_entries', 'e') )
					   ->innerJoin( $db->qn('#__rsdirectory_entries_custom', 'ec') . ' ON ' . $db->qn('e.id') . ' = ' . $db->qn('ec.entry_id') )
					   ->innerJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') .  ' = ' . $db->qn('u.id') )
					   ->where( '(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' >= ' . $db->q( JFactory::getDate()->toSql() ) . ')' )
					   ->where( $db->qn('u.block') . ' = ' . $db->q(0) )
					   ->where( $column . ' LIKE ' . $db->q( '%' . $db->escape($item->value, true) . '%' ) );

				if (!$can_view_all_unpublished_entries)
				{
					if ($can_view_own_unpublished_entries)
					{
					   $query->where(
							'( (' . $db->qn('e.published') . ' = ' . $db->q(1) . ' AND ' . $db->qn('e.published_time') . ' <= ' . $db->q( JFactory::getDate()->toSql() ) . ')' .
							' OR ' . $db->qn('e.user_id') . ' = ' . $db->q($user->id) . ')'
						);
					}
					else
					{
						$query->where( $db->qn('e.published') . ' = ' . $db->q(1) )
							  ->where( $db->qn('e.published_time') . ' <= ' . $db->q( JFactory::getDate()->toSql() ) );
					}
				}

				$db->setQuery($query);
				$item->entries_count = $db->loadResult();
			}
		}

		return $items;
	}

	/**
     * Output dependencies child HTML.
     *
     * @access public
     *
     * @static
     *
     * @param int $parent
     * @param mixed $value
     * @param JRegistry $params
     * @param int $itemid
     */
	public static function outputDependenciesChildHTML($parent, $value, $params, $itemid) {
		if ($dependencies = self::getDependencies(0, $parent, $value))
		{
			foreach ($dependencies as $dependency)
			{
				$items = self::getDependenciesItems($dependency,$params);

				foreach ($items as $item)
				{
					echo '<div class="media">';
					echo '<div class="media-body">';
					echo '<h5 class="media-heading">';

					$uri = new JUri( JRoute::_( "index.php?option=com_rsdirectory&view=entries" . ($itemid ? "&Itemid=$itemid" : '') ) );
					$uri->setVar( "f[$item->form_field_name]", urlencode($item->value) );

					echo '<a href="'.htmlspecialchars( $uri->toString() ).'">'.self::escapeHTML($item->text).'</a>';
					echo $params->get('num_entries') ? ' (' . $item->entries_count . ')' : '';
					echo '</h5>';

					self::outputDependenciesChildHTML($item->parent, $item->value, $params, $itemid);

					echo '</div>';
					echo '</div>';
				}
			}
		}
	}

    /**
     * Check wether a field is a dependency parent for other fields.
     *
     * @access public
     *
     * @static
     *
     * @param int $field_id
     *
     * @return bool
     */
    public static function isDependencyParent($field_id)
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
               ->select('COUNT(*)')
               ->from( $db->qn('#__rsdirectory_fields_properties') )
               ->where( $db->qn('property_name') . ' = ' . $db->q('dependency') )
               ->where( $db->qn('value') . ' = ' . $db->q($field_id) );

        $db->setQuery($query);
        return (bool)$db->loadResult();
    }

    /**
     * Remove the elements of the array that are not integers.
     *
     * @access public
     *
     * @static
     *
     * @param array $array
     * @param bool $reset_keys
     * @param bool $remove_empty Remove the elements whose values are empty.
     *
     * @return mixed
     */
    public static function arrayInt($array, $reset_keys = true, $remove_empty = false)
    {
        if ( !is_array($array) || !$array )
            return array();

        foreach ($array as $key => $value)
        {
            if ( !is_numeric($value) || ( $remove_empty && empty($value) ) )
            {
                unset($array[$key]);
            }
        }

        return array_values($array);
    }

    /**
     * Find a category in a hierarchical list of categories.
     *
     * @access public
     *
     * @static
     *
     * @param int $category_id
     * @param array $categories
     * @param object &$category
     */
    public static function findCategory($category_id, $categories, &$category)
    {
        if ( !$category_id || !$categories || !is_array($categories) )
            return;

        foreach ($categories as $cat)
        {
            if ($cat->id == $category_id)
            {
                $category = $cat;
                break;
            }

            if ( $children = $cat->getChildren() )
            {
                self::findCategory($category_id, $children, $category);
            }
        }
    }

    /**
     * Check if an user exists.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $condition An integer (id) or an array of column => value pairs.
     */
    public static function userExists($condition)
        {
        if ( !is_array($condition) )
        {
            $condition = array('id' => $condition);
        }

        $db = JFactory::getDBO();

        $query = $db->getQuery(true)
               ->select('COUNT(*)')
               ->from( $db->qn('#__users') );

        foreach ($condition as $column => $value)
        {
            $query->where( $db->qn($column) . ' = ' . $db->q($value) );
        }

        $db->setQuery($query, 0, 1);

        return (bool)$db->loadResult();
    }

    /**
     * SEF.
     *
     * @access public
     *
     * @static
     *
     * @param int $id
     * @param string $name
     *
     * @return string
     */
    public static function sef($id, $name)
    {
        return intval($id) . ':' . JFilterOutput::stringURLSafe($name);
    }

    /**
     * Get review HTML.
     *
     * @access public
     *
     * @static
     *
     * @param object $review
     *
     * @return string
     */
	public static function getReviewHTML($review)
    {
		$view = new JViewLegacy(array(
			'name'		=> 'entry',
			'base_path'	=> JPATH_SITE.'/components/com_rsdirectory',
			'layout'	=> 'default'
		));
		
		// Allow view to be overrided by the template
		$view->addTemplatePath(JPATH_THEMES . '/' . JFactory::getApplication()->getTemplate() . '/html/com_rsdirectory/' . $view->getName());
		
		$model = JModelLegacy::getInstance('Entry', 'RSDirectoryModel', array(
			'option' 		=> 'com_rsdirectory',
			'table_path' 	=> JPATH_ADMINISTRATOR.'/components/com_rsdirectory/tables'
		));
		
		$view->setModel($model, true);
		
		$view->review = $review;
		$view->config = RSDirectoryConfig::getInstance();
		
		return $view->loadTemplate('review');
    }
   
	// detect if a text is html or not
	public static function isHTML($text)
	{
		return htmlspecialchars($text) !== $text;
	}

    /**
     * Get owner reply HTML.
     *
     * @access public
     *
     * @param string $owner_reply
     *
     * @return string
     */
    public static function getOwnerReplyHTML($owner_reply)
    {
        $view = new JViewLegacy(array(
            'name'		=> 'entry',
            'base_path'	=> JPATH_SITE.'/components/com_rsdirectory',
            'layout'	=> 'default'
        ));

        // Allow view to be overrided by the template
        $view->addTemplatePath(JPATH_THEMES . '/' . JFactory::getApplication()->getTemplate() . '/html/com_rsdirectory/' . $view->getName());

        $model = JModelLegacy::getInstance('Entry', 'RSDirectoryModel', array(
            'option' 		=> 'com_rsdirectory',
            'table_path' 	=> JPATH_ADMINISTRATOR.'/components/com_rsdirectory/tables'
        ));

        $view->setModel($model, true);

        $view->owner_reply = $owner_reply;

        return $view->loadTemplate('owner_reply');
    }

	public static function getRatingText($avg_rating, $ratings_count) {
		$text = JText::_('COM_RSDIRECTORY_AVERAGE_RATING_VALUE');

		return sprintf( $text, number_format($avg_rating, 1), $ratings_count );
	}

    /**
     * Build the entries filtering query.
     *
     * @access public
     *
     * @static
     *
     * @param array $fields
     * @param array $filters
     * @param object &$query
     */
    public static function buildEntriesFilteringQuery($fields, $filters, &$query)
    {
        if (!$fields || !$filters || !$query)
            return;

        $db = JFactory::getDbo();

        foreach ($filters as $form_field_name => $values)
        {
            if ( is_string($values) && $values === '' )
                continue;

            $field = RSDirectoryHelper::findElements( array('form_field_name' => $form_field_name), $fields );

            if (!$field)
                continue;

            $table = $field->create_column ? 'ec' : 'e';

            if ($field->field_type == 'map')
            {
                $column_name = $db->qn("$table.{$field->column_name}_address");
            }
            else
            {
                $column_name = $db->qn("$table.$field->column_name");
            }

            $searchable_advanced = $field->properties->get('searchable_advanced');

            // Process the range filter.
            if ($searchable_advanced == 'range')
            {
                // Proceed if everything's ok.
                if ( $values && is_array($values) )
                {
                    unset($from);
                    unset($to);

                    // Initialize the ranges conditions array.
                    $ranges_cond = array();

                    $column_cast = "CAST( $column_name AS DECIMAL(30, 6) )";

                    foreach ($values as $range)
                    {
                        $range = trim($range);

                        // Less than.
                        if ( strpos($range, 'lt') !== false )
                        {
                            $range = trim( str_replace('lt-', '', $range) );

                            if ( !is_numeric($range) )
                                continue;

                            $ranges_cond[] = "$column_cast <= CAST( " . $db->q($range) . " AS DECIMAL(30, 6) )";
                        }
                        // Greater than.
                        else if ( strpos($range, 'gt') !== false )
                        {
                            $range = trim( str_replace('gt-', '', $range) );

                            if ( !is_numeric($range) )
                                continue;

                            $ranges_cond[] = "$column_cast >= CAST( " . $db->q($range) . " AS DECIMAL(30, 6) )";
                        }
                        // Custom range.
                        else if ( strpos($range, 'from') !== false || strpos($range, 'to') !== false )
                        {
                            // From.
                            if ( strpos($range, 'from-') !== false )
                            {
                                $range = trim( str_replace('from-', '', $range) );

                                if ( is_numeric($range) )
                                {
                                    $from = $range;
                                }
                            }

                            // To.
                            if ( strpos($range, 'to-') !== false )
                            {
                                $range = trim( str_replace('to-', '', $range) );

                                if ( is_numeric($range) )
                                {
                                    $to = $range;
                                }
                            }
                        }
                        // Normal range i - j.
                        else
                        {
                            $range = explode('-', $range);

                            foreach ($range as &$v)
                            {
                                $v = trim($v);
                            }

                            if ( isset($range[2]) || !is_numeric($range[0]) || !is_numeric($range[1]) )
                                continue;

                            if ($range[0] > $range[1])
                            {
                                $ranges_cond[] = "( $column_cast >= CAST( " . $db->q($range[1]) . " AS DECIMAL(30, 6) ) AND $column_cast <= CAST( " . $db->q($range[0]) . " AS DECIMAL(30, 6) ) )";
                            }
                            else
                            {
                                $ranges_cond[] = "( $column_cast >= CAST( " . $db->q($range[0]) . " AS DECIMAL(30, 6) ) AND $column_cast <= CAST( " . $db->q($range[1]) . " AS DECIMAL(30, 6) ) )";
                            }
                        }
                    }

                    if ( isset($from, $to) )
                    {
                        if ($from > $to)
                        {
                            $aux = $from;
                            $from = $to;
                            $to = $aux;
                        }

                        $ranges_cond[] = "( $column_cast >= CAST( " . $db->q($from) . " AS DECIMAL(30, 6) ) AND $column_cast <= CAST( " . $db->q($to) . " AS DECIMAL(30, 6) ) )";
                    }
                    else if ( isset($from) )
                    {
                        $ranges_cond[] = "$column_cast >= CAST( " . $db->q($from) . " AS DECIMAL(30, 6) )";
                    }
                    else if ( isset($to) )
                    {
                        $ranges_cond[] = "$column_cast <= CAST( " . $db->q($to) . " AS DECIMAL(30, 6) )";
                    }

                    if ($ranges_cond)
                    {
                        $query->where( '(' . implode(' OR ', $ranges_cond) . ')' );
                    }
                }
            }
            else if ($searchable_advanced == 'date_range')
            {
                if ( !empty($values['from']) )
                {
                    $query->where( $column_name . ' >= STR_TO_DATE(' . $db->q( JFactory::getDate($values['from'])->toSql() ) . ', ' . $db->q('%Y-%m-%d %H:%i:%s') . ')' );
                }

                if ( !empty($values['to']) )
                {
                    $query->where( $column_name . ' <= STR_TO_DATE(' . $db->q( JFactory::getDate($values['to'])->toSql() ) . ', ' . $db->q('%Y-%m-%d %H:%i:%s') . ')' );
                }
            }
            else if ($searchable_advanced == 1)
            {
                if ( !in_array( $field->field_type, array('images', 'image_upload', 'fileupload') ) )
                    continue;

                $alias = "fr$field->id";

                $query->innerJoin( $db->qn('#__rsdirectory_uploaded_files_fields_relations', $alias) . ' ON ' . $db->qn("$alias.entry_id") . ' = ' . $db->qn('e.id') . ' AND ' . $db->qn("$alias.field_id") . ' = ' . $db->q($field->id) );
            }
            else
            {
                // Initialize the condition array.
                $cond = array();

                if ( !is_array($values) )
                {
                    $values = (array)$values;
                }

                $cond_type =  $field->properties->get('searchable_advanced_condition_type', 'strict');
				
				if ($cond_type == 'containing' && $searchable_advanced == 'checkboxgroup') {
					$operator = $field->properties->get('searchable_advanced_condition_operator', 'OR');
				} else {
					$operator = 'OR';
				}

                foreach ($values as $value)
                {
                    if ( is_string($value) && $value === '' )
                        continue;

                    $cond_str = $column_name;

                    switch ( $cond_type )
                    {
                        case 'starts_with':

                            $cond_str .= ' LIKE ' . $db->q( $db->escape($value, true) . '%', false);

                            break;

                        case 'ending_with':

                            $cond_str .= ' LIKE ' . $db->q( '%' . $db->escape($value, true), false);

                            break;

                        case 'containing':
							if ($operator === 'AND')
							{
								// Value\r\n
								$cond_str = $column_name . ' LIKE ' . $db->q($db->escape($value, true) . $db->escape("\r\n%"), false) . ' OR ';
								// \r\nValue\r\n
								$cond_str .= $column_name . ' LIKE ' . $db->q($db->escape("%\r\n") . $db->escape($value, true) . $db->escape("\r\n%"), false) . ' OR ';
								// \r\nValue
								$cond_str .= $column_name . ' LIKE ' . $db->q($db->escape("%\r\n") . $db->escape($value, true), false) . ' OR ';
								// Value
								$cond_str .= $column_name . ' LIKE ' . $db->q($db->escape($value, true), false);
								
								$cond_str = '(' . $cond_str . ')';
							}
							else							
							{
								$cond_str .= ' LIKE ' . $db->q( '%' . $db->escape($value, true) . '%', false);
							}

                            break;

                        default:

                            $cond_str .= ' LIKE ' . $db->q($value);

                            break;
                    }

                    $cond[] = $cond_str;
                }

                if ($cond)
                {
					$query->where( '(' . implode(' '.$operator.' ', $cond) . ')' );
                }
            }
        }
    }

    /**
     * Add an entry id to a list of recently visited entries and save the array in a cookie.
     *
     * @access public
     *
     * @static
     *
     * @param int $id
     */
    public static function addRecentlyVisited($id)
    {
        if ( !is_numeric($id) )
            return;

        $conf = JFactory::getConfig();

        $name = 'rsdir_recently_visited';
        $expire = JFactory::getDate()->toUnix() + 7776000; // Remember for 90 days.
        $domain = $conf->get('config.cookie_domain', '');
		$path = $conf->get('config.cookie_path', '/');

        // Values as stored like this: id,id,id,id
        $values = explode( ',', JFactory::getApplication()->input->cookie->getString($name) );
        $values = self::arrayInt($values, true, true);

        // Delete the value if it already exists in the array to update it's position. 
        if ( ( $key = array_search($id, $values) ) !== false )
        {
            unset($values[$key]);
        }

        $values[] = $id;

        // Get the last 10 entries.
        $values = array_slice($values, -10);

		setcookie( $name, implode(',', $values), $expire, $path, $domain );
    }

    /**
     * Return the HTML code for a cached file table row.
     *
     * @access public
     *
     * @static
     *
     * @param object $file
     * @param int $i
     *
     * @return string
     */
    public static function getBackupCachedFileRowHTML($file, $i = 0)
    {
        $str = '<tr>' .
                '<td class="align-middle"><input class="cached-file form-check-input" type="checkbox" value="' . $file->hash . '" /></td>' .
                '<td class="align-middle">' . $i . '</td>' .
                '<td class="align-middle"><a href="' . $file->url . '">' . $file->name . '</a></td>' .
                '<td class="align-middle">' . $file->date . '</td>' .
                '<td class="align-middle"><button class="restore-cache btn btn-light" data-path="' . self::escapeHTML($file->path) . '">' . JText::_('COM_RSDIRECTORY_RESTORE') . '</button></td>' .
                '</tr>';

        return $str;
    }

    /**
     * Trim data.
     *
     * @access public
     *
     * @static
     *
     * @param string $data
     * @param string $character_mask
     *
     * @return mixed
     */
    public static function trim($data)
    {
        if ( is_string($data) )
        {
            return trim($data);
        }

        return array_map( array('self', 'trim'), $data );
    }

    /**
	 * Method to get an array formed of a single column from an array.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param array $items
	 * @param string $column
	 *
	 * @return array
	 */
	public static function getColumn($items, $column)
	{
		$results = array();

		if ( $items && is_array($items) )
		{
			foreach ($items as $item)
			{
				if ( is_array($item) && isset($item[$column]) )
				{
					$results[] = $item[$column];
				}
				else if ( is_object($item) && isset($item->$column) )
				{
					$results[] = $item->$column;
				}
			}
		}

		return $results;
	}

    /**
     * Method to generate a hierarchy tree from an array.
     *
     * @access public
     *
     * @static
     *
     * @param array $array
     * @param int $parent_id
     * @param string $pk The name of the primary key.
     * @param string $ppk The name of the parent key.
     *
     * @return array
     */
    public static function arrayToTree($array, $parent_id = 0, $pk = 'id', $ppk = 'parent_id')
    {
        $results = array();

        foreach ($array as $i => $v)
        {
            if ( is_array($v) && $v[$ppk] == $parent_id )
            {
                $results[$i] = $v;
                $results[$i]['children'] = self::arrayToTree($array, $v[$pk], $pk, $ppk);
            }
            else if ( is_object($v) && $v->$ppk == $parent_id )
            {
                $results[$i] = $v;
                $results[$i]->children = self::arrayToTree($array, $v->$pk, $pk, $ppk);
            }
        }

        return $results;
    }

    /**
     * Method to print human-readable information about a variable inside a <pre> tag.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $expression
     */
    public static function print_r($expression)
    {
        echo '<pre>';
        print_r($expression);
        echo '</pre>';
    }

    /**
     * Method to check if a file is an image.
     *
     * @access public
     *
     * @static
     *
     * @param string $file_path
     *
     * @return bool
     */
    public static function isImage($file_path)
    {
        if ( empty($file_path) || !file_exists($file_path) || !is_readable($file_path) )
            return false;

        $return = false;

        if ( function_exists('exif_imagetype') )
        {
            @$return = (bool)exif_imagetype($file_path);
        }
        else if ( function_exists('finfo_open') )
        {
            $finfo = finfo_open(FILEINFO_MIME_TYPE);

            if ( $mime = finfo_file($finfo, $file_path) )
            {
                if ( substr($mime, 0, strpos($mime, '/') ) == 'image' )
                {
                    $return = true;
                }
            }

            finfo_close($finfo);
        }
        else if ( function_exists('getimagesize') )
        {
            @$size = getimagesize($file_path);

            if ( !empty($size['mime']) )
            {
                if ( substr($size['mime'], 0, strpos($size['mime'], '/') ) == 'image' )
                {
                    $return = true;
                }
            }
        }
        else if ( function_exists('mime_content_type') )
        {
            if ( $mimetype = mime_content_type($file_path) )
            {
                if ( substr($mimetype, 0, strpos($mimetype, '/') ) == 'image' )
                {
                    $return = true;
                }
            }
        }

        return $return;
    }

    /**
     * Method to get tooltip text based on the Joomla! version.
     *
     * @access public
     *
     * @static
     *
     * @param string $text
     *
     * @return string
     */
    public static function getTooltipText($text, $content = '')
    {
        if ($content === '')
        {
            if (strpos($text, '::') !== false)
            {
                list($text, $content) = explode('::', $text);
            }
            else
            {
                $content = $text;
            }
        }
        // title="
        $result = htmlspecialchars(trim($text, ':')) . '"';

        // "
        if (self::isJ4())
        {
            if (JFactory::getLanguage()->isRtl())
            {
                $result .= ' data-bs-placement="left" ';
            }

            $result .= ' data-bs-content="' . htmlspecialchars($content);
        }
        else
        {
            if (JFactory::getLanguage()->isRtl())
            {
                $result .= ' data-placement="left" ';
            }

            $result .= ' data-content="' . htmlspecialchars($content);
        }

        return $result;
    }

    /**
     * Method to get tooltip class based on the Joomla! version.
     *
     * @access public
     *
     * @param bool $force
     *
     * @static
     *
     * @return string
     */
    public static function getTooltipClass($force = false)
    {
        static $class = array();
        $force = (int) $force;

        if (!isset($class[$force]))
        {
            if (!$force && self::isJ4())
            {
                JHtml::_('bootstrap.popover', '.hasPopover', array('trigger' => 'hover focus'));
                $class[$force] = 'hasPopover';
            }
            else
            {

                JHtml::_('bootstrap.tooltip', '.hasTooltip');
                $class[$force] = 'hasTooltip';
            }

        }

        return $class[$force];
    }

    /**
     * Method to calculate the difference betweem two dates.
     *
     * @access public
     *
     * @param string $date1
     * @param string $date2
     * @param int $divisor
     *
     * @return int
     */
    public static function getDateDiff($date1, $date2, $divisor = 1)
    {
        return round( abs( strtotime($date1) - strtotime($date2) ) / ($divisor ? $divisor : 1) );
    }

    /**
     * Quote an array of values.
     *
     * @access public
     *
     * @static
     *
     * @param array $array
     *
     * @return string
     */
    public static function quoteImplode($array)
    {
		$db = JFactory::getDbo();

		foreach ($array as &$value)
        {
			$value = $db->q($value);
		}

		return implode(',', $array);
	}

	/**
     * Get the ids of the subcategories of a category
     *
     * @access public
     *
     * @static
     *
     * @param int $id
     *
     * @return array
     */
	public static function getSubcategoriesIds($id, &$array = array())
	{
		$array[] = $id;

		if ($subcategories = self::getSubcategories($id)) {
			foreach ($subcategories as $subcategory) {
				$array[] = $subcategory->id;
				self::getSubcategoriesIds($subcategory->id,$array);
			}
		}
	}

	/**
     * Count the number of entries a category has
     *
     * @access public
     *
     * @static
     *
     * @param int $id
     *
     * @return int
     */
    public static function countEntriesFromCategory($id)
    {
		$db = JFactory::getDbo();

		$user = JFactory::getUser();

		$jinput = JFactory::getApplication()->input;

		self::getSubcategoriesIds($id, $categories);
		$categories = array_map('intval', $categories);

		$query = $db->getQuery(true)
			   ->select('COUNT(e.id)')
			   ->from( $db->qn('#__rsdirectory_entries', 'e') )
			   ->innerJoin( $db->qn('#__rsdirectory_entries_custom', 'ec') . ' ON ' . $db->qn('e.id') . ' = ' . $db->qn('ec.entry_id') )
			   ->innerJoin( $db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id') )
			   ->innerJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') .  ' = ' . $db->qn('u.id') )
			   ->leftJoin( $db->qn('#__rsdirectory_favorites', 'f') . ' ON ' . $db->qn('e.id') . ' = ' . $db->qn('f.entry_id') . ' AND ' . $db->qn('f.user_id') .  ' = ' . $db->q($user->id) )
			   ->where( $db->qn('u.block') . ' = ' . $db->q(0) );


		// Process permissions.
		$can_view_all_unpublished_entries = RSDirectoryHelper::checkUserPermission('can_view_all_unpublished_entries');
		$can_view_own_unpublished_entries = RSDirectoryHelper::checkUserPermission('can_view_own_unpublished_entries');

		if ($can_view_all_unpublished_entries || $can_view_own_unpublished_entries)
		{
			$statuses = $jinput->get( 'status', array(0, 1), 'status' );

			$status_conditions = array();

			if ( in_array(0, $statuses) )
			{
				$unpublished_condition[] = '(' . $db->qn('e.published') . ' = ' . $db->q(0) . ' OR ' . $db->qn('e.published_time') . ' > ' . $db->q( JFactory::getDate()->toSql() ) . ')';

				if (!$can_view_all_unpublished_entries)
				{
					$unpublished_condition[] = '(' . $db->qn('e.user_id') . ' = ' . $db->q($user->id) . ')';
				}

				$status_conditions[] = implode(' AND ', $unpublished_condition);
			}

			if ( in_array(1, $statuses) )
			{
				$status_conditions[] = '((' . $db->qn('e.published') . ' = ' . $db->q(1) . ' AND ' . $db->qn('e.published_time') . ' <= ' . $db->q( JFactory::getDate()->toSql() ) . ') AND (' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' >= ' . $db->q( JFactory::getDate()->toSql() ) . '))';
			}

			if ($status_conditions)
			{
				$query->where( '(' . implode(' OR ', $status_conditions) . ')' );
			}
		}
		else
		{
			$query->where( $db->qn('e.published') . ' = ' . $db->q(1) )
				  ->where( $db->qn('e.published_time') . ' <= ' . $db->q( JFactory::getDate()->toSql() ) )
				  ->where( '(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' >= ' . $db->q( JFactory::getDate()->toSql() ) . ')' );
		}

		$query->where( $db->qn('e.category_id') . ' IN ('. implode(',',$categories).')' );
		$db->setQuery($query);
		return (int) $db->loadResult();
	}
	
	/**
     * Check if the entry is expired
     * @return bool
     */
	public static function isEntryExpired($entry_id) {
		static $status = array();
		
		if (!isset($status[$entry_id])) {
			// Get DBO.
			$db = JFactory::getDbo();
			
			$query = $db->getQuery(true);
			$query->select('COUNT('.$db->qn('e.id').')')
				  ->from($db->qn('#__rsdirectory_entries', 'e'))
				  ->where( $db->qn('e.expiry_time') . ' != ' . $db->q('0000-00-00 00:00:00') )
				  ->where( $db->qn('e.expiry_time') . ' < ' . $db->q( JFactory::getDate()->toSql() ) )
				  ->where( $db->qn('e.id') . ' = ' . $db->q($entry_id) );
			$db->setQuery($query);
			$status[$entry_id] = (bool) $db->loadResult();
		}
		
		return $status[$entry_id];
	}

	public static function buildCaptcha($type, $template = '') {
		// check for captcha settings

		$use_captcha   = RSDirectoryHelper::checkUserPermission($type.'_captcha');

		$config 	   = RSDirectoryConfig::getInstance();

		$captcha	   = $config->get('captcha_type');


        $output_captcha_html = '';

        if ($use_captcha) {

            if ( $captcha == 'built_in' )
            {
                $width = 30 * $config->get('captcha_characters_number') + 50;

                $output_captcha_html .= '<img id="rsdir-captcha" src="'.JRoute::_( "index.php?option=com_rsdirectory&task=field.captcha&type=".$type."&random=" . mt_rand() ).'" width="'.$width.'" height="80" alt="CAPTCHA" />';
                $output_captcha_html .= '<i id="rsdir-captcha-refresh" class="fa fa-refresh" title="'.JText::_('COM_RSDIRECTORY_REFRESH_CAPTCHA_DESC').'"></i>';
                $output_captcha_html .= JHtml::_('image', 'com_rsdirectory/loader.gif', '', 'id="rsdir-captcha-loader" class="rsdir-hide" width="16" height="16"', true);
                $output_captcha_html .= '<br />';
                if ($type == 'add_entry')
                {
                    $output_captcha_html .= '<input id="rsdir-captcha-input"'.RsdirectoryAdapterGrid::formControl(true).' type="text" name="fields[captcha]" />';
                }
                else
                {
                    $output_captcha_html .= '<input id="rsdir-captcha-input"'.RsdirectoryAdapterGrid::formControl(true).' type="text" name="jform[captcha]" required="true" aria-required="true" />';
                }
            }
            else
            {
                //for all others use declaration
                if ($type != 'contact')
                {
                    // load captcha dependecies
                    self::loadCaptchaDependencies($type);

                    JFactory::getDocument()->addScriptDeclaration('
                        RSDirectoryReCAPTCHAv2.loaders.push(function(){
                            if (typeof RSDirectoryReCAPTCHAv2.ctypes[\''.$type.'\'] === "undefined") {
                                var id = grecaptcha.render("g-recaptcha-'.$type.'", {
                                "sitekey": "'.self::escapeHTML($config->get('recaptcha_new_site_key')).'",
                                "theme": "'.self::escapeHTML($config->get('recaptcha_new_theme')).'",
                                "type": "'.self::escapeHTML($config->get('recaptcha_new_type')).'"
                            });
                                RSDirectoryReCAPTCHAv2.ctypes[\''.$type.'\'] = id;
                            }
                        });
                    ');
                } else {
                    $output_captcha_html .= '
					<script type="text/javascript">
						var rsdir_new_captcha_load = grecaptcha.render("g-recaptcha-contact", {
							"sitekey": "'.self::escapeHTML($config->get('recaptcha_new_site_key')).'",
							"theme": "'.self::escapeHTML($config->get('recaptcha_new_theme')).'",
							"type": "'.self::escapeHTML($config->get('recaptcha_new_type')).'"
						});
					</script>';
                }
                $output_captcha_html .= '<div id="g-recaptcha-'.$type.'"></div>';
            }

            if (empty($template))
            {
                $template .= '<div class="control-group rsdir-field-wrapper clearfix">';
                $template .= '<label class="rsdir-label">' . JText::_('COM_RSDIRECTORY_CAPTCHA_LABEL') . '</label>%s</div>';
            }
		}

		return $use_captcha ? sprintf($template, $output_captcha_html) : '';
	}

    public static function loadCaptchaDependencies($type)
    {
        static $loaded_script;
        static $loaded = array();
        if (!isset($loaded[$type]) && $type != 'contact') {

            $doc = JFactory::getDocument();
            if ($doc->getType() == 'html')
            {
                JHtml::_('script', 'com_rsdirectory/recaptcha.js', array('relative' => true, 'version' => 'auto'));
                $loaded[$type] = 'loaded';
            }
        }

        if (is_null($loaded_script))
        {
            $doc = JFactory::getDocument();
            if ($doc->getType() == 'html')
            {
                $doc->addCustomTag('<script type="text/javascript" src="https://www.google.com/recaptcha/api.js?hl=' . JFactory::getLanguage()->getTag() . '" defer async></script>');
                $loaded_script = true;
            }
        }
    }

	public static function loadRaty($selector = null, $params = array()) {
		$init_params	= '';

        JHtml::_('stylesheet', 'com_rsdirectory/jquery.raty.css', array('relative' => true, 'version' => 'auto'));
        JHtml::_('script', 'com_rsdirectory/jquery.raty.min.js', array('relative' => true, 'version' => 'auto'));

		if ($params) {
			foreach ($params as $key => $value) {
				if ($value == 'true' || $value == 'false') {
					$init_params .= $key . ': '. $value .',';
				} else {
					$init_params .= $key . ': \''. $value .'\',';
				}
			}
		}

		if ($selector) {
			$config 	= RSDirectoryConfig::getInstance();
			$starType 	= $config->get('ratings_star_type', 'i');
			
			static $cache = array();

			JText::script('COM_RSDIRECTORY_RATY_BAD');
			JText::script('COM_RSDIRECTORY_RATY_POOR');
			JText::script('COM_RSDIRECTORY_RATY_REGULAR');
			JText::script('COM_RSDIRECTORY_RATY_GOOD');
			JText::script('COM_RSDIRECTORY_RATY_GORGEOUS');

			$script = ' jQuery(function($)
						{
							if ($(\'' . $selector . '\').length > 0 && $.fn.raty) {
								$(\'' . $selector . '\').raty(
								{
									path		: "' . JUri::root(true) . '/media/com_rsdirectory/images/raty/",
									hints		: [Joomla.JText._(\'COM_RSDIRECTORY_RATY_BAD\'), Joomla.JText._(\'COM_RSDIRECTORY_RATY_POOR\'), Joomla.JText._(\'COM_RSDIRECTORY_RATY_REGULAR\'), Joomla.JText._(\'COM_RSDIRECTORY_RATY_GOOD\'), Joomla.JText._(\'COM_RSDIRECTORY_RATY_GORGEOUS\')],
									score		: function()
									{
										return $(this).attr("data-rating");
									},
									readOnly	: true,
									starType	: \''.$starType.'\',
									' . $init_params . '
								});
							}
						});';

			$hash = md5($selector.serialize($params));

			if (!isset($cache[$hash])) {
                $doc = JFactory::getDocument();
				$doc->addScriptDeclaration($script);
			}

			$cache[$hash] = true;
		}
	}

    public static function isCode($value) {
        if (self::hasCode($value)) {
            return eval($value);
        }

        return $value;
    }

    public static function hasCode($value) {
        return (strpos($value, '<code>') !== false);
    }
	
	public static function mb_strtolower($str)
    {
        if (function_exists('mb_strtolower'))
        {
            return mb_strtolower($str);
        }
        else
        {
            return strtolower($str);
        }
    }
	
	public static function mb_strtoupper($str)
    {
        if (function_exists('mb_strtoupper'))
        {
            return mb_strtoupper($str);
        }
        else
        {
            return strtoupper($str);
        }
    }
	
	public static function render_jquery_calendar($params = array()) {
        static $dependencies;

        if (is_null($dependencies)) {
            if (!self::isJ4())
            {
                // make sure the jquery ui is loaded
                JHtml::_( 'jquery.ui');
            }
            else
            {
                // load the jQuery datePicker
                JHtml::_('script', 'com_rsdirectory/jquery-ui-1.10.4.custom.min.js', array('relative' => true, 'version' => 'auto'));
            }

            // load the jQuery datePicker
            JHtml::_('script', 'com_rsdirectory/jquery.ui.datepicker.min.js', array('relative' => true, 'version' => 'auto'));

            // load our jQuery calendar class
            require_once dirname(__FILE__) . '/jquerycalendar.php';

            $dependencies = true;

            // build the calendars translations
            $datepicker_lang = array(
                'closeText' 		=> JText::_('COM_RSDIRECTORY_CAL_CLOSE'),
                'prevText' 			=> JText::_('COM_RSDIRECTORY_CAL_PREV'),
                'nextText' 			=> JText::_('COM_RSDIRECTORY_CAL_NEXT'),
                'currentText' 		=> JText::_('COM_RSDIRECTORY_CAL_TODAY'),
                'monthNames' 		=> array(
                    JText::_('JANUARY'),
                    JText::_('FEBRUARY'),
                    JText::_('MARCH'),
                    JText::_('APRIL'),
                    JText::_('MAY'),
                    JText::_('JUNE'),
                    JText::_('JULY'),
                    JText::_('AUGUST'),
                    JText::_('SEPTEMBER'),
                    JText::_('OCTOBER'),
                    JText::_('NOVEMBER'),
                    JText::_('DECEMBER')
                ),
                'monthNamesShort' 	=> array(
                    JText::_('JANUARY_SHORT'),
                    JText::_('FEBRUARY_SHORT'),
                    JText::_('MARCH_SHORT'),
                    JText::_('APRIL_SHORT'),
                    JText::_('MAY_SHORT'),
                    JText::_('JUNE_SHORT'),
                    JText::_('JULY_SHORT'),
                    JText::_('AUGUST_SHORT'),
                    JText::_('SEPTEMBER_SHORT'),
                    JText::_('OCTOBER_SHORT'),
                    JText::_('NOVEMBER_SHORT'),
                    JText::_('DECEMBER_SHORT')
                ),
                'dayNames' 			=> array(
                    JText::_('SUNDAY'),
                    JText::_('MONDAY'),
                    JText::_('TUESDAY'),
                    JText::_('WEDNESDAY'),
                    JText::_('THURSDAY'),
                    JText::_('FRIDAY'),
                    JText::_('SATURDAY')
                ),
                'dayNamesShort' 	=> array(
                    JText::_('SUN'),
                    JText::_('MON'),
                    JText::_('TUE'),
                    JText::_('WED'),
                    JText::_('THU'),
                    JText::_('FRI'),
                    JText::_('SAT')
                ),
                'dayNamesMin' 		=> array(
                    JText::_('COM_RSDIRECTORY_CAL_SU'),
                    JText::_('COM_RSDIRECTORY_CAL_MO'),
                    JText::_('COM_RSDIRECTORY_CAL_TU'),
                    JText::_('COM_RSDIRECTORY_CAL_WE'),
                    JText::_('COM_RSDIRECTORY_CAL_TH'),
                    JText::_('COM_RSDIRECTORY_CAL_FR'),
                    JText::_('COM_RSDIRECTORY_CAL_SA')
                ),
                'weekHeader' 		=> JText::_('COM_RSDIRECTORY_CAL_WEEK'),
                'isRTL' 			=> JFactory::getLanguage()->isRTL(),
                'yearSuffix' 		=> JText::_('COM_RSDIRECTORY_CAL_YEAR_SUFFIX')
            );

            $script = "\n\n".'// Set up the regional translations for the calendars'."\n";
            $script .= 'var rsdir_datepicker_regional = {'."\n";
            foreach ($datepicker_lang as $key => $value) {
                $script .= json_encode($key).':'.json_encode($value).",\n";
            }
            $script .= '}'."\n";

            JFactory::getDocument()->addScriptDeclaration($script);
        }

        try{
            $calendar = new RSDirectoryJqueryCalendar($params);

            return $calendar->render();
        } catch(Exception $e) {
            JFactory::getApplication()->enqueueMessage($e->getMessage(), 'warning');
            return false;
        }
    }

    public static function anonymise($id, $anonymiseJoomlaData = null)
    {
        $db     = JFactory::getDbo();
        $query  = $db->getQuery(true);

        $config = RSDirectoryConfig::getInstance();

        $fake_text      = JText::_('COM_RSDIRECTORY_DATA_HAS_BEEN_ANONYMISED');
        $fake_ip        = '0.0.0.0';
        $fake_email     = JUserHelper::genRandomPassword(mt_rand(10, 16)) . '@' . JUserHelper::genRandomPassword(mt_rand(10, 16));

        if ($anonymiseJoomlaData === null)
        {
            $anonymiseJoomlaData = $config->get('anonymise_joomla_data', 1);
        }

        // Let's create a fake email & fake username
        if ($anonymiseJoomlaData) {
            $fake_username = JUserHelper::genRandomPassword(mt_rand(10, 16));
            // Make sure this email is free
            $query->clear()
                ->select($db->qn('id'))
                ->from($db->qn('#__users'))
                ->where($db->qn('email') . ' = ' . $db->q($fake_email));
            while ($db->setQuery($query)->loadResult()) {
                $fake_email .= JUserHelper::genRandomPassword(mt_rand(1, 2));
                $query->clear()
                    ->select($db->qn('id'))
                    ->from($db->qn('#__users'))
                    ->where($db->qn('email') . ' = ' . $db->q($fake_email));
            }

            // Make sure this username is free
            $query->clear()
                ->select($db->qn('id'))
                ->from($db->qn('#__users'))
                ->where($db->qn('username') . ' = ' . $db->q($fake_username));
            while ($db->setQuery($query)->loadResult()) {
                $fake_username .= JUserHelper::genRandomPassword(mt_rand(1, 2));
                $query->clear()
                    ->select($db->qn('id'))
                    ->from($db->qn('#__users'))
                    ->where($db->qn('username') . ' = ' . $db->q($fake_username));
            }

            // #__users data
            $query->clear()
                ->update($db->qn('#__users'))
                ->set($db->qn('name') . ' = ' . $db->q($fake_username))
                ->set($db->qn('username') . ' = ' . $db->q($fake_username))
                ->set($db->qn('email') . ' = ' . $db->q($fake_email))
                ->set($db->qn('password') . ' = ' . $db->q(JUserHelper::hashPassword(JUserHelper::genRandomPassword(20))))
                ->where($db->qn('id') . ' = ' . $db->q($id));
            $db->setQuery($query)->execute();
        }

        // Anonymise entries
        $fields = array(
            'rsdirectory_entries' => array (
                'ip'             => $fake_ip,
                'published'      => 0,
                'renew'          => 0,
                'title'          => $fake_text,
                'big_subtitle'   => $fake_text,
                'small_subtitle' => $fake_text,
                'description'    => $fake_text
            )
        );
        $query->clear()
            ->select($db->qn('id'))
            ->select($db->qn('form_id'))
            ->from($db->qn('#__rsdirectory_entries'))
            ->where($db->qn('user_id') . ' = '. $db->q($id));
        $db->setQuery($query);
        $user_entries = $db->loadObjectList();

        if (!empty($user_entries)) {
            foreach ($user_entries as $entry) {
                // Using static method to not load the form fields every time
                static $form_custom_fields = array();

                if (!isset($form_custom_fields[$entry->form_id])) {
                    $form_custom_fields[$entry->form_id] = RSDirectoryHelper::getFormFields($entry->form_id);
                    // keep only the field types that may hold personal data information
                    $column_fields = array();
                    foreach ($form_custom_fields[$entry->form_id] as $i => $field) {
                        if ((int) $field->create_column == 0) {
                            unset($form_custom_fields[$entry->form_id][$i]);
                            continue 1;
                        }

                        // replace the values of the columns with the proper data
                        switch($field->field_type) {
                            case 'calendar':
                            case 'dropdown_date_picker':
                            $column_fields[$field->column_name] = '0000-00-00 00:00:00';
                            break;

                            case 'map':
                                $column_fields[$field->column_name.'_address']   = $fake_text;
                                $column_fields[$field->column_name.'_lat']       = '0.000000';
                                $column_fields[$field->column_name.'_lng']      = '0.000000';
                            break;

                            case 'youtube':
                                $column_fields[$field->column_name] = 'https://youtu.be/1';
                            break;

                            case 'link':
                                $column_fields[$field->column_name] = 'http://www.example.com';
                            break;

                            default:
                                $column_fields[$field->column_name] = $fake_text;
                            break;
                        }
                    }
                    $form_custom_fields[$entry->form_id] = $column_fields;
                }

                // append the custom fields
                if (!empty($form_custom_fields[$entry->form_id])) {
                    $fields['rsdirectory_entries_custom'] = $form_custom_fields[$entry->form_id];
                }

                // Anonymise the entries
                foreach ($fields as $table => $table_fields) {
                    // set the table to update
                    $query->clear()->update($db->qn('#__'.$table));

                    foreach ($table_fields as $column => $value) {
                        $query->set($db->qn($column) . ' = ' . $db->q($value));
                    }

                    $index_column = ($table == 'rsdirectory_entries' ? 'id' : 'entry_id');
                    $query->where($db->qn($index_column) . ' = ' . $db->q($entry->id));
                    $db->setQuery($query)->execute();
                }
            }
        }


        // Remove the uploaded files for this user
        $query->clear()
            ->select($db->qn('hash'))
            ->from($db->qn('#__rsdirectory_uploaded_files'))
            ->where($db->qn('user_id') . ' = '. $db->q($id));
        $db->setQuery($query);
        $file_hashes = $db->loadColumn();

        if (!empty($file_hashes)) {
            foreach ($file_hashes as $hash) {
                $file = RSDirectoryHelper::getFileObject(0, $hash);
                if (!$file) {
                    continue;
                }

                RSDirectoryHelper::deleteFiles($file);
            }
        }

        // Reported Entries
        $query->clear()
            ->update($db->qn('#__rsdirectory_entries_reported'))
            ->set($db->qn('ip') . ' = ' . $db->q($fake_ip))
            ->set($db->qn('name') . ' = ' . $db->q($fake_text))
            ->set($db->qn('email') . ' = ' . $db->q($fake_email))
            ->where($db->qn('user_id') . ' = ' . $db->q($id));
        $db->setQuery($query)->execute();

        // Reviews
        $query->clear()
            ->update($db->qn('#__rsdirectory_reviews'))
            ->set($db->qn('ip') . ' = ' . $db->q($fake_ip))
            ->set($db->qn('name') . ' = ' . $db->q($fake_text))
            ->set($db->qn('email') . ' = ' . $db->q($fake_email))
            ->where($db->qn('user_id') . ' = ' . $db->q($id));
        $db->setQuery($query)->execute();

        // Transactions
        $query->clear()
            ->update($db->qn('#__rsdirectory_users_transactions'))
            ->set($db->qn('ip') . ' = ' . $db->q($fake_ip))
            ->where($db->qn('user_id') . ' = ' . $db->q($id));
        $db->setQuery($query)->execute();

        // log the anonymisation for the user
        $log_params = array(
            'type' => 'user_anonymise',
            'user_id' => $id
        );
        RSDirectoryHelper::logAction($log_params);
    }
	
	/**
    Legacy functions, will just keep them so that modules and plugins using this functions will not cause errors until they are updated
   */

    /**
     * Is Joomla! 3.0?
     */
    public static function isJ30()
    {
        return self::isJoomlaCompatible('3.0');
    }

    /**
     * Is Joomla! 2.5?
     */
    public static function isJ25()
    {
        return self::isJoomlaCompatible('2.5') && !self::isJoomlaCompatible('3.0');
    }

    /**
     * Method to check Joomla! version.
     *
     * @access public
     *
     * @static
     *
     * @param string $version
     *
     * @return bool
     */
    public static function isJoomlaCompatible($version)
    {
        static $jversion;

        if (!$jversion)
        {
            $jversion = new JVersion();
        }

        return $jversion->isCompatible($version);
    }


    /**
     * Method to load the Radius Map.
     *
     * @access public
     *
     * @static
     *
     * @param array $params
     */
    public static function loadRadiusMap($params) {
        $config = RSDirectoryConfig::getInstance();

        $map_type = $config->get('map_type', 'google');
        // load the proper files
        self::loadMapsDependencies($config, $map_type, true);

        // The opacity is used in both map types
        $opacity = $params['circle_opacity'];

        // Constrain the opacity between 0 and 100
        $opacity = $opacity > 100 ? 100 : $opacity;
        $opacity = $opacity < 0 ? 0 : $opacity;
        $opacity = round(($opacity / 100), 2);

        $script = '';
        switch($map_type) {
            case 'google':
                $script = 'jQuery(function($)
                {
                    jQuery( document.getElementById("rsdir-map-canvas") ).rsRadiusSearch(
                    {
                        address: ' . json_encode( $params['location'] ) . ',
                        form: document.getElementById("rsdir-radius-search"),
                        zoom: ' . (int)$params['default_zoom'] . ',
                        infoWindowWidth: ' . (int)$params['info_window_width'] . ',
                        infoWindowHeight: ' . (int)$params['info_window_height'] .  ',
                        fillColor: ' . json_encode( $params['circle_color'] ) . ',
                        fillOpacity: ' . $opacity. ',
                        strokeColor: ' . json_encode( $params['circle_stroke_color'] ) . ',
                        hideCircle: ' . $params['hide_circle']. ',
                        useShowMore: ' . $params['use_show_more'] . ',
                        limit: '.($params['limit'] - 1).'
                    });

                });';
            break;

            case 'openstreetmapbox':
            case 'openstreetthunderforest':
            case 'openstreetstamen':
            case 'openstreetesri':

                // specific attributes
                $attr = self::getMapAttribution($map_type, $config);
                $tileType    = $attr['tileType'];
                $accessToken = $attr['accessToken'];
                $attribution = $attr['attribution'];
                $tileLayer   = $attr['tileLayer'];

                $script = 'jQuery(function($)
                {
                    jQuery( document.getElementById("rsdir-map-canvas") ).rsORadiusSearch(
                    {
                        address: ' . json_encode( $params['location'] ) . ',
                        form: document.getElementById("rsdir-radius-search"),
                        zoom: ' . (int)$params['default_zoom'] . ',
                        infoWindowWidth: ' . (int)$params['info_window_width'] . ',
                        infoWindowHeight: ' . (int)$params['info_window_height'] .  ',
                        fillColor: ' . json_encode( $params['circle_color'] ) . ',
                        color: ' . json_encode( $params['circle_stroke_color'] ) . ',
                        hideCircle: ' . $params['hide_circle']. ',
                        fillOpacity: ' . $opacity. ',
                        useShowMore: ' . $params['use_show_more'] . ',
                        limit: '.($params['limit'] - 1).',

                        tileLayer: \''.$tileLayer.'\',
                        tileType: \''.$tileType.'\',
                        attribution: \''.$attribution.'\',
                        accessToken: \''.$accessToken.'\'
                    });

                });';
            break;
        }

        if (!empty($script)) {
            // Add the script declaration.
            JFactory::getDocument()->addScriptDeclaration($script);
        }
    }

    /**
     * Method to load the Normal Map.
     *
     * @access public
     *
     * @static
     *
     * @param array $params
     */
    public static function loadMap($params, $geocoder = '') {
        $config = RSDirectoryConfig::getInstance();

        $map_type = $config->get('map_type', 'google');
        // load the proper files
        self::loadMapsDependencies($config, $map_type, false);

        $script = '';
        switch($map_type) {
            case 'google':
                $script = 'jQuery(function($)
                {
                    jQuery( document.getElementById("' . $params['map_id'] . '") ).rsMap(
                    {
                        address: ' . (isset($params['address']) ? json_encode($params['address']) : '""') . ',
                        lat: ' . (isset($params['lat']) ? $params['lat'] : '""') . ',
                        lng: ' . (isset($params['lng']) ? $params['lng'] : '""') . ',
                        zoom: ' . (isset($params['default_zoom']) ? $params['default_zoom'] : '5') . ',
                        inputAddress: '.(isset($params['inputAddress']) ? $params['inputAddress'] : 'null').',
                        inputLat: '.(isset($params['inputLat']) ? $params['inputLat'] : 'null').',
                        inputLng: '.(isset($params['inputLng']) ? $params['inputLng'] : 'null').',
                        markerDraggable: '.(isset($params['markerDraggable']) ? $params['markerDraggable'] : 'false').',
                        mapIcon: '.(isset($params['icon_params']) ? json_encode($params['icon_params']) : 'null').'
                    });
                    '.(!empty($geocoder) ? $geocoder : '').'
                });';
                break;

            case 'openstreetmapbox':
            case 'openstreetthunderforest':
            case 'openstreetstamen':
            case 'openstreetesri':

                // specific attributes
                $attr = self::getMapAttribution($map_type, $config);
                $tileType    = $attr['tileType'];
                $accessToken = $attr['accessToken'];
                $attribution = $attr['attribution'];
                $tileLayer   = $attr['tileLayer'];

                $script = 'jQuery(function($)
                {
                    jQuery( document.getElementById("' . $params['map_id'] . '") ).rsOMap(
                    {
                        address: ' . (isset($params['address']) ? json_encode($params['address']) : '""') . ',
                        lat: ' . (isset($params['lat']) ? $params['lat'] : '""') . ',
                        lng: ' . (isset($params['lng']) ? $params['lng'] : '""') . ',
                        zoom: ' . (isset($params['default_zoom']) ? $params['default_zoom'] : '5') . ',
                        inputAddress: '.(isset($params['inputAddress']) ? $params['inputAddress'] : 'null').',
                        inputLat: '.(isset($params['inputLat']) ? $params['inputLat'] : 'null').',
                        inputLng: '.(isset($params['inputLng']) ? $params['inputLng'] : 'null').',
                        markerDraggable: '.(isset($params['markerDraggable']) ? $params['markerDraggable'] : 'false').',
                        mapIcon: '.(isset($params['icon_params']) ? json_encode($params['icon_params']) : 'null').',

                        tileLayer: \''.$tileLayer.'\',
                        tileType: \''.$tileType.'\',
                        attribution: \''.$attribution.'\',
                        accessToken: \''.$accessToken.'\'
                    });
                    '.(!empty($geocoder) ? $geocoder : '').'
                });';
                break;
        }

        if (!empty($script)) {
            // Add the script declaration.
            JFactory::getDocument()->addScriptDeclaration($script);
        }
    }

    /**
     * Method to load get the OpenStreetMaps attributuion types and settings
     *
     * @access public
     *
     * @static
     *
     * @param $map_type, $config
     *
     * @return string
     */
    public static function getMapAttribution($map_type, $config) {
        $attributions = array(
          'openstreetmapbox'        => array(
              'tileType'        => self::legacyMapboxTiles($config->get('openstreet_mapbox_tile_type', 'mapbox.streets')),
              'accessToken'     => $config->get('openstreet_mapbox_access_token', ''),
              'tileLayer'       => 'https://api.mapbox.com/styles/v1/{id}/tiles/{z}/{x}/{y}?access_token={accessToken}',
              'attribution'     => 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, <a href="https://creativecommons.org/licenses/by-sa/2.0/">CC-BY-SA</a>, Imagery  &copy; <a href="https://www.mapbox.com/">Mapbox</a>'
          ),
          'openstreetthunderforest' => array(
              'tileType'        => '',
              'accessToken'     => $config->get('openstreet_thunderforest_api_key', ''),
              'tileLayer'       => 'https://tile.thunderforest.com/'.$config->get('openstreet_thunderforest_tile_type', 'neighbourhood').'/{z}/{x}/{y}.png?apikey={accessToken}',
              'attribution'     => 'Map data &copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a> contributors, Maps &copy; <a href="http://www.thunderforest.com">Thunderforest</a>'
          ),
          'openstreetstamen'            => array(
              'tileType'        => '',
              'accessToken'     => '',
              'tileLayer'       => 'https://stamen-tiles.a.ssl.fastly.net/'.$config->get('openstreet_stamen_tile_type', 'terrain').'/{z}/{x}/{y}.'.($config->get('openstreet_stamen_tile_type', 'terrain') == 'toner' ? 'png' : 'jpg'),
              'attribution'     => '<a href="http://leafletjs.com" title="A JS library for interactive maps">Leaflet</a> | Map tiles by <a href="http://stamen.com/">Stamen Design</a>, under <a href="http://creativecommons.org/licenses/by/3.0">CC BY 3.0</a>. Data by <a href="http://openstreetmap.org">OpenStreetMap</a>, under <a href="http://www.openstreetmap.org/copyright">ODbL</a>.'
          ),
          'openstreetesri'            => array(
                'tileType'        => '',
                'accessToken'     => '',
                'tileLayer'       => 'https://server.arcgisonline.com/ArcGIS/rest/services/'.$config->get('openstreet_esri_tile_type', 'World_Street_Map').'/MapServer/tile/{z}/{y}/{x}',
                'attribution'     => 'Tiles &copy; Esri -- {tile_tile_attribution}. Data by <a href="http://openstreetmap.org">OpenStreetMap</a>, under <a href="http://www.openstreetmap.org/copyright">ODbL</a>.'
          )
        );

        if ($map_type == 'openstreetesri') {
            switch($config->get('openstreet_esri_tile_type', 'World_Street_Map')) {
                case 'World_Street_Map':
                    $attributions['openstreetesri']['attribution'] = str_replace('{tile_tile_attribution}', 'Source: Esri, DeLorme, NAVTEQ, USGS, Intermap, iPC, NRCAN, Esri Japan, METI, Esri China (Hong Kong), Esri (Thailand), TomTom, 2012', $attributions['openstreetesri']['attribution']);
                break;
                case 'World_Topo_Map':
                    $attributions['openstreetesri']['attribution'] = str_replace('{tile_tile_attribution}', 'Source: Esri, DeLorme, NAVTEQ, TomTom, Intermap, iPC, USGS, FAO, NPS, NRCAN, GeoBase, Kadaster NL, Ordnance Survey, Esri Japan, METI, Esri China (Hong Kong), and the GIS User Community', $attributions['openstreetesri']['attribution']);
                break;
                case 'World_Imagery':
                    $attributions['openstreetesri']['attribution'] = str_replace('{tile_tile_attribution}', 'Source: Esri, i-cubed, USDA, USGS, AEX, GeoEye, Getmapping, Aerogrid, IGN, IGP, UPR-EGP, and the GIS User Community', $attributions['openstreetesri']['attribution']);
                break;
            }
        }

        return (isset($attributions[$map_type]) ? $attributions[$map_type] : '');
    }
	
	protected static function legacyMapboxTiles($type) {
        if ($type == 'mapbox.streets') {
            return 'mapbox/streets-v11';
        } elseif ($type == 'mapbox.dark') {
            return 'mapbox/dark-v10';
        } elseif ($type == 'mapbox.light') {
            return 'mapbox/light-v10';
        } elseif ($type == 'mapbox.outdoors') {
            return 'mapbox/outdoors-v11';
        } elseif ($type == 'mapbox.satellite') {
            return 'mapbox/satellite-streets-v11';
        }

        return $type;
    }

    /**
     * Method to load the Google or OpenStreetMaps dependencies
     *
     * @access protected
     *
     * @static
     *
     * @param $config, $map_type, $radius
     */
    protected static function loadMapsDependencies($config, $map_type, $radius = false) {
        static $loaded = array();

        if (!isset($loaded[$map_type])) {
            if  ($map_type == 'google') {
                JHtml::_('script', 'https://maps.google.com/maps/api/js?key='.urlencode($config->get('google_api_key', '')).'&amp;libraries=geometry');
                if ($radius) {
                    JHtml::_('script', 'com_rsdirectory/jquery.radius-search.js', array('relative' => true, 'version'=> 'auto'));
                } else {
                    JHtml::_('script', 'com_rsdirectory/jquery.rsmap.js', array('relative' => true, 'version'=> 'auto'));
                }
            } else {
                // leaflet stylesheet
                JHtml::_('stylesheet', 'com_rsdirectory/leaflet.css', array('relative' => true, 'version'=> 'auto'));

                // leafelt
                JHtml::_('script', 'https://unpkg.com/leaflet@1.4.0/dist/leaflet.js', array(), array('integrity' => 'sha512-QVftwZFqvtRNi0ZyCtsznlKSWOStnDORoefr1enyq5mVL4tmKB3S/EnC3rRJcxCPavG10IcrVGSmPh6Qw5lwrg==', 'crossorigin' => ''));

                // esri
                JHtml::_('script', 'https://unpkg.com/esri-leaflet@2.2.4/dist/esri-leaflet.js', array(), array('integrity' => 'sha512-tyPum7h2h36X52O2gz+Pe8z/3l+Y9S1yEUscbVs5r5aEY5dFmP1WWRY/WLLElnFHa+k1JBQZSCDGwEAnm2IxAQ==', 'crossorigin' => ''));

                JHtml::_('script', 'https://unpkg.com/esri-leaflet-geocoder@2.2.13/dist/esri-leaflet-geocoder.js', array(), array('integrity' => 'sha512-zdT4Pc2tIrc6uoYly2Wp8jh6EPEWaveqqD3sT0lf5yei19BC1WulGuh5CesB0ldBKZieKGD7Qyf/G0jdSe016A==', 'crossorigin' => ''));

                if ($radius) {
                    JHtml::_('script', 'com_rsdirectory/jquery.radius-search.os.js', array('relative' => true, 'version'=> 'auto'));
                } else {
                    JHtml::_('script', 'com_rsdirectory/jquery.rsmap.os.js', array('relative' => true, 'version'=> 'auto'));
                }
            }
        }
    }

    /**
     * Method that handles exipired entries redirect
     *
     * @access public
     *
     * @static
     *
     * @param $entry
     *
     * @throws exception on 404 case
     */
    public static function handleExpirationRedirect($entry) {
        $config 	= RSDirectoryConfig::getInstance();

        if ($config->get('use_not_found_redirect', 0))
        {
            throw new Exception(JText::_('COM_RSDIRECTORY_ENTRY_EXPIRED'), 404);
        }

		$redirect_link = JUri::base(false);
        if ($config->get('use_expired_redirect', 0))
        {
            $days = (int) $config->get('expired_redirect_days', 30);

            $status = 301;

            // set the redirect link
			$redirect_to = $config->get('expired_redirect_to', 'category');

			switch ($redirect_to)
			{
				case 'category':
					// get the Itemid
					$Itemid = (int) JFactory::getApplication()->input->get('Itemid', 0);
					$redirect_link = RSDirectoryRoute::getCategoryEntriesURL($entry->category_id, '', $Itemid, true);
					break;

				case 'homepage':
					$redirect_link 	= JUri::base(false);
					break;

				case 'url':
					$redirect_link = $config->get('expired_redirect_to_custom', '');
					break;
			}


            if ($days > 0)
            {
                $until = JFactory::getDate($entry->expiry_time)->toUnix() + ($days * 86400);

                if (JFactory::getDate()->toUnix() < $until)
                {
                    $status = 302;
                }
            }

			JFactory::getApplication()->redirect($redirect_link, $status);
        }
    }

    public static function logAction($params = array()) {
        static $log_class_loaded;

        if ( is_null($log_class_loaded)) {
            JPluginHelper::importPlugin('actionlog');
            $log_class_loaded = true;
        }

        JFactory::getApplication()->triggerEvent('onRsdirActionLog', array($params));
    }

    public static function getRandomSeed($type = '')
    {
        $session = JFactory::getSession();
        $seed = $session->get('com_rsdirectory.'.$type.'random_seed');
        if ($seed === null)
        {
            $seed = mt_rand();
            $session->set('com_rsdirectory.'.$type.'random_seed', $seed);
        }

        return $seed;
    }

    /**
     * Get Login Form for use in frontend
     *
     * @return object $form
     */
    public static function getLoginForm()
    {
        JForm::addFormPath(JPATH_SITE . '/components/com_rsdirectory/models/forms');

        $form = JForm::getInstance( 'com_rsdirectory.login', 'login', array('control' => '') );

        //check for two factor auth
        $twofactormethods = JAuthenticationHelper::getTwoFactorMethods();


        // Add "rsdir-login-" prefix to the ID's so that no conflicts would emerge
        foreach ($form->getFieldsets() as $fieldset)
        {
            if ($fields = $form->getFieldset($fieldset->name))
            {
                foreach ($fields as $field)
                {
                    if (count($twofactormethods) <= 1 && $field->fieldname == 'secretkey')
                    {
                        $form->removeField($field->fieldname);
                    }
                    $form->setFieldAttribute($field->fieldname, 'id', 'rsdir_login_'.$field->id);
                }
            }
        }

        return $form;
    }

    /**
     * Get User Details Form for use in frontend
     *
     * @return object $form
     */
    public static function getUserDetailsForm($master_group = 'jform')
    {
        JForm::addFormPath(JPATH_SITE . '/components/com_rsdirectory/models/forms');

        return JForm::getInstance( 'com_rsdirectory.userdetails', 'userdetails', array('control' => $master_group) );
    }
}