<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2020 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

class RSDirectoryScripting
{
    protected $placeholders = array();

    public function __construct($placeholders){
        $this->placeholders = $placeholders;
    }

    public static function getInstance($placeholders) {
        static $script = array();

        $hash = implode('-', $placeholders);
        $hash = md5($hash);

        if (!isset($script[$hash])) {
            $script[$hash] = new RSDirectoryScripting($placeholders);
        }

        return $script[$hash];
    }

    /**
     * Method to process conditional placeholders.
     *
     * E.g.: {if field_name}this text gets displayed only if the placeholder has a value{/if}
     *
     * @access protected
     *
     * @param string $string
     * @param array $map
     * @param array $form_fields
     * @param array $custom_fields
     *
     * @return $string
     */
    public function processConditional($string, $map =  array(), $form_fields = array(), $custom_fields = array())
    {
        // clean the string for editor spaces that are not normal ones
        $string = preg_replace('~\x{00c2}\x{00a0}~','',$string);

        $condition 	= '({[a-z0-9\-]+})';
        $inner = '((?:(?!{/?if).)*?)';
        $comparisonPattern  = '(<=|&lt;=|>=|&gt;=|<-|&lt;-|->|-&gt;|<>|&lt;&gt;|<|&lt;|>|&gt;|!=|===|==|=)';

        // Placeholder1 > Placeholder2
        $pattern = '#{if(\s+)?' . $condition . '(\s+)?' . $comparisonPattern . '?(\s+)?' . $condition . '?\s?(\s+)?}' . $inner . '{/if}#is';
        while ( preg_match($pattern, $string, $match) )
        {
            $f_name            = str_replace(array('{', '}'), '',$match[2]);
            $operand	       = htmlspecialchars_decode(trim($match[4]));
            $f_name_compare    = trim(str_replace(array('{', '}'), '',$match[6]));
            $content           = $match[8];

            $f_name_additional         = $this->findAdditional($f_name);
            $f_name_compare_additional = $this->findAdditional($f_name_compare);

            $additional                = $f_name_additional->additional;
            $skip_field_check          = $f_name_additional->skip_field_check;

            $additional_compare        =  $f_name_compare_additional->additional;
            $skip_field_check_compare  =  $f_name_compare_additional->skip_field_check;


            if (!$skip_field_check)
            {
                $form_field = RSDirectoryHelper::findElements( array('name' => $f_name), $form_fields );

                // check the custom_fields too
                if (empty($form_field) && isset($custom_fields[$f_name.$additional]))
                {
                    $form_field = true;
                }

                // check global placeholders too
                if (empty($form_field) && in_array($f_name.$additional, $this->placeholders)) {

                    $form_field = true;
                }

                $value = empty($form_field) || !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : $map[$f_name.$additional];//addcslashes($content, '$');
            }
            else
            {
                $value = !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : $map[$f_name.$additional];//addcslashes($content, '$');
            }

            if (!$skip_field_check_compare)
            {
                $form_field = RSDirectoryHelper::findElements( array('name' => $f_name_compare), $form_fields );

                // check the custom_fields to
                if (empty($form_field) && isset($custom_fields[$f_name_compare.$additional_compare]))
                {
                    $form_field = true;
                }

                $compare = empty($form_field) || !isset($map[$f_name_compare.$additional_compare]) || $map[$f_name_compare.$additional_compare] == '' ? '' : $map[$f_name_compare.$additional_compare];//addcslashes($content, '$');
            }
            else
            {
                $compare = !isset($map[$f_name_compare.$additional_compare]) || $map[$f_name_compare.$additional_compare] == '' ? '' : $map[$f_name_compare.$additional_compare];//addcslashes($content, '$');
            }

            if (strlen($f_name_compare) > 0 && strlen($compare) == 0)
            {
                break;
            }

            $result = $this->doComparison($value, $compare, $operand);

            $newline = '{/if}[\r]?[\n]?';

            // If empty value remove whole line else show line but remove pseudo-code.
            $string = preg_replace(str_replace('{/if}', $newline, $pattern), $result ? addcslashes($content, '$') : '', $string, 1);
        }

        // Placeholder1 > 1
        $pattern = '#{if(\s+)?' . $condition . '(\s+)?' . $comparisonPattern . '?(\s+)?' . $inner . '?\s?(\s+)?}' . $inner . '{/if}#is';
        while ( preg_match($pattern, $string, $match) )
        {
            $f_name            = str_replace(array('{', '}'), '',$match[2]);
            $operand	       = htmlspecialchars_decode(trim($match[4]));
            $compare           = trim($match[6], '\'" ');
            $content           = $match[8];

            $f_name_additional         = $this->findAdditional($f_name);

            $additional                = $f_name_additional->additional;
            $skip_field_check          = $f_name_additional->skip_field_check;


            if (!$skip_field_check)
            {
                $form_field = RSDirectoryHelper::findElements( array('name' => $f_name), $form_fields );

                // check the custom_fields too
                if (empty($form_field) && isset($custom_fields[$f_name.$additional]))
                {
                    $form_field = true;
                }

                // check global placeholders too
                if (empty($form_field) && in_array($f_name.$additional, $this->placeholders)) {

                    $form_field = true;
                }
                $value = empty($form_field) || !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : $map[$f_name.$additional];//addcslashes($content, '$');
            }
            else
            {
                $value = !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : $map[$f_name.$additional];//addcslashes($content, '$');
            }

            if (strlen($f_name) > 0 && strlen($value) == 0 && !isset($map[$f_name.$additional]))
            {
                break;
            }

            $result = $this->doComparison($value, $compare, $operand);

            $newline = '{/if}[\r]?[\n]?';

            // If empty value remove whole line else show line but remove pseudo-code.
            $string = preg_replace(str_replace('{/if}', $newline, $pattern), $result ? addcslashes($content, '$') : '', $string, 1);
        }

        // 1 < Placeholder1
        $pattern = '#{if(\s+)?' . $inner . '(\s+)?' . $comparisonPattern . '?(\s+)?' . $condition . '?\s?(\s+)?}' . $inner . '{/if}#is';
        while ( preg_match($pattern, $string, $match) )
        {
            $f_name            = str_replace(array('{', '}'), '',$match[6]);
            $operand	       = htmlspecialchars_decode(trim($match[4]));
            $compare           = trim($match[2], '\'" ');
            $compare           = trim($compare);
            $content           = $match[8];
			
			//check the field name to exist because if it doesn't the next match (legacy one) will be skipped
            if (!strlen($f_name)) {
                break;
            }

            $f_name_additional         = $this->findAdditional($f_name);

            $additional                = $f_name_additional->additional;
            $skip_field_check          = $f_name_additional->skip_field_check;


            if (!$skip_field_check)
            {
                $form_field = RSDirectoryHelper::findElements( array('name' => $f_name), $form_fields );

                // check the custom_fields too
                if (empty($form_field) && isset($custom_fields[$f_name.$additional]))
                {
                    $form_field = true;
                }

                // check global placeholders too
                if (empty($form_field) && in_array($f_name.$additional, $this->placeholders)) {

                    $form_field = true;
                }

                $value = empty($form_field) || !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : $map[$f_name.$additional];//addcslashes($content, '$');
            }
            else
            {
                $value = !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : $map[$f_name.$additional];//addcslashes($content, '$');
            }

            $result = $this->doComparison($value, $compare, $operand);

            $newline = '{/if}[\r]?[\n]?';

            // If empty value remove whole line else show line but remove pseudo-code.
            $string = preg_replace(str_replace('{/if}', $newline, $pattern), $result ? addcslashes($content, '$') : '', $string, 1);
        }

        // legacy scripting - {if field_name} without brackets but with brackets also for new style use
        $condition 	= '([a-z0-9\-]+|{[a-z0-9\-]+})';
        $inner = '((?:(?!{/?if).)*?)';
        $pattern = '#{if(\s+)?' . $condition . '(\s+)?}' . $inner . '{/if}#is';

        while ( preg_match($pattern, $string, $match) )
        {
            $f_name = str_replace(array('{', '}'), '',$match[2]);
            $content = $match[4];

            $f_name_additional         = $this->findAdditional($f_name);

            $additional                = $f_name_additional->additional;
            $skip_field_check          = $f_name_additional->skip_field_check;

            if (!$skip_field_check)
            {
                $form_field = RSDirectoryHelper::findElements( array('name' => $f_name), $form_fields );

                // check the custom_fields too
                if (empty($form_field) && isset($custom_fields[$f_name.$additional]))
                {
                    $form_field = true;
                }

                // check global placeholders too
                if (empty($form_field) && in_array($f_name.$additional, $this->placeholders)) {

                    $form_field = true;
                }

                $replace = empty($form_field) || !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : addcslashes($content, '$');
            }
            else
            {
                $replace = !isset($map[$f_name.$additional]) || $map[$f_name.$additional] == '' ? '' : addcslashes($content, '$');
            }

            $newline = '{/if}[\r]?[\n]?';

            // If empty value remove whole line else show line but remove pseudo-code.
            $string = preg_replace(str_replace('{/if}', $newline, $pattern), $replace, $string, 1);
        }

        return $string;
    }

    protected function findAdditional(&$field_name)
    {
        $checks_custom = array('-small-thumb', '-big-thumb', '-normal-thumb');

        $additional = '';
        $skip_field_check = false;

        if (strpos($field_name, '-src-')) {
            $field_name_data = explode('-src-', $field_name);

            if (count($field_name_data) == 2)
            {
                // detect if custom field
                $found_custom = false;
                $prefix = '';
                foreach ($checks_custom as $ck)
                {
                    if (strpos($field_name_data[0], $ck) !== false)
                    {
                        $field_name = str_replace($ck, '', $field_name_data[0]);
                        $prefix = $ck;
                        $found_custom = true;
                    }
                }

                if (!$found_custom)
                {
                    $field_name = $field_name_data[0];
                }
                $additional = $prefix.'-src-'.$field_name_data[1];
                $skip_field_check = true;
            }
        }

        return (object) array('additional' => $additional, 'skip_field_check' => $skip_field_check);
    }

    protected function doComparison($value, $compare, $operand)
    {
        switch ($operand)
        {
            default:
                $result = $value;
                break;

            case '<=':
                $result = $value <= $compare;
                break;

            case '>=':
                $result = $value >= $compare;
                break;

            case '<>':
                $result = $value <> $compare;
                break;

            case '<':
                $result = $value < $compare;
                break;

            case '>':
                $result = $value > $compare;
                break;

            case '!=':
                $result = $value != $compare;
                break;

            case '=':
            case '==':
                $result = $value == $compare;
                break;

            case '===':
                $result = $value === $compare;
                break;

            case '<-':
                $result = $this->inclusion($value, $compare, 'left');
                break;

            case '->':
                $result = $this->inclusion($value, $compare, 'right');
                break;
        }

        return $result;
    }

    public function inclusion($value, $compare, $direction) {
        $values = explode(',', $value);
        $compares = explode(',', $compare);

        $intersect = array_intersect($values, $compares);

        return ($direction == 'left' ? count($intersect) == count($compares) : count($intersect) == count($values));
    }

}