<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */


class RSDirectorySnippets {
    public $snippet = '';
    protected $ids = array();

    public function __construct($ids = array())
    {
        $this->ids = $ids;
    }

    public static function getInstance($ids = array())
    {
        static $instance = array();

        $hash = implode(',', $ids);
        $hash = md5($hash);

        if (!isset($instance[$hash]))
        {
            $instance[$hash] = new RSDirectorySnippets($ids);
        }

        return $instance[$hash];
    }

    public function buildEntriesSnippets($listing_type = 'listing_detail')
    {
        $snippets = array();

        foreach ($this->ids as $id)
        {
            if ($entry_data = RSDirectoryHelper::getEntry($id))
            {
                $entry_data = RSDirectoryHelper::getEntryData($entry_data);

                if (!$entry_data)
                {
                	continue;
                }

                $form = RSDirectoryHelper::getForm($entry_data->form_id);

                if (empty($form->use_entry_snippet))
                {
                    continue;
                }

                $files_list = RSDirectoryHelper::getFilesObjectList(0, $id);

                // Load the files required by the image gallery.
                $images = false;
                if ($images_field = RSDirectoryHelper::findFormField('images', $entry_data->form->fields))
                {
                    if ($files_list && $form->listing_detail_show_gallery)
                    {
                        $images = RSDirectoryHelper::findElements(array('field_id' => $images_field->id), $files_list, false);
                    }
                }

                // if the admin selected to use built in snippet format
                if (!empty($form->use_entry_snippet_auto))
                {
                    $images_src = array();
                    if ($images)
                    {
                        foreach ($images as $i => $image)
                        {
                            $images_src[] = RSDirectoryHelper::getImageURL($image->hash, 'big');
                        }
                    }

                    $desc = RSDirectoryHelper::cut(strip_tags($entry_data->description), 300);
                    $desc = str_replace(array("\r\n", "\n"), ' ', $desc);


                    $json_snippet = new stdClass();
                    $json_snippet->{'@context'} = 'https://schema.org/';
                    $json_snippet->{'@type'} = 'Product';
                    $json_snippet->name = RSDirectoryHelper::escapeHTML($entry_data->title);

                    if (!empty($images_src))
                    {
                        $json_snippet->image =  $images_src;
                    }
                    $json_snippet->description = $desc;

                    // if the SKU is set use it
                    if (!empty($form->entry_snippet_sku))
                    {
                        $json_snippet->sku = RSDirectoryHelper::escapeHTML($form->entry_snippet_sku);
                    }

                    // Use Global Identifier if its set and he placehoder is not empty
                    if (!empty($form->entry_snippet_identifier) && $form->entry_snippet_identifier != 'none' && !empty($form->entry_snippet_identifier_placeholder))
                    {
                        $json_snippet->{$form->entry_snippet_identifier} = RSDirectoryHelper::escapeHTML($form->entry_snippet_identifier_placeholder);
                    }

                    // Get the reviews for this entry
                    $ratings_model = RSDirectoryModel::getInstance('Ratings');
                    // set the entry id in the model
                    $ratings_model->setEntryId($id);
                    $reviews = $ratings_model->getItems();

                    $snippet_reviews = array();
                    foreach ($reviews as $review) {
                        // keep only the reviews that have a positive score ( > 0)
                        if ((int)$review->score == 0)
                        {
                            continue;
                        }

                        $review_snippet_obj = new stdClass();

                        $review_snippet_obj->{'@type'} = 'Review';
                        $review_snippet_obj->reviewRating =  new stdClass();
                        $review_snippet_obj->reviewRating->{'@type'} =  'Rating';
                        $review_snippet_obj->reviewRating->ratingValue =  (int)$review->score;
                        $review_snippet_obj->reviewRating->bestRating =  5;

                        if (!empty($review->review))
                        {
                            $review_snippet_obj->reviewBody = RSDirectoryHelper::escapeHTML($review->review);
                            $review_snippet_obj->reviewBody = strip_tags($review_snippet_obj->reviewBody);
                            $review_snippet_obj->reviewBody = str_replace(array("\r\n", "\n"), ' ',$review_snippet_obj->reviewBody);
                        }

                        $review_snippet_obj->author = new stdClass();
                        $review_snippet_obj->author->{'@type'} = 'Person';
                        $review_snippet_obj->author->name = RSDirectoryHelper::escapeHTML($review->name);

                        $snippet_reviews[] = $review_snippet_obj;
                    }

                    // if there are reviews then add the aggregate rating
                    if (!empty($snippet_reviews))
                    {
                        $json_snippet->review =  $snippet_reviews;

                        $json_snippet->aggregateRating = new stdClass();
                        $json_snippet->aggregateRating->{'@type'} = 'AggregateRating';
                        $json_snippet->aggregateRating->ratingValue = $entry_data->avg_rating;
                        $json_snippet->aggregateRating->reviewCount = $entry_data->ratings_count;
                    }

                    if (!empty($entry_data->price) && $form->listing_detail_show_price && RSDirectoryHelper::findFormField('price', $entry_data->form->fields))
                    {
                        $item_url = RSDirectoryRoute::getEntryURL($entry_data->id, $entry_data->title, '', 0, true);

                        $config = RSDirectoryConfig::getInstance();
                        $currency = $config->get('currency');
                        $decimals = $config->get('decimals');
                        $content_price = number_format($entry_data->price, $decimals, '.', '');

                        $json_snippet->offers = new stdClass();
                        $json_snippet->offers->{'@type'} = 'Offer';
                        $json_snippet->offers->url = $item_url;
                        $json_snippet->offers->priceCurrency = $currency;
                        $json_snippet->offers->price = $content_price;

                        if ($entry_data->expiry_time != '0000-00-00 00:00:00')
                        {
                            $json_snippet->offers->priceValidUntil = RSDirectoryHelper::escapeHTML($entry_data->expiry_time);
                        }

                        // Set the product availability
                        if (!empty($form->entry_snippet_offer_availability))
                        {
                            $json_snippet->offers->availability = 'https://schema.org/'.RSDirectoryHelper::escapeHTML($form->entry_snippet_offer_availability);
                        }
                        else
                        {
                            // if not set use the InStock
                            $json_snippet->offers->availability = 'https://schema.org/InStock';
                        }

                        // Set the item condition if exists
                        if (!empty($form->entry_snippet_offer_item_condition))
                        {
                            $json_snippet->offers->itemCondition = 'https://schema.org/'.RSDirectoryHelper::escapeHTML($form->entry_snippet_offer_item_condition);
                        }

                        if ($form->{$listing_type . '_show_author'})
                        {
                            $json_snippet->offers->seller = new stdClass();
                            $json_snippet->offers->seller->{'@type'} = 'Person';
                            $json_snippet->offers->seller->{'@type'} = 'Person';
                            $json_snippet->offers->seller->name = $entry_data->author;
                        }
                    }

                    // encode it without escape
                    $json_snippet = json_encode($json_snippet, JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT);

                    // replace the placeholders
                    require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/placeholders.php';
                    $json_snippet = RSDirectoryPlaceholders::getInstance($json_snippet, $entry_data->form->fields, $entry_data, $form)->process();

                    // strip all html tags if there are any left
                    $json_snippet = strip_tags($json_snippet);

                    // add to the snippets array
                    $snippets[] = $json_snippet;
                }
                // if the admin selected to use own snippet format (pro users)
                else
                {
                    if (empty($form->entry_snippet))
                    {
                        continue;
                    }
                    if ($json_snippet = json_decode($form->entry_snippet))
                    {
                        // handle images
                        if (isset($json_snippet->image) && $json_snippet->image == '{snippet.images}')
                        {
                           $images_src = array();
                           if ($images)
                           {
                               // add the images
                               foreach ($images as $i => $image)
                               {
                                   $images_src[] = RSDirectoryHelper::getImageURL($image->hash, 'big');
                               }

                               if (!empty($images_src))
                               {
                                  $json_snippet->image = $images_src;
                               }
                           }
                           else
                           {
                               unset($json_snippet->images);
                           }
                        }

                        // handle reviews
                        if (isset($json_snippet->review))
                        {
                            $ratings_model = RSDirectoryModel::getInstance('Ratings');
                            // set the entry id in the model
                            $ratings_model->setEntryId($id);
                            $reviews = $ratings_model->getItems();

                            $snippet_reviews = array();
                            if (empty($reviews))
                            {
                               unset($json_snippet->review);
                            }
                            else {
                                foreach ($reviews as $review) {
                                    // keep only the reviews that have a positive score ( > 0)
                                    if ((int)$review->score == 0) {
                                        continue;
                                    }
                                    $review_model = $json_snippet->review;

                                    // set the score
                                    if (isset($review_model->reviewRating->ratingValue)) {
                                        $review_model->reviewRating->ratingValue = str_replace('{snippet.review_rating_value}', $review->score, $review_model->reviewRating->ratingValue);
                                    }

                                    // set the review body
                                    if (isset($review_model->reviewBody) && !empty($review->review)) {
                                        $review_model->reviewBody = str_replace('{snippet.review_body}', RSDirectoryHelper::escapeHTML($review->review), $review_model->reviewBody);
                                        $review_model->reviewBody = str_replace(array("\r\n", "\n"), ' ',$review_model->reviewBody);
                                    } else {
                                        unset($review_model->reviewBody);
                                    }

                                    // set the author
                                    if (isset($review_model->author->name)) {
                                        $review_model->author->name = str_replace('{snippet.review_author_name}', RSDirectoryHelper::escapeHTML($review->name), $review_model->author->name);
                                    }

                                    $snippet_reviews[] = $review_model;
                                }

                                if (!empty($snippet_reviews))
                                {
                                    $json_snippet->review = count($snippet_reviews) == 1 ? $snippet_reviews[0] : $snippet_reviews;
                                }
                            }
                        }

                        // handle aggregate rating
                        if (isset($json_snippet->aggregateRating) && isset($json_snippet->review))
                        {
                            $json_snippet->aggregateRating->ratingValue = str_replace('{snippet.aggregate_rating_value}', $entry_data->avg_rating, $json_snippet->aggregateRating->ratingValue);
                            $json_snippet->aggregateRating->reviewCount = str_replace('{snippet.aggregate_review_count}', $entry_data->ratings_count, $json_snippet->aggregateRating->reviewCount);
                        }
                        else
                        {
                            if(isset($json_snippet->aggregateRating))
                            {
                                unset($json_snippet->aggregateRating);
                            }
                        }

                        // handle offers
                        if (isset($json_snippet->offers))
                        {
                            if (!empty($entry_data->price) && RSDirectoryHelper::findFormField('price', $entry_data->form->fields))
                            {
                                $config = RSDirectoryConfig::getInstance();
                                $currency = $config->get('currency');
                                $decimals = $config->get('decimals');
                                $content_price = number_format($entry_data->price, $decimals, '.', '');

                                $json_snippet->offers->priceCurrency = str_replace('{snippet.currency}', $currency, $json_snippet->offers->priceCurrency);
                                $json_snippet->offers->price = str_replace('{snippet.price_value}', $content_price, $json_snippet->offers->price);

                                if (isset($json_snippet->offers->priceValidUntil) && $entry_data->expiry_time != '0000-00-00 00:00:00')
                                {
                                    $json_snippet->offers->priceValidUntil = str_replace('{snippet.price_valid_until}', RSDirectoryHelper::escapeHTML($entry_data->expiry_time), $json_snippet->offers->priceValidUntil);
                                }
                                else
                                {
                                    if(isset($json_snippet->offers->priceValidUntil))
                                    {
                                        unset($json_snippet->offers->priceValidUntil);
                                    }
                                }
                            } else {
                                unset($json_snippet->offers);
                            }
                        }

                        // process the other placeholders
                        $json_snippet = json_encode($json_snippet, JSON_UNESCAPED_SLASHES);



                        require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/placeholders.php';
                        $json_snippet = RSDirectoryPlaceholders::getInstance($json_snippet, $entry_data->form->fields, $entry_data, $form)->process();

                        // strip all html tags
                        $json_snippet = strip_tags($json_snippet);
                        // remove new lines so that the json_decode will still work
                        $json_snippet = str_replace(array("\r\n", "\n"),' ',$json_snippet);


                        // small workaround for the description
                        $json_snippet = json_decode($json_snippet);

                        if (isset($json_snippet->description) && !empty($json_snippet->description)) {
                            $json_snippet->description =  RSDirectoryHelper::cut(strip_tags($json_snippet->description), 300);
                        }

                        // encode it again without escape
                        $json_snippet = json_encode($json_snippet, JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT);

                        // add to the snippets array
                        $snippets[] = $json_snippet;
                    }
                }
            }
        }



        // add to the public snippet variable
        if (!empty($snippets))
        {
            if (count($snippets) == 1 && $listing_type == 'listing_detail')
            {
                $this->snippet = $snippets[0];
            }
            else
            {
                $this->snippet = '['.implode(','."\n", $snippets).']';
            }
        }
    }
}