<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Vimeo embed code generator helper.
 */
class RSDirectoryVimeo
{
	/**
	 * The Vimeo field properties.
	 *
	 * @access private
	 *
	 * @var JRegistry
	 */
	private $properties;
		
	/**
	 * The Vimeo video url.
	 *
	 * @access private
	 *
	 * @var string
	 */
	private $url;
		
	/**
     * The class constructor.
     *
     * @access public
     *
     * @param JRegistry $properties The Vimeo field properties.
     * @param string $url The Vimeo video url.
     */
	public function __construct($properties, $url)
	{
		$this->properties = empty($properties) ? new RSDirectoryFormFieldProperties : $properties;
		$this->url = $url;
	}
		
	/**
	 * Generate the embed code.
	 *
	 * @access public
	 *
	 * @return string
	 */
	public function generate()
	{
		$properties = $this->properties;
			
		// Get the video id.	
		$uri = new JUri($this->url);
		$id = substr( $uri->getPath(), 1 );

		$video_size = $properties->get('video_size');
		$player_prefix = '';
		$player_suffix = '';
		$player_style = '';
		$size = '';

		if ( $video_size == 'custom' )
		{
			$size = ' width="'.(int)$properties->get('video_width').'" height="'.(int)$properties->get('video_height').'" ';
		}
		else if ($video_size != 'custom' && $video_size != 'responsive')
		{
			list($width, $height) = explode( 'x', $properties->get('video_size') );

			$size = ' width="'.(int)$width.'" height="'.(int)$height.'" ';
		}
		else
		{
			$player_prefix = '<div style="padding:56.25% 0 0 0;position:relative;">';
			$player_suffix = '</div><script src="https://player.vimeo.com/api/player.js"></script>';
			$player_style = ' style="position:absolute;top:0;left:0;width:100%;height:100%;" ';
		}

		$color = $properties->get('color', '#00adef');
		$color = str_replace('#','', $color);
		$url_attr = array();
		$url_attr[] = 'color='.$color;

		if (!(int)$properties->get('video_portrait', '0'))
		{
			$url_attr[] = 'portrait=0';
		}

		if (!(int)$properties->get('video_title', '0'))
		{
			$url_attr[] = 'title=0';
		}

		if (!(int)$properties->get('video_byline', '0'))
		{
			$url_attr[] = 'byline=0';
		}

		if ((int)$properties->get('autoplay', '0'))
		{
			$url_attr[] = 'autoplay=1';
		}

		if ((int)$properties->get('loop', '0'))
		{
			$url_attr[] = 'loop=1';
		}

		if (!empty($url_attr))
		{
			$url_attr = '?'.implode('&', $url_attr);
		}
		$url = '//player.vimeo.com/video/'.$id.$url_attr;



		return $player_prefix.'<iframe src="' . RSDirectoryHelper::escapeHTML($url) . '"'.$size.$player_style.' frameborder="0" allow="autoplay; fullscreen; picture-in-picture" allowfullscreen></iframe>'.$player_suffix;
	}
}