<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

if (RSDirectoryHelper::isJ4()) {
    require_once JPATH_ADMINISTRATOR . '/components/com_categories/src/Model/CategoriesModel.php';
} else {
    require_once JPATH_ADMINISTRATOR . '/components/com_categories/models/categories.php';
}

if ( !class_exists('RSDirectoryCategories') )
{
    if (RSDirectoryHelper::isJ4())
    {
        class RSDirectoryCategoriesParent extends Joomla\Component\Categories\Administrator\Model\CategoriesModel{}
    }
    else
    {
        class RSDirectoryCategoriesParent extends CategoriesModelCategories {}
    }

    class RSDirectoryCategories extends RSDirectoryCategoriesParent
    {
        /**
         * Method to get a object for retrieving the data set from a database.
         *
         * @access protected
         *
         * @return  object
         */
        protected function getListQuery()
        {
            // Get the query from the parent class.
            $query = parent::getListQuery();

            // Get DBO.
            $db = JFactory::getDbo();

            $query->select( $db->qn('a.params') );

            return $query;
        }
    }
}


/**
 * Categories model.
 */
class RSDirectoryModelCategories extends RSDirectoryCategories
{
    /**
    * Constructor.
    *
    * @param array An optional associative array of configuration settings.
    */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'ordering',
                'lft',
                'published',
                'level',
                'title',
                'access',
                'language',
                'id',
            );
        }
            
        parent::__construct($config);
    }
    
    /**
     * Method to get a list of categories.
     *
     * @access public
     *
     * @return mixed An array of objects on success, false on failure.
     */
    public function getItems()
    {
        $items = parent::getItems();
            
        if (!$items)
            return $items;
            
        // Get the forms.
        $forms = RSDirectoryHelper::getForms();
            
        $forms_aux = array();
           
        // Group the forms by id. 
        foreach($forms as $form)
        {
            $forms_aux[$form->id] = $form;
        }
            
        // Get all the categories that belong to RSDirectory!
        $categories = RSDirectoryHelper::getCategories();
            
        foreach ($items as &$item)
        {
            // Get the category params.
            $params = new JRegistry($item->params);
                
            // Get the form id.
            $form_id = $params->get('form_id');
                
            // We have a form assigned to the category.
            if ( isset($forms_aux[$form_id]) )
            {
                $item->form = $forms_aux[$form_id];
            }
            // Inherited form.
            else
            {
                $form_id = RSDirectoryHelper::getCategoryInheritedFormId($item->id, $categories);
                    
                // Found the inherited form.
                if ( isset($forms_aux[$form_id]) )
                {
                    $item->form = clone $forms_aux[$form_id];
                    $item->form->title = JText::sprintf('COM_RSDIRECTORY_INHERITED', " ({$item->form->title})");
                }
                // No form found.
                else
                {
                    $item->form = (object)array(
                        'title' => JText::sprintf( 'COM_RSDIRECTORY_INHERITED', ' (' . JText::_('JNONE') . ')' ),
                    );
                }
            }
        }
            
        return $items;
    }
        
    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @param string An optional ordering field.
     * @param string An optional direction (asc|desc).
     */
    protected function populateState($ordering = null, $direction = null)
    {
        // List state information.
        parent::populateState('a.lft', 'asc');
            
        $this->setState('filter.extension', 'com_rsdirectory');
    }
}