<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Credit History model.
 */
class RSDirectoryModelCreditHistory extends JModelAdmin
{
    /**
     * Method to get a table object, load it if necessary.
     * 
     * @access public
     * 
     * @param string $type
     * @param string $prefix
     * @param array $config
     * 
     * @return object
     */
    public function getTable( $type = 'CreditHistory', $prefix = 'RSDirectoryTable', $config = array() )
    {
        return JTable::getInstance($type, $prefix, $config);
    }
        
    /**
     * Method for getting the form from the model.
     *
     * @access public
     * 
     * @param array $data
     * @param bool $loadData
     * 
     * @return mixed
     */
    public function getForm( $data = array(), $loadData = true )
    {
        return false;
    }
        
    /**
     * Method to mark one or more entries as paid/unpaid.
     *
     * @access public
     *
     * @return bool
     */
    public function markAsPaid($pks, $paid)
    {
        if ( empty($pks) )
            return false;
            
        if ( !is_array($pks) )
        {
            $pks = (array)$pks;
        }
            
        // Sanitize array.
        $pks = RSDirectoryHelper::arrayInt($pks, true, true);
            
        if ( empty($pks) )
            return false;
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        $pks = array_unique($pks);
        $pks_unquoted = $pks;
            
        foreach ($pks as &$pk)
        {
            $pk = $db->q($pk);
        }
            
        $in_pks = implode(',' , $pks);
            
        $paid = $paid ? 1 : 0;
            
        // Update paid status.
        $query = $db->getQuery(true)
               ->update( $db->qn('#__rsdirectory_entries_credits') )
               ->set( $db->qn('paid') . ' = ' . $db->q($paid) )
               ->where( $db->qn('id') . ' IN (' . $in_pks . ')' );
               
        $db->setQuery($query);
        $db->execute();
            
        // Get the entries ids.
        $query = $db->getQuery(true)
               ->select( $db->qn('id') )
               ->select( $db->qn('entry_id') )
               ->from( $db->qn('#__rsdirectory_entries_credits') )
               ->where( $db->qn('id') . ' IN (' . $in_pks . ')' );
              // ->group( $db->qn('entry_id') );
               
        $db->setQuery($query);
        $assoc_list = $db->loadAssocList('id', 'entry_id');

        // remove duplicates
        $entries_ids = array_unique($assoc_list);

        // rewrite indexes
        $entries_ids = array_values($entries_ids);

        if ($entries_ids)
        {
            if ($paid)
            {
                // Get the ids of the entries that have unpaid entries.
                $query = $db->getQuery(true)
                       ->select( $db->qn('entry_id') )
                       ->from( $db->qn('#__rsdirectory_entries_credits') )
                       ->where( $db->qn('entry_id') . ' IN (' . implode(',', $entries_ids) . ')' )
                       ->where( $db->qn('paid') . ' = ' . $db->q(0) )
                       ->group( $db->qn('entry_id') );
                       
                $db->setQuery($query);
                $unpaid_entries_ids = $db->loadColumn();
                    
                // Get the ids of the entries that have no unpaid entries.
                $paid_entries_ids = array_diff($entries_ids, $unpaid_entries_ids);
                    
                // Mark entrie as paid.
                if ($paid_entries_ids)
                {
                    $query = $db->getQuery(true)
                           ->update( $db->qn('#__rsdirectory_entries') )
                           ->set( $db->qn('paid') . ' = ' . $db->q(1) )
                           ->where( $db->qn('id') . ' IN (' . implode(',', $paid_entries_ids) . ')' );
                           
                    $db->setQuery($query);
                    $db->execute();
                }
            }
            else
            {
                $query = $db->getQuery(true)
                       ->update( $db->qn('#__rsdirectory_entries') )
                       ->set( $db->qn('paid') . ' = ' . $db->q(0) )
                       ->where( $db->qn('id') . ' IN (' . implode(',', $entries_ids) . ')' );
                        
                $db->setQuery($query);
                $db->execute();
                    
                // Unpublish entries.
                JTable::getInstance('Entry', 'RSDirectoryTable')->publish($entries_ids, 0);
            }
        }

        // log every credit
        foreach ($pks_unquoted as $pk)
        {
            $log_params = array(
                'type' => $paid ? 'credits_history_mark_paid' : 'credits_history_mark_unpaid',
                'credit_id' => $pk
            );

            // if the credit still has an entry associated
            if (isset($assoc_list[$pk]))
            {
                $log_params['entry_id'] = $assoc_list[$pk];
            }

            RSDirectoryHelper::logAction($log_params);
        }
            
        return true;
    }
}