<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Credits History model.
 */
class RSDirectoryModelCreditsHistory extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'title',
				'user',
				'object_type',
				'credits',
				'free',
				'created_time',
				'paid',
				'id',
            );
        }
            
        parent::__construct($config);
    }

    /**
     * Method to overwrite the filter layout for the entry_id field.
     *
     * @access public
     *
     * @return object.
     */
    public function getFilterForm($data = array(), $loadData = true) {
        $form = parent::getFilterForm($data, $loadData);

        if (RSDirectoryHelper::isJ4())
        {
            $form->setFieldAttribute('entry_id', 'layout', 'joomla.form.field.list-fancy-select','filter');
        }

        return $form;
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
			
		// Get entry author display info (name, username, email..).
		$author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');
			
		$select = array(
			$db->qn('ec') . '.*',
			$db->qn("u.$author", 'user'),
			$db->qn('e.title'),
			$db->qn('f.name', 'field_name'),
		);
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_entries_credits', 'ec') )
			   ->leftJoin( $db->qn('#__rsdirectory_entries', 'e') . ' ON ' . $db->qn('ec.entry_id') . ' = ' . $db->qn('e.id') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('ec.user_id') . ' = ' . $db->qn('u.id') )
			   ->leftJoin( $db->qn('#__rsdirectory_fields', 'f') . ' ON ' . $db->qn('ec.object_id') . ' = ' . $db->qn('f.id') . ' AND ' . $db->qn('ec.object_type') . ' = ' . $db->q('form_field') );
			   
		// Filter by entry.  
        if ( $entry_id = $this->getState('filter.entry_id') )
        {
            $query->where( $db->qn('ec.entry_id') . ' = ' . $db->q($entry_id) );
        }
			
		// Filter by user.  
        if ( $user_id = $this->getState('filter.user_id') )
        {
            $query->where( $db->qn('ec.user_id') . ' = ' . $db->q($user_id) );
        }
			
		// Filter by type.  
        if ( $type = $this->getState('filter.type') )
        {
            $query->where( $db->qn('ec.object_type') . ' = ' . $db->q($type) );
        }
			
		// Filter by credits.      
        if ( $credits = $this->getState('filter.credits') )
        {
            $query->where( $db->qn('ec.credits') . ' = ' . $db->q($credits) );
        }
			
		// Filter by credits.
		$is_free = $this->getState('filter.is_free');
			
        if ($is_free !== '')
        {
            $query->where( $db->qn('ec.free') . ' = ' . $db->q($is_free) );
        }
			
		// Paid Y/N ?
        $paid = $this->getState('filter.paid');
            
        if ( is_numeric($paid) )
        {
            $query->where( $db->qn('ec.paid') . ' = ' . $db->q($paid) );    
        }
			
		// Filter by search.
        $search = $this->getState('filter.search');
			
		if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $condition = array(
                $db->qn('e.title') . " LIKE $search",
                $db->qn('u.name') . " LIKE $search",
            );
                
            $query->where( '(' . implode(' OR ', $condition) . ')' );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
            
        $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );
            
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * 
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
		$search = $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search');
        $this->setState('filter.search', $search);
			
		$entry_id = $this->getUserStateFromRequest("$this->context.filter.entry_id", 'filter_entry_id', '');
        $this->setState('filter.entry_id', $entry_id);
			
		$user_id = $this->getUserStateFromRequest("$this->context.filter.user_id", 'filter_user_id', '');
        $this->setState('filter.user_id', $user_id);
			
		$type = $this->getUserStateFromRequest("$this->context.filter.type", 'filter_type', '');
        $this->setState('filter.type', $type);
			
		$credits = $this->getUserStateFromRequest("$this->context.filter.credits", 'filter_credits', '');
        $this->setState('filter.credits', $credits);
			
		$is_free = $this->getUserStateFromRequest("$this->context.filter.is_free", 'filter_is_free', '');
        $this->setState('filter.is_free', $is_free);
			
		$paid = $this->getUserStateFromRequest("$this->context.filter.paid", 'filter_paid');
        $this->setState('filter.paid', $paid);
			
        // List state information.
        parent::populateState('ec.created_time', 'desc');
    }
}