<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Email Message model.
 */
class RSDirectoryModelEmailMessage extends JModelAdmin
{
    /**
     * Method to get a table object, load it if necessary.
     * 
     * @access public
     * 
     * @param string $type
     * @param string $prefix
     * @param array $config
     * 
     * @return object
     */
    public function getTable( $type = 'EmailMessage', $prefix = 'RSDirectoryTable', $config = array() )
    {
        return JTable::getInstance($type, $prefix, $config);
    }
        
    /**
     * Method for getting the form from the model.
     *
     * @access public
     * 
     * @param array $data
     * @param bool $loadData
     * 
     * @return mixed
     */
    public function getForm( $data = array(), $loadData = true )
    {
        // Get the form.
        $form = $this->loadForm( 'com_rsdirectory.emailmessage', 'emailmessage', array('control' => 'jform', 'load_data' => $loadData) );
            
        if ( empty($form) )
            return false;
            
        return $form;
    }
        
    /**
     * Method to get the data that should be injected in the form.
     *
     * @access protected
     * 
     * @return array
     */
    protected function loadFormData()
    {
        // Check for data in the session.
        $data = JFactory::getApplication()->getUserState('com_rsdirectory.edit.emailmessage.data');
            
        return $data ? $data : $this->getItem();
    }
        
    /**
     * Validate form data.
     *
     * @access public
     * 
     * @param object $form The form to validate against.
     * @param array $data The data to validate.
     * @param string $group The name of the field group to validate.
     * 
     * @return mixed
     */
    public function validate($form, $data, $group = null)
    {
        $return = parent::validate($form, $data, $group);
            
        if ( !empty($data['type']) && ($data['type'] == 'entry_expiration_notice' || $data['type'] == 'entry_expiration_renew') && empty($data['entry_expiration_period']) )
        {
            $this->setError( JText::_('COM_RSDIRECTORY_EMAIL_MESSAGES_ENTRY_EXPIRATION_PERIOD_ERROR') );
            $return = false;
        }
            
        $fields = array(
            'to' => JText::_('COM_RSDIRECTORY_EMAIL_MESSAGE_TO_LABEL'),
            'bcc' => JText::_('COM_RSDIRECTORY_EMAIL_MESSAGE_BCC_LABEL'),
            'cc' => JText::_('COM_RSDIRECTORY_EMAIL_MESSAGE_CC_LABEL'),
        );
        
        foreach ($fields as $field => $label)
        {
            if ( !empty($data[$field]) )
            {
                $emails = explode(',', $data[$field]);
                    
                if ($emails)
                {
                    foreach ($emails as &$email)
                    {
                        $email = trim($email);
                            
                        if ( !RSDirectoryHelper::email($email) )
                        {
                            $this->setError( JText::sprintf('COM_RSDIRECTORY_EMAIL_FIELDS_INVALID_EMAIL_MESSAGE', $label) );
                            $return = false;
                            continue 2;
                        }
                    }
                }
            }
        }
            
        return $return;
    }
        
    /**
     * Save the email message.
     *
     * @access public
     * 
     * @param array $data
     * 
     * @return bool
     */
    public function save($data)
    {
        if ( JFactory::getApplication()->input->get('task') == 'save2copy' )
        {
            $this->setError( JText::_('COM_RSDIRECTORY_OPERATION_NOT_ALLOWED') );
            return false;
        }
            
        $return = parent::save($data);
            
        return $return;
    }
}