<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

/**
 * The EmailMessages model.
 */
class RSDirectoryModelEmailMessages extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'published', 'subject', 'type', 'title', 'id',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get filtering states.
        $search = $this->getState('filter.search');
        $type = $this->getState('filter.type');
        $published = $this->getState('filter.published');
            
        $query = $db->getQuery(true)
               ->select( $db->qn('em') . '.*, ' . $db->qn('c.title') )
               ->from( $db->qn('#__rsdirectory_email_messages', 'em') )
               ->join( 'LEFT', $db->qn('#__categories', 'c') . ' ON ' . $db->qn('em.category_id') . ' = ' . $db->qn('c.id') );
              
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $where = array(
                $db->qn('em.subject') . " LIKE $search",
                $db->qn('em.text') . " LIKE $search",
                $db->qn('c.title') . " LIKE $search",
            );
                
            $query->where( '(' .implode(' OR ', $where) . ')' );
        }
            
        if ($type)
        {
            $query->where( '(' . $db->qn('em.type') . ' = ' . $db->q($type) . ')' );    
        }        
            
        if ($published != '')
        {
            $query->where( '(' . $db->qn('em.published') . ' = ' . $db->q($published) . ')' );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
            
        $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );
            
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $this->setState( 'filter.search', $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search') );
        $this->setState( 'filter.type', $this->getUserStateFromRequest("$this->context.filter.type", 'filter_type') );
        $this->setState( 'filter.published', $this->getUserStateFromRequest("$this->context.filter.type", 'filter_published') );
            
        // List state information.
        parent::populateState('id', 'asc');
    }
}