<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('checkboxes');

/**
 * Custom fields list.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldCustomFieldsList extends JFormFieldCheckboxes
{
    /**
     * The form field type.
     *
     * @var string
     */
    protected $type = 'customfieldslist';

    /**
     * Method to get the field options.
     *
     * @return  array  The field option objects.
     *
     */
    protected function getOptions()
    {
        // Get DBO.
        $db = JFactory::getDbo();

        // Get the form id.
        $form_id = JFactory::getApplication()->input->getInt('id');

        $options = array();

        $where = array(
            $db->qn('ff.form_id') . ' = ' . $db->q($form_id),
            $db->qn('ft.core') . ' = ' . $db->q(0),
            $db->qn('ft.expect_value') . ' = ' . $db->q(1),
            $db->qn('ft.create_column') . ' = ' . $db->q(1),
        );

        $query = $db->getQuery(true)
            ->select( $db->qn('f') . '.*' )
            ->from( $db->qn('#__rsdirectory_fields', 'f') )
            ->innerJoin( $db->qn('#__rsdirectory_forms_fields', 'ff') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('ff.field_id') )
            ->innerJoin( $db->qn('#__rsdirectory_field_types', 'ft') . ' ON ' . $db->qn('f.field_type_id') . ' = ' . $db->qn('ft.id') )
            ->where($where);

        $db->setQuery($query);

        if ($results = $db->loadObjectList()) {
            foreach ($results as $option) {

                if (!$option->published)
                {
                    $option->name .= ' <span class="'.RsdirectoryAdapterGrid::badgeClass('warning').'">' . JText::_('JUNPUBLISHED') . '</span>';
                }
                $tmp = JHtml::_('select.option', $option->id, $option->name);

                $tmp->checked = false;
                if ($this->value && is_array($this->value) && in_array($option->id, $this->value)) {
                    $tmp->checked = true;
                }

                // Add the option object to the result set.
                $options[] = $tmp;
            }

            if ($options) {
                $this->hasValues = true;
            }
        }

        reset($options);

        return $options;
    }

    /**
     * Method to get the field input markup.
     *
     * @access protected
     * @return string The field input markup.
     */
    protected function getInput()
    {
        // it is a getter
        $options = $this->options;
        if (empty($options))
        {
            return JText::_('COM_RSDIRECTORY_NO_CUSTOM_FORM_FIELDS');
        }
        else
        {
            $input = parent::getInput();

            if (RSDirectoryHelper::isJ4())
            {
                $input = str_replace(' form-check-inline', '', $input);
            }
            return $input;
        }
    }
}