<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('list');

/**
 * Entry Category Select options field.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldEntryCategory extends JFormFieldList
{
    /**
     * A flexible category list that respects access controls
     *
     * @var string
     */
    public $type = 'entrycategory';

    /**
	 * Method to get the field input markup for a combo box field.
	 *
	 * @access protected
	 * 
	 * @return string The field input markup.
	 */
	protected function getInput()
	{
        // Initialize the options array.
        $options = $this->getCategories();
		
		$app = JFactory::getApplication();
		
		// Frontend must be treated differently
		if ($app->isClient('site')) {
			// Is this an Add Entry menu item with a category added?
			if (($item = $app->getMenu()->getActive()) && ($catid = $item->getParams()->get('catid')) && $item->component == 'com_rsdirectory') {
				// Get current category
				if ($category = $this->getCategory($catid)) {
					// Parent item with subcategories
					if ($category->hasChildren()) {
						$options = $this->getCategories($catid);
					} else {
						// Just this category then, no need for a dropdown here
						return '<strong>'.$this->escape($category->title).'</strong><input type="hidden" name="'.$this->name.'" value="'.$category->id.'" id="'.$this->id.'" />';
					}
				}
			}
		}
		
		// Editing a current entry
		if ($this->value && $app->input->getInt('id')) {
			// Get current category
			$category = $this->getCategory($this->value, true);
			// If it's unpublished, allow it to remain selected
			if (!$category->published) {
				$options[] = (object) array(
					'value' 	=> $category->id,
					'text'		=> str_repeat('- ', $category->level - 1) . '['. $category->title . ']'
				);
			}
		}
            
        // Initialize variables.
		$html = array();
		$attr = '';
			
		// Initialize some field attributes.
		$attr .= empty($this->class) ? '' : ' class="' . $this->class . '"';
		$attr .= $this->disabled ? ' disabled' : '';
		$attr .= $this->size ? ' size="' . $this->size . '"' : '';
		$attr .= $this->multiple ? ' multiple' : '';
		$attr .= $this->required ? ' required aria-required="true"' : '';
		$attr .= $this->autofocus ? ' autofocus' : '';
		$attr .= ' data-current-category-id="' . $this->value . '"';
			
		// Initialize JavaScript field attributes.
		$attr .= empty($this->onchange) ? '' : ' onchange="' . $this->onchange . '"';
		$attr .= empty($this->onclick) ? '' : ' onclick="' . $this->onclick . '"';

		if (RSDirectoryHelper::isJ4())
		{
			$html[] = '<div class="row g-2"><div class="col-9">';
		}
		// Add the select field.	
		$html[] = JHtml::_( 'select.genericlist', $options, $this->name, array('list.attr' => $attr, 'list.select' => $this->value, 'id' => $this->id) );

		if (RSDirectoryHelper::isJ4())
		{
			$html[] = '</div><div class="col-3 d-grid">';
		}

		// Add the button (if requested and if there are multiple options to select from)
		if ((count($options) > 1) && ( !isset($this->element['change-button']) || $this->element['change-button'] == 'true' ))
		{
			$html[] = ' <button id="' . $this->id . '_change" class="' . RSDirectoryHelper::getTooltipClass() . ' btn btn-light" title="' . RSDirectoryHelper::getTooltipText( JText::_('COM_RSDIRECTORY_CHANGE_CATEGORY_DESC') ) . '" disabled="disabled">' . JText::_('COM_RSDIRECTORY_CHANGE_CATEGORY_LABEL') . '</button>';
		}
			
		// Add loader.
		$html[] = ' '.JHtml::_('image', 'com_rsdirectory/loader.gif', '', 'id="' . $this->id . '_loader" class="hide" width="16" height="16"', true);

		if (RSDirectoryHelper::isJ4())
		{
			$html[] = '</div></div>';
		}

		return implode($html);
    }
	
	protected function getCategories($id = 'root') {
		$options = array(
			'extension' 	=> 'com_rsdirectory',
			'table' 		=> '#__rsdirectory_entries',
			'field' 		=> 'category_id',
			'statefield' 	=> 'published',
			'countItems' 	=> false
		);

		// Create a new JCategories object.
		$categories = new JCategories($options);
		
		if ($root = $categories->get($id)) {
			if ($children = $root->getChildren()) {				
				return $this->getCategoriesOptions($children);
			}
		}
		
		return array();
	}
	
	protected function getCategoriesOptions($categories) {
		$ids = array();

		foreach ($categories as $category) {
			$option = (object) array(
				'value' 	=> $category->id,
				'text'		=> str_repeat('- ', $category->level - 1) . $category->title,
				'disable' 	=> $category->hasChildren()
			);
			$ids[] = $option;
			
			if ($category->hasChildren()) {
				$ids = array_merge($ids, $this->getCategoriesOptions($category->getChildren()));
			}
		}
		
		return $ids;
	}
	
	protected function getCategory($id, $db = false) {
		if ($db) {
			$category = JTable::getInstance('Category', 'JTable');
			if ($category->load($id)) {
				return $category;
			}
			
			return false;
		}
		return RSDirectoryHelper::getCategory($id);
	}
	
	protected function escape($string) {
		return htmlentities($string, ENT_COMPAT, 'utf-8');
	}
}