<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('checkboxes');

/**
 * Fields List.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldFieldsList extends JFormFieldCheckboxes
{
    /**
     * The form field type.
     *
     * @var string
     */
    protected $type = 'fieldslist';


    /**
     * Method to get the field options.
     *
     * @return  array  The field option objects.
     *
     */
    protected function getOptions()
    {
        // Get DBO.
        $db = JFactory::getDbo();

        $options = array();

        $select = array(
            $db->qn('f') . '.*',
            $db->qn('ft.all_forms'),
        );

        $query = $db->getQuery(true)
            ->select($select)
            ->from( $db->qn('#__rsdirectory_fields', 'f') )
            ->innerJoin( $db->qn('#__rsdirectory_field_types', 'ft') . ' ON ' . $db->qn('f.field_type_id') . ' = ' . $db->qn('ft.id') );

        $form_id = JFactory::getApplication()->input->getInt('id');

        // Keep the form's fields ordering if we are existing form.
        if ($form_id)
        {
            $query->leftJoin( $db->qn('#__rsdirectory_forms_fields', 'ff') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('ff.field_id') . ' AND ' . $db->qn('ff.form_id') . ' = ' . $db->q($form_id) );

            // Put the non-null values 1st and then the null values.
            $query->order( $db->qn('ff.ordering') . ' IS NULL, ' . $db->qn('ff.ordering') );
        }

        // Also order the fields by id.
        $query->order( $db->qn('f.id') );

        $db->setQuery($query);

        if ($results = $db->loadObjectList()) {
            foreach ($results as $option) {

                if (!$option->published)
                {
                    $option->name .= ' <span class="'.RsdirectoryAdapterGrid::badgeClass('warning').'">' . JText::_('JUNPUBLISHED') . '</span>';
                }
                $tmp = JHtml::_('select.option', $option->id, $option->name);

                $tmp->checked = false;
                if ($this->value && is_array($this->value) && in_array($option->id, $this->value)) {
                    $tmp->checked = true;
                }

                // Add the option object to the result set.
                $options[] = $tmp;
            }

            if ($options) {
                $this->hasValues = true;
            }
        }

        reset($options);

        return $options;
    }

    /**
     * Method to get the field input markup.
     *
     * @access protected
     * @return string The field input markup.
     */
    protected function getInput()
    {

        $input = parent::getInput();

        if (RSDirectoryHelper::isJ4())
        {
            $input = str_replace(' form-check-inline', '', $input);
        }

        return $input;

    }
}