<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Support a modal entry picker.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldModal_Entry extends JFormField
{
    /**
     * The form field type.
     *
     * @var string
     */
    protected $type = 'Modal_Entry';

    /**
     * Method to get the field input markup.
     *
     * @access protected
     *
     * @return string The field input markup.
     */
    protected function getInput()
    {
        // Build the script.
        $script = array();
        $script[] = 'function jSelectEntry_' . $this->id . '(id, title, catid, object) {';
        $script[] = 'document.getElementById("' . $this->id . '_id").value = id;';
        $script[] = 'document.getElementById("' . $this->id . '_name").value = title;';
        if (RSDirectoryHelper::isJ4())
        {
            $script[] = 'var rsdir_modal_element = document.getElementById(\'modal_entry_'. $this->id.'\');';
            $script[] = 'var rsdir_modal = bootstrap.Modal.getInstance(rsdir_modal_element);';
            $script[] = 'rsdir_modal.hide();';
        }
        else{
            $script[] = 'jQuery(\'#modal_entry_'. $this->id.'\').modal(\'hide\');';//modal close
        }

        $script[] = '}';

        // Add the script to the document head.
        JFactory::getDocument()->addScriptDeclaration( implode("\n", $script) );

        // Setup variables for display.
        $html = array();
        $link = 'index.php?option=com_rsdirectory&amp;view=entries&amp;layout=modal&amp;tmpl=component&amp;function=jSelectEntry_' . $this->id;

        // Get DBO.
        $db = JFactory::getDBO();

        $query = $db->getQuery(true)
            ->select( $db->qn('title') )
            ->from( $db->qn('#__rsdirectory_entries') )
            ->where( $db->qn('id') . ' = ' . $db->q($this->value) );

        $db->setQuery($query);

        try
        {
            $title = $db->loadResult();
        }
        catch (RuntimeException $e)
        {
            JFactory::getApplication()->enqueueMessage($e->getMessage(), 'warning');
        }

        if ( empty($title) )
        {
            $title = JText::_('COM_RSDIRECTORY_SELECT_AN_ENTRY');
        }

        $title = htmlspecialchars($title, ENT_QUOTES, 'UTF-8');

        // The current user display field.
        $input  = '<input type="text" class="input-medium'.(RSDirectoryHelper::isJ4() ? ' form-control' : '').'" id="' . $this->id . '_name" value="' . $title . '" disabled="disabled" size="35" />';
        $append = '<a class="btn'.(RSDirectoryHelper::isJ4() ? ' btn-primary' : '').'" data'.(RSDirectoryHelper::isJ4() ? '-bs' : '').'-toggle="modal" role="button" title="' . JText::_('COM_RSDIRECTORY_CHANGE_ENTRY') . '"  href="#modal_entry_'.$this->id.'"><i class="icon-file"></i> ' . JText::_('JSELECT') . '</a>';
        $html[] = RsdirectoryAdapterGrid::inputAppend($input, $append, true);



        // The active entry id field.
        $value = 0 == (int) $this->value ? '' : (int) $this->value;

        // class='required' for client side validation.
        $class = $this->required ? ' class="required modal-value"' : '';

        // Build the modal
        $html[] = JHtml::_('bootstrap.renderModal',
            'modal_entry_'. $this->id,
            array(
                'title'       => JText::_('COM_RSDIRECTORY_CHANGE_ENTRY'),
                'url' 	      =>  $link . '&amp;' . JSession::getFormToken() . '=1',
                'height'      => 500,
                'backdrop'    => 'static',
                'closeButton' => true,
                'footer'      => '<a class="btn" data'.(RSDirectoryHelper::isJ4() ? '-bs' : '').'-dismiss="modal">' . JText::_('JCANCEL') . '</a>'
            )
        );

        $html[] = '<input type="hidden" id="' . $this->id . '_id"' . $class . ' name="' . $this->name . '" value="' . $value . '" />';

        return implode("\n", $html);
    }

    /**
     * Method to get the field label markup.
     *
     * @access protected
     *
     * @return  string  The field label markup.
     */
    protected function getLabel() {
        $this->id .= '_id';

        return parent::getLabel();
    }
}
