<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * The Form model.
 */
class RSDirectoryModelForm extends JModelAdmin
{
    /**
     * Method to get a table object, load it if necessary.
     * 
     * @access public
     * 
     * @param string $type
     * @param string $prefix
     * @param array $config
     * 
     * @return object
     */
    public function getTable( $type = 'Form', $prefix = 'RSDirectoryTable', $config = array() )
    {
        return JTable::getInstance($type, $prefix, $config);
    }
        
    /**
     * Method for getting the form from the model.
     *
     * @access public
     * 
     * @param array $data
     * @param bool $loadData
     * 
     * @return mixed
     */
    public function getForm( $data = array(), $loadData = true )
    {
        // Get the form.
        $form = $this->loadForm( 'com_rsdirectory.form', 'form', array('control' => 'jform', 'load_data' => $loadData) );
            
        if ( empty($form) )
            return false;
            
        return $form;
    }
        
    /**
     * Method to get the data that should be injected in the form.
     *
     * @access protected
     * 
     * @return array
     */
    protected function loadFormData()
    {
        // Check for data in the session.
        $data = JFactory::getApplication()->getUserState('com_rsdirectory.edit.form.data');
            
        return $data ? $data : $this->getItem();
    }
        
    /**
     * Method to get a single record.
     *
     * @access public
     *
     * @param int $pk The id of the primary key.
     *
     * @return mixed Object on success, false on failure.
     */
    public function getItem($pk = null)
    {
        // Get the id of the primary key.
        $pk = $pk ? $pk : (int) $this->getState( $this->getName() . '.id' );
            
        $item = parent::getItem($pk);
            
        if ($pk)
        {
            // Get DBO.
            $db = JFactory::getDbo();
                
            // Get the form fields ids.
            $query = $db->getQuery(true)
                   ->select( $db->qn('field_id') )
                   ->from( $db->qn('#__rsdirectory_forms_fields') )
                   ->where( $db->qn('form_id') . ' = ' . $db->q($pk) );
                  
            $db->setQuery($query);
                
            $item->form_fields = $db->loadColumn();
			
			// must be string format so that checkboxes can be checked
            $item->form_fields = array_map('strval', $item->form_fields);
                
            // Get the custom form fields ids.
            $query = $db->getQuery(true)
                   ->select( $db->qn('field_id') )
                   ->from( $db->qn('#__rsdirectory_forms_custom_fields') )
                   ->where( $db->qn('form_id') . ' = ' . $db->q($pk) );
                   
            $db->setQuery($query);
                
            $item->listing_detail_custom_fields = $db->loadColumn();
			
			// must be string format so that checkboxes can be checked
            $item->listing_detail_custom_fields = array_map('strval', $item->listing_detail_custom_fields);
        }
            
        return $item;
    }

    public function validate($form, $data, $group = null)
    {
        $validate = parent::validate($form, $data, $group);


        //check the snippet value
        if (!empty($data['entry_snippet']))
        {

            $test_json = json_decode($data['entry_snippet']);
            if (empty($test_json)) {
                JFactory::getApplication()->enqueueMessage('COM_RSDIRECTORY_SNIPPET_NOT_OK', 'error');
                return false;
            }
        }

        return $validate;
    }

    /**
     * Save the field.
     *
     * @access public
     * 
     * @param array $data
     * 
     * @return bool
     */
    public function save($data)
    {
        // Exit the function if the data array is invalid.
        if (!$data)
            return false;
            
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get mainframe.
        $app = JFactory::getApplication();
            
        // Get the JInput object.
        $jinput = $app->input;
            
        // Get the field id.
        $id = $jinput->getInt('id');
          
          
        // Get the form table.
        $form = $this->getTable();
            
        // current form fields
        $current_form_fields = array();
        if ( $id && $jinput->get('task') != 'save2copy' )
        {
            // Load field object.
            $form->load($id);

            // get the current fields assigned
            $query = $db->getQuery(true)
                ->select($db->qn('field_id'))
                ->from($db->qn('#__rsdirectory_forms_fields'))
                ->where( $db->qn('form_id') . ' = ' . $db->q($id) );

            $db->setQuery($query);
            $current_form_fields = $db->loadColumn();
                
            // Delete all the form fields for this form.
            $query = $db->getQuery(true)
                   ->delete( $db->qn('#__rsdirectory_forms_fields') )
                   ->where( $db->qn('form_id') . ' = ' . $db->q($id) );
                   
            $db->setQuery($query);
            $db->execute();
                
            // Delete all the custom form fields for this form.
            $query = $db->getQuery(true)
                   ->delete( $db->qn('#__rsdirectory_forms_custom_fields') )
                   ->where( $db->qn('form_id') . ' = ' . $db->q($id) );
                   
            $db->setQuery($query);
            $db->execute();
        }
            
            
        if (!$form->save($data))
        {
            $this->setError(JText::_('COM_RSDIRECTORY_GENERAL_SQL_ERROR'));
            return false;
        }
            
        $id = $form->id;
            
        // Remember the id of the newly inserted form.
        $this->setState( $this->getName() . '.id', $id );
           
        if ( empty($data['form_fields']) )
        {
            $data['form_fields'] = array();
        }
            
        // Add the all_forms fields to the form fields array.
        $query = $db->getQuery(true)
               ->select( $db->qn('f.id') )
               ->from( $db->qn('#__rsdirectory_fields', 'f') )
               ->innerJoin( $db->qn('#__rsdirectory_field_types', 'ft') . ' ON ' . $db->qn('f.field_type_id') . ' = ' . $db->qn('ft.id') )
               ->where( $db->qn('ft.all_forms') . ' = ' . $db->q(1) );
               
        $db->setQuery($query);
            
        if ( $all_forms_field_ids = $db->loadColumn() )
        {
            $data['form_fields'] = array_merge($data['form_fields'], $all_forms_field_ids);
        }
            
        if ( !empty($data['form_fields']) )
        {
            $data['form_fields'] = array_unique($data['form_fields']);
                
            // Insert columns.
            $columns = array('form_id', 'field_id', 'ordering');
                
            // Insert the form fields in the database.
            $query = $db->getQuery(true)
                   ->insert( $db->qn('#__rsdirectory_forms_fields') )
                   ->columns( $db->qn($columns) );
                    
            foreach ($data['form_fields'] as $i => $field_id)
            {
                $values = array(
                    $db->q($id),
                    $db->q($field_id),
                    $db->q($i + 1),
                );
                    
                $query->values( implode(',', $values) );
            }
                
            $db->setQuery($query);
            $db->execute();

            foreach ($data['form_fields'] as $field_id)
            {
                // log the assignation of the fields
                if (!in_array($field_id, $current_form_fields))
                {
                    // Do not log the all_forms fields, they are always added
                    if (!empty($all_forms_field_ids) && in_array($field_id, $all_forms_field_ids)) {
                        continue;
                    }

                    $field = JTable::getInstance('Field', 'RSDirectoryTable');
                    $field->load($field_id);

                    if ($field)
                    {
                        $field_type = JTable::getInstance('FieldType', 'RSDirectoryTable');
                        $field_type->load($field->field_type_id);

                        if ($field_type)
                        {
                            // log the form assignation
                            $log_params = array(
                                'type'          => 'field_assign',
                                'field_id'      => $field_id,
                                'field_type'    => $field_type->type,
                                'field_name'    => $field->name,
                                'form_id'       => $id,
                                'form_title'    => $form->title
                            );
                            RSDirectoryHelper::logAction($log_params);
                        }
                    }
                }
            }

            $unassigned_fields = array_diff($current_form_fields, $data['form_fields']);

            if (!empty($unassigned_fields))
            {
                foreach ($unassigned_fields as $field_id)
                {
                    $field = JTable::getInstance('Field', 'RSDirectoryTable');
                    $field->load($field_id);

                    if ($field)
                    {
                        $field_type = JTable::getInstance('FieldType', 'RSDirectoryTable');
                        $field_type->load($field->field_type_id);

                        if ($field_type)
                        {
                            // log the form assignation
                            $log_params = array(
                                'type'          => 'field_unassign',
                                'field_id'      => $field_id,
                                'field_type'    => $field_type->type,
                                'field_name'    => $field->name,
                                'form_id'       => $id,
                                'form_title'    => $form->title
                            );
                            RSDirectoryHelper::logAction($log_params);
                        }
                    }
                }
            }
        }
            
        if ( !empty($data['listing_detail_custom_fields']) )
        {
            $data['listing_detail_custom_fields'] = array_unique($data['listing_detail_custom_fields']);
                
            // Insert columns.
            $columns = array('form_id', 'field_id');
                
            // Insert the custom form fields in the database.
            $query = $db->getQuery(true)
                   ->insert( $db->qn('#__rsdirectory_forms_custom_fields') )
                   ->columns( $db->qn($columns) );
                    
            foreach ($data['listing_detail_custom_fields'] as $field_id)
            {
                $values = array(
                    $db->q($id),
                    $db->q($field_id),
                );
                    
                $query->values( implode(',', $values) );
            }
                
            $db->setQuery($query);
            $db->execute();
        }
            
        // Clean the session data.
        $app->setUserState('com_rsdirectory.edit.form.data', null);
            
        return true;
    }
        
    /**
     * Get RSTabs.
     *
     * @access public
     * 
     * @return RSTabs
     */
    public function getRSTabs()
    {
        return new RsdirectoryAdapterTabs('com-rsdirectory-form');
    }
}