<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Forms model.
 */
class RSDirectoryModelForms extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'title', 'id',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get filtering states.
        $search = $this->getState('filter.search');
            
        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_forms') );
              
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $query->where( $db->qn('title') . " LIKE $search" );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
            
        $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );
            
        return $query;
    }
        
    /**
     * Get forms options.
     *
     * @access public
     * @return mixed
     */
    public function getFormsOptions()
    {
        // Get forms object list.
        $forms = RSDirectoryHelper::getForms();
            
        // Initialize the options array.
        $options = array();
            
        if ($forms)
        {
            foreach ($forms as $form)
            {
                $options[] = (object)array(
                    'value' => $form->id,
                    'text' => $form->title,
                );
            }
        }
            
        return $options;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $this->setState( 'filter.search', $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search') );
            
        // List state information.
        parent::populateState('id', 'asc');
    }
}