<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Ratings model.
 */
class RSDirectoryModelRatings extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'published',
                'title',
                'entry_author',
                'score',
                'subject',
                'review_author',
                'name',
                'created_time',
                'id',
	            'rating',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get entry author display info (name, username, email..).
		$author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');
            
        $select = array(
            $db->qn('r') . '.*',
            $db->qn('e.title'),
            $db->qn('u.id', 'entry_author_id'),
            $db->qn("u.$author", 'entry_author'),
            $db->qn('ru.id', 'review_author_id'),
            $db->qn("ru.$author", 'review_author'),
        );
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_reviews', 'r') )
               ->leftJoin( $db->qn('#__rsdirectory_entries', 'e') . ' ON ' . $db->qn('r.entry_id') . ' = ' . $db->qn('e.id') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id') )
               ->leftJoin( $db->qn('#__users', 'ru') . ' ON ' . $db->qn('r.user_id') . ' = ' . $db->qn('ru.id') );
               
        // Filter by status.  
        $published = $this->getState('filter.published');
            
        if ($published !== '')
        {
            $query->where( $db->qn('r.published') . ' = ' . $db->q($published) );
        }
        
        // Filter by rating.  
        $rating = $this->getState('filter.rating');
            
        if ($rating !== '')
        {
            $query->where( $db->qn('r.score') . ' = ' . $db->q($rating) );
        }
            
        // Filter by search.
        $search = $this->getState('filter.search');
            
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $condition = array(
                $db->qn('e.title') . " LIKE $search",
                $db->qn('r.subject') . " LIKE $search",
                $db->qn('r.review') . " LIKE $search",
                $db->qn('u.email') . " LIKE $search",
                $db->qn('u.name') . " LIKE $search",
                $db->qn('r.email') . " LIKE $search",
                $db->qn('r.name') . " LIKE $search",
            );
                
            $query->where( '(' . implode(' OR ', $condition) . ')' );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
            
        $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );
            
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * 
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $search = $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search');
        $this->setState('filter.search', $search);
            
        $published = $this->getUserStateFromRequest("$this->context.filter.published", 'filter_published', '');
        $this->setState('filter.published', $published);
            
        $rating = $this->getUserStateFromRequest("$this->context.filter.rating", 'filter_rating', '');
        $this->setState('filter.rating', $rating);
            
        // List state information.
        parent::populateState('id', 'asc');
    }
}