<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Reported Entries model.
 */
class RSDirectoryModelReportedEntries extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'published',
                'status',
                'title',
                'entry_author',
                'reason',
                'report_author',
                'name',
                'email',
                'created_time',
                'entry_id',
                'entries_author_id',
                'reports_author_id',
                'id',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get entry author display info (name, username, email..).
		$author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');
            
        $select = array(
            $db->qn('r') . '.*',
            $db->qn('e.title'),
            $db->qn("u.$author", 'entry_author'),
            $db->qn('u.id', 'entry_author_id'),
            $db->qn("ru.$author", 'report_author'),
            $db->qn('ru.id', 'report_author_id'),
        );
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_entries_reported', 'r') )
               ->leftJoin( $db->qn('#__rsdirectory_entries', 'e') . ' ON ' . $db->qn('r.entry_id') . ' = ' . $db->qn('e.id') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id') )
               ->leftJoin( $db->qn('#__users', 'ru') . ' ON ' . $db->qn('r.user_id') . ' = ' . $db->qn('ru.id') );
               
        // Filter by status.  
        $status = $this->getState('filter.status');
            
        if ($status !== '')
        {
            $query->where( $db->qn('r.published') . ' = ' . $db->q($status) );
        }
            
        // Filter by entry.  
        $entry_id = $this->getState('filter.entry_id');
            
        if ($entry_id !== '')
        {
            $query->where( $db->qn('r.entry_id') . ' = ' . $db->q($entry_id) );
        }
            
        // Filter by entries author.  
        $entries_author_id = $this->getState('filter.entries_author_id');
            
        if ($entries_author_id !== '')
        {
            $query->where( $db->qn('e.user_id') . ' = ' . $db->q($entries_author_id) );
        }
            
        // Filter by reason.  
        $reason = $this->getState('filter.reason');
            
        if ($reason !== '')
        {
            $query->where( $db->qn('r.reason') . ' = ' . $db->q($reason) );
        }
            
        // Filter by reports author.  
        $reports_author_id = $this->getState('filter.reports_author_id');
            
        if ($reports_author_id !== '')
        {
            $query->where( $db->qn('r.user_id') . ' = ' . $db->q($reports_author_id) );
        }
            
        // Filter by email author.  
        $email = $this->getState('filter.email');
            
        if ($email !== '')
        {
            $query->where( $db->qn('r.email') . ' = ' . $db->q($email) );
        }
            
        // Filter by search.
        $search = $this->getState('filter.search');
            
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $condition = array(
                $db->qn('e.title') . " LIKE $search",
                $db->qn('r.name') . " LIKE $search",
                $db->qn('r.email') . " LIKE $search",
                $db->qn('r.message') . " LIKE $search",
                $db->qn('r.reason') . " LIKE $search",
            );
                
            $query->where( '(' . implode(' OR ', $condition) . ')' );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
            
        $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );
            
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * 
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $search = $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search');
        $this->setState('filter.search', $search);
            
        $status = $this->getUserStateFromRequest("$this->context.filter.status", 'filter_status', '');
        $this->setState('filter.status', $status);
            
        $entry_id = $this->getUserStateFromRequest("$this->context.filter.entry_id", 'filter_entry_id', '');
        $this->setState('filter.entry_id', $entry_id);
            
        $entries_author_id = $this->getUserStateFromRequest("$this->context.filter.entries_author_id", 'filter_entries_author_id', '');
        $this->setState('filter.entries_author_id', $entries_author_id);
            
        $reason = $this->getUserStateFromRequest("$this->context.filter.reason", 'filter_reason', '');
        $this->setState('filter.reason', $reason);
            
        $reports_author_id = $this->getUserStateFromRequest("$this->context.filter.reports_author_id", 'filter_reports_author_id', '');
        $this->setState('filter.reports_author_id', $reports_author_id);
            
        $email = $this->getUserStateFromRequest("$this->context.filter.email", 'filter_email', '');
        $this->setState('filter.email', $email);
            
        // List state information.
        parent::populateState('id', 'asc');
    }
}