<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Transaction model.
 */
class RSDirectoryModelTransaction extends JModelAdmin
{
    /**
     * Method to get a table object, load it if necessary.
     * 
     * @access public
     * 
     * @param string $type
     * @param string $prefix
     * @param array $config
     * 
     * @return object
     */
    public function getTable( $type = 'Transaction', $prefix = 'RSDirectoryTable', $config = array() )
    {
        return JTable::getInstance($type, $prefix, $config);
    }
        
    /**
     * Method for getting the form from the model.
     *
     * @access public
     * 
     * @param array $data
     * @param bool $loadData
     * 
     * @return bool
     */
    public function getForm( $data = array(), $loadData = true )
    {
        return false;
    }
        
    /**
     * Method to get a single record.
     *
     * @access public
     *
     * @param int $pk The id of the primary key.
     *
     * @return mixed Object on success, false on failure.
     */
    public function getItem($pk = null)
    {
        // Get the id of the primary key.
        $pk = $pk ? $pk : (int) $this->getState( $this->getName() . '.id' );
            
        if (!$pk)
            return false;
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        // Get entry author display info (name, username, email..).
		$author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');
            
        $select = array(
            $db->qn('t') . '.*',
            $db->qn("u.$author", 'user'),
            $db->qn('e.id', 'entry_id'),
            $db->qn('e.title', 'entry_title'),
        );
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_users_transactions', 't') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('t.user_id') . ' = ' . $db->qn('u.id') )
               ->leftJoin( $db->qn('#__rsdirectory_entries', 'e') . ' ON ' . $db->qn('t.entry_id') . ' = ' . $db->qn('e.id') )
               ->where( $db->qn('t.id') . ' = ' . $db->q($pk) );
               
        $db->setQuery($query);
            
        return $db->loadObject();
    }
	
	/**
     * Method to finalize the transaction.
     *
     * @access public
     *
     * @param array $pks The ids of the primary key.
     * @param int $value The new state.
     *
     * @return bool.
     */
    public function publish(&$pks, $value = 1) {
        // check the joomla version
        $jversion = new JVersion();

        // If is later than 3.9.1 need to rewrite the publish function
        if (!$jversion->isCompatible('3.9.1')) {
            return parent::publish($pks, $value);
        }

        // If is 3.9.1 or later
        $user = JFactory::getUser();
        $table = $this->getTable();
        $pks = (array) $pks;

        // Check if there are items to change
        if (!count($pks))
        {
            return true;
        }

        // Attempt to change the state of the records.
        if (!$table->publish($pks, $value, $user->get('id')))
        {
            $this->setError($table->getError());

            return false;
        }
		
        return true;
    }
}
    