<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Transactions model.
 */
class RSDirectoryModelTransactions extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'credit_title',
                'user',
                'entry_title',
                'entry_paid',
                'gateway',
                'currency',
                'price',
                'tax',
                'total',
                'credits',
                'status',
                'date_created',
                'date_finalized',
                'id',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get entry author display info (name, username, email..).
		$author = RSDirectoryConfig::getInstance()->get('entry_author', 'name');
            
        $select = array(
            $db->qn('t') . '.*',
            $db->qn("u.$author", 'user'),
            $db->qn('e.id', 'entry_id'),
            $db->qn('e.title', 'entry_title'),
            $db->qn('e.paid', 'entry_paid'),
        );
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_users_transactions', 't') )
               ->leftJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('t.user_id') . ' = ' . $db->qn('u.id') )
               ->leftJoin( $db->qn('#__rsdirectory_entries', 'e') . ' ON ' . $db->qn('t.entry_id') . ' = ' . $db->qn('e.id') );
               
        // Filter by credit package.  
        if ( $credit_package = $this->getState('filter.credit_package') )
        {
            $query->where( $db->qn('t.credit_title') . ' = ' . $db->q($credit_package) );
        }
            
        // Filter by user.  
        if ( $user_id = $this->getState('filter.user_id') )
        {
            $query->where( $db->qn('t.user_id') . ' = ' . $db->q($user_id) );
        }
            
        // Filter by entry.  
        if ( $entry_id = $this->getState('filter.entry_id') )
        {
            $query->where( $db->qn('t.entry_id') . ' = ' . $db->q($entry_id) );
        }
            
        // Filter by gateway.
        if ( $gateway = $this->getState('filter.gateway') )
        {
            $query->where( $db->qn('t.gateway') . ' = ' . $db->q($gateway) );
        }
            
        // Filter by currency.      
        if ( $currency = $this->getState('filter.currency') )
        {
            $query->where( $db->qn('t.currency') . ' = ' . $db->q($currency) );
        }
            
        // Filter by price.  
        if ( $price = $this->getState('filter.price') )
        {
            $query->where( $db->qn('t.price') . ' = ' . $db->q($price) );
        }
            
        // Filter by credits.      
        if ( $credits = $this->getState('filter.credits') )
        {
            $query->where( $db->qn('t.credits') . ' = ' . $db->q($credits) );
        }
            
        // Filter by status.      
        if ( $status = $this->getState('filter.status') )
        {
            $query->where( $db->qn('t.status') . ' = ' . $db->q($status) );
        }
            
        // Filter by search.
        $search = $this->getState('filter.search');
            
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $conditions = array(
                $db->qn('t.credit_title') . " LIKE $search",
                $db->qn('t.gateway_order_number') . " LIKE $search",
                $db->qn('t.gateway_order_type') . " LIKE $search",
                $db->qn('t.gateway_params') . " LIKE $search",
                $db->qn('t.gateway_log') . " LIKE $search",
                $db->qn('e.title') . " LIKE $search",
            );
                
            $query->where( '(' . implode(' OR ', $conditions) . ')' );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
            
        if ($ordering == 't.credits')
        {
            $query->order( $db->qn($ordering) . ' = 0 ' . $db->escape($direction) . ', ' . $db->qn($ordering) . ' ' . $db->escape($direction) );
        }
        else
        {
            $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );   
        }
            
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * 
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $search = $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search');
        $this->setState('filter.search', $search);
            
        $credit_package = $this->getUserStateFromRequest("$this->context.filter.credit_package", 'filter_credit_package');
        $this->setState('filter.credit_package', $credit_package);
            
        $user_id = $this->getUserStateFromRequest("$this->context.filter.user_id", 'filter_user_id');
        $this->setState('filter.user_id', $user_id);
            
        $entry_id = $this->getUserStateFromRequest("$this->context.filter.entry_id", 'filter_entry_id');
        $this->setState('filter.entry_id', $entry_id);
            
        $gateway = $this->getUserStateFromRequest("$this->context.filter.gateway", 'filter_gateway');
        $this->setState('filter.gateway', $gateway);
            
        $currency = $this->getUserStateFromRequest("$this->context.filter.currency", 'filter_currency');
        $this->setState('filter.currency', $currency);
            
        $price = $this->getUserStateFromRequest("$this->context.filter.price", 'filter_price');
        $this->setState('filter.price', $price);
            
        $credits = $this->getUserStateFromRequest("$this->context.filter.credits", 'filter_credits');
        $this->setState('filter.credits', $credits);
            
        $status = $this->getUserStateFromRequest("$this->context.filter.status", 'filter_status');
        $this->setState('filter.status', $status);
            
        // List state information.
        parent::populateState('t.date_created', 'desc');
    }
}