<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * User model.
 */
class RSDirectoryModelUser extends JModelAdmin
{
    public function getTable($type = 'Users', $prefix = 'RSDirectoryTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Stock method to auto-populate the model state.
     *
     * @access protected
     */
    protected function populateState()
    {
        // Get the pk of the record from the request.
        $pk = JFactory::getApplication()->input->getInt('id');
        $this->setState( $this->getName() . '.id', $pk );
    }

    /**
     * Method for getting the form from the model.
     *
     * @access public
     *
     * @param array $data
     * @param bool $loadData
     *
     * @return mixed
     */
    public function getForm( $data = array(), $loadData = true )
    {
        // Get the form.
        $form = $this->loadForm( 'com_rsdirectory.user', 'user', array('control' => 'jform', 'load_data' => $loadData) );

        if ( empty($form) )
        {
            return false;
        }

        // get the current user
        $current_user_id = JFactory::getApplication()->input->getInt('id');

        // select the saved or default credits of the user
        $available_credits = RSDirectoryCredits::getUserCredits($current_user_id);

        // do not save the "unlimited" value to the credits
        if ($available_credits == 'unlimited')
        {
            $available_credits = $form->getValue('credits');
        }

        $form->setValue('credits', null, $available_credits);
        $form->setValue('user_id', null, $current_user_id);

        return $form;
    }

    /**
     * Method to get the data that should be injected in the form.
     *
     * @access protected
     *
     * @return array
     */
    protected function loadFormData()
    {
        // Check for data in the session.
        $data = JFactory::getApplication()->getUserState('com_rsdirectory.edit.user.data');

        return $data ? $data : $this->getItem();
    }

    /**
     * Method to get a single record.
     *
     * @access public
     *
     * @param int $pk The id of the primary key.
     *
     * @return mixed Object on success, false on failure.
     */
    public function getItem($pk = null)
    {
        // Get the id of the primary key.
        $pk = $pk ? $pk : (int) $this->getState( $this->getName() . '.id' );

        // Get DBO.
        $db = JFactory::getDBO();

        $select = array(
            $db->qn('u.id'),
            $db->qn('u.id', 'user_id'),
            $db->qn('u.name'),
            $db->qn('u.username'),
            $db->qn('uc.credits'),
            $db->qn('uc.unlimited_credits'),
            $db->qn('uc.enable_contact_form'),
            $db->qn('ec.spent_credits'),
            $db->qn('e.entries_count'),
            $db->qn('rev.reviews_count'),
            $db->qn('rep.reports_count'),
            $db->qn('t.transactions_count'),
        );

        // Create the entries subquery.
        $subquery1 = $db->getQuery(true)
            ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('entries_count') )
            ->from( $db->qn('#__rsdirectory_entries') )
            ->group( $db->qn('user_id') );

        // Create the entries credits subquery.
        $subquery2 = $db->getQuery(true)
            ->select( $db->qn('user_id') . ', SUM(' . $db->qn('credits') . ') AS ' . $db->qn('spent_credits') )
            ->from( $db->qn('#__rsdirectory_entries_credits') )
            ->where( $db->qn('free') . ' = ' . $db->q(0) )
            ->where( $db->qn('paid') . ' = ' . $db->q(1) )
            ->group( $db->qn('user_id') );

        // Create the reviews subquery.
        $subquery3 = $db->getQuery(true)
            ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('reviews_count') )
            ->from( $db->qn('#__rsdirectory_reviews') )
            ->group( $db->qn('user_id') );

        // Create the reports subquery.
        $subquery4 = $db->getQuery(true)
            ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('reports_count') )
            ->from( $db->qn('#__rsdirectory_entries_reported') )
            ->group( $db->qn('user_id') );

        // Create the transactions subquery.
        $subquery5 = $db->getQuery(true)
            ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('transactions_count') )
            ->from( $db->qn('#__rsdirectory_users_transactions') )
            ->group( $db->qn('user_id') );

        $query = $db->getQuery(true)
            ->select($select)
            ->from( $db->qn('#__users', 'u') )
            ->leftJoin( $db->qn('#__rsdirectory_users', 'uc') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('uc.user_id') )
            ->leftJoin( '(' . $subquery1 . ') AS ' . $db->qn('e') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('e.user_id') )
            ->leftJoin( '(' . $subquery2 . ') AS ' . $db->qn('ec') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('ec.user_id') )
            ->leftJoin( '(' . $subquery3 . ') AS ' . $db->qn('rev') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('rev.user_id') )
            ->leftJoin( '(' . $subquery4 . ') AS ' . $db->qn('rep') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('rep.user_id') )
            ->leftJoin( '(' . $subquery5 . ') AS ' . $db->qn('t') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('t.user_id') )
            ->where( $db->qn('u.id') . ' = ' . $db->q($pk) );

        $db->setQuery($query);

        return $db->loadObject();
    }

    /**
     * Save the user data.
     *
     * @access public
     *
     * @param array $data
     *
     * @return bool
     */
    public function save($data)
    {
        $result = parent::save($data);

        if ($result)
        {
            // log the edit for the user
            $log_params = array(
                'type' => 'user_edit',
                'user_id' => $data['user_id']
            );
            RSDirectoryHelper::logAction($log_params);
        }

        return $result;
    }

    public function anonymise($id)
    {
        // Don't anonymise current user to avoid issues
        if (JFactory::getUser()->id == $id)
        {
            throw new Exception(JText::_('COM_RSDIRECTORY_CANNOT_ANONYMISE_LOGGED_IN_USER'));
        }

        RSDirectoryHelper::anonymise($id);
    }
}