<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Users model.
 */
class RSDirectoryModelUsers extends JModelList
{
    /**
     * Class constructor.
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'name',
                'username',
                'credits',
                'spent_credits',
                'transactions_count',
                'entries_count',
                'reviews_count',
                'reports_count',
                'id',
                'posted_reports',
                'posted_reviews',
                'posted_entries',
                'has_transactions',
                'has_credits',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        $select = array(
            $db->qn('u.id'),
            $db->qn('u.name'),
            $db->qn('u.username'),
            'IFNULL('.$db->qn('uc.credits').', (SELECT DISTINCT '.$db->qn('ug.initial_credits').' FROM '.$db->qn('#__rsdirectory_groups_relations').' AS '.$db->qn('ugr').'
INNER JOIN '.$db->qn('#__user_usergroup_map').' AS '.$db->qn('ugm').' ON '.$db->qn('ugm.group_id').' =  '.$db->qn('ugr.jgroup_id').'
LEFT JOIN '.$db->qn('#__rsdirectory_groups').' AS '.$db->qn('ug').' ON '.$db->qn('ugr.group_id').' = '.$db->qn('ug.id').'
WHERE '.$db->qn('ugm.user_id').' = '.$db->qn('u.id').' ORDER BY '.$db->qn('ug.initial_credits').' DESC LIMIT 1)) AS '.$db->qn('credits'),
            $db->qn('uc.unlimited_credits'),
            $db->qn('ec.spent_credits'),
            $db->qn('e.entries_count'),
            $db->qn('rev.reviews_count'),
            $db->qn('rep.reports_count'),
            $db->qn('t.transactions_count'),
        );
           
        // Create the entries subquery. 
        $subquery1 = $db->getQuery(true)
                       ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('entries_count') )
                       ->from( $db->qn('#__rsdirectory_entries') )
                       ->group( $db->qn('user_id') );
                       
        // Create the entries credits subquery.
        $subquery2 = $db->getQuery(true)
                   ->select( $db->qn('user_id') . ', SUM(' . $db->qn('credits') . ') AS ' . $db->qn('spent_credits') )
                   ->from( $db->qn('#__rsdirectory_entries_credits') )
                   ->where( $db->qn('free') . ' = ' . $db->q(0) )
                   ->where( $db->qn('paid') . ' = ' . $db->q(1) )
                   ->group( $db->qn('user_id') );
                   
        // Create the reviews subquery.
        $subquery3 = $db->getQuery(true)
                   ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('reviews_count') )
                   ->from( $db->qn('#__rsdirectory_reviews') )
                   ->group( $db->qn('user_id') );
                   
        // Create the reports subquery.
        $subquery4 = $db->getQuery(true)
                   ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('reports_count') )
                   ->from( $db->qn('#__rsdirectory_entries_reported') )
                   ->group( $db->qn('user_id') );
                   
        // Create the transactions subquery.
        $subquery5 = $db->getQuery(true)
                   ->select( $db->qn('user_id') . ', COUNT(*) AS ' . $db->qn('transactions_count') )
                   ->from( $db->qn('#__rsdirectory_users_transactions') )
                   ->group( $db->qn('user_id') );
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__users', 'u') )
               ->leftJoin( $db->qn('#__rsdirectory_users', 'uc') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('uc.user_id') )
               ->leftJoin( '(' . $subquery1 . ') AS ' . $db->qn('e') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('e.user_id') )
               ->leftJoin( '(' . $subquery2 . ') AS ' . $db->qn('ec') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('ec.user_id') )
               ->leftJoin( '(' . $subquery3 . ') AS ' . $db->qn('rev') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('rev.user_id') )
               ->leftJoin( '(' . $subquery4 . ') AS ' . $db->qn('rep') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('rep.user_id') )
               ->leftJoin( '(' . $subquery5 . ') AS ' . $db->qn('t') . ' ON ' . $db->qn('u.id') . ' = ' . $db->qn('t.user_id') );
               
        // Filter by available credits.
        $has_credits = $this->getState('filter.has_credits');
            
        if ($has_credits !== '')
        {
            if ($has_credits)
            {
                $query->having( '(' . $db->qn('credits') . ' > ' . $db->q(0) . ' OR ' . $db->qn('uc.unlimited_credits') . ' = ' . $db->q(1) . ')' );
            }
            else
            {
                $query->having( $db->qn('credits') . ' = ' . $db->q(0) )
                      ->having( '(' . $db->qn('uc.unlimited_credits') . ' = ' . $db->q(0) . ' OR ' . $db->qn('uc.unlimited_credits') . ' IS NULL)' );
            }
        }
            
        // Filter by spent credits.
        $spent_credits = $this->getState('filter.spent_credits');
            
        if ($spent_credits !== '')
        {
            $query->where( $db->qn('ec.spent_credits') . ($spent_credits ? ' > ' . $db->q(0) : ' IS NULL') );
        }
            
        // Filter by transactions.
        $has_transactions = $this->getState('filter.has_transactions');
            
        if ($has_transactions !== '')
        {
            $query->where( $db->qn('t.transactions_count') . ($has_transactions ? ' > ' . $db->q(0) : ' IS NULL') );
        }
            
        // Filter by entries.
        $posted_entries = $this->getState('filter.posted_entries');
            
        if ($posted_entries !== '')
        {
            $query->where( $db->qn('e.entries_count') . ($posted_entries ? ' > ' . $db->q(0) : ' IS NULL') );
        }
            
        // Filter by reviews.
        $posted_reviews = $this->getState('filter.posted_reviews');
            
        if ($posted_reviews !== '')
        {
            $query->where( $db->qn('rev.reviews_count') . ($posted_reviews ? ' > ' . $db->q(0) : ' IS NULL') );
        }
            
        // Filter by reports.
        $posted_reports = $this->getState('filter.posted_reports');
            
        if ($posted_reports !== '')
        {
            $query->where( $db->qn('rep.reports_count') . ($posted_reports ? ' > ' . $db->q(0) : ' IS NULL') );
        }
            
        // Filter by search.
        $search = $this->getState('filter.search');
            
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $condition = array(
                $db->qn('u.name') . " LIKE $search",
                $db->qn('u.username') . " LIKE $search",
            );
                
            $query->where( '(' . implode(' OR ', $condition) . ')' );
        }
            
        $ordering = $this->getState('list.ordering', 'id');
        $direction = $this->getState('list.direction', 'asc');
        
        if ($ordering == 'uc.credits')
        {
            $query->order( $db->qn('uc.unlimited_credits') . ' = 1 ' . $db->escape($direction) . ', ' . $db->qn($ordering) . ' ' . $db->escape($direction) );
        }
        else
        {
            $query->order( $db->qn($ordering) . ' ' . $db->escape($direction) );   
        }
            
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * 
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = 'u.id', $direction = 'asc')
    {
        $search = $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search');
        $this->setState('filter.search', $search);
            
        $has_credits = $this->getUserStateFromRequest("$this->context.filter.has_credits", 'filter_has_credits', '');
        $this->setState('filter.has_credits', $has_credits);
            
        $spent_credits = $this->getUserStateFromRequest("$this->context.filter.spent_credits", 'filter_spent_credits', '');
        $this->setState('filter.spent_credits', $spent_credits);
            
        $has_transactions = $this->getUserStateFromRequest("$this->context.filter.has_transactions", 'filter_has_transactions', '');
        $this->setState('filter.has_transactions', $has_transactions);
            
        $posted_entries = $this->getUserStateFromRequest("$this->context.filter.posted_entries", 'filter_posted_entries', '');
        $this->setState('filter.posted_entries', $posted_entries);
            
        $posted_reviews = $this->getUserStateFromRequest("$this->context.filter.posted_reviews", 'filter_posted_reviews', '');
        $this->setState('filter.posted_reviews', $posted_reviews);
            
        $posted_reports = $this->getUserStateFromRequest("$this->context.filter.posted_reports", 'filter_posted_reports', '');
        $this->setState('filter.posted_reports', $posted_reports);
            
        // List state information.
        parent::populateState($ordering, $direction);
    }
}