<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Email Message table.
 */
class RSDirectoryTableEmailMessage extends JTable
{
    public $description = '';
    public $to_name = '';
    public $to_email = '';
    public $bcc = '';
    public $cc = '';
    public $subject = '';
    public $text = '';

    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object Database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__rsdirectory_email_messages', 'id', $db);
    }

    /**
     * Method to store a row in the database from the JTable instance properties.
     *
     * @param boolean $updateNulls True to update fields even if they are null.
     *
     * @return boolean True on success.
     */
    public function store($updateNulls = false)
    {
        // Get the name of the primary key.
        $k = $this->_tbl_key;

        // check to see if there is an id
        $id = $this->$k;

        if ($id)
        {
            $email = JTable::getInstance('EmailMessage', 'RSDirectoryTable');
            $email->load($id);

            $old_published_state = $email->published;
        }

        $return = parent::store($updateNulls);
        if ($return)
        {
            $state_changed = !$id ? true : false;

            // check to see if the published state has been changed
            if ($id && $old_published_state != $this->published)
            {
                $state_changed = true;
            }

            $new_id = $this->$k;
            // log the create/edit for the form
            $log_params = array(
                'type'              => 'email_message_' . (! $id ? 'create' : 'update'),
                'email_id'          => $new_id,
                'email_description' => $this->description
            );

            RSDirectoryHelper::logAction($log_params);

            // log the published state if its changed
            if ($state_changed)
            {
                // log the create/edit for the form
                $log_params = array(
                    'type'              => 'email_message_' . (! $this->published ? 'unpublished' : 'published'),
                    'email_id'          => $new_id,
                    'email_description' => $this->description
                );

                RSDirectoryHelper::logAction($log_params);
            }
        }

        return $return;
    }


    /**
     * Method to set the publishing state for a row or list of rows in the database table.
     *
     * @param mixed $pks
     * @param integer $state
     * @param integer $userId
     *
     * @return boolean
     */
    public function publish($pks = null, $state = 1, $userId = 0)
    {
        $k = $this->_tbl_key;

        // Sanitize input.
        $pks = array_map('intval', $pks);

        // If there are no primary keys set check to see if the instance key is set.
        if ( empty($pks) )
        {
            if (!$this->$k)
                return false;

            $pks = array($this->$k);
        }

        $return = parent::publish($pks, $state, $userId);

        if ($return) {
            foreach  ($pks as $pk) {
                $email = JTable::getInstance('EmailMessage', 'RSDirectoryTable');
                $email->load($pk);

                if ($email) {
                    $log_params = array(
                        'type'              => 'email_message_' . (!$state ? 'unpublished' : 'published'),
                        'email_id'          => $pk,
                        'email_description' => $email->description
                    );
                    RSDirectoryHelper::logAction($log_params);
                }
            }
        }

        return $return;

    }

    /**
     * Method to delete a row from the database table by primary key value.
     *
     * @access public
     *
     * @param mixed
     *
     * @return boolean
     *
     * @throws UnexpectedValueException
     */
    public function delete($pk = null)
    {
        $k = $this->_tbl_key;
        $pk = is_null($pk) ? $this->$k : $pk;

        // If no primary key is given, return false.
        if ( is_null($pk) )
        {
            throw new UnexpectedValueException('Null primary key not allowed.');
        }

        $return = parent::delete($pk);

        if ($return) {
            $log_params = array(
                'type'              => 'email_message_delete',
                'email_description' => $this->description
            );
            RSDirectoryHelper::logAction($log_params);
        }

        return $return;
    }
}
