<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Form table.
 */
class RSDirectoryTableForm extends JTable
{
    public $title_template = '';
    public $big_subtitle_template = '';
    public $small_subtitle_template = '';
    public $description_template = '';
    public $entry_submit_thank_you_message = '';
    public $contact_from_name = '';
    public $contact_from_email = '';
    public $contact_to_name = '';
    public $contact_to_email = '';
    public $contact_cc = '';
    public $contact_bcc = '';
    public $contact_subject = '';
    public $contact_message = '';
    public $entry_snippet = '';
    public $metadata_description = '';
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object Database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__rsdirectory_forms', 'id', $db);
    }
        
    /**
     * Method to store a row in the database from the JTable instance properties.
     *
     * @param boolean $updateNulls True to update fields even if they are null.
     *
     * @return boolean True on success.
     */
    public function store($updateNulls = false)
    {
        // Get the name of the primary key.
        $k = $this->_tbl_key;

        // check to see if there is an id
        $id = $this->$k;

        $return = parent::store($updateNulls);

        if ($return)
        {

            // log the create/edit for the form
            $new_id = $this->$k;
            $log_params = array(
                'type' => 'form_' . (! $id ? 'create' : 'update'),
                'form_id' =>  $new_id,
                'form_title' => $this->title
            );
            RSDirectoryHelper::logAction($log_params);
        }



        // Get mainframe.
        $app = JFactory::getApplication();
            
        // Get the JInput object.
        $category_id = $app->input->getInt('category_id');
            
        if ( !$app->isClient('site') && $category_id )
        {
            $category = JTable::getInstance('Category', 'RSDirectoryTable');
                
            $category->load($category_id);
                
            if ($category->id)
            {
                $params = new JRegistry($category->params);
                    
                $params->set('form_id', $this->$k);
                    
                $category->params = $params->toString();
                    
                $category->store();
            }
        }

        return $return;
    }
        
    /**
     * Method to delete a row from the database table by primary key value.
     *
     * @param mixed
     *
     * @return boolean
     *
     * @throws UnexpectedValueException
     */
    public function delete($pk = null)
    {
        $k = $this->_tbl_key;
        $pk = is_null($pk) ? $this->$k : $pk;
            
        // If no primary key is given, return false.
        if ( is_null($pk) )
        {
            throw new UnexpectedValueException('Null primary key not allowed.');
        }
            
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Delete the fields assigned to the form.
        $query = $db->getQuery(true)
               ->delete( $db->qn('#__rsdirectory_forms_fields') )
               ->where( $db->qn('form_id') . ' = ' . $db->q($pk) );
                
        $db->setQuery($query);
        $db->execute();

        $form = self::getInstance('Form', 'RSDirectoryTable');
        $form->load($pk);
            
        $return = parent::delete($pk);

        if ($return) {
            $log_params = array(
                'type' => 'form_delete',
                'form_title' => $form->title
            );
            RSDirectoryHelper::logAction($log_params);
        }

        return $return;
    }
}
