<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Group table.
 */
class RSDirectoryTableGroup extends JTable
{
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object Database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__rsdirectory_groups', 'id', $db);
    }

    /**
     * Method to store a row in the database from the JTable instance properties.
     *
     * @param boolean $updateNulls True to update fields even if they are null.
     *
     * @return boolean True on success.
     */
    public function store($updateNulls = false)
    {
        // Get the name of the primary key.
        $k = $this->_tbl_key;

        // check to see if there is an id
        $id = $this->$k;

        if ($id)
        {
            $group = JTable::getInstance('Group', 'RSDirectoryTable');
            $group->load($id);

            $old_published_state = $group->published;
        }

        $return = parent::store($updateNulls);
        if ($return)
        {
            $state_changed = !$id ? true : false;

            // check to see if the published state has been changed
            if ($id && $old_published_state != $this->published)
            {
                $state_changed = true;
            }

            $new_id = $this->$k;
            // log the create/edit for the form
            $log_params = array(
                'type'       => 'group_' . (! $id ? 'create' : 'update'),
                'group_id'   => $new_id,
                'group_name' => $this->name
            );

            RSDirectoryHelper::logAction($log_params);

            // log the published state if its changed
            if ($state_changed)
            {
                // log the create/edit for the form
                $log_params = array(
                    'type'       => 'group_' . (! $this->published ? 'unpublished' : 'published'),
                    'group_id'   => $new_id,
                    'group_name' => $this->name
                );

                RSDirectoryHelper::logAction($log_params);
            }
        }

        return $return;
    }
        
    /**
     * Method to delete a row from the database table by primary key value.
     *
     * @access public
     *
     * @param mixed
     *
     * @return boolean
     *
     * @throws UnexpectedValueException
     */
    public function delete($pk = null)
    {
        $k = $this->_tbl_key;
        $pk = is_null($pk) ? $this->$k : $pk;
            
        // If no primary key is given, return false.
        if ( is_null($pk) )
        {
            throw new UnexpectedValueException('Null primary key not allowed.');
        }
            
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Delete group - jgroup relations.
        $query = $db->getQuery(true)
               ->delete( $db->qn('#__rsdirectory_groups_relations') )
               ->where( $db->qn('group_id') . ' = ' . $db->q($pk) );
                
        $db->setQuery($query);
        $db->execute();

        $return = parent::delete($pk);

        if ($return) {
            $log_params = array(
                'type' => 'group_delete',
                'group_name' => $this->name
            );
            RSDirectoryHelper::logAction($log_params);
        }

        return $return;
    }

    /**
     * Method to set the publishing state for a row or list of rows in the database table.
     *
     * @param mixed $pks
     * @param integer $state
     * @param integer $userId
     *
     * @return boolean
     */
    public function publish($pks = null, $state = 1, $userId = 0)
    {
        $k = $this->_tbl_key;

        // Sanitize input.
        $pks = array_map('intval', $pks);

        // If there are no primary keys set check to see if the instance key is set.
        if ( empty($pks) )
        {
            if (!$this->$k)
                return false;

            $pks = array($this->$k);
        }

        $return = parent::publish($pks, $state, $userId);

        if ($return) {
            foreach  ($pks as $pk) {
                $group = JTable::getInstance('Group', 'RSDirectoryTable');
                $group->load($pk);

                if ($group) {
                    $log_params = array(
                        'type' => 'group_' . (!$state ? 'unpublished' : 'published'),
                        'group_id'   => $pk,
                        'group_name' => $group->name
                    );
                    RSDirectoryHelper::logAction($log_params);
                }
            }
        }

        return $return;

    }
}
