<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Reported Entry table.
 */
class RSDirectoryTableReportedEntry extends JTable
{

    public $message = '';
    public $reason = '';
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object Database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__rsdirectory_entries_reported', 'id', $db);
    }

    public function publish($pks = null, $state = 1, $userId = 0)
    {
        $pks = (array) $pks;

        // Sanitize input.
        $pks = array_map('intval', $pks);

        if (empty($pks)) {
            return false;
        }

        $return = parent::publish($pks, $state, $userId);

        if ($return)
        {
            $db = JFactory::getDbo();

            $report_ids = array();
            foreach ($pks as $id)
            {
                $report_ids[] = $db->q($id);
            }


            $query = $db->getQuery(true)
                ->select($db->qn('entry_id'))
                ->select($db->qn('id'))
                ->from($db->qn('#__rsdirectory_entries_reported'))
                ->where( $db->qn('id') . ' IN (' . implode(',', $report_ids) . ')' );
            $db->setQuery($query);

            if ($entries_data =  $db->loadAssocList()) {
                $entries_ids = array();
                $reports_entries_ids = array();

                foreach ($entries_data as $entry) {
                    $entries_ids[] = (int) $entry['entry_id'];
                    $reports_entries_ids[$entry['entry_id']] = (int) $entry['id'];
                }

                $entries = RSDirectoryHelper::getEntriesObjectListByIds($entries_ids);

                if ($entries)
                {
                    foreach ($entries as $entry)
                    {
                        // user log
                        if (!empty($entry)) {
                            $log_params = array(
                                'type' => ($state == 1 ? 'report_read' : 'report_unread'),
                                'entry_id' => $entry->id,
                                'report_id' => isset($reports_entries_ids[$entry->id]) ? $reports_entries_ids[$entry->id] : 0,
                                'entry_name' => $entry->title
                            );
                            RSDirectoryHelper::logAction($log_params);
                        }
                    }
                }
            }
        }

        return $return;
    }

    public function save($src, $orderingFilter = '', $ignore = '')
    {
        $return = parent::save($src, $orderingFilter, $ignore);

        if ($return)
        {
            if (isset($src['entry_id'])) {
                $entry = RSDirectoryHelper::getEntry($src['entry_id']);

                if (!empty($entry)) {
                    $log_params = array(
                        'type' => 'report_add',
                        'entry_id' => $entry->id,
                        'report_id' => $this->id,
                        'entry_name' => $entry->title
                    );
                    RSDirectoryHelper::logAction($log_params);
                }
            }
        }

        return $return;
    }

    public function delete($pk = null) {
        $pks = (array) $pk;

        // Sanitize input.
        $pks = array_map('intval', $pks);


        $db = JFactory::getDbo();

        $report_ids = array();
        foreach ($pks as $id)
        {
            $report_ids[] = $db->q($id);
        }


        $query = $db->getQuery(true)
            ->select($db->qn('entry_id'))
            ->from($db->qn('#__rsdirectory_entries_reported'))
            ->where( $db->qn('id') . ' IN (' . implode(',', $report_ids) . ')' );
        $db->setQuery($query);

        //loadColumn
        if ($entries_ids =  $db->loadColumn()) {
            $entries = RSDirectoryHelper::getEntriesObjectListByIds($entries_ids);
            if ($entries)
            {
                foreach ($entries as $entry)
                {
                    // user log
                    if (!empty($entry)) {
                        $log_params = array(
                            'type' => 'report_delete',
                            'entry_id' => $entry->id,
                            'entry_name' => $entry->title
                        );
                        RSDirectoryHelper::logAction($log_params);
                    }
                }
            }
        }


        return parent::delete($pk);
    }
}