<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Entry view.
 */
class RSDirectoryViewEntry extends JViewLegacy
{
    public function display($tpl = null)
    {
        // Hide the menu
        JFactory::getApplication()->input->set('hidemainmenu', true);

        // Get the mainframe.
        $app = JFactory::getApplication();
            
        // Get the JInput object.
        $jinput = $app->input;
            
        // Get stored data.
        $data = $app->getUserState('com_rsdirectory.edit.entry.data');
            
        // Get the entry id.
        $id = $jinput->getInt('id');
            
        // Get the category id.
        $category_id = empty($data['category_id']) ? $jinput->getInt('category_id') : $data['category_id'];
            
        if ($id)
        {
            // Get entry.
            $entry = $this->get('Item');
                
            if (!$entry)
            {
                throw new Exception(JText::_('COM_RSDIRECTORY_PAGE_NOT_FOUND'), 404);
            }
                
            if (!$category_id)
            {
                $category_id = $entry->category_id;    
            }
                
            $this->entry = $entry;
        }
            
        // Get the form associated to the category.
        $form = RSDirectoryHelper::getCategoryInheritedForm($category_id);
            
        $this->form = $form;
            
        if ($form->id)
        {
            $form_fields = RSDirectoryHelper::getFormFields($form->id, 1);
                
            if ($form->use_title_template || $form->use_big_subtitle_template || $form->use_small_subtitle_template || $form->use_description_template)
            {
                foreach ($form_fields as $i => $form_field)
                {
                    if (
                        ($form->use_title_template && $form_field->field_type == 'title') ||
                        ($form->use_big_subtitle_template && $form_field->field_type == 'big_subtitle') ||
                        ($form->use_small_subtitle_template && $form_field->field_type == 'small_subtitle') ||
                        ($form->use_description_template && $form_field->field_type == 'description')
                    )
                    {
                        unset($form_fields[$i]);
                    }
                }
            }
                
            $this->form_fields = $form_fields;
               
            require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/formfield.php';
			
			if (!RSDirectoryHelper::isJ4())
            {
                JHtml::_('stylesheet', 'com_rsdirectory/font-awesome.min.css', array('relative' => true, 'version' => 'auto'));
            }
        }
        else
        {
            $url = JRoute::_("index.php?option=com_rsdirectory&task=category.edit&id=$category_id");

            JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_RSDIRECTORY_SELECTED_CATEGORY_NO_FORM_ASSIGNED', $url), 'error');

            return;
        }
            
        $this->addToolBar();
        $this->id = $id;
        $this->category_id = $category_id;
        $this->jform = $this->get('Form');
            
        parent::display($tpl);
            
        // Clear the stored data.
        $app->setUserState('com_rsdirectory.edit.entry.data', null);
            
        // Clear errors.
        $app->setUserState('com_rsdirectory.edit.entry.error_field_ids', null);
    }
        
    /**
     * Add toolbar.
     *
     * @access protected
     */
    protected function addToolBar()
    {
        // Set title.
        if ( JFactory::getApplication()->input->getInt('id') )
        {
            JToolBarHelper::title( JText::_('COM_RSDIRECTORY_PAGE_EDIT_ENTRY'), 'rsdirectory' );
        }
        else
        {
            JToolBarHelper::title( JText::_('COM_RSDIRECTORY_PAGE_ADD_ENTRY'), 'rsdirectory' );
        }

        RSDirectoryToolbarHelper::addToolbar('entries');
            
        JToolBarHelper::apply('entry.apply');
        JToolBarHelper::save('entry.save');
        JToolBarHelper::save2new('entry.save2new');
        JToolBarHelper::save2copy('entry.save2copy');
        JToolBarHelper::cancel('entry.cancel');
    }
}