<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

class RsdirectoryViewTransaction extends JViewLegacy
{
    public function display( $tpl = null )
    {
        $app 		 = JFactory::getApplication();
        $config 	 = RSDirectoryConfig::getInstance();
        $use_invoice = $config->get('use_general_invoice', 0);

        $transaction_id = $app->input->get('id', 0);
        $user_id = $app->input->get('user_id', 0);

        if (!$use_invoice || !$transaction_id)
        {
            throw new Exception(JText::_('COM_RSDIRECTORY_ERROR_INVOICE_NOT_ACTIVATED'), 500);
        }
		
		 if (!$user_id)
        {
            throw new Exception(JText::_('COM_RSDIRECTORY_ERROR_INVOICE_NO_USER_ID'), 500);
        }

        require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/invoice.php';

        // get the invoice
        $invoice = RSDirectoryInvoice::getInstance($transaction_id);

        // set the user id
        $invoice->setUserId($user_id);

        // Build the template
        $this->template    = $invoice->getTemplate();

        // retrieve the document data
        $document    = $invoice->getDocument();

        if (empty($this->template) || is_null($document)) {
            throw new Exception(JText::_('COM_RSDIRECTORY_ERROR_INVOICE_NO_TEMPLATE'), 500);
        }

        parent::display($tpl);

        // Build the PDF Document string from the document buffer
        $contents = ob_get_contents();
        ob_end_clean();

        $filename = $document->title;

        // Build root without Joomla! folder
        if ($folder = JUri::root(true))
        {
            $site_path = substr(str_replace(DIRECTORY_SEPARATOR, '/', JPATH_SITE), 0, -strlen($folder));
        }
        else
        {
            $site_path = str_replace(DIRECTORY_SEPARATOR, '/', JPATH_SITE);
        }

        // Add own CSS
        if ($css_path = realpath($site_path.'/' . JHtml::_('stylesheet', 'com_rsdirectory/invoice.css', array('pathOnly' => true, 'relative' => true))))
        {
            $contents = '<link rel="stylesheet" href="' . $this->escape($css_path) . '" type="text/css"/>' . $contents;
        }

        // Allow plugins to use their own PDF library
        $app->triggerEvent('rsdir_onPDFView', array($contents, $filename));

        /*
         * Setup external configuration options
         */
        define('K_TCPDF_EXTERNAL_CONFIG', true);
        define("K_PATH_MAIN", JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/tcpdf');
        define("K_PATH_URL", JPATH_BASE);
        define("K_PATH_FONTS", K_PATH_MAIN.'/fonts/');
        define("K_PATH_CACHE", K_PATH_MAIN."/cache");
        define("K_PATH_URL_CACHE", K_PATH_URL."/cache");
        define("K_PATH_IMAGES", K_PATH_MAIN."/images");
        define("K_BLANK_IMAGE", K_PATH_IMAGES."/_blank.png");
        define("K_CELL_HEIGHT_RATIO", 1.25);
        define("K_TITLE_MAGNIFICATION", 1.3);
        define("K_SMALL_RATIO", 2/3);
        define("HEAD_MAGNIFICATION", 1.1);

        /*
         * Create the pdf document
         */

		if (!class_exists('TCPDF'))
		{
			require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/tcpdf/tcpdf.php';
		}

        $pdf = new TCPDF();
        $pdf->SetMargins(15, 15, 15);
        $pdf->SetAutoPageBreak(true, 25);
        $pdf->SetHeaderMargin(0);
        $pdf->SetFooterMargin(0);
        $pdf->setImageScale(4);
        $pdf->setPrintHeader(false);
        $pdf->setPrintFooter(false);

        $document = JFactory::getDocument();

        // Set PDF Metadata
        $pdf->SetCreator($document->getGenerator());
        $pdf->SetTitle($document->getTitle());
        $pdf->SetSubject($document->getDescription());
        $pdf->SetKeywords($document->getMetaData('keywords'));


        // Set RTL
        $lang = JFactory::getLanguage();
        $pdf->setRTL($lang->isRTL());

        // Set Font
        $font = $config->get('invoice_pdf_font', 'freesans');
        $font_size = $config->get('invoice_pdf_font_size', 10);
        $pdf->setFont($font, '', $font_size);
		
        $pdf->setHeaderFont(array($font, '', 10));
        $pdf->setFooterFont(array($font, '', 8));

        // Initialize PDF Document
        if (is_callable(array($pdf, 'AliasNbPages'))) {
            $pdf->AliasNbPages();
        }
        $pdf->AddPage();

        $pdf->WriteHTML($contents, true);
        $data = $pdf->Output('', 'S');

        // Build the PDF Document string from the document buffer
        header('Content-Type: application/pdf; charset=utf-8');
        header('Content-disposition: attachment; filename="'.$filename.'.pdf"', true);

        echo $data;

        $app->close();
    }
}