<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JHtml::_('behavior.keepalive');
JHtml::_('behavior.formvalidator');

JText::script('COM_RSDIRECTORY_ANONYMISE_BUTTON_WARNING');

$script =
    "Joomla.submitbutton = function(task) {
	if (task == 'user.cancel' || task == 'user.anonymise' || document.formvalidator.isValid( document.getElementById('adminForm'))) {
		Joomla.submitform( task, document.getElementById('adminForm') );
	}
}";

JFactory::getDocument()->addScriptDeclaration($script);
?>
<form id="adminForm" class="form-validate" action="<?php echo JRoute::_('index.php?option=com_rsdirectory&view=user&layout=edit&id=' . $this->id); ?>" method="post">
    <h3><?php echo JText::_('COM_RSDIRECTORY_USER_DETAILS_LABEL'); ?></h3>
    <table class="table table-striped">
        <tbody>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_NAME'); ?></th>
            <td>
                <a class="<?php echo RSDirectoryHelper::getTooltipClass(); ?>" href="<?php echo JRoute::_('index.php?option=com_users&task=user.edit&id=' . $this->item->id); ?>" title="<?php echo RSDirectoryHelper::getTooltipText( JText::sprintf( 'COM_RSDIRECTORY_EDIT_USER', $this->escape($this->item->name) ) ); ?>">
                    <?php echo $this->escape($this->item->name); ?>
                </a>
            </td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_USERNAME'); ?></th>
            <td><?php echo $this->escape($this->item->username); ?></td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_AVAILABLE_CREDITS'); ?></th>
            <td>
                <?php
                if ($this->item->unlimited_credits)
                {
                    echo JText::_('COM_RSDIRECTORY_UNLIMITED');
                }
                else
                {
                    echo $this->item->credits ? $this->escape($this->item->credits) : 0;
                }
                ?>
            </td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_SPENT_CREDITS'); ?></th>
            <td><?php echo $this->escape($this->item->spent_credits); ?></td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_CREDITS_HISTORY'); ?></th>
            <td>
                <a class="<?php echo RSDirectoryHelper::getTooltipClass(); ?>" href="<?php echo JRoute::_('index.php?option=com_rsdirectory&view=creditshistory&filter_user_id=' . $this->item->id); ?>" title="<?php echo RSDirectoryHelper::getTooltipText( JText::sprintf( 'COM_RSDIRECTORY_CREDITS_HISTORY_FOR', $this->escape($this->item->name) ) ); ?>">
                    <?php echo JText::_('COM_RSDIRECTORY_CREDITS_HISTORY'); ?>
                </a>
            </td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_TRANSACTIONS_COUNT'); ?></th>
            <td>
                <a class="<?php echo RSDirectoryHelper::getTooltipClass(); ?>" href="<?php echo JRoute::_('index.php?option=com_rsdirectory&view=transactions&filter_user_id=' . $this->item->id); ?>" title="<?php echo RSDirectoryHelper::getTooltipText( JText::sprintf( 'COM_RSDIRECTORY_TRANSACTIONS_MADE_BY', $this->escape($this->item->name) ) ); ?>">
                    <?php echo empty($this->item->transactions_count) ? 0 : $this->item->transactions_count; ?>
                </a>
            </td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_ENTRIES_COUNT'); ?></th>
            <td>
                <a class="<?php echo RSDirectoryHelper::getTooltipClass(); ?>" href="<?php echo JRoute::_('index.php?option=com_rsdirectory&view=entries&filter_author_id=' . $this->item->id); ?>" title="<?php echo RSDirectoryHelper::getTooltipText( JText::sprintf( 'COM_RSDIRECTORY_ENTRIES_POSTED_BY', $this->escape($this->item->name) ) ); ?>">
                    <?php echo empty($this->item->entries_count) ? 0 : $this->item->entries_count; ?>
                </a>
            </td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_REVIEWS_COUNT'); ?></th>
            <td>
                <a class="<?php echo RSDirectoryHelper::getTooltipClass(); ?>" href="<?php echo JRoute::_('index.php?option=com_rsdirectory&view=ratings&filter_reviews_author_id=' . $this->item->id); ?>" title="<?php echo RSDirectoryHelper::getTooltipText( JText::sprintf( 'COM_RSDIRECTORY_REVIEWS_POSTED_BY', $this->escape($this->item->name) ) ); ?>">
                    <?php echo empty($this->item->reviews_count) ? 0 : $this->item->reviews_count; ?>
                </a>
            </td>
        </tr>
        <tr>
            <th nowrap style="width: 1%"><?php echo JText::_('COM_RSDIRECTORY_REPORTS_COUNT'); ?></th>
            <td>
                <a class="<?php echo RSDirectoryHelper::getTooltipClass(); ?>" href="<?php echo JRoute::_('index.php?option=com_rsdirectory&view=reportedentries&filter_reports_author_id=' . $this->item->id); ?>" title="<?php echo RSDirectoryHelper::getTooltipText( JText::sprintf( 'COM_RSDIRECTORY_REPORTS_POSTED_BY', $this->escape($this->item->name) ) ); ?>">
                    <?php echo empty($this->item->reports_count) ? 0 : $this->item->reports_count; ?>
                </a>
            </td>
        </tr>
        </tbody>
    </table>

    <div class="alert alert-danger">
        <p class="text-center"><?php echo JText::_('COM_RSDIRECTORY_ANONYMISE_DESC'); ?></p>
        <p class="text-center"><button type="button" onclick="if (confirm(Joomla.JText._('COM_RSDIRECTORY_ANONYMISE_BUTTON_WARNING'))) { Joomla.submitbutton('user.anonymise'); }" class="btn btn-danger btn-large"><?php echo JText::_('COM_RSDIRECTORY_ANONYMISE_BUTTON'); ?></button></p>
    </div>

    <h3><?php echo JText::_('COM_RSDIRECTORY_EDIT_USER_LABEL'); ?></h3>

    <?php
    echo $this->form->renderFieldset('general');
    ?>

    <div>
        <input type="hidden" name="task" value="" />
        <input type="hidden" name="id" value="<?php echo $this->item->user_id; ?>" />
        <?php echo JHtml::_('form.token') . "\n"; ?>
    </div>

</form><!-- #user -->