;(function($)
{
    $.rsRadiusSearch = function(el, options)
	{
        var base = this;
			
        base.$el = $(el);
        base.el = el;
			
        base.$el.data("rsRadiusSearch", base);
			
        base.init = function()
		{
            base.options = $.extend({},$.rsRadiusSearch.defaultOptions, options);
				
			// Initialize the Geocoder.
			base.geocoder = new google.maps.Geocoder();
				
			base.form = $(base.options.form);
			base.location = base.form.find('[name="location"]');
			base.radius = base.form.find('[name="radius"]');
			base.unit = base.form.find('[name="unit"]');
			base.loader = base.form.find('.loader');
				
			base.circle = null;
			base.markers = [];
			base.cache = [];
				
			base.initMap();
			base.setPos();
			base.locationOnKeyUp();
			base.formOnSubmit();
        };
		// Initialize map.
		base.initMap = function()
		{
			base.map = new google.maps.Map(el,
			{
				zoom: base.options.zoom,
				mapTypeId: base.options.mapType,
				streetViewControl: base.options.streetViewControl,
				scrollwheel: base.options.scrollwheel,
				zoomControl: base.options.zoomControl,
				styles: base.options.styles,
			});
		}
			
		// Set the map center and marker position.
		base.setPos = function()
		{	
			var coords = {
				initialLat  : '',
				initialLong : ''
			};		
			base.process('initial');
			
			if(navigator.geolocation && rsdir_use_geolocation) {
				navigator.geolocation.getCurrentPosition(function(position) {
					coords.initialLat 	= position.coords.latitude;
					coords.initialLong 	= position.coords.longitude;
					base.process('initial', coords)
				}, function() {
					base.process('initial');
				});
			}
			
		};
			
		// Add a key up event to the address input.	
		base.locationOnKeyUp = function()
		{
			base.location.bind('keyup', function()
			{
				var srcEl = event.currentTarget;
				if( srcEl.delayTimer )
				{
					clearTimeout (srcEl.delayTimer );
				}

				srcEl.delayTimer = setTimeout(function()
				{
					base.location.parent().find('.' + base.options.placeholdersWrapperClass).remove();

					if ( $.trim( base.location.val() ) )
					{
						base.geocoder.geocode( {address: base.location.val()}, function(results, status)
						{
							if (status == google.maps.GeocoderStatus.OK)
							{
								var results_wrapper = $('<div class="' + base.options.placeholdersWrapperClass + '"><ul class="' + base.options.placeholdersClass + '"></ul></div>');
								base.location.after(results_wrapper);

								$(results).each(function(index, item)
								{
									li = $('<li>' + item.formatted_address + '</li>').click(function()
									{
										base.location.val(item.formatted_address);
										base.map.setCenter(item.geometry.location);

										results_wrapper.remove();
									});

									results_wrapper.find('ul').append(li);
								});

								$(document).click( function(event)
								{
									if( $(event.target).parents().index(results_wrapper) == -1 )
									{
										results_wrapper.remove();
									}
								});
							}
						});
					}
				}, 500);
			});
		};
			
		// Set the form on submit event.
		base.formOnSubmit = function()
		{	
			base.form.submit(function(e)
			{
				e.preventDefault();
					
				errors = false;
					
				// Remove errors.
				base.form.find('.error').removeClass('error');
					
				// Validate location.
				if ( !$.trim( base.location.val() ) )
				{
					base.location.parents('.control-group').addClass('error');
					errors = true;
				}
					
				// Validate radius.
				if ( !/^\d+$/.test( base.radius.val() ) || parseInt( base.radius.val(), 10 ) <= 0 )
				{
					base.radius.parents('.control-group').addClass('error');
					errors = true;
				}
					
				// Stop the execution of the function if there are errors.
				if (errors)
				{
					$('html,body').animate({scrollTop: base.form.offset().top});
					return;
				}
					
				base.loader.removeClass('hide');
					
				base.clearMarkers();
				
				// get the max and min coordinates for accurate search in database
				
				base.geocoder.geocode( {'address': base.location.val()}, function(results, status)
				{
					// set the data
					var data = base.form.serialize();
					if (status == google.maps.GeocoderStatus.OK)
					{						
						data = data + '&startpoint=' + results[0].geometry.location.lat() + ',' + results[0].geometry.location.lng();
						base.formOnSubmitAjax(data);
					} else {
						// in case geocoder doesn't work
						base.formOnSubmitAjax(data);
					}
				});	
			});
		}
		
		base.formOnSubmitAjax = function (data) {
			$.ajax(
			{
				type: "POST",
				dataType: "json",
				url: base.form.attr('action'),
				data: data,
				success: function(json)
				{
					
					try
					{
						if ($('#rsdir-map-results').length) {
							$('#rsdir-map-results').empty().parent().hide();
						}
						
						// json is valid if json.length does not throw an error.
						var results = json.length;
							
						base.initInfoWindow();
						base.offsetCount = 0;
						base.offsetMarkers = [];
						base.showMoreLoaded = false;
							
						// Create the markers.
						$(json).each(function(i, element)
						{
							$(element.coords).each(function(j, coords)
							{
								base.createMarker(element.id, coords, element);
								if (base.options.useShowMore) {
									if (results > base.options.limit ){
										if (!base.showMoreLoaded) {
											var loadMoreButton = $('<a href="" class="rsdir-map-show-more">'+Joomla.JText._('COM_RSDIRECTORY_SHOW_MORE')+'</a>').click(function(e){
												e.preventDefault();
												base.loadMoreResults(this);
											});
											var $loadMore = $('<tr>').append($('<td colspan="3">').append(loadMoreButton));
											$('#rsdir-map-results').append($loadMore);
											base.showMoreLoaded = true;
										}
									}
								}
							});
						});
						base.loadMoreResults();
					}
					catch(e)
					{	
					}
					base.track = 0;
					base.process();
						
					base.loader.addClass('hide');
				}
			});
		}
		
		base.showMoreLoaded = false;
		base.offsetCount = 0;
		base.offsetMarkers = [];
		base.loadMoreStart = 0;
		
		base.loadMoreResults = function(showMoreButton) {
			base.offsetCount = 0;
			var isLoadMore = $('#rsdir-map-results').find('.rsdir-map-show-more').length;
			var lasItemShown = 0;
			var lastItemIndex = base.offsetMarkers.length - 1;
			$.each(base.offsetMarkers, function(i, item) {
				i = parseInt(i);
				if (!base.options.useShowMore || (base.offsetCount <= base.options.limit && (typeof showMoreButton == 'undefined' || base.loadMoreStart < i))) {
					var $row 	= $('<tr>');
					
					if (item.element.promoted && item.element.promoted == 1)
					{
						$row.addClass('rsdir-map-promoted');
					}
					
					// Image
					var $image 	= $(item.element.image);
					var $cell 	= $('<td>').append($image);
					$row.append($cell);
					
					// Title + Location
					var $title		= $(item.element.link)
					var $location 	= $('<p>').text(item.element.address);
					var $cell 		= $('<td>').append($title).append($location);
					$row.append($cell);
					
					// Button
					var $cell 	= $('<td>');
					var $button = $('<button type="button" class="btn btn-secondary"><i class="fa fa-map-marker"></i></button>').click(function(e){
						e.preventDefault();
						base.map.setCenter(new google.maps.LatLng(item.coords.lat, item.coords.lng));
						google.maps.event.trigger(item.marker, 'click');
						$('html, body').animate({
							scrollTop: $(base.el).offset().top
						}, 1000);
					});
					$cell.append($button);
					$row.append($cell);
					
					// Append row
					if (typeof showMoreButton != 'undefined') { // First request
						$(showMoreButton).parent().parent().before($row);
					} else {
						// When the "Show More" button is pressed
						var $table 	= $('#rsdir-map-results');
						if (isLoadMore) {
							$('#rsdir-map-results').find('.rsdir-map-show-more').parent().parent().before($row);
						} else {
							$table.append($row);
						}
						$table.parent().show();
					}
					
					if (base.options.useShowMore && base.offsetCount == base.options.limit) {
						base.loadMoreStart = i;
					}
					if (base.options.useShowMore) base.offsetCount++;
					
					lasItemShown = i;
				}
			});
			// Remove the "Show More" button beacause there are no more results to show
			if (base.options.useShowMore && isLoadMore && lasItemShown == lastItemIndex) {
				$('#rsdir-map-results').find('.rsdir-map-show-more').parent().parent().hide();
			}
		}
			
		// Initialize the info window.	
		base.initInfoWindow = function()
		{
			base.infoWindow = new google.maps.InfoWindow();	
		}
		
		// Create marker.	
		base.createMarker = function(id, coords, element)
		{
			if (typeof element.map_icon != 'undefined' && element.map_icon != '') {
				var marker = new google.maps.Marker(
					{
						position: new google.maps.LatLng(coords.lat, coords.lng),
						map: base.map,
						icon: element.map_icon.icon
					});
			}
			else
			{
				var marker = new google.maps.Marker(
					{
						position: new google.maps.LatLng(coords.lat, coords.lng),
						map: base.map
					});
			}
			
			if ($('#rsdir-map-results').length) {
				var elementData = {
					coords : coords,
					element : element,
					marker : marker
				}
				base.offsetMarkers.push(elementData);
			}
				
			google.maps.event.addListener(marker, 'click', function()
			{
				if (base.cache[id] == undefined)
				{
					infoWindow = $('<div class="' + base.options.infoWindowClass + '" style=" width: ' + base.options.infoWindowWidth + 'px; height: ' + base.options.infoWindowHeight + 'px;">' + Joomla.JText._('COM_RSDIRECTORY_LOADING') + '</div>')
						
					base.infoWindow.setContent(infoWindow[0]);
					base.infoWindow.open(base.map, marker);
						
					// Serialize the form data.
					data = base.form.serializeArray();
						
					// Add the entry id.
					data.push({name: 'id', value: id});
						
					// Change the task.
					for (i in data)
					{
						if (data[i].name == 'task')
						{
							data[i].value = 'radius.getInfoWindow'
						}
					}
						
					$.ajax(
					{
						type: "POST",
						url: base.form.attr('action'),
						data: data,
						success: function(response)
						{
							infoWindow = $(response);
								
							rating = infoWindow.find('.rating');
								
							if (rating.length > 0 && $.fn.raty)
							{
								rating.raty(
								{
									path: rsdir.root + 'media/com_rsdirectory/images/raty/',
									hints: [Joomla.JText._('COM_RSDIRECTORY_RATY_BAD'), Joomla.JText._('COM_RSDIRECTORY_RATY_POOR'), Joomla.JText._('COM_RSDIRECTORY_RATY_REGULAR'), Joomla.JText._('COM_RSDIRECTORY_RATY_GOOD'), Joomla.JText._('COM_RSDIRECTORY_RATY_GORGEOUS')],
									readOnly: true,
									score: function()
									{
										return $(this).attr('data-rating');
									},
									starType: 'i'
								});
							}
								
							eventName = 'click.radiusSearchFav';
								
							infoWindow.find('.fav').off(eventName).on(eventName, function()
							{
								addToFavorites( $(this) );
							});
								
							base.cache[id] = infoWindow[0];
								
							base.infoWindow.setContent(infoWindow[0]);
						}
					});
				}
				else
				{
					base.infoWindow.setContent(base.cache[id]);
					base.infoWindow.open(base.map, marker);	
				}
			});
				
			base.markers.push(marker);
		}
			
		// Clear markers.
		base.clearMarkers = function()
		{
			for (var i = 0; i < base.markers.length; i++)
			{
				base.markers[i].setMap(null);
			}
				
			base.markers = [];
		}
		
		// track the load of the map
		base.track = 0;
			
		base.process = function(type, coords)
		{	
			var use_geocoder = true;
			if (typeof coords != 'undefined') {
				use_geocoder = false;
			}
			
			if (use_geocoder) {
				
				base.geocoder.geocode( {'address': base.location.val()}, function(results, status)
				{
					
					if (status == google.maps.GeocoderStatus.OK)
					{
						if (base.track == 0) {
							base.map.setCenter(results[0].geometry.location);
							var searchCenter = results[0].geometry.location;
							base.processCircle(searchCenter);
							if (typeof type != 'undefined' && type == 'initial') {
								base.processCreateMarkers(results[0].geometry.location.lat(), results[0].geometry.location.lng());
							}
							base.track++;
						}
					}
				});		
			} else {
				var initialLocation = new google.maps.LatLng(coords.initialLat, coords.initialLong);
				if (initialLocation) {
					base.map.setCenter(initialLocation);
					var searchCenter = initialLocation;
					base.processCircle(searchCenter);
					base.geocoder.geocode({'latLng': initialLocation}, function(results, status){
						if (status == google.maps.GeocoderStatus.OK) {
							 if (results[1]) {
								jQuery('#rsdir-location').val(results[1].formatted_address);
								if (typeof type != 'undefined' && type == 'initial') {
									base.processCreateMarkers(coords.initialLat, coords.initialLong);
								}
							 }
						}
					});
					base.track++;
				}
			}
		},
		
		base.processCreateMarkers = function (lat, lng) {
			base.loader.removeClass('hide');
			
			var data = base.form.serialize();
			data = data + '&startpoint=' + lat + ',' + lng;
			
			$.ajax(
				{
					type: "POST",
					dataType: "json",
					url: base.form.attr('action'),
					data: data,
					success: function(json)
					{
						if ($('#rsdir-map-results').length) {
							$('#rsdir-map-results').empty().parent().hide();
						}
						
						try
						{
							// json is valid if json.length does not throw an error.
							var results = json.length;
								
							base.initInfoWindow();
							base.offsetCount = 0;
							base.offsetMarkers = [];
							base.showMoreLoaded = false;
							base.loadMoreStart = 0;
								
							// Create the markers.
							$(json).each(function(i, element)
							{
								$(element.coords).each(function(j, coords)
								{
									base.createMarker(element.id, coords, element);
									if (base.options.useShowMore) {
										if (results > base.options.limit ){
											if (!base.showMoreLoaded) {
												var loadMoreButton = $('<a href="" class="rsdir-map-show-more btn btn-primary">'+Joomla.JText._('COM_RSDIRECTORY_SHOW_MORE')+'</a>').click(function(e){
													e.preventDefault();
													base.loadMoreResults(this);
												});
												var $loadMore = $('<tr>').append($('<td colspan="3">').append(loadMoreButton));
												$('#rsdir-map-results').append($loadMore);
												base.showMoreLoaded = true;
											}
										}
									}
								});
							});
							base.loadMoreResults();
						}
						catch(e)
						{	
						}
						base.loader.addClass('hide');
					}
				});
		
		},
		
		base.processCircle = function (searchCenter) {
			var unit_value = base.unit.val() == 'mi' || base.unit.val() == 'miles' ? 1609.34 : 1000;
				
			var radius = parseInt( base.radius.val(), 10) * unit_value;
			
			if (base.circle)
			{
				base.circle.setMap(null);
			}
				
			base.circle = new google.maps.Circle(
			{
				center:searchCenter,
				radius: radius,
				fillOpacity: base.options.hideCircle ? 0 : base.options.fillOpacity,
				fillColor: base.options.fillColor,
				strokeWeight: 1,
				strokeColor: base.options.strokeColor,
				strokeOpacity: base.options.hideCircle ? 0 : 1,
				map: base.map
			});
				
			var bounds = new google.maps.LatLngBounds();
			var foundMarkers = 0;
				
			for (var i = 0; i < base.markers.length; i++)
			{
				if ( google.maps.geometry.spherical.computeDistanceBetween( base.markers[i].getPosition(), searchCenter ) < radius )
				{
					bounds.extend( base.markers[i].getPosition() );
					base.markers[i].setMap(base.map);
					foundMarkers++;
				}
				else
				{
					base.markers[i].setMap(null);
				}
			}
				
			if (foundMarkers > 0)
			{
				if ( bounds.getNorthEast().equals( bounds.getSouthWest() ) )
				{
					var extendPoint1 = new google.maps.LatLng( bounds.getNorthEast().lat() + 0.001, bounds.getNorthEast().lng() + 0.001 );
					var extendPoint2 = new google.maps.LatLng( bounds.getNorthEast().lat() - 0.001, bounds.getNorthEast().lng() - 0.001 );
					bounds.extend(extendPoint1);
					bounds.extend(extendPoint2);
				}
			 
				base.map.fitBounds(bounds);
			}
			else
			{
				base.map.fitBounds( base.circle.getBounds() );
			}
		}
			
        base.init();
    };
		
    $.rsRadiusSearch.defaultOptions = {
		form: null,
		address: 'Statue of Liberty National Monument, New York, NY 10004, United States',
		zoom: 5,
		mapType: google.maps.MapTypeId.ROADMAP, // See: https://developers.google.com/maps/documentation/javascript/maptypes#BasicMapTypes
		streetViewControl: false,
		scrollwheel: false,
		zoomControl: true,
		placeholdersWrapperClass: 'rsdir-placeholders-wrapper',
		placeholdersClass: 'rsdir-placeholders',
		infoWindowClass: 'rsdir-info-window media',
		infoWindowWidth: 300,
		infoWindowHeight: 120,
		fillColor: '#ff8080',
		strokeColor: '#000000',
		fillOpacity: 0.35,
		useShowMore: 0,
		hideCircle: 0,
		limit: 5,
		styles: [],
    };
		
    $.fn.rsRadiusSearch = function(options)
	{
        return this.each(function()
		{
            (new $.rsRadiusSearch(this, options));
        });
    };
		
})(jQuery);