(function($)
{
	$.rsOMap = function(el, options)
	{
		var base = this;
			
		base.$el = $(el);
		base.el = el;
			
		base.$el.data("rsOMap", base);
			
		base.init = function()
		{
			base.options = $.extend({},$.rsOMap.defaultOptions, options);
				
			// Initialize the Geocoder.
			base.geocoder = L.esri.Geocoding.geocodeService(); // http://esri.github.io/esri-leaflet/api-reference/services/geocode-service.html
				
			base.inputAddress = $(base.options.inputAddress);
			base.inputLat = $(base.options.inputLat);
			base.inputLng = $(base.options.inputLng);
				
			base.initMap();
			base.initMarker();
			base.initPos();
			base.setMarkerOnDragEnd();
			base.inputAddressOnKeyUp();
			base.inputLatLngOnChange();
		};
			
		// Initialize map.
		base.initMap = function()
		{
			base.map = L.map(el,
			{
				zoom: base.options.zoom,
				zoomControl: base.options.zoomControl,
				scrollWheelZoom: base.options.scrollwheel,
			});
			
			var tileLayer = L.tileLayer(base.options.tileLayer,
			{
				maxZoom: 18,
				attribution: base.options.attribution,
				id: base.options.tileType,
				accessToken: base.options.accessToken
			});
			
			tileLayer.addTo(base.map);
		}
			
		// Initialize marker.
		base.initMarker = function()
		{
			if (!base.options.markerDisplay)
				return;

			var options = {
				draggable: base.options.markerDraggable
			};

			if (base.options.mapIcon) {
				var markerIcon = L.icon({
					iconUrl: base.options.mapIcon.icon,
					iconSize: [base.options.mapIcon.w, base.options.mapIcon.h],
					iconAnchor: [base.options.mapIcon.icon_anchor_x, base.options.mapIcon.icon_anchor_y],
					popupAnchor: [0, base.options.mapIcon.popup_anchor_y],
					shadowUrl: base.options.mapIcon.shadow
				});
				options.icon = markerIcon;
			}
				
			base.marker = L.marker([0, 0], options);
			
			base.marker.addTo(base.map);
		}
			
		// Initialize the map and marker position.
		base.initPos = function()
		{
			// 1st priority: the lat and lng options.
			if ( parseFloat(base.options.lat) || parseFloat(base.options.lng) )
			{
				base.setPos( L.latLng(base.options.lat, base.options.lng) );
			}
			// 2nd priority: the address option.
			else if (base.options.address)
			{
				base.geocoder.geocode().text(base.options.address).run(function(error, result){
					if (typeof error == 'undefined' && result.results.length > 0) {
						base.setPos(result.results[0].latlng);
					}
				});
			}
			// 3rd priority: the lat and lng input values.
			else if ( parseFloat( base.inputLat.val() ) || parseFloat( base.inputLng.val() ) )
			{
				base.setPos( L.latLng( base.inputLat.val(), base.inputLng.val() ) );
			}
			// 4th priority: the address input value.
			else if ( base.inputAddress.val() )
			{
				base.geocoder.geocode().text(base.inputAddress.val()).run(function(error, result){
					if (typeof error == 'undefined' && result.results.length > 0) {
						base.setPos(result.results[0].latlng);
					}
				});
			}
			else
			{
				base.setPos( L.latLng(0, 0) );
			}
		};
			
		// Set the map and marker positon.
		base.setPos = function(latLng)
		{
			base.map.setView(latLng);
				
			if (base.options.markerDisplay)
			{
				base.marker.setLatLng(latLng);
			}
		}
			
		// Add a on drag end event to the marker.
		base.setMarkerOnDragEnd = function()
		{
			if (!base.options.markerDisplay)
				return;
			
			base.marker.on('dragend', function(event) {
				base.inputLat.val( base.marker.getLatLng().lat );
				base.inputLng.val( base.marker.getLatLng().lng );
				
				// update the cost if there is any
				if (typeof updateTextCreditsCost === 'function') {
					updateTextCreditsCost( base.inputLat, base.inputLat.val() );
				}
				
				base.geocoder.reverse().latlng(base.marker.getLatLng()).run(function(error, result) {
					if (typeof error == 'undefined') {
						base.inputAddress.val(result.address.Match_addr);
						
						if (typeof base.options.markerOnDragEnd != 'function')
							return;
							
						// Call the user defined on drag event.
						base.options.markerOnDragEnd(result);
					} 
					else 
					{
						base.inputAddress.val('');
					}
				});
				
			});
		}
			
		// Add a on key up event to the address input.
		base.inputAddressOnKeyUp = function()
		{
			base.inputAddress.bind('keyup', function()
			{
				var srcEl = event.currentTarget;
				if( srcEl.delayTimer )
				{
					clearTimeout (srcEl.delayTimer );
				}

				srcEl.delayTimer = setTimeout(function()
				{
					base.inputAddress.parent().find('.' + base.options.placeholdersWrapperClass).remove();
					
					if ( $.trim( base.inputAddress.val() ) )
					{
						base.geocoder.geocode().text(base.inputAddress.val()).run(function(error, result){
							if (typeof error == 'undefined' && result.results.length > 0) {

								if ( base.inputAddress.parent().find('.'+base.options.placeholdersWrapperClass).length) {
									base.inputAddress.parent().find('.'+base.options.placeholdersWrapperClass).remove();
								}

								var results_wrapper = $('<div class="' + base.options.placeholdersWrapperClass + '"><ul class="' + base.options.placeholdersClass + '"></ul></div>');
								base.inputAddress.after(results_wrapper);

								$(result.results).each(function(index, item)
								{
									li = $('<li>' + item.text + '</li>').click(function()
									{
										base.inputAddress.val(item.text);
										base.inputLat.val( item.latlng.lat);
										base.inputLng.val( item.latlng.lng );

										// update the cost if there is any
										if (typeof updateTextCreditsCost === 'function') {
											updateTextCreditsCost( base.inputLat, base.inputLat.val() );
										}

										base.setPos(item.latlng);

										results_wrapper.remove();
									});

									results_wrapper.find('ul').append(li);
								});

								$(document).click( function(event)
								{
									if( $(event.target).parents().index(results_wrapper) == -1 )
									{
										results_wrapper.remove();
									}
								});
							}
						});
					}
					else
					{
						base.inputLat.val(0);
						base.inputLng.val(0);

						base.setPos( L.latLng(0, 0) );
					}
				}, 500);
			});
		};
			
		// 	Add a on change event to the lat and lng inputs.
		base.inputLatLngOnChange = function()
		{
			inputLatLngOnChange = function()
			{
				base.geocoder.reverse().latlng(base.marker.getLatLng()).run(function(error, result) {
					if (typeof error == 'undefined') 
					{
						base.inputAddress.val(result.address.Match_addr);
						// check if the address field has remain completed
						if (base.inputAddress.val().trim() != '' && typeof updateTextCreditsCost === 'function') {
							updateTextCreditsCost( base.inputAddress, base.inputAddress.val() );
						}	
					} 
					else 
					{
						base.inputAddress.val('');
					}
				});
				
				var filteredLat = base.filterLatLng(base.inputLat.val(), 'lat');
				var filteredLng = base.filterLatLng(base.inputLng.val(), 'lng');
				
				// update the cost if there is any
				if (typeof updateTextCreditsCost === 'function') {
					if (base.inputLat.val() != '' && base.inputLat.val() != '0' && base.inputLng.val() != '' && base.inputLng.val() != '0') {
						updateTextCreditsCost( base.inputLat, base.inputLat.val() );
					} else if ((base.inputLat.val() == '' || base.inputLat.val() != '0') || (base.inputLng.val() != '' && base.inputLng.val() != '0')){
						updateTextCreditsCost( base.inputLat, '' );
					}
				}
				
				base.setPos( L.latLng( filteredLat, filteredLng ) );
			}
				
			base.inputLat.bind('keyup change', inputLatLngOnChange);
			base.inputLng.bind('keyup change', inputLatLngOnChange);
		}
		
		base.filterLatLng = function(value, type) {
			// strip all unneeded characters
			value = value.replace(/[^0-9\.\-]+/g, '');
			
			// force a number
			value = parseFloat(value);
			
			// check the value
			if (type == 'lat' && (value < -90 || value > 90)) {
				value = 0;
			} else if (type == 'lng' && (value < -180 || value > 180)) {
				value = 0;
			}
			
			if (isNaN(value)) {
				value = 0;
			}
			
			return value;
		}
		
		base.geoLocationProcess = function(position) {
			var pos = {
				lat: position.coords.latitude,
				lng: position.coords.longitude
			};
			
			var hasAddress = false;
			base.geocoder.reverse().latlng(base.marker.getLatLng()).run(function(error, result) {
				if (typeof error == 'undefined') {
					base.inputAddress.val(result.address.Match_addr);
					hasAddress = true;
				} 
				else 
				{
					base.inputAddress.val('');
				}
			});
			
			base.inputLat.val(position.coords.latitude);
			base.inputLng.val(position.coords.longitude);
			
			if (typeof updateTextCreditsCost === 'function') {
				if (hasAddress || (position.coords.longitude && position.coords.latitude)) {
					if (hasAddress) {
						updateTextCreditsCost(base.inputAddress.context, base.inputAddress.val());
					} else  {
						updateTextCreditsCost(base.inputLat.context, base.inputLat.val());
						updateTextCreditsCost(base.inputLng.context, base.inputLng.val());
					}
				}
			}
			base.setPos(pos);
		}
			
		base.init();
	};
		
	$.rsOMap.defaultOptions = {
		address: '',
		lat: null,
		lng: null,
		zoom: 5,
		tileType: '',
		tileLayer: '',
		accessToken: '',
		attribution: '',
		mapIcon: null,
		
		scrollwheel: false,
		zoomControl: true,
		inputAddress: null,
		inputLat: null,
		inputLng: null,
		markerDisplay: true,
		markerDraggable: false,
		markerOnDragEnd: null,
		placeholdersWrapperClass: 'rsdir-placeholders-wrapper',
		placeholdersClass: 'rsdir-placeholders'
	};
		
	$.fn.rsOMap = function(options)
	{
		return this.each(function()
		{
			(new $.rsOMap(this, options));
		});
	};
})(jQuery);