<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('list');

/**
 * Category Select options field.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldCategorySelect extends JFormFieldList
{
    /**
     * A flexible category list that respects access controls
     *
     * @var string
     */
    public $type = 'categoryselect';

    /**
     * Method to get a list of categories that respects access controls and can be used for
     * either category assignment or parent category assignment in edit screens.
     * Use the parent element to indicate that the field will be used for assigning parent categories.
     *
     * @return array The field option objects.
     */
    protected function getOptions()
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true)
            ->select( $db->qn( array('id', 'title', 'level') ) )
            ->from( $db->qn('#__categories') )
            ->where( $db->qn('parent_id') . ' > ' . $db->q(0) )
            ->where( $db->qn('extension') . ' = ' . $db->q('com_rsdirectory') )
            ->where( $db->qn('published') . ' IN (' . $db->q(0) . ', ' . $db->q(1) . ')' )
            ->order( $db->qn('lft') );

        $db->setQuery($query);
        $items = $db->loadObjectList();

        $options = array();
        $options[] = JHtml::_('select.option', '', JText::_('PLG_RSDIRECTORY_IMPORTCSV_CATEGORY_OPTION'), 'value', 'text');

        if ($items)
        {
            foreach ($items as $i => $item)
            {
                $repeat = $item->level - 1 >= 0 ? $item->level - 1 : 0;
                $item->title = str_repeat('- ', $repeat) . $item->title;
                $disable = isset($items[$i + 1]) && $items[$i + 1]->level > $item->level;
                $options[] = JHtml::_('select.option', $item->id, $item->title, 'value', 'text', $disable);
            }
        }

        return $options;
    }
}