<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2018 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

class plgActionlogRsdirectoryInstallerScript
{

	/**
	 * @param $type
	 * @param $parent
	 *
	 * @return bool
	 */
	public function preflight($type, $parent) {
		if ($type == 'uninstall')
		{
			// Must delete RSDirectory! option from the action log extensions 
            $db = JFactory::getDbo();
			try {
				$query = $db->getQuery(true);
				$query->delete($db->qn('#__action_logs_extensions'))
					  ->where($db->qn('extension') .' = '.$db->q('com_rsdirectory'));

				$db->setQuery($query);
				if ($deleted = $db->execute()) 
				{
					$com_actionlogs = JComponentHelper::getComponent('com_actionlogs');
					$params = $com_actionlogs->getParams();

					$loggableExtensions = $params->get('loggable_extensions', array());
					
					if (in_array('com_rsdirectory', $loggableExtensions))
					{
						$loggableExtensions = array_diff($loggableExtensions, array('com_rsdirectory'));
						$loggableExtensions = array_values($loggableExtensions);
						
						// set the new loggable extensions without com_rsdirectory
						$params->set('loggable_extensions', $loggableExtensions);
						
						$table = JTable::getInstance('extension');
						$table->load($com_actionlogs->id);
						$table->bind(array('params' => $params->toString()));
						$table->store();
					}
				}
			} 
			catch (RuntimeException $e)
			{
				JFactory::getApplication()->enqueueMessage($e->getMessage());
				return false;
			}
			
			return true;
		}
		try
		{
			if (!file_exists(JPATH_ADMINISTRATOR.'/components/com_rsdirectory/helpers/version.php'))
			{
				throw new Exception('Please install RSDirectory! first before you attempt to install this plugin.');
				
			}
			
			// Usable only in Joomla 3.9 and above
			$jversion = new JVersion();

			if (!$jversion->isCompatible('3.9.0'))
			{
				throw new Exception('This plugin works only with Joomla 3.9 and above versions!');
			}
		} 
		catch (Exception $e)
		{
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
			return false;
		}

		if ($type == 'update')
		{
			if (class_exists('RSDirectoryVersion'))
			{
				// make some updates from old version
				if (version_compare((string) RSDirectoryVersion::$version, '2.0.0', '<')) {
					// check to see if the database can support regex_replace
					$db = JFactory::getDbo();
					$db_version= $db->getVersion();
					$db_server_type = $db->getServerType();

					if ($db_server_type == 'mysql') {
						$db_type = stripos($db_version,'-maria') !== false ? 'maria' : 'sql';

						$can_run = false;

						switch($db_type) {
							case 'maria':
								$clean_version = explode('-', $db_type, 1);
								if (!version_compare((string) $clean_version, '10.0.5', '<'))
								{
									$can_run = true;
								}
							break;

							case 'sql':
								if (!version_compare((string) $db_version, '8.0', '<'))
								{
									$can_run = true;
								}
							break;
						}


						if ($can_run) {
							// try and clean the previous logs
							try
							{
								$db->setQuery("UPDATE `#__action_logs` SET `message` = REGEXP_REPLACE(`message`, ".$db->q('\\\\">(.+)<\\\\/a>').", '') WHERE `extension`='com_rsdirectory'");
								$db->execute();

								$db->setQuery("UPDATE `#__action_logs` SET `message` = REGEXP_REPLACE(`message`, ".$db->q("<a href=\\\\\"\\\\\\".JUri::root(true)."\\\\/administrator\\\\/").", '') WHERE `extension`='com_rsdirectory'");
								$db->execute();
							}
							catch (Exception $e) {}
						}
					}
				}
			}
		}

		return true;
	}
	
	 public function postflight($type, $parent)
    {
        if ($type == 'uninstall')
		{
            return true;
		}
        
        // Process install. 
        if ($type == 'install')
        {
			// Must insert in the action log extensions the RSDirectory! option
            $db = JFactory::getDbo();
			try {
				// If it fails, it will throw a RuntimeException
				$object = (object) array(
					'extension' => 'com_rsdirectory'
				);
				
				$db->insertObject('#__action_logs_extensions', $object);
			} 
			catch (RuntimeException $e)
			{
				JFactory::getApplication()->enqueueMessage($e->getMessage());
				return false;
			}
        }
    }
}