<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2020 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

class com_rsdirectoryInstallerScript
{
    /**
     * An array of plugins to be installed with the component.
     *
     * @var array
     *
     * @access private
     *
     * @static
     */
    private static $defaultPlugins = array(
        'plg_rsdirectory' => array(
            'name' => 'System Plugin ...',
            'min_version' => '1.1.5',
            'core' => true
        ),
        'plg_rsdirectorywiretransfer' => array(
            'name' => 'RSDirectory! Wire Transfer Payment Plugin ...',
            'min_version' => '1.2.0',
        ),
        'plg_rsdirectory_importcsv' => array(
			'element' => 'importcsv',
            'name' => 'RSDirectory! CSV Import Plugin ...',
            'group' => 'rsdirectory',
            'min_version' => '1.1.0',
        ),
        'plg_rsdirectoryprivacy' => array(
            'name' => 'RSDirectory! Privacy Plugin ...',
            'min_version' => '1.0.0',
            'group' => 'privacy',
            'element' => 'rsdirectory'
        ),
        'plg_rsdirectoryactionlog' => array(
            'name' => 'Action Log - RSDirectory! Plugin ...',
            'min_version' => '1.1.0',
            'group' => 'actionlog',
            'element' => 'rsdirectory'
        ),
        'plg_installer' => array(
            'name' => 'Installer - RSDirectory! ...',
            'min_version' => '1.0.0',
            'group' => 'installer',
            'element' => 'rsdirectory',
            'core' => true
        ),
		'plg_rsdirectoryimportcsv' => array(
            'name' => 'RSDirectory! CSV Import Plugin ...',
            'group' => 'system',
			'check_new' => 'plg_rsdirectory_importcsv',
			'old' => true
        )
    );

    /**
     * An array of other RSDirectory! plugins.
     *
     * @var array
     *
     * @access private
     *
     * @static
     */
    private static $otherPlugins = array(
        'rsdirectory2co' => array(
            'name' => 'RSDirectory! 2Checkout Payment Plugin ...',
            'min_version' => '1.2.0'
        ),
        'rsdirectoryauthorize' => array(
            'name' => 'RSDirectory! Authorize Payment Plugin ...',
            'min_version' => '1.2.0'
        ),
        'rsdirectoryideal' => array(
            'name' => 'RSDirectory! iDEAL Payment Plugin ...',
            'min_version' => '1.1.0'
        ),
        'rsdirectorypayfast' => array(
            'name' => 'RSDirectory! PayFast Payment Plugin ...',
            'min_version' => '1.1.0'
        ),
        'rsdirectorypaypal' => array(
            'name' => 'RSDirectory! PayPal Payment Plugin ...',
            'min_version' => '1.2.0'
        ),
        'rsdirectoryfinder' => array(
            'name' => 'Smart Search - RSDirectory! ...',
            'min_version' => '1.1.0'
        ),
		'importmosetstree' => array(
            'name' => 'RSDirectory! Mosets Tree Import Plugin ...',
            'group' => 'rsdirectory',
            'min_version' => '1.1.0'
        ),
        'importsobipro' => array(
            'name' => 'RSDirectory! SobiPro Import Plugin ...',
            'group' => 'rsdirectory',
            'min_version' => '1.1.0'
        ),
		
		// old plugins
		'rsdirectoryimportmosetstree' => array(
            'name' => 'RSDirectory! Mosets Tree Import Plugin ...',
			'group' => 'system',
			'old' => true
        ),
		'rsdirectoryimportsobipro' => array(
            'name' => 'RSDirectory! SobiPro Import Plugin ...',
            'group' => 'system',
			'old' => true
        )
    );

    /**
     * The install hook.
     *
     * @access public
     *
     * @param object $parent
     */
    public function install($parent)
    {
    }

    /**
     * The uninstall hook.
     *
     * @access public
     *
     * @param object $parent
     */
    public function uninstall($parent)
    {
        // Get the database object.
        $db = JFactory::getDbo();

        // Get all the RSDirectory! plugins.
        $plugins = array_merge(self::$defaultPlugins, self::$otherPlugins);

        // Uninstall plugins.
        if ($plugins)
        {
            foreach ($plugins as $plugin => $data)
            {
                $group = isset($data['group']) ? $data['group'] : 'system';
                $element = isset($data['element']) ? $data['element'] : str_replace('plg_', '', $plugin);

                $query = $db->getQuery(true)
                    ->select( $db->qn('extension_id') )
                    ->from( $db->qn('#__extensions') )
                    ->where( $db->qn('element') . '=' . $db->q($element) )
                    ->where( $db->qn('folder') . '=' . $db->q($group) )
                    ->where( $db->qn('type') . '=' . $db->q('plugin') );

                $db->setQuery($query);
				
                if ( $extension_id = $db->loadResult() )
				{
					// Initialize JInstaller.
                    $plg_installer = new JInstaller();

                    $plg_installer->uninstall('plugin', $extension_id);
				}
            }
        }
    }

    /**
     * The preflight hook.
     *
     * @access public
     *
     * @param string $type
     * @param object $parent
     *
     * @return  mixed
     */
    public function preflight($type, $parent)
    {
        $app 		= JFactory::getApplication();
        $jversion 	= new JVersion();

        // Running Joomla! 2.5
        if (!$jversion->isCompatible('3.7.0'))
        {
            $app->enqueueMessage('RSDirectory! Could not be installed on your Joomla! version. Please consider updating to minimum 3.7.0 version of Joomla! if you\'d like to still use RSDirectory!.', 'error');
            return false;
        }
    }

    /**
     * The postflight hook.
     *
     * @access public
     *
     * @param string $type
     * @param object $parent
     *
     * @return  mixed
     */
    public function postflight($type, $parent)
    {
        if ($type == 'uninstall')
            return true;

        $messages = array();

        // Process update.
        if ($type == 'update')
        {
            $this->processUpdate($type, $parent, $messages);
        }

        // Process plugins.
        $this->processPlugins($type, $parent, $messages);

        // Output install screen.
        $this->outputInstallScreen($messages);
    }

    /**
     * Process update.
     *
     * @access public
     *
     * @param string $type
     * @param object $parent
     * @param array &$messages
     */
    public function processUpdate($type, $parent, &$messages)
    {
        $source = $parent->getParent()->getPath('source');

        // Update config variables
        $this->runSQL($source, 'config.sql');

        $db = JFactory::getDbo();

        // Rename the users table before the install.sql so that the users table is not added
        $db->setQuery( 'SHOW TABLES LIKE ' . $db->q( $db->getPrefix() . 'rsdirectory_users_credits' ) );

        if ( $db->loadResult() )
        {
            $db->setQuery( 'RENAME TABLE ' . $db->qn('#__rsdirectory_users_credits') . ' TO ' . $db->qn('#__rsdirectory_users') );
            $db->execute();
        }

        // Run the install.sql so that new tables or new rows could be added
        $this->runSQL($source, 'install.sql');
		
		// Add default values to the old tables for Joomla 4
		$jversion 	= new JVersion();

        if ($jversion->isCompatible('4.0.0'))
        {
			if ($tables = $db->getTableList())
			{
				foreach ($tables as $table)
				{
					if (strpos($table, $db->getPrefix() . 'rsdirectory') !== false)
					{
						if ($fields = $db->getTableColumns($table, false))
						{
							foreach ($fields as $field)
							{
								$fieldType = strtolower($field->Type);
								$fieldKey = strtolower($field->Key);

								if (!is_null($field->Default))
								{
									continue;
								}

								if (strpos($fieldType, 'int') !== false || strpos($fieldType, 'float') !== false || strpos($fieldType, 'decimal') !== false)
								{
									if ($fieldKey != 'pri')
									{
										$db->setQuery('ALTER TABLE ' . $db->qn($table) . ' ALTER ' . $db->qn($field->Field) . ' SET DEFAULT ' . $db->q(0));
										$db->execute();
									}
								} 
								elseif (strpos($fieldType, 'varchar') !== false)
								{
									$db->setQuery('ALTER TABLE ' . $db->qn($table) . ' ALTER ' . $db->qn($field->Field) . ' SET DEFAULT ' . $db->q(''));
									$db->execute();
								}
								elseif (strpos($fieldType, 'datetime') !== false)
								{
									$db->setQuery('ALTER TABLE ' . $db->qn($table) . ' ALTER ' . $db->qn($field->Field) . ' SET DEFAULT ' . $db->q($db->getNullDate()));
									$db->execute();
								}
							}
						}
					}
				}
			}
		}

        // Rename the users > unlimited column.
        $columns = $db->getTableColumns('#__rsdirectory_users');

        if ( isset($columns['unlimited']) )
        {
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_users') . ' CHANGE ' . $db->qn('unlimited') . ' ' . $db->qn('unlimited_credits') . ' TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"' );
            $db->execute();
        }

        // Rename the entries credits > unlimited column / remove the credits column 'unsigned' property
        $columns = $db->getTableColumns('#__rsdirectory_entries_credits');

        if ( isset($columns['unlimited']) )
        {
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_entries_credits') . ' CHANGE ' . $db->qn('unlimited') . ' ' . $db->qn('unlimited_credits') . ' TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"' );
            $db->execute();
        }

        if ( isset($columns['credits']) )
        {
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_entries_credits') . ' CHANGE ' . $db->qn('credits') . ' ' . $db->qn('credits') . ' INT(11) NOT NULL DEFAULT "0"' );
            $db->execute();
        }

        // Process email messages columns.
        $columns = $db->getTableColumns('#__rsdirectory_email_messages');

        if ( isset($columns['to']) )
        {
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_email_messages') . ' CHANGE ' . $db->qn('to') . ' ' . $db->qn('to_email') . ' TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""' );
            $db->execute();
        }

        if ($columns['subject'] != 'text')
        {
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_email_messages') . ' CHANGE ' . $db->qn('subject') . ' ' . $db->qn('subject') . ' TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""' );
            $db->execute();
        }

        $entries_custom_columns = $db->getTableColumns('#__rsdirectory_entries_custom');
        if (!isset($entries_custom_columns['idc']))
        {
            // remove the `entry_id` old Primary Key
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_entries_custom') . ' DROP PRIMARY KEY' );
            $db->execute();

            // add the new `idc` as the Primary Key
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_entries_custom') . ' ADD ' . $db->qn('idc') . ' INT NOT NULL AUTO_INCREMENT FIRST, ADD PRIMARY KEY (' . $db->qn('idc') . ')' );
            $db->execute();

            // set the `entry_id` column as UNIQUE
            $db->setQuery( 'ALTER TABLE ' . $db->qn('#__rsdirectory_entries_custom') . ' ADD CONSTRAINT ' . $db->qn('entry_id') . ' UNIQUE KEY(`entry_id`)' );
            $db->execute();
        }

        // Update tables.
        $tables = array(
            '#__rsdirectory_entries' => array(
                'promoted_expiry' => array(
                    'alter' => 'DATETIME NOT NULL DEFAULT "0000-00-00 00:00:00" AFTER `promoted`',
                ),
                'paid' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'expiration_notice_time' => array(
                    'alter' => 'DATETIME NOT NULL DEFAULT "0000-00-00 00:00:00"',
                ),
                'language' => array(
                    'alter' => 'VARCHAR(7) DEFAULT "*"',
                ),
                'activation_code' => array(
                    'alter' => 'VARCHAR(100) NOT NULL DEFAULT ""',
                ),
                // metadata
                'metadata_title' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'metadata_keywords' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'metadata_description' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                // new promoted field
                'promoted_selected' => array(
                    'alter' => 'VARCHAR(20) NOT NULL DEFAULT "-1"',
                ),
                'promoted_period' => array(
                    'alter' => 'INT(11) UNSIGNED NOT NULL DEFAULT "0"',
                )
            ),

            '#__rsdirectory_entries_credits' => array(
                'paid' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'is_diff' => array(
                    'alter' => 'TINYINT(1) NOT NULL DEFAULT "0" AFTER `credits`',
                )
            ),

            '#__rsdirectory_email_messages' => array(
                'entry_expiration_period' => array(
                    'alter' => 'INT( 11 ) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'description' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'to_name' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'send_html' => array(
                    'alter' => 'TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT "0"',
                ),
            ),

            '#__rsdirectory_entries_reported' => array(
                'token' => array(
                    'alter' => 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
            ),

            '#__rsdirectory_field_types' => array(
                'all_forms' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
            ),

            '#__rsdirectory_forms' => array(
                'listing_detail_show_contact' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'contact_from_name' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                    'update' => '{contact.name}',
                ),
                'contact_from_email' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                    'update' => '{contact.email}',
                ),
                'contact_to_name' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                    'update' => '{name}',
                ),
                'contact_to_email' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                    'update' => '{email}',
                ),
                'contact_cc' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'contact_bcc' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'contact_subject' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                    'update' => 'Contact message from your entry {title}',
                ),
                'contact_send_html' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'contact_message' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                    'update' => '<p>From: {contact.name} ({contact.email})</p><p>Message body:<br />{contact.message}</p><p>---<br />This email was sent via the contact form on <a href="{url}">{title}</a></p>',
                ),
                'listing_row_show_category_hierarchy' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'listing_detail_show_category_hierarchy' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                // snippet stuff
                'use_entry_snippet' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'use_entry_snippet_auto' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'entry_snippet_sku' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'entry_snippet_identifier' => array(
                    'alter' => 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT "none"',
                ),
                'entry_snippet_identifier_placeholder' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'entry_snippet_offer_item_condition' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'entry_snippet_offer_availability' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'entry_snippet' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),

                // metadata
                'metadata_title' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'metadata_keywords' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'metadata_description' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                )
            ),

            '#__rsdirectory_groups' => array(
                'add_entry_captcha' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'can_vote_own_entries' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'can_contact_entries_authors' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
                'contact_captcha' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'review_captcha' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'can_change_category' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                    'update' => '1'
                ),
                'report_captcha' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "0"',
                ),
                'initial_credits' => array(
                    'alter' => 'INT(11) UNSIGNED NOT NULL DEFAULT "0"',
                ),
            ),

            '#__rsdirectory_reviews' => array(
                'name' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'email' => array(
                    'alter' => 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'owner_reply' => array(
                    'alter' => 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
                'token' => array(
                    'alter' => 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT ""',
                ),
            ),

            '#__rsdirectory_users' => array(
                'enable_contact_form' => array(
                    'alter' => 'TINYINT(1) UNSIGNED NOT NULL DEFAULT "1"',
                ),
            ),

            '#__rsdirectory_users_transactions' => array(
                'entry_id' => array(
                    'alter' => 'INT(11) UNSIGNED NOT NULL DEFAULT "0"',
                ),
            ),
        );

        foreach ($tables as $table => $columns)
        {
            $existing_columns = $db->getTableColumns($table);

            foreach ($columns as $column => $type)
            {
                if ( !isset($existing_columns[$column]) )
                {
                    if ( isset($type['alter']) )
                    {
                        $db->setQuery( 'ALTER TABLE ' . $db->qn($table) . ' ADD ' . $db->qn($column) . ' ' . $type['alter'] );
                        $db->execute();
                    }

                    if ( isset($type['update']) )
                    {
                        $query = $db->getQuery(true)
                            ->update( $db->qn($table) )
                            ->set( $db->qn($column) . ' = ' . $db->q($type['update']) );

                        $db->setQuery($query);
                        $db->execute();
                    }
                }
            }
        }

        // change the price field from FLOAT to DECIMAL(20,5)
        $rsdirectory_entries_columns = $db->getTableColumns('#__rsdirectory_entries', false);
        if (isset($rsdirectory_entries_columns['price']) && $rsdirectory_entries_columns['price']->Type != "decimal(20,5) unsigned")
        {
            $db->setQuery('ALTER TABLE ' . $db->qn('#__rsdirectory_entries') . ' CHANGE ' . $db->qn('price') . ' ' . $db->qn('price') . ' DECIMAL(20,5) UNSIGNED NOT NULL DEFAULT "0"');
            $db->execute();
        }

        $keys = array('promoted', 'renew', 'published_time', 'expiry_time', 'promoted_expiry', 'avg_rating', 'created_time', 'modified_time', 'new', 'paid', 'published');
        foreach ($keys as $key)
        {
            if ($rsdirectory_entries_columns[$key]->Key != 'MUL')
            {
                $db->setQuery('ALTER TABLE ' . $db->qn('#__rsdirectory_entries') . ' ADD INDEX(' . $db->qn($key) . ')')->execute();
            }
        }

        //check for previous keys on the email_messages table
        $db->setQuery('SHOW INDEX FROM '.$db->qn('#__rsdirectory_email_messages'));
        $keys = $db->loadAssocList();
        if (!empty($keys)) {
            $already_deleted = array();
            foreach ($keys as $key){
                if ($key['Key_name'] == 'category_id' && !in_array('category_id', $already_deleted)){
                    $db->setQuery('ALTER TABLE '.$db->qn('#__rsdirectory_email_messages').' DROP INDEX '.$db->qn('category_id'));
                    $db->execute();

                    $already_deleted[] = 'category_id';
                } else {
                    continue;
                }
            }
        }

        // Run the install_data.sql so that new rows could be added
        $this->runSQL($source, 'install_data.sql');

        $this->runSQL($source, 'import_tmp.structure.sql');
        $this->runSQL($source, 'restore_tmp.structure.sql');
        $this->runSQL($source, 'tokens.structure.sql');

        // Add image upload field.
        $query = $db->getQuery(true)
            ->select('COUNT(*)')
            ->from( $db->qn('#__rsdirectory_field_types') )
            ->where( $db->qn('type') . ' = ' . $db->q('image_upload') );

        $db->setQuery($query, 0, 1);

        if ( !$db->loadResult() )
        {
            // Get the position of the file upload field.
            $query = $db->getQuery(true)
                ->select( $db->qn('ordering') )
                ->from( $db->qn('#__rsdirectory_field_types') )
                ->where( $db->qn('type') . ' = ' . $db->q('fileupload') );

            $db->setQuery($query);
            $odering = $db->loadResult();

            // Increment the positions of all field types above the file upload field.
            $query = $db->getQuery(true)
                ->update( $db->qn('#__rsdirectory_field_types') )
                ->set( $db->qn('ordering') . ' = ' . $db->qn('ordering') . ' + ' . $db->q(1) )
                ->where( $db->qn('ordering') . ' > ' . $db->q($odering) );

            $db->setQuery($query);
            $db->execute();

            // Insert the image upload field.
            $object = (object)array(
                'type' => 'image_upload',
                'xml_file' => 'image_upload',
                'core' => 0,
                'always_published' => 0,
                'create_column' => 0,
                'expect_value' => 1,
                'ordering' => $odering + 1,
            );

            $db->insertObject('#__rsdirectory_field_types', $object);
        }

        // Add link field.
        $query = $db->getQuery(true)
            ->select('COUNT(*)')
            ->from( $db->qn('#__rsdirectory_field_types') )
            ->where( $db->qn('type') . ' = ' . $db->q('link') );

        $db->setQuery($query, 0, 1);

        if ( !$db->loadResult() )
        {
            // Get the position of the youtube field.
            $query = $db->getQuery(true)
                ->select( $db->qn('ordering') )
                ->from( $db->qn('#__rsdirectory_field_types') )
                ->where( $db->qn('type') . ' = ' . $db->q('youtube') );

            $db->setQuery($query);
            $odering = $db->loadResult();

            // Increment the positions of all field types above the youtube field.
            $query = $db->getQuery(true)
                ->update( $db->qn('#__rsdirectory_field_types') )
                ->set( $db->qn('ordering') . ' = ' . $db->qn('ordering') . ' + ' . $db->q(1) )
                ->where( $db->qn('ordering') . ' > ' . $db->q($odering) );

            $db->setQuery($query);
            $db->execute();

            // Insert the image upload field.
            $object = (object)array(
                'type' => 'link',
                'xml_file' => 'link',
                'core' => 0,
                'always_published' => 0,
                'create_column' => 1,
                'expect_value' => 1,
                'ordering' => $odering + 1,
            );

            $db->insertObject('#__rsdirectory_field_types', $object);
        }


        // Add email messages data.
        $query = $db->getQuery(true)
            ->select('COUNT(*)')
            ->from( $db->qn('#__rsdirectory_email_messages') );

        $db->setQuery($query, 0, 1);

        if ( !$db->loadResult() )
        {
            $this->runSQL($source, 'email_messages.data.sql');
        }

        // Add entry_expiration_renew email if it's not present
        $query = $db->getQuery(true)
            ->select('COUNT(*)')
            ->from( $db->qn('#__rsdirectory_email_messages') )
            ->where( $db->qn('type') .' = ' .$db->q('entry_expiration_renew'));

        $db->setQuery($query, 0, 1);

        if (!$db->loadResult()) {
            // Insert the proper email up.
            $object = (object)array(
                'description' => 'Message to entry author 48 hours prior of entry expiration if autorenew and no credits left',
                'type' => 'entry_expiration_renew',
                'entry_expiration_period' => 48,
                'to_name' => '{name}',
                'to_email' => '{email}',
                'subject' => 'Your entry "{title}" will expire on {expiry-date} due tu insufficient credits',
                'send_html' => 1,
                'text' => '<p>Hello {name}!</p>\r\n<p>Your entry, <a href="{url}">{title}</a>, will expire on {expiry-date} due to insufficient credits.</p>\r\n<p>Kind Regards,<br /><a href="{site.url}">{site.name}</a> staff</p>'
            );

            $db->insertObject('#__rsdirectory_email_messages', $object);
        }

        // Set all_forms.
        $query = $db->getQuery(true)
            ->update( $db->qn('#__rsdirectory_field_types') )
            ->set( $db->qn('all_forms') . ' = ' . $db->q(1) )
            ->where( $db->qn('type') . ' IN (' . $this->quoteImplode( array('title', 'publishing_period') ) . ')' );

        $db->setQuery($query);
        $db->execute();


        // Refine the promoted field properties
        $query = $db->getQuery(true)
            ->select('field_id')
            ->from( $db->qn('#__rsdirectory_fields_properties') )
            ->where( $db->qn('property_name') . ' = ' . $db->q('promoted_period') );

        $db->setQuery($query, 0, 1);

        if ( $field_id = $db->loadResult() )
        {
            $query = $db->getQuery(true)
                ->select('value')
                ->from( $db->qn('#__rsdirectory_fields_properties') )
                ->where( $db->qn('field_id').'='.$db->q($field_id) )
                ->where( $db->qn('property_name') . ' = ' . $db->q('credits') );

            $db->setQuery($query, 0, 1);

            $credits = $db->loadResult();

            $query = $db->getQuery(true)
                ->select('value')
                ->from( $db->qn('#__rsdirectory_fields_properties') )
                ->where( $db->qn('field_id').'='.$db->q($field_id) )
                ->where( $db->qn('property_name') . ' = ' . $db->q('promoted_period') );

            $db->setQuery($query, 0, 1);

            $promoted_period = $db->loadResult();

            // If there is one of these set add it as an item to the promoted field
            if (!empty($promoted_period) || !empty($credits))
            {
                $items = array(
                    (object)array(
                        'period' => (int)$promoted_period,
                        'credits' => (int) $credits,
                    )
                );

                $value = serialize($items);

                // this property should not exist yet
                $query = $db->getQuery(true)
                    ->select('value')
                    ->from( $db->qn('#__rsdirectory_fields_properties') )
                    ->where( $db->qn('field_id').'='.$db->q($field_id) )
                    ->where( $db->qn('property_name') . ' = ' . $db->q('items') );

                $db->setQuery($query, 0, 1);

                if (empty($db->loadResult()))
                {
                    $q_items = (object)array(
                        'field_id' => $field_id,
                        'property_name' => 'items',
                        'value' => $value
                    );

                    $db->insertObject('#__rsdirectory_fields_properties', $q_items);
                }
            }

            // remove these, as we do not use them anymore
            $query 	= $db->getQuery(true);
            $query->delete('#__rsdirectory_fields_properties')
                ->where( $db->qn('field_id').'='.$db->q($field_id) )
                ->where( '('.$db->qn('property_name').'='.$db->q('promoted_period') . ' OR ' . $db->qn('property_name').'='.$db->q('credits').' )' );
            $db->setQuery($query)->execute();
        }
    }

    /**
     * Process plugins.
     *
     * @access public
     *
     * @param string $type
     * @param object $parent
     * @param array &$messages
     */
    public function processPlugins($type, $parent, &$messages)
    {
        $source = $parent->getParent()->getPath('source');

        // Get the database object.
        $db = JFactory::getDbo();

        $messages['items'] = array(
            'installed' => array(),
            'disabled' => array(),
        );

        // Install default plugins.
        if (self::$defaultPlugins)
        {
            foreach (self::$defaultPlugins as $plugin => $data)
            {
                // Initialize JInstaller.
                $installer = new JInstaller();
				
				if (empty($data['old']))
				{
					if ( $installer->install("$source/other/$plugin") )
					{
						$group = isset($data['group']) ? $data['group'] : 'system';
						$element = isset($data['element']) ? $data['element'] : str_replace('plg_', '', $plugin);
						$core = !empty($data['core']);
	 
						if ($type == 'install' || $core)
						{
							$query = $db->getQuery(true)
								->update('#__extensions')
								->set( $db->qn('enabled') . '=' . $db->q(1) )
								->where( $db->qn('element') . '=' . $db->q($element) )
								->where( $db->qn('folder') . '=' . $db->q($group) )
								->where( $db->qn('type') . '=' . $db->q('plugin') );
	 
							$db->setQuery($query);
							$db->execute();
						}
	 
						$messages['items']['installed'][] = (object)array(
							'name' => $data['name'],
							'status' => 'ok',
							'text' => 'Installed',
						);
					}
					else
					{
						$messages['items']['installed'][] = (object)array(
							'name' => $data['name'],
							'status' => 'not-ok',
							'text' => 'Error installing!',
						);
					}
				}
				// remove the old plugins
				else if ($data['old'] === true)
                {
                    $element = isset($data['element']) ? $data['element'] : str_replace('plg_', '', $plugin);
                    $query = $db->getQuery(true)
                        ->select( $db->qn('extension_id') )
                        ->from( $db->qn('#__extensions') )
                        ->where( $db->qn('element') . '=' . $db->q($element) )
                        ->where( $db->qn('folder') . '=' . $db->q($data['group']) )
                        ->where( $db->qn('type') . '=' . $db->q('plugin') );

                    $db->setQuery($query);

                    if ( $extension_id = $db->loadResult() )
                    {
                        // keep the current status
                        $query = $db->getQuery(true)
                            ->select( $db->qn('enabled') )
                            ->from( $db->qn('#__extensions') )
                            ->where( $db->qn('extension_id') . '=' . $db->q($extension_id) );

                        $db->setQuery($query);
                        $enabled = $db->loadResult();
                        
                        // Uninstall the plugins
                        if ($installer->uninstall('plugin', $extension_id))
                        {
                            $messages['items']['uninstalled'][] = (object)array(
                                'name' => $data['name'],
                                'status' => 'ok',
                                'text' => 'Uninstalled old '.$data['group'].' plugin.',
                            );
							
							// if there is a new plugin reference add the enabled status to it
							if (!empty($data['check_new']))
							{
								$reference_plugin = isset(self::$defaultPlugins[$data['check_new']]) ? self::$defaultPlugins[$data['check_new']] : null;
								if (!empty($reference_plugin))
								{
									$element = isset($reference_plugin['element']) ? $reference_plugin['element'] : str_replace('plg_', '', $data['check_new']);
									$group = isset($reference_plugin['group']) ? $reference_plugin['group'] : '';
									if ($enabled && !empty($group))
									{
										$query = $db->getQuery(true)
											->update('#__extensions')
											->set( $db->qn('enabled') . '=' . $db->q(1) )
											->where( $db->qn('element') . '=' . $db->q($element) )
											->where( $db->qn('folder') . '=' . $db->q($group) )
											->where( $db->qn('type') . '=' . $db->q('plugin') );

										$db->setQuery($query);
										$db->execute();
									}
								}
							}
                        } 
                        else
                        {
                            $messages['items']['uninstalled'][] = (object)array(
                                'name' => $data['name'],
                                'status' => 'not-ok',
                                'text' => 'Error uninstalling!',
                            );
                        }
                    }
                }
            }
        }

        // Check other plugins.
        if ( $installedPlugins = $this->getPlugins( array_keys(self::$otherPlugins) ) )
        {
            foreach ($installedPlugins as $plugin)
            {
                $data = self::$otherPlugins[$plugin->element];

                $file = JPATH_SITE . '/plugins/' . $plugin->folder . '/' . $plugin->element . '/' . $plugin->element . '.xml';

                if ( file_exists($file) )
                {
					$disable_pugin = false;
					
					// for the old plugins do not check the version, just disable them
					if (!empty($data['old']) && $data['old'] === true)
					{
						$disable_pugin = true;
					}
					else
					{
						$xml = file_get_contents($file);

						preg_match('/<version>(.*)<\/version>/', $xml, $matches);

						if ( !isset($matches[1]) || version_compare($matches[1], $data['min_version']) == -1 )
						{
							$disable_pugin = true;
						}
					}
					
					if ($disable_pugin)
					{
						$this->disableExtension($plugin->extension_id);

						$messages['items']['disabled'][] = (object)array(
							'name' => $data['name'],
							'status' => 'not-ok',
							'text' => 'Disabled'
						);
					}
                }
            }
        }
    }
	
	protected function escape($string) {
		return htmlentities($string, ENT_COMPAT, 'utf-8');
	}

    /**
     * Ouput install message.
     *
     * @access private
     *
     * @param object $messages
     */
    private function outputInstallScreen($messages)
    {
        // Get the database object.
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
            ->select( $db->qn('value') )
            ->from( $db->qn('#__rsdirectory_config') )
            ->where( $db->qn('name') . ' = ' . $db->q('credit_cost') );

        $db->setQuery($query);
        $credit_cost = $db->loadResult();

        ?>

        <style type="text/css">
            .installer-title {
                font-size: 18px;
                line-height: 20px;
                margin: 12px 0;
                padding-bottom: 5px;
                border-bottom: 1px solid #999;
            }

            .install-ok {
                background: #7dc35b;
                color: #fff;
                padding: 3px;
            }

            .install-not-ok {
                background: #E9452F;
                color: #fff;
                padding: 3px;
            }

            .uninstall-ok {
                background: #c38306;
                color: #fff;
                padding: 3px;
            }

            .uninstall-not-ok {
                background: #E9452F;
                color: #fff;
                padding: 3px;
            }

            .version-history {
                margin: 0 0 1em 0;
                padding: 0;
                list-style-type: none;
            }

            .version-history > li {
                margin: 0 0 0.5em 0;
                padding: 0 0 0 4em;
            }

            .version-new,
            .version-fixed,
            .version-upgraded {
                float: left;
                font-size: 0.8em;
                margin-left: -4.9em;
                width: 4.5em;
                color: white;
                text-align: center;
                font-weight: bold;
                text-transform: uppercase;
                -webkit-border-radius: 4px;
                -moz-border-radius: 4px;
                border-radius: 4px;
            }

            .version-new {
                background: #7dc35b;
            }

            .version-fixed {
                background: #e9a130;
            }

            .version-upgraded {
                background: #61b3de;
            }

            .big-warning {
                background: #FAF0DB;
                border: solid 1px #EBC46F;
                padding: 5px;
            }

            .big-warning strong {
                color: red;
            }

            .big-info {
                background: #D5FFDC;
                border: solid 1px #EBC46F;
                padding: 5px;
            }
			
			.rsdirectory-row {
				width: 100%;
				display: block;
				margin-bottom: 2%;
			}

			.rsdirectory-row:after {
				clear: both;
				display: block;
				content: "";
			}

			.rsdirectory-column-2 {
				width: 19%;
				margin-right: 1%;
				float: left;
			}

			.rsdirectory-column-10 {
				width: 80%;
				float: left;
			}
        </style>
        <div class="rsdirectory-row">
            <div class="rsdirectory-column-2 center text-center">
				<?php echo JHtml::_('image', 'com_rsdirectory/rsdirectory-box.png', 'RSTickets! Pro Box', array(), true); ?>
            </div>
            <div class="rsdirectory-column-10">
                <?php if ( !empty($messages['items']['installed']) ) { ?>
                    <?php foreach ($messages['items']['installed'] as $item) { ?>
						<p><?php echo $this->escape($item->name); ?> ...
							<b class="install-<?php echo $item->status; ?>"><?php echo $item->text; ?></b>
						</p>
                    <?php } ?>
                <?php } ?>

                <?php if ( !empty($messages['items']['disabled']) ) { ?>
                    <p class="big-warning"><strong>Warning!</strong> The following plugins have been temporarily disabled to prevent any errors being shown on your website. Please <a href="https://www.rsjoomla.com/downloads.html">download the latest versions</a> from your account and update your installation before enabling them.</p>
                    <?php foreach ($messages['items']['disabled'] as $item) { ?>
						<p><?php echo $this->escape($item->name); ?> ...
							<b class="install-<?php echo $item->status; ?>"><?php echo $item->text; ?></b>
						</p>
                    <?php } ?>
                <?php } ?>

                <?php if ( !empty($messages['items']['uninstalled']) ) { ?>
                    <?php foreach ($messages['items']['uninstalled'] as $item) { ?>
                        <p><?php echo $this->escape($item->name); ?> ...
                            <b class="uninstall-<?php echo $item->status; ?>"><?php echo $item->text; ?></b>
                        </p>
                    <?php } ?>
                <?php } ?>
				
				<h2>Changelog v2.0.4</h2>
                <ul class="version-history">
                    <li><span class="version-fixed">Fix</span> The "Total Credits" message was sometimes loaded incorrectly when an entry was submitted.</li>
                </ul>

                <p>Click <a href="https://www.rsjoomla.com/support/documentation/rsdirectory/changelog/rsdirectory-changelog.html" target="_blank">here</a> to access the full changelog.</p>

                <p>
                    <a class="btn btn-large btn-primary" href="index.php?option=com_rsdirectory">Start using RSDirectory!</a>
                    <a class="btn btn-large btn-secondary" href="https://www.rsjoomla.com/support/documentation/rsdirectory.html" target="_blank">Read the RSDirectory! User Guide</a>
                    <a class="btn btn-large btn-secondary" href="https://www.rsjoomla.com/support.html" target="_blank">Get Support!</a>
                </p>

                <?php if ( empty($credit_cost) ) { ?>
                    <div class="big-warning"><strong>Warning!</strong> The credit cost is set to 0. You should change this, if you wish to use the credits functionality, by accessing the Payments tab on the Configuration page.</div>
                <?php } ?>

            </div>
        </div>

        <?php
    }

    /**
     * Get plugins data.
     *
     * @access private
     *
     * @param array $plugins
     *
     * @return mixed
     */
    private function getPlugins($plugins)
    {
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
            ->select('*')
            ->from('#__extensions')
            ->where( $db->qn('type') . '=' . $db->q('plugin') )
            ->where( $db->qn('folder') . ' IN (' . $this->quoteImplode( array('rsdirectory', 'search', 'system') ) . ')' )
            ->where( $db->qn('element') . ' IN (' . $this->quoteImplode($plugins) . ')' );

        $db->setQuery($query);

        return $db->loadObjectList();
    }

    /**
     * Disable extension.
     *
     * @access private
     *
     * @param int $extension_id
     */
    private function disableExtension($extension_id)
    {
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
            ->update('#__extensions')
            ->set( $db->qn('enabled') . '=' . $db->q(0) )
            ->where( $db->qn('extension_id') . '=' . $db->q($extension_id) );

        $db->setQuery($query);
        $db->execute();
    }

    /**
     * Quote an array of values.
     *
     * @access private
     *
     * @param array $array
     *
     * @return string
     */
    private function quoteImplode($array)
    {
        $db = JFactory::getDbo();

        foreach ($array as &$value)
        {
            $value = $db->q($value);
        }

        return implode(',', $array);
    }

    /**
     * Load and execute a SQL file.
     *
     * @access private
     *
     * @param string $source
     * @param string $file
     */
    private function runSQL($source, $file)
    {
        $db = JFactory::getDbo();
        $driver = strtolower($db->name);

        if (strpos($driver, 'mysql') !== false)
        {
            $driver = 'mysql';
        }
        else if ($driver == 'sqlsrv')
        {
            $driver = 'sqlazure';
        }

        $sqlfile = $source . '/admin/sql/' . $driver . '/' . $file;

        if ( file_exists($sqlfile) )
        {
            $buffer = file_get_contents($sqlfile);

            if ($buffer !== false)
            {
                $queries = $db->splitSql($buffer);

                foreach ($queries as $query)
                {
                    $db->setQuery($query);

                    if ( !$db->execute() )
                    {
                        JFactory::getApplication()->enqueueMessage(JText::sprintf( 'JLIB_INSTALLER_ERROR_SQL_ERROR', $db->stderr(true) ), 'warning');
                    }
                }
            }
        }
    }
}