<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Entry controller.
 */
class RsdirectoryControllerEntryReport extends JControllerForm
{
    /**
     * Method to save a record.
     *
     * @access public
     *
     * @param string $key The name of the primary key of the URL variable.
     * @param string $urlVar The name of the URL variable if different from the primary key (sometimes required to avoid router collisions).
     *
     * @return bool True if successful, false otherwise.
     */
    public function save($key = null, $urlVar = null)
    {

        // Check for request forgeries.
        $this->checkToken();
			
		// Get the mainframe.
        $app = JFactory::getApplication();
			
		 // Get the data.
        $data = $app->input->get( 'jform', array(), 'array' );

        // RSDirectory configuration
        $config = RSDirectoryConfig::getInstance();

			
		// Get the Entry model.
        $model = $this->getModel();
            
        // Get the form.
        $form = $model->getForm();
            
        // Validate the posted data.
        $return = $model->validate($form, $data);

        if ($config->get('use_consent_checkbox_on_report_form', 1)) {
            // Check the consent if user is not logged in
            if (!JFactory::getUser()->id && !$app->input->getInt('i_consent_to_privacy_policy_and_submit_my_details', 0)) {
                $app->enqueueMessage(JText::_('COM_RSDIRECTORY_CONSENT_DEFAULT_ERROR'), 'error');
                $return = false;
            }
        }
            
        // Check for validation errors.
        if ($return === false)
        {
            // Save the data in the session.
            $app->setUserState('com_rsdirectory.edit.entryreport.data', $data);
                
            // Redirect back to the edit screen.
            $app->redirect( JUri::getInstance()->toString() );
        }
			
		// Attempt to save the configuration.
        $data = $return;
        $return = $model->save($data);
            
            
        // Check the return value.
        if ($return === false)
        {
            // Save the data in the session.
            $app->setUserState('com_rsdirectory.edit.entryreport.data', $data);
                
            $message = JText::sprintf( 'COM_RSDIRECTORY_ENTRY_REPORT_FAILED', $model->getError() );
                
            // Redirect back to the edit screen.
            $app->enqueueMessage($message, 'error');
            $app->redirect( JUri::getInstance()->toString() );
        }
			
		$app->enqueueMessage( JText::_('COM_RSDIRECTORY_ENTRY_REPORTED') );
		$app->redirect( JUri::getInstance()->toString() );
			
		return true;
	}
}