<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Entries view.
 */
class RSDirectoryViewEntries extends JViewLegacy
{
    /**
     * The display function.
     *
     * @access public
     * 
     * @param mixed $tpl
     */
    public function display($tpl = null)
    {
		$doc = JFactory::getDocument();
		$app = JFactory::getApplication();
		$pagination_params = array();
		
		// Check if category can be accessed by current view level
		if ($requestedCategory = $app->input->getInt('category')) {
			$category = JTable::getInstance('Category', 'JTable');
			if ($category->load($requestedCategory)) {
				$levels = JFactory::getUser()->getAuthorisedViewLevels();
				$path 	= $category->getPath();
				
				foreach ($path as $parentCategory) {
					if (!in_array($parentCategory->access, $levels)) {
						throw new Exception(JText::_('COM_RSDIRECTORY_ACCESS_DENIED_TO_CATEGORY'), 403);
					}
				}
			}
			$category = RSDirectoryHelper::getCategory($requestedCategory);
			$pagination_params['category'] = RSDirectoryRoute::sef($requestedCategory, $category->alias);
		}
			
		if ( !$app->input->get('Itemid') )
		{
			// Add breadcrumb item.
			$pathway = JFactory::getApplication()->getPathway();
			$pathway->addItem( JText::_('COM_RSDIRECTORY_ENTRIES_VIEW_DEFAULT_TITLE') );
		}

		$breadcrumbsTree = $this->get('CategoriesTree');
		
		if ($breadcrumbsTree) {
			$pathway = JFactory::getApplication()->getPathway();
			foreach ($breadcrumbsTree as $treeItem) {
				$pathway->addItem( $treeItem->title, $treeItem->link );
			}
		}
		
		RSDirectoryHelper::loadRaty('.rsdir-entry-rating');

		JHtml::_('script', 'com_rsdirectory/script.js', array('relative' => true, 'version'=> 'auto'));

		$state = $this->get('State');
			
		// Get the page/component configuration.
		$params = $state->params;
			
		$this->pageclass_sfx = htmlspecialchars( $params->get('pageclass_sfx') );
		$this->items = $this->get('Items');
		$this->pagination = $this->get('Pagination');
		$this->sort_field = $this->get('SortField');
		$this->params = $params;

		// letters
        if ($params->get('use_alphabetical')) {
            $this->letters = $this->get('Letters');
            $this->currentLetter = $app->input->getString('alpha', '');
            // check for special chars group
            if ($this->currentLetter == JText::_('COM_RSDIRECTORY_SEF_SPECIAL'))
			{
                $this->currentLetter = '#';
            }
			
			if ($params->get('use_reset', 1) && strlen($this->currentLetter)) {
				$this->resetUrl = $this->get('LettersUrl');
			} else {
				$this->resetUrl = false;
			}
        }
			
		// Get the view.
		$this->view = $app->input->get('view');
		$pagination_params['view'] = $this->view;
		
		// Add the view and category if it's present
		if (!empty($pagination_params)) {
			foreach ($pagination_params as $param => $value) {
				$this->pagination->setAdditionalUrlParam($param, $value);
			}
		}
			
		$config = RSDirectoryConfig::getInstance();
			
		// Get the date and time display setting.
		$this->date_and_time_display = $config->get('date_and_time_display');
		$this->width = $config->get('small_thumbnail_width');
		$this->height = $config->get('small_thumbnail_height');
		$this->normal_width = $config->get('normal_thumbnail_width');
		$this->normal_height = $config->get('normal_thumbnail_height');
		$this->big_width = $config->get('big_thumbnail_width');
		$this->big_height = $config->get('big_thumbnail_height');
		$this->images_listing_detail_position = $config->get('images_listing_detail_position');
			
		// Get permissions.
		$this->can_edit_all_entries = RSDirectoryHelper::checkUserPermission('can_edit_all_entries');
		$this->can_edit_own_entries = RSDirectoryHelper::checkUserPermission('can_edit_own_entries');
		$this->can_delete_all_entries = RSDirectoryHelper::checkUserPermission('can_delete_all_entries');
        $this->can_delete_own_entries = RSDirectoryHelper::checkUserPermission('can_delete_own_entries');
			
		$this->user = JFactory::getUser();
			
		JText::script('COM_RSDIRECTORY_ENTRY_DELETION_CONFIRMATION');
		JText::script('COM_RSDIRECTORY_ENTRY_ADD_TO_FAVORITES');
		JText::script('COM_RSDIRECTORY_ENTRY_REMOVE_FROM_FAVORITES');
			
		// Add feed links
		if ( $params->get('show_feed_link', 1) )
		{
			$link = '&format=feed&limitstart=';
				
			$attribs = array('type' => 'application/rss+xml', 'title' => 'RSS 2.0');
			$this->document->addHeadLink( JRoute::_($link . '&type=rss'), 'alternate', 'rel', $attribs );
				
			$attribs = array('type' => 'application/atom+xml', 'title' => 'Atom 1.0');
			$this->document->addHeadLink( JRoute::_($link . '&type=atom'), 'alternate', 'rel', $attribs );
		}


		if ($config->get('use_listing_snippets')) {
			// add the schema.org snippets in json-ld format
			require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/snippets.php';

			// get the displayed entries ids
			$ids = array();
			if (!empty($this->items)) {
				foreach ($this->items as $entry) {
					$ids[] = (int)$entry->id;
				}
				try {
					$snippet = RSDirectorySnippets::getInstance($ids);
					$snippet->buildEntriesSnippets('listing_row');

					if (!empty($snippet->snippet)) {
						$doc->addScriptDeclaration($snippet->snippet, 'application/ld+json');
					}
				} catch (Exception $e) {

				}
			}
		}
		// end snippets
			
		$this->prepareDocument();
			
		parent::display($tpl);
    }
		
	/**
	 * Prepares the document.
	 *
	 * @access protected
	 */
	protected function prepareDocument()
	{
		$app = JFactory::getApplication();
		$menus = $app->getMenu();
			
		// Because the application sets a default page title, we need to get it from the menu item itself.
		$menu = $menus->getActive();
			
		// Get the categories filter.
		$categories = $app->input->get( 'categories', array(), 'array' );
			
		if ( isset($categories[0]) && !isset($categories[1]) )
		{
			$category = RSDirectoryHelper::getCategory($categories[0]);	
		}
			
		// Get the users filter.
		$users = $app->input->get( 'users', array(), 'array' );
			
		if ( isset($users[0]) && !isset($users[1]) )
		{
			$user = JFactory::getUser($users[0]);
		}
			
		if ( !empty($category) && $category->id != 'root' )
		{
			$page_heading = $category->title;
			$title = $category->title;
			$metadesc = $category->metadesc;
			$metakey = $category->metakey;
				
			$registry = new JRegistry($category->metadata);
			$metadata = $registry->toArray();
		}
		else
		{
			if ( empty($user->id) )
			{
				$page_heading = $this->params->get( 'page_title', empty($menu) ? JText::_('COM_RSDIRECTORY_ENTRIES_VIEW_DEFAULT_TITLE') : $menu->title );	
			}
			else
			{
				$page_heading = JText::sprintf( 'COM_RSDIRECTORY_ENTRIES_POSTED_BY', $this->escape($user->name) );
			}
				
			$title = $this->params->get('page_title', '');
			$metadesc = $this->params->get('menu-meta_description');
			$metakey = $this->params->get('menu-meta_keywords');
				
			$metadata = array(
				'robots' => $this->params->get('robots'),
			);
		}
		
		$alternateive_page_heading = $this->params->get('page_heading', '');
		if (empty($alternateive_page_heading)) {
			$this->params->set('page_heading', $page_heading);
		}
			
		// Set the browser title.
		$sitename = $app->get('sitename');
		$sitename_pagetitles = $app->get('sitename_pagetitles', 0);
			
		if ( empty($title) )
		{
			$title = $sitename;
		}
		else if ($sitename_pagetitles == 1)
		{
			$title = JText::sprintf('JPAGETITLE', $sitename, $title);
		}
		else if ($sitename_pagetitles == 2)
		{
			$title = JText::sprintf('JPAGETITLE', $title, $sitename);
		}
			
		$this->document->setTitle($title);
			
		// Set the meta description.
		if ($metadesc)
		{
			$metadesc = str_replace( array("\r\n", "\n"), ' ', $metadesc );
				
			$this->document->setDescription($metadesc);
		}
			
		// Set the meta keywords.
		if ($metakey)
		{
			$this->document->setMetadata('keywords', $metakey);
		}
			
		// Set the robots tag.
		if ( !empty($metadata['robots']) )
		{
			$this->document->setMetadata('robots', $metadata['robots']);
		}
	}
}