<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

$config = $this->config;
$form = $this->form;
$form_fields = $this->form_fields;
$entry = $this->entry;
$span = empty($this->images) ? 12 : 8
?>

<div class="<?php echo RsdirectoryAdapterGrid::column($span); ?>">

<?php
	$badges = array();
	$links  = array();
		
	if ($entry->promoted)
	{
		$badges[] = '<span class="rsdir-top-entry '.RsdirectoryAdapterGrid::badgeClass('info').'">' . JText::_('COM_RSDIRECTORY_TOP_ENTRY') . '</span>';
	}
		
	if ($this->expired)
	{
		$badges[] = '<span class="rsdir-expired '.RsdirectoryAdapterGrid::badgeClass('warning').'">' . JText::_('COM_RSDIRECTORY_EXPIRED') . '</span>';
	}
	else if (!$this->published)
	{ 
		$badges[] = '<span class="rsdir-unpublished '.RsdirectoryAdapterGrid::badgeClass('warning').'">' . JText::_('COM_RSDIRECTORY_UNPUBLISHED') . '</span>';
	}
	
	if ( empty($this->print) )
	{
		if ($this->can_edit_entry)
		{
			$links[] = '<li><a class="rsdir-edit'.(RSDirectoryHelper::isJ4() ? ' dropdown-item' : '').'" href="' . RSDirectoryRoute::getEntryURL($entry->id, $entry->title, 'edit') . '" title="' . JText::_('COM_RSDIRECTORY_EDIT_LINK_TITLE') . '"><i class="fa fa-edit"></i> ' . JText::_('COM_RSDIRECTORY_EDIT') . '</a></li>';
		}

		if ($this->can_delete_entry)
		{
			$links[] = '<li><a class="rsdir-detail-delete-entry'.(RSDirectoryHelper::isJ4() ? ' dropdown-item' : '').'" href="#" data-entry-id="' .  $entry->id . '" title="' . JText::_('COM_RSDIRECTORY_DELETE_LINK_TITLE') . '"><i class="fa fa-trash-o"></i> ' . JText::_('COM_RSDIRECTORY_DELETE') . '</a></li>';
		}
		
		if ($this->can_edit_entry && !$entry->paid)
		{
			$links[] = '<li class="'.(RSDirectoryHelper::isJ4() ? 'dropdown-' : '').'divider"></li>';
			$links[] = '<li><a class="rsdir-finalize'.(RSDirectoryHelper::isJ4() ? ' dropdown-item' : '').'" href="' . JRoute::_("index.php?option=com_rsdirectory&task=entry.finalize&id=$entry->id") . '" title="' . JText::_('COM_RSDIRECTORY_FINALIZE_LINK_TITLE') . '"><i class="fa fa-shopping-cart"></i> ' . JText::_('COM_RSDIRECTORY_FINALIZE') . '</a></li>';
		}
	}
	?>
	
	<?php if ($badges || $links) { ?>
	<div class="clearfix">
		<?php if ($badges) { ?>
			<?php echo implode(' ', $badges); ?>
		<?php } ?>
		
		<?php if ($links) { ?>
		<div class="<?php echo RSDirectoryHelper::isJ4() ? 'dropdown float-end' : 'btn-group pull-right'; ?>">
			<a class="btn dropdown-toggle<?php echo RSDirectoryHelper::isJ4() ? ' btn-sm btn-light' : ' btn-mini';?>" data-<?php echo RSDirectoryHelper::isJ4() ? 'bs-' : '';?>toggle="dropdown" href="#">
				<i class="fa fa-cog"></i> <span class="caret"></span>
			</a>
			<ul class="dropdown-menu">
				<?php echo implode('', $links); ?>
			</ul>
		</div>
		<?php } ?>
	</div>
	<?php } ?>
        
    <?php if ($form->listing_detail_show_title) { ?>
    <div class="page-header">
        <h2 class="rsdir-detail-title"><?php echo $this->escape($entry->title); ?></h2>
    </div>
    <?php } ?>
	
	<?php
		if ($form->listing_detail_show_ratings)
		{
			RSDirectoryHelper::loadRaty('.rsdir-detail-rating');
			?>
			<div class="rsdir-detail-rating-wrapper rsdir-entry-rating rsdir-detail-section">
				<div class="rsdir-detail-rating" id="rsdir-ratings-avg-rating" data-rating="<?php echo $this->escape($entry->avg_rating); ?>"><span><?php echo $entry->avg_rating; ?></span></div>
				<span id="rsdir-ratings-count-text">
				<?php
				if ($entry->ratings_count) {
					echo RSDirectoryHelper::getRatingText($entry->avg_rating, $entry->ratings_count);
				}
				?>
				</span>
			</div>
			<?php
		}
	?>
        
	<?php
	if ($form->listing_detail_show_big_subtitle)
	{
		if ( $big_subtitle_field = RSDirectoryHelper::findFormField('big_subtitle', $form_fields) )
		{
			$big_subtitle = RSDirectoryHelper::cleanText( $this->entry->big_subtitle, true, $big_subtitle_field->properties->get('clean_links') );
				
			if ($big_subtitle)
			{
				$big_subtitle = $big_subtitle_field->properties->get('allow_html') ? $big_subtitle : nl2br( strip_tags($big_subtitle) );
					
				?>
					
				<h3 class="rsdir-detail-big-subtitle rsdir-detail-section"><?php echo $big_subtitle; ?></h3>
					
				<?php
			}
		}
	}
		
	if ($form->listing_detail_show_small_subtitle)
	{
		if ( $small_subtitle_field = RSDirectoryHelper::findFormField('small_subtitle', $form_fields) )
		{
			$small_subtitle = RSDirectoryHelper::cleanText( $this->entry->small_subtitle, true, $small_subtitle_field->properties->get('clean_links') );
				
			if ($small_subtitle)
			{
				$small_subtitle = $small_subtitle_field->properties->get('allow_html') ? $small_subtitle : nl2br( strip_tags($small_subtitle) );
					
				?>
					
				<div class="rsdir-detail-small-subtitle rsdir-detail-section"><?php echo $small_subtitle; ?></div>
					
				<?php
			}
		}
	}
		
	$price_field = RSDirectoryHelper::findFormField('price', $form_fields)
		
	?>
        
    <?php if ( ($form->listing_detail_show_price && $price_field) ) { ?>
    <div class="rsdir-price-wrapper <?php echo RSDirectoryHelper::isJ4() ? 'card mb-3' : 'well well-small' ?> clearfix">
		<div class="rsdir-detail-price<?php echo RSDirectoryHelper::isJ4() ? ' card-body' : '' ?>">
			<i class="fa fa-tag"></i>
			<strong class="rsdir-detail-price-label hidden-tablet d-none d-md-inline-block">
				<?php echo $this->escape( $price_field->properties->get('listing_caption', JText::_('COM_RSDIRECTORY_LISTING_PRICE_LABEL')) ); ?>
			</strong>
			<?php echo RSDirectoryHelper::formatPrice($entry->price); ?>
			<?php if ($entry->expiry_time != '0000-00-00 00:00:00') { ?>
				<time datetime="<?php echo $this->escape($entry->expiry_time); ?>"></time>
			<?php } ?>
		</div>
    </div>
    <?php } ?>
        
    <?php if ( ($entry_meta = RSDirectoryHelper::getEntryMeta($entry, $form, 'listing_detail')) || $form->listing_detail_show_expiry_time ) { ?>
	<div class="rsdir-detail-meta rsdir-detail-section clearfix">
		<?php echo $entry_meta; ?>
        <?php if ($form->listing_detail_show_expiry_time) { ?>
			<i class="fa fa-clock-o <?php echo RSDirectoryHelper::getTooltipClass(); ?>" title="<?php echo JText::_('COM_RSDIRECTORY_ENTRY_TOOLTIP_EXPIRES'); ?>"></i>
				<span class="muted"><?php echo $this->escape(RSDirectoryHelper::formatExpiryDate($entry->expiry_time)); ?></span>
		<?php } ?>
	</div>
    <?php } ?>
		
	<?php if ( empty($this->print) && !empty($this->contact) && ( empty($this->images) || !RSDirectoryHelper::findFormField('images', $form_fields) ) ) { ?>
	<div class="rsdir-contact-wrapper">
		<?php echo $this->contact; ?>
	</div>
	<?php } ?>
</div>