<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * My Account view.
 */
class RSDirectoryViewMyAccount extends JViewLegacy
{
    /**
     * The display function.
     *
     * @access public
     * 
     * @param mixed $tpl
     */
    public function display($tpl = null)
    {
        // Get mainframe.
        $app = JFactory::getApplication();
            
        // Only logged in users can access this page.
        $user = JFactory::getUser();
        if ( !$user->id )
        {
            return $app->enqueueMessage( JText::_('JGLOBAL_YOU_MUST_LOGIN_FIRST') );
        }
            
        if ( !$app->input->get('Itemid') )
		{
			// Add breadcrumb item.
			$pathway = $app->getPathway();
			$pathway->addItem( JText::_('COM_RSDIRECTORY_YOUR_ACCOUNT') );
		}
            
        if ( $tab = $app->input->get('tab') )
        {
            $script = 'jQuery(function($){ $( document.getElementById("com-rsdirectory-configuration") ).find("a[href=\'#' . $tab . '\']").click(); });';
                
            JFactory::getDocument()->addScriptDeclaration($script);
        }
            
        $state = $this->get('State');
            
        // Get the page/component configuration.
        $params = $state->params;
            
        // Get the credits history model.
        $creditshistory_model = RSDirectoryModel::getInstance('CreditsHistory');
            
        $this->pageclass_sfx = htmlspecialchars( $params->get('pageclass_sfx') );
        $this->params = $params;
        $this->rstabs = $this->get('RSTabs');
        $this->form = $this->get('Form');
        $this->current_credits = RSDirectoryCredits::getUserCredits();
        $this->spent_credits = RSDirectoryCredits::getUserSpentCredits();
        $this->posted_entries = $this->get('PostedEntriesCount');
        $this->transactions = $this->get('Transactions');
        $this->credits_history = $creditshistory_model->getItems();

        $config 	= RSDirectoryConfig::getInstance();
        $this->allow_self_anonymisation = $config->get('allow_self_anonymisation', 0) && !$user->authorise('core.admin');
        $this->anonymise_joomla_data    = $config->get('anonymise_joomla_data', 1);
        $this->email = $user->email;

        $this->use_invoice = $config->get('use_general_invoice', 0);
            
        $pagination = $creditshistory_model->getPagination();
        $pagination->setAdditionalUrlParam('view', 'myaccount');
        $pagination->setAdditionalUrlParam('tab', 'credits-history');
            
        $this->pagination = $pagination;

        $this->prepareDocument();

        parent::display($tpl);
    }

    protected function prepareDocument() {
        $app = JFactory::getApplication();

        // Description
        if ($this->params->get('menu-meta_description')) {
            $this->document->setDescription($this->params->get('menu-meta_description'));
        }

        // Keywords
        if ($this->params->get('menu-meta_keywords')) {
            $this->document->setMetadata('keywords', $this->params->get('menu-meta_keywords'));
        }

        // Robots
        if ($this->params->get('robots')) {
            $this->document->setMetadata('robots', $this->params->get('robots'));
        }

        // Add meta information from category, if one has been requested
        if (!empty($this->category)) {
            if (!empty($this->category->meta_description)) {
                $this->document->setMetaData('description', $this->category->meta_description);
            }
            if (!empty($this->category->meta_keywords)) {
                $this->document->setMetaData('keywords', $this->category->meta_keywords);
            }
        }

        // Add site name
        $title = $this->params->get('page_title', '');

        // Check for empty title and add site name if param is set
        if (empty($title))
        {
            $title = $app->get('sitename');
        }
        elseif ($app->get('sitename_pagetitles', 0) == 1)
        {
            $title = JText::sprintf('JPAGETITLE', $app->get('sitename'), $title);
        }
        elseif ($app->get('sitename_pagetitles', 0) == 2)
        {
            $title = JText::sprintf('JPAGETITLE', $title, $app->get('sitename'));
        }

        $this->document->setTitle($title);
    }
}