<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

$item = $this->item;
$form = $item->form;
$form_fields = $form->fields;

?>

<div class="<?php echo RsdirectoryAdapterGrid::column(9); ?> rsdir-listing-detail-part">
		
	<?php

	$badges = array();
	$links  = array();

	if ($item->promoted)
	{
		$badges[] = '<span class="rsdir-top-entry '.RsdirectoryAdapterGrid::badgeClass('info').'">' . JText::_('COM_RSDIRECTORY_TOP_ENTRY') . '</span>';
	}

	if ($this->expired)
	{
		$badges[] = '<span class="rsdir-expired '.RsdirectoryAdapterGrid::badgeClass('warning').'">' . JText::_('COM_RSDIRECTORY_EXPIRED') . '</span>';
	}
	else if (!$this->published)
	{
		$badges[] = '<span class="rsdir-unpublished '.RsdirectoryAdapterGrid::badgeClass('warning').'">' . JText::_('COM_RSDIRECTORY_UNPUBLISHED') . '</span>';
	}

	if ( empty($this->print) )
	{
		$canEdit = !empty($this->can_edit_all_entries) || ( !empty($this->can_edit_own_entries) && $this->user->id == $item->user_id && $this->user->id && $item->user_id );

		if ( $canEdit )
		{
			$links[] = '<li><a class="rsdir-edit'.(RSDirectoryHelper::isJ4() ? ' dropdown-item' : '').'" href="' . RSDirectoryRoute::getEntryURL($item->id, $item->title, 'edit') . '" title="' . JText::_('COM_RSDIRECTORY_EDIT_LINK_TITLE') . '"><i class="fa fa-edit"></i> ' . JText::_('COM_RSDIRECTORY_EDIT') . '</a></li>';
		}

		if ( !empty($this->can_delete_all_entries) || ( !empty($this->can_delete_own_entries) && $this->user->id == $item->user_id && $this->user->id && $item->user_id ) )
		{
			$links[] = '<li><a class="rsdir-listing-delete'.(RSDirectoryHelper::isJ4() ? ' dropdown-item' : '').'" href="#" data-entry-id="' .  $item->id . '" title="' . JText::_('COM_RSDIRECTORY_DELETE_LINK_TITLE') . '"><i class="fa fa-trash-o"></i> ' . JText::_('COM_RSDIRECTORY_DELETE') . '</a></li>';
		}

		if ($canEdit && !$this->item->paid)
		{
			$links[] = '<li class="'.(RSDirectoryHelper::isJ4() ? 'dropdown-' : '').'divider"></li>';
			$links[] = '<li><a class="rsdir-finalize'.(RSDirectoryHelper::isJ4() ? ' dropdown-item' : '').'" href="' . JRoute::_("index.php?option=com_rsdirectory&task=entry.finalize&id=$item->id") . '" title="' . JText::_('COM_RSDIRECTORY_FINALIZE_LINK_TITLE') . '"><i class="fa fa-shopping-cart"></i> ' . JText::_('COM_RSDIRECTORY_FINALIZE') . '</a></li>';
		}
	}
		
	?>
		
	<?php if ($badges || $links) { ?>
	<div class="clearfix<?php echo $this->images_listing_detail_position == 'left' ? ' text-right' : ''; ?>">
		<?php if ($badges) { ?>
			<?php echo implode(' ', $badges); ?>
		<?php } ?>

		<?php if ($links) { ?>
			<div class="<?php echo RSDirectoryHelper::isJ4() ? 'dropdown float-end' : 'btn-group pull-right'; ?>">
				<button type="button" class="btn dropdown-toggle<?php echo RSDirectoryHelper::isJ4() ? ' btn-sm btn-light' : ' btn-mini';?>" data-<?php echo RSDirectoryHelper::isJ4() ? 'bs-' : '';?>toggle="dropdown">
					<i class="fa fa-cog"></i> <span class="caret"></span>
				</button>
				<ul class="dropdown-menu">
					<?php echo implode('', $links); ?>
				</ul>
			</div>
		<?php } ?>
	</div>
	<?php } ?>
		
	<h2 class="rsdir-listing-title">
		<a href="<?php echo $this->item_url; ?>" title="<?php echo $this->title_str; ?>"><?php echo $this->title_str; ?></a>
	</h2>
		
	<?php
	if ($form->listing_row_show_big_subtitle)
	{
		if ( $big_subtitle_field = RSDirectoryHelper::findFormField('big_subtitle', $form_fields) )
		{
			$big_subtitle = RSDirectoryHelper::cleanText( $item->big_subtitle, true, $big_subtitle_field->properties->get('clean_links') );
				
			if ($big_subtitle)
			{
				$big_subtitle = $big_subtitle_field->properties->get('allow_html') ? $big_subtitle : nl2br( strip_tags($big_subtitle) );
					
				?>
					
				<h3 class="rsdir-listing-big-subtitle"><?php echo $big_subtitle; ?></h3>
					
				<?php
			}
		}
	}
		
	if ($form->listing_row_show_small_subtitle)
	{
		if ( $small_subtitle_field = RSDirectoryHelper::findFormField('small_subtitle', $form_fields) )
		{
			$small_subtitle = RSDirectoryHelper::cleanText( $item->small_subtitle, true, $small_subtitle_field->properties->get('clean_links') );
				
			if ($small_subtitle)
			{
				$small_subtitle = $small_subtitle_field->properties->get('allow_html') ? $small_subtitle : nl2br( strip_tags($small_subtitle) );
					
				?>
					
				<div class="rsdir-listing-small-subtitle"><?php echo $small_subtitle; ?></div>
					
				<?php
			}
		}
	}
		
	if (($meta = RSDirectoryHelper::getEntryMeta($item, $form)) || $form->listing_row_show_expiry_time)
	{
		?>
			
		<div class="rsdir-listing-meta clearfix">
			<div class="<?php echo RsdirectoryAdapterGrid::column(12)?>">
				<?php echo $meta; ?>
				<?php if ($form->listing_row_show_expiry_time) { ?>
					<i class="fa fa-clock-o <?php echo RSDirectoryHelper::getTooltipClass(); ?>" title="<?php echo JText::_('COM_RSDIRECTORY_ENTRY_TOOLTIP_EXPIRES'); ?>"></i>
					<span class="muted"><?php echo $this->escape(RSDirectoryHelper::formatExpiryDate($item->expiry_time)); ?></span>
				<?php } ?>
			</div>
		</div><!-- .rsdir-listing-meta -->
	<?php
	}
	?>
        
</div>