<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * My Entries view.
 */
class RSDirectoryViewMyEntries extends JViewLegacy
{
    /**
     * The display function.
     *
     * @access public
     * 
     * @param mixed $tpl
     */
    public function display($tpl = null)
    {
        $app = JFactory::getApplication();
            
        // Only logged in users can access this page.
        if ( !JFactory::getUser()->id )
        {
            return $app->enqueueMessage( JText::_('JGLOBAL_YOU_MUST_LOGIN_FIRST') );
        }
            
        if ( !$app->input->get('Itemid') )
		{
			// Add breadcrumb item.
			$pathway = $app->getPathway();
			$pathway->addItem( JText::_('COM_RSDIRECTORY_YOUR_ENTRIES') );
		}

        JHtml::_('script', 'com_rsdirectory/script.js', array('relative' => true, 'version'=> 'auto'));
            
        $state = $this->get('State');
            
        // Get the page/component configuration.
        $params = $state->params;
            
        $this->pageclass_sfx = htmlspecialchars( $params->get('pageclass_sfx') );
        $this->items = $this->get('Items');
        $this->pagination = $this->get('Pagination');
        $this->sort_field = $this->get('SortField');
        $this->params = $params;
			
		// Get the view.
        $this->view = JFactory::getApplication()->input->get('view');
            
        $config = RSDirectoryConfig::getInstance();
            
        $this->date_and_time_display = $config->get('date_and_time_display');
        $this->width = $config->get('small_thumbnail_width');
        $this->height = $config->get('small_thumbnail_height');
		$this->normal_width = $config->get('normal_thumbnail_width');
		$this->normal_height = $config->get('normal_thumbnail_height');
		$this->big_width = $config->get('big_thumbnail_width');
		$this->big_height = $config->get('big_thumbnail_height');
        $this->images_listing_detail_position = $config->get('images_listing_detail_position');
            
		// Get permissions.
        $this->can_edit_own_entries = RSDirectoryHelper::checkUserPermission('can_edit_own_entries');
        $this->can_delete_own_entries = RSDirectoryHelper::checkUserPermission('can_delete_own_entries');
		
		$this->user = JFactory::getUser();
			
        JText::script('COM_RSDIRECTORY_ENTRY_DELETION_CONFIRMATION');
        JText::script('COM_RSDIRECTORY_ENTRY_ADD_TO_FAVORITES');
        JText::script('COM_RSDIRECTORY_ENTRY_REMOVE_FROM_FAVORITES');

        $this->prepareDocument();

        parent::display($tpl);
    }

    protected function prepareDocument() {
        $app = JFactory::getApplication();

        // Description
        if ($this->params->get('menu-meta_description')) {
            $this->document->setDescription($this->params->get('menu-meta_description'));
        }

        // Keywords
        if ($this->params->get('menu-meta_keywords')) {
            $this->document->setMetadata('keywords', $this->params->get('menu-meta_keywords'));
        }

        // Robots
        if ($this->params->get('robots')) {
            $this->document->setMetadata('robots', $this->params->get('robots'));
        }

        // Add meta information from category, if one has been requested
        if (!empty($this->category)) {
            if (!empty($this->category->meta_description)) {
                $this->document->setMetaData('description', $this->category->meta_description);
            }
            if (!empty($this->category->meta_keywords)) {
                $this->document->setMetaData('keywords', $this->category->meta_keywords);
            }
        }

        // Add site name
        $title = $this->params->get('page_title', '');

        // Check for empty title and add site name if param is set
        if (empty($title))
        {
            $title = $app->get('sitename');
        }
        elseif ($app->get('sitename_pagetitles', 0) == 1)
        {
            $title = JText::sprintf('JPAGETITLE', $app->get('sitename'), $title);
        }
        elseif ($app->get('sitename_pagetitles', 0) == 2)
        {
            $title = JText::sprintf('JPAGETITLE', $title, $app->get('sitename'));
        }

        $this->document->setTitle($title);
    }
}