<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Radius Search view.
 */
class RSDirectoryViewRadius extends JViewLegacy
{
    /**
     * The display function.
     *
     * @access public
     * 
     * @param mixed $tpl
     */
    public function display($tpl = null)
    {
		if ($this->getLayout() == 'window') {
			// Set parameters
			$this->params = $this->get('params');
			
			// Get entry
			$this->entry = $this->get('Entry');
			
			if (empty($this->entry)) {
				throw new Exception(JText::_('COM_RSDIRECTORY_PAGE_NOT_FOUND'), 404);
			}
			
			// Check if category can be accessed by current view level
			$category = JTable::getInstance('Category', 'JTable');
			if ($category->load($this->entry->category_id)) {
				$levels = JFactory::getUser()->getAuthorisedViewLevels();
				$path 	= $category->getPath();
				
				foreach ($path as $parentCategory) {
					if (!in_array($parentCategory->access, $levels)) {
						throw new Exception(JText::_('COM_RSDIRECTORY_ACCESS_DENIED_TO_ENTRY'), 403);
					}
				}
			}

			// get the Item Id
			$itemId = $this->params->get('itemid', 0);
			
			// Get entry URL
			$this->url = RSDirectoryRoute::getEntryURL($this->entry->id, $this->entry->title, '', $itemId);
			
			// Info window dimensions
			$this->width 	= (int) $this->params->get('info_window_width', 300);
			$this->height 	= (int) $this->params->get('info_window_height', 120);
			
			// Thumbnail dimensions
			$this->thumbnails_width  = (int) $this->params->get('thumbnails_width');
			$this->thumbnails_height = (int) $this->params->get('thumbnails_height');
			
			// Find images
			$images_field = RSDirectoryHelper::findFormField('images', $this->entry->form->fields);
			if ( empty($images_field) || empty($images_field->files) ) {
				$this->thumbnail_url = false;
			} else {
				$this->thumbnail_url = RSDirectoryHelper::getImageURL($images_field->files[0]->hash, 'small');
			}
		} else {
			$app = JFactory::getApplication();
			$doc = JFactory::getDocument();
				
			if ( !$app->input->get('Itemid') )
			{
				// Add breadcrumb item.
				$pathway = JFactory::getApplication()->getPathway();
				$pathway->addItem( JText::_('COM_RSDIRECTORY_RADIUS_SEARCH') );
			}
				
			$state = $this->get('State');
				
				
			// Get the page/component configuration.
			$params = $state->params;
			
			$this->radius 			= (int) $params->get('default_radius', '100');
			$this->location 		= $params->get('default_location');
			$this->unit		 		= $params->get('default_unit');
			$this->use_geolocation 	= $params->get('use_geolocation', 1);
			
			$use_show_more = (int) $params->get('use_show_more', 0);
			$config = JFactory::getConfig();
			$limit = (int) $config->get('list_limit');

			// Load the Radius Map
			$map_params = array(
				'location' => $this->location,
				'default_zoom' => $params->get('default_zoom', '5'),
				'info_window_width' => $params->get('info_window_width', 300),
				'info_window_height' => $params->get('info_window_height', 120),
				'circle_color' => $params->get('circle_color', '#ff8080'),
				'circle_stroke_color' => $params->get('circle_stroke_color', '#000000'),
				'circle_opacity' => (int) $params->get('circle_opacity', '35'),
				'hide_circle' => (int) $params->get('hide_radius_circle', '0'),
				'use_show_more' => $use_show_more,
				'limit' => $limit
			);

			RSDirectoryHelper::loadRadiusMap($map_params);
				
			if ( $params->get('filtering') )
			{
				require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/filter.php';
					
				$this->fields = RSDirectoryHelper::getFilterFields( $params->get('featured_categories') );
			}
				
			if ( $params->get('show_ratings') )
			{
				JText::script('COM_RSDIRECTORY_RATY_BAD');
				JText::script('COM_RSDIRECTORY_RATY_POOR');
				JText::script('COM_RSDIRECTORY_RATY_REGULAR');
				JText::script('COM_RSDIRECTORY_RATY_GOOD');
				JText::script('COM_RSDIRECTORY_RATY_GORGEOUS');
				
				RSDirectoryHelper::loadRaty();
			}
			
			if ($use_show_more) {
				JText::script('COM_RSDIRECTORY_SHOW_MORE');
			}
				
			if ( $params->get('show_favorites_button') )
			{
				JText::script('COM_RSDIRECTORY_ENTRY_ADD_TO_FAVORITES');
				JText::script('COM_RSDIRECTORY_ENTRY_REMOVE_FROM_FAVORITES');	
			}
				
			$this->width = $params->get('width', '100%');
				
			if ( is_numeric($this->width) )
			{
				$this->width .= 'px';
			}
				
			$this->pageclass_sfx = htmlspecialchars( $params->get('pageclass_sfx') );
			$this->params = $params;
			$this->options = array(
				'more' => $params->get('more'),
			);
				
			if ( $params->get('more') )
			{
				JText::script('COM_RSDIRECTORY_SHOW_MORE');
				JText::script('COM_RSDIRECTORY_SHOW_LESS');
			}
				
			JText::script('COM_RSDIRECTORY_LOADING');

			$this->prepareDocument();
		}
			
		parent::display($tpl);
    }

	protected function prepareDocument() {
		$app = JFactory::getApplication();

		// Description
		if ($this->params->get('menu-meta_description')) {
			$this->document->setDescription($this->params->get('menu-meta_description'));
		}

		// Keywords
		if ($this->params->get('menu-meta_keywords')) {
			$this->document->setMetadata('keywords', $this->params->get('menu-meta_keywords'));
		}

		// Robots
		if ($this->params->get('robots')) {
			$this->document->setMetadata('robots', $this->params->get('robots'));
		}

		// Add meta information from category, if one has been requested
		if (!empty($this->category)) {
			if (!empty($this->category->meta_description)) {
				$this->document->setMetaData('description', $this->category->meta_description);
			}
			if (!empty($this->category->meta_keywords)) {
				$this->document->setMetaData('keywords', $this->category->meta_keywords);
			}
		}

		// Add site name
		$title = $this->params->get('page_title', '');

		// Check for empty title and add site name if param is set
		if (empty($title))
		{
			$title = $app->get('sitename');
		}
		elseif ($app->get('sitename_pagetitles', 0) == 1)
		{
			$title = JText::sprintf('JPAGETITLE', $app->get('sitename'), $title);
		}
		elseif ($app->get('sitename_pagetitles', 0) == 2)
		{
			$title = JText::sprintf('JPAGETITLE', $title, $app->get('sitename'));
		}

		$this->document->setTitle($title);
	}
}